@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -83,7 +83,7 @@ namespace Constants {
     const char *const XML_ATTRIB_FORMAT           = "format";
 }
 
-inline static void warnXmlReadError(const QString &file, const QString &msg, const int line = 0, const int col = 0)
+static inline void warnXmlReadError(const QString &file, const QString &msg, const int line = 0, const int col = 0)
 {
     LOG_ERROR_FOR("CommandLine",
                   Trans::ConstantTranslations::tkTr(Trans::Constants::FILE_1_ISNOT_READABLE).arg(file) + " ; " +
@@ -97,6 +97,9 @@ public:
     CommandLinePrivate()
     {
         params.insert(Core::Constants::CL_MedinTux,          "--medintux");
+        params.insert(Core::Constants::CL_Independant,       "--independant");
+        params.insert(Core::Constants::CL_Movements,         "--movements");
+        params.insert(Core::Constants::CL_PreferedReceipts,  "--prefered-receipts");
         params.insert(Core::Constants::CL_Test,              "--test");
         params.insert(Core::Constants::CL_ReceiptsCreator,   "--receipts-creator");
         params.insert(Core::Constants::CL_EMR_Name,          "--emr-name");
@@ -147,6 +150,9 @@ public:
             switch (params.key(k,-1))
             {
             case Core::Constants::CL_MedinTux :          value.insert(Core::Constants::CL_MedinTux, true); break;
+            case Core::Constants::CL_Independant :       value.insert(Core::Constants::CL_Independant, true); break;
+            case Core::Constants::CL_Movements :         value.insert(Core::Constants::CL_Movements, true); break;
+            case Core::Constants::CL_PreferedReceipts :  value.insert(Core::Constants::CL_PreferedReceipts, true); break;
             case Core::Constants::CL_Test :              value.insert(Core::Constants::CL_Test, true); break;
             case Core::Constants::CL_ReceiptsCreator :   value.insert(Core::Constants::CL_ReceiptsCreator, true); break;
             case Core::Constants::CL_EMR_Name :          value.insert(Core::Constants::CL_EMR_Name, a.mid(a.indexOf("=")+1).remove("\"")); break;
@@ -277,6 +283,7 @@ public:
 //            <ICD10 value="J11.0;A22;Z23"/>
 //        </Patient>
         QDomElement element = rootElement.firstChildElement();
+
         while (!element.isNull()) {
             if (element.tagName() == Internal::Constants::XML_PATIENT_IDENTITY) {
                 value.insert(Core::Constants::CL_PatientName, element.attribute(Internal::Constants::XML_ATTRIB_LASTNAMES));
@@ -294,7 +301,7 @@ public:
     {
         patient->setData(patient->index(0, IPatient::Uid),            value.value(Core::Constants::CL_PatientUid));
         patient->setData(patient->index(0, IPatient::DateOfBirth),    value.value(Core::Constants::CL_DateOfBirth));
-        patient->setData(patient->index(0, IPatient::BirthName),      value.value(Core::Constants::CL_PatientName));
+        patient->setData(patient->index(0, IPatient::UsualName),      value.value(Core::Constants::CL_PatientName));
         patient->setData(patient->index(0, IPatient::Firstname),      value.value(Core::Constants::CL_PatientFirstname));
         patient->setData(patient->index(0, IPatient::Gender),         value.value(Core::Constants::CL_PatientGender));
     }
