@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developper : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main Developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "mainwindow.h"
@@ -46,6 +46,7 @@
 #include <coreplugin/actionmanager/mainwindowactions.h>
 #include <coreplugin/actionmanager/mainwindowactionhandler.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/contextmanager/contextmanager.h>
 #include <coreplugin/dialogs/plugindialog.h>
 #include <coreplugin/dialogs/settingsdialog.h>
@@ -57,6 +58,8 @@
 #include <accountplugin/accountview.h>
 
 #include <accountplugin/receipts/receiptviewer.h>
+#include <accountplugin/movements/movementsviewer.h>
+#include <accountplugin/receipts/preferredreceipts.h>
 
 #include <extensionsystem/pluginerrorview.h>
 #include <extensionsystem/pluginview.h>
@@ -76,6 +79,7 @@
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 #include <QDockWidget>
+#include <QMenu>
 
 using namespace MainWin;
 using namespace MainWin::Internal;
@@ -122,6 +126,7 @@ bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
     Q_UNUSED(errorString);
     // create menus
     createFileMenu();
+    createFileNewSubMenu();
     Core::ActionContainer *fmenu = actionManager()->createMenu(Core::Constants::M_FILE_RECENTFILES);
     connect(fmenu->menu(), SIGNAL(aboutToShow()),this, SLOT(aboutToShowRecentFiles()));
     createEditMenu();
@@ -131,7 +136,6 @@ bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
 
     Core::MainWindowActions actions;
     actions.setFileActions(
-            Core::MainWindowActions::A_FileNew  |
             Core::MainWindowActions::A_FileOpen |
             Core::MainWindowActions::A_FileSave |
             Core::MainWindowActions::A_FileSaveAs |
@@ -163,21 +167,10 @@ bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
 
 void MainWindow::extensionsInitialized()
 {
-    // Update countdown to dosage transmission
-    int count = settings()->value(Internal::SETTINGS_COUNTDOWN,0).toInt();
-    ++count;
-//    if ((count==30) || (commandLine()->value(Core::Internal::CommandLine::CL_TransmitDosage).toBool())) {
-//        messageSplash(tr("Transmitting posologies..."));
-//        settings()->setValue(Internal::SETTINGS_COUNTDOWN,0);
-//        transmitDosage();
-//    } else {
-        settings()->setValue(Internal::SETTINGS_COUNTDOWN,count);
-//    }
-
-        // Creating MainWindow interface
-        m_ui = new Internal::Ui::MainWindow();
-        m_ui->setupUi(this);
-        setWindowTitle(qApp->applicationName() + " - " + qApp->applicationVersion());
+    // Creating MainWindow interface
+    m_ui = new Internal::Ui::MainWindow();
+    m_ui->setupUi(this);
+    setWindowTitle(qApp->applicationName() + " - " + qApp->applicationVersion());
 
     // Disable some actions when starting as medintux plugin
     if (commandLine()->value(Core::Constants::CL_MedinTux).toBool()) {
@@ -215,24 +208,9 @@ void MainWindow::extensionsInitialized()
 
     // Start the update checker
     if (updateChecker()->needsUpdateChecking(settings()->getQSettings())) {
-        messageSplash(tkTr(Trans::Constants::CHECKING_UPDATES));
-        Utils::Log::addMessage(this, tkTr(Trans::Constants::CHECKING_UPDATES));
-        statusBar()->addWidget(new QLabel(tkTr(Trans::Constants::CHECKING_UPDATES), this));
-        statusBar()->addWidget(updateChecker()->progressBar(this),1);
-        connect(updateChecker(), SIGNAL(updateFound()), this, SLOT(updateFound()));
-        connect(updateChecker(), SIGNAL(done(bool)), this, SLOT(updateCheckerEnd()));
-        updateChecker()->check(Utils::Constants::FREEACCOUNT_UPDATE_URL);
-        settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
-    }
-
-
-    // Here we set the UI according to the commandline parser
-    if (commandLine()->value(Core::Constants::CL_ReceiptsCreator).toBool()) {
-        setCentralWidget(new ReceiptViewer(this));
-        if (WarnLogMessage)
-            LOG("receiptGUI initialized");
-    } else {
-        setCentralWidget(new ReceiptViewer(this));
+        settings()->setPath(Core::ISettings::UpdateUrl, Utils::Constants::FREEACCOUNT_UPDATE_URL);
+        if (checkUpdate())
+            settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
     }
 
     userChanged();
@@ -258,6 +236,31 @@ void MainWindow::postCoreOpened()
         qWarning() << "MainWindow::postCoreOpened()";
 
     finishSplash(this);
+
+    // Here we set the UI according to the commandline parser
+    if (commandLine()->value(Core::Constants::CL_ReceiptsCreator).toBool())
+    {
+        setCentralWidget(new ReceiptViewer(this));
+        if (WarnLogMessage)
+            LOG("receiptGUI initialized");
+    }
+    else if (commandLine()->value(Core::Constants::CL_PreferedReceipts).toBool())
+    {
+        setCentralWidget(new PreferredReceipts(this));
+        if (WarnLogMessage)
+            LOG("prefered receipts GUI initialized");
+        }
+    else if (commandLine()->value(Core::Constants::CL_Movements).toBool())
+    {
+        setCentralWidget(new MovementsViewer(this));
+        if (WarnLogMessage)
+            LOG("movements GUI initialized");
+        }
+    else {
+        setCentralWidget(new ReceiptViewer(this));
+    }
+
+
     actionManager()->retranslateMenusAndActions();
     contextManager()->updateContext();
     raise();
@@ -266,7 +269,7 @@ void MainWindow::postCoreOpened()
 
 /**
   \brief Refresh the ui data refering to the patient
-  \sa Core::Internal::CoreImpl::instance()->patient(), diPatient
+  \sa Core::Internal::CoreImpl::instance()->patient()
 */
 void MainWindow::refreshPatient()
 {
@@ -301,7 +304,7 @@ void MainWindow::closeEvent( QCloseEvent *event )
 //        if (commandLine()->value(Core::Internal::CommandLine::CL_MedinTux).toBool()) {
 //            QString tmp = DrugsDB::DrugsIO::instance()->prescriptionToHtml(drugModel());
 //            tmp.replace("font-weight:bold;", "font-weight:600;");
-//            Utils::saveStringToFile(Utils::toHtmlAccent(tmp) , exfile, Utils::DontWarnUser);
+//            Utils::saveStringToFile(Utils::htmlReplaceAccents(tmp) , exfile, Utils::DontWarnUser);
 //        } else {
 //            savePrescription(exfile);
 //        }
@@ -356,7 +359,7 @@ void MainWindow::userChanged()
     setWindowTitle(qApp->applicationName() + " - " + qApp->applicationVersion() + " // " + user()->value(Core::IUser::Uuid).toString());
 }
 
-void MainWindow::updateCheckerEnd()
+void MainWindow::updateCheckerEnd(bool)
 {
     delete statusBar();
 }
@@ -398,10 +401,7 @@ bool MainWindow::newFile()
     return true;
 }
 
-/**
-  \brief Open the preferences dialog
-  \sa mfDrugsPreferences
-*/
+/** Open the preferences dialog */
 bool MainWindow::applicationPreferences()
 {
     Core::SettingsDialog dlg(this);
@@ -409,19 +409,12 @@ bool MainWindow::applicationPreferences()
     return true;
 }
 
-/** \brief Change the font of the viewing widget */
 void MainWindow::changeFontTo(const QFont &font)
 {
     Q_UNUSED(font);
-//    m_ui->m_CentralWidget->changeFontTo(font);
-//    m_ui->patientName->setFont(font);
 }
 
 
-/**
-  \brief Prints the prescription using the header, footer and watermark.
-  \sa tkPrinter
-*/
 bool MainWindow::print()
 {
 //    return m_ui->m_CentralWidget->printPrescription();
@@ -443,37 +436,14 @@ bool MainWindow::configureMedintux()
 
 bool MainWindow::saveAsFile()
 {
-    return savePrescription();
+    return true;
 }
 
 bool MainWindow::saveFile()
 {
-    return savePrescription();
-}
-
-/** Saves a prescription. If fileName is empty, user is ask about a file name. */
-bool MainWindow::savePrescription(const QString &fileName)
-{
-    Q_UNUSED(fileName);
-    // TODO: code here */
-//    QString xmlExtra = patient()->toXml();
-//    if (commandLine()->value(Core::Internal::CommandLine::CL_EMR_Name).isValid()) {
-//        xmlExtra.append(QString("<EMR name=\"%1\"").arg(commandLine()->value(Core::Internal::CommandLine::CL_EMR_Name).toString()));
-//        if (commandLine()->value(Core::Internal::CommandLine::CL_EMR_Name).isValid()) {
-//            xmlExtra.append(QString(" uid=\"%1\"").arg(commandLine()->value(Core::Internal::CommandLine::CL_EMR_Uid).toString()));
-//        }
-//        xmlExtra.append("/>");
-//    }
-//    qWarning() << xmlExtra;
-//    return DrugsDB::DrugsIO::savePrescription(drugModel(), xmlExtra, fileName);
     return true;
 }
 
-/**
-  \brief Opens a prescription saved using savePrescription().
-  \sa savePrescription()
-  \sa DrugsIO
-*/
 bool MainWindow::openFile()
 {
 //    QString f = QFileDialog::getOpenFileName(this,
@@ -524,58 +494,3 @@ void MainWindow::createDockWindows()
 //    menu->addAction(dock->toggleViewAction());
 }
 
-
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientName_textChanged(const QString &text)
-//{
-//    patient()->setValue(Core::Patient::Name, text);
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientSurname_textChanged(const QString &text)
-//{
-//    patient()->setValue(Core::Patient::Surname, text);
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_sexCombo_currentIndexChanged(const QString &text)
-//{
-//    patient()->setValue(Core::Patient::Gender, text);
-//    refreshPatient();
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientWeight_valueChanged(const QString &text)
-//{
-//    patient()->setValue(Core::Patient::Weight, text);
-//    refreshPatient();
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientSize_valueChanged(const QString & text)
-//{
-//    patient()->setValue(Core::Patient::Height, text);
-//    refreshPatient();
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientClCr_valueChanged(const QString & text)
-//{
-//    patient()->setValue(Core::Patient::CreatinClearance, text);
-//    refreshPatient();
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_patientCreatinin_valueChanged(const QString & text)
-//{
-//    patient()->setValue(Core::Patient::Creatinine, text);
-//    refreshPatient();
-//}
-//
-///** \brief Always keep uptodate patient's datas */
-//void MainWindow::on_listOfAllergies_textChanged(const QString &text)
-//{
-//    // TODO: manage allergies */
-//    patient()->setValue(Core::Patient::DrugsAtcAllergies, text);
-//    refreshPatient();
-//}
