@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -30,6 +30,7 @@
 #include <coreplugin/actionmanager/mainwindowactions.h>
 #include <coreplugin/actionmanager/mainwindowactionhandler.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/contextmanager/contextmanager.h>
 #include <coreplugin/dialogs/plugindialog.h>
 #include <coreplugin/dialogs/helpdialog.h>
@@ -46,10 +47,12 @@
 #include <drugsbaseplugin/drugbasecore.h>
 #include <drugsbaseplugin/drugsbase.h>
 #include <drugsbaseplugin/constants.h>
+#include <drugsbaseplugin/constants_html.h>
 #include <drugsbaseplugin/drugsio.h>
 #include <drugsbaseplugin/globaldrugsmodel.h>
 #include <drugsbaseplugin/drugsmodel.h>
 #include <drugsbaseplugin/idrugengine.h>
+#include <drugsbaseplugin/prescriptionprinter.h>
 
 //#include <drugsbaseplugin/interactionmanager.h>
 //#include <drugsbaseplugin/engines/allergyengine.h>
@@ -84,6 +87,7 @@
 #include <QDockWidget>
 #include <QDataWidgetMapper>
 #include <QStandardItemModel>
+#include <QMenu>
 
 using namespace MainWin;
 using namespace MainWin::Internal;
@@ -98,10 +102,12 @@ static inline Core::ITheme *theme()  { return Core::ICore::instance()->theme();
 static inline Core::ActionManager *actionManager() { return Core::ICore::instance()->actionManager(); }
 static inline Core::IPatient *patient() { return Core::Internal::CoreImpl::instance()->patient(); }
 static inline Core::FileManager *fileManager() { return Core::ICore::instance()->fileManager(); }
-inline static DrugsDB::DrugsModel *drugModel() { return DrugsWidget::DrugsWidgetManager::instance()->currentDrugsModel(); }
+static inline DrugsDB::DrugsModel *drugModel() { return DrugsWidget::DrugsWidgetManager::instance()->currentDrugsModel(); }
 static inline Core::IDocumentPrinter *printer() {return ExtensionSystem::PluginManager::instance()->getObject<Core::IDocumentPrinter>();}
 static inline ExtensionSystem::PluginManager *pluginManager() {return ExtensionSystem::PluginManager::instance();}
 static inline DrugsDB::DrugsBase &drugsBase() {return DrugsDB::DrugBaseCore::instance().drugsBase();}
+static inline DrugsDB::DrugsIO &drugsIo() {return DrugsDB::DrugBaseCore::instance().drugsIo();}
+static inline DrugsDB::PrescriptionPrinter &prescriptionPrinter() {return DrugsDB::DrugBaseCore::instance().prescriptionPrinter();}
 
 // SplashScreen Messagers
 static inline void messageSplash(const QString &s) {theme()->messageSplashScreen(s); }
@@ -137,6 +143,7 @@ public:
     bool readExchangeFile(const QString &msg)
     {
         QString exfile = commandLine()->value(Core::CommandLine::CL_ExchangeOutFile).toString();
+        LOG_FOR(q, "Reading exchange out prescription file: " + exfile);
         if (!exfile.isEmpty()) {
             messageSplash(msg);
             if (QFileInfo(exfile).isRelative())
@@ -146,21 +153,20 @@ public:
             if (QFile(exfile).exists())
                 tmp = Utils::readTextFile(exfile, Utils::DontWarnUser);
             //            LOG("Content of the exchange file : " + tmp);
-            DrugsDB::DrugsIO io;
             if (tmp.contains(DrugsDB::Constants::ENCODEDHTML_FREEDIAMSTAG)) {
                 int begin = tmp.indexOf(DrugsDB::Constants::ENCODEDHTML_FREEDIAMSTAG) + QString(DrugsDB::Constants::ENCODEDHTML_FREEDIAMSTAG).length();
                 int end = tmp.indexOf("\"", begin);
                 QString encoded = tmp.mid( begin, end - begin );
-                io.prescriptionFromXml(drugModel(), QByteArray::fromBase64(encoded.toAscii()));
+                drugsIo().prescriptionFromXml(drugModel(), QByteArray::fromBase64(encoded.toAscii()));
             } else if (tmp.contains("DrugsInteractionsEncodedPrescription:")) {
                 // TODO: Manage wrong file encoding
                 int begin = tmp.indexOf("DrugsInteractionsEncodedPrescription:") + QString("DrugsInteractionsEncodedPrescription:").length();
                 int end = tmp.indexOf("\"", begin);
                 QString encoded = tmp.mid( begin, end - begin );
-                io.prescriptionFromXml(drugModel(), QByteArray::fromBase64(encoded.toAscii()));
+                drugsIo().prescriptionFromXml(drugModel(), QByteArray::fromBase64(encoded.toAscii()));
             } else if (tmp.startsWith("<?xml") && tmp.contains("<FreeDiams>", Qt::CaseInsensitive) && tmp.contains("</FreeDiams>", Qt::CaseInsensitive)) {
                 // TODO: Read patients datas ?
-                io.prescriptionFromXml(drugModel(), tmp);
+                drugsIo().prescriptionFromXml(drugModel(), tmp);
             } else {
                 return false;
             }
@@ -173,12 +179,13 @@ public:
         if (!m_Mapper)
             m_Mapper = new QDataWidgetMapper(q);
         m_Mapper->setModel(patient());
-        m_Mapper->addMapping(q->m_ui->patientName, Core::IPatient::BirthName);
+        m_Mapper->addMapping(q->m_ui->patientName, Core::IPatient::UsualName);
         m_Mapper->addMapping(q->m_ui->patientFirstname, Core::IPatient::Firstname);
-        m_Mapper->addMapping(q->m_ui->patientSize, Core::IPatient::Height);
-        m_Mapper->addMapping(q->m_ui->sizeUnit, Core::IPatient::HeightUnit);
-        m_Mapper->addMapping(q->m_ui->patientWeight, Core::IPatient::Weight);
-        m_Mapper->addMapping(q->m_ui->weightUnit, Core::IPatient::WeightUnit);
+        // TODO: manage weight and height
+//        m_Mapper->addMapping(q->m_ui->patientSize, Core::IPatient::Height);
+//        m_Mapper->addMapping(q->m_ui->sizeUnit, Core::IPatient::HeightUnit);
+//        m_Mapper->addMapping(q->m_ui->patientWeight, Core::IPatient::Weight);
+//        m_Mapper->addMapping(q->m_ui->weightUnit, Core::IPatient::WeightUnit);
         m_Mapper->addMapping(q->m_ui->sexCombo, Core::IPatient::GenderIndex);
         m_Mapper->addMapping(q->m_ui->patientCreatinin, Core::IPatient::Creatinine);
         m_Mapper->addMapping(q->m_ui->creatinineUnit, Core::IPatient::CreatinineUnit);
@@ -224,6 +231,31 @@ public:
         }
     }
 
+    QString getXmlExtraData()
+    {
+        QString extraData = patient()->toXml();
+        Q_ASSERT(printer());
+        if (printer()) {
+            const QList<Core::PrintedDocumentTracer> &pdfs = printer()->printedDocs();
+            foreach(const Core::PrintedDocumentTracer &t, pdfs) {
+                extraData += QString("<Printed file=\"%1\" docName=\"%2\" dateTime=\"%3\" userUid=\"%4\"/>\n")
+                        .arg(t.fileName())
+                        .arg(t.documentName())
+                        .arg(t.dateTime().toString(Qt::ISODate))
+                        .arg(t.userUid());
+            }
+        }
+        // Add EMR informations
+        if (commandLine()->value(Core::CommandLine::CL_EMR_Name).isValid()) {
+            extraData.append(QString("<EMR name=\"%1\"").arg(commandLine()->value(Core::CommandLine::CL_EMR_Name).toString()));
+            if (commandLine()->value(Core::CommandLine::CL_EMR_Name).isValid()) {
+                extraData.append(QString(" uid=\"%1\"").arg(commandLine()->value(Core::CommandLine::CL_EMR_Uid).toString()));
+            }
+            extraData.append("/>");
+        }
+        return extraData;
+    }
+
 public:
     QDataWidgetMapper *m_Mapper;
     QStandardItemModel *m_AllergiesModel;
@@ -256,7 +288,8 @@ static const char* const  SETTINGS_COUNTDOWN = "transmissionCountDown";
 //--------------------------------------------------------------------------------------------------------
 MainWindow::MainWindow( QWidget * parent ) :
         Core::IMainWindow(parent),
-        aClearPatient(0),
+        aClearPatient(new QAction(this)),
+        m_ui(0),
         d(new Internal::MainWinPrivate(this))
 {
     setObjectName("MainWindow");
@@ -282,16 +315,18 @@ bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
     // Create prescription menu
     Core::ActionContainer *pmenu = actionManager()->actionContainer(DrugsWidget::Constants::M_PLUGINS_DRUGS);
     if (!pmenu) {
-        pmenu = actionManager()->createMenu(DrugsWidget::Constants::M_PLUGINS_DRUGS);
-        pmenu->appendGroup(DrugsWidget::Constants::G_PLUGINS_VIEWS);
-        pmenu->appendGroup(DrugsWidget::Constants::G_PLUGINS_MODES);
-        pmenu->appendGroup(DrugsWidget::Constants::G_PLUGINS_SEARCH);
-        pmenu->appendGroup(DrugsWidget::Constants::G_PLUGINS_DRUGS);
-        pmenu->appendGroup(DrugsWidget::Constants::G_PLUGINS_INTERACTIONS);
-        pmenu->setTranslations(DrugsWidget::Constants::DRUGSMENU_TEXT, DrugsWidget::Constants::DRUGCONSTANTS_TR_CONTEXT);
+        using namespace DrugsWidget::Constants;
+        pmenu = actionManager()->createMenu(M_PLUGINS_DRUGS);
+        pmenu->appendGroup(Core::Id(G_PLUGINS_VIEWS));
+        pmenu->appendGroup(Core::Id(G_PLUGINS_MODES));
+        pmenu->appendGroup(Core::Id(G_PLUGINS_SEARCH));
+        pmenu->appendGroup(Core::Id(G_PLUGINS_DRUGS));
+        pmenu->appendGroup(Core::Id(G_PLUGINS_INTERACTIONS));
+        pmenu->appendGroup(Core::Id(Core::Constants::G_PLUGINS_PADTOOLS));
+        pmenu->setTranslations(DRUGSMENU_TEXT, DRUGCONSTANTS_TR_CONTEXT);
     }
     Q_ASSERT(pmenu);
-    actionManager()->actionContainer(Core::Constants::MENUBAR)->addMenu(pmenu, Core::Constants::G_PLUGINS);
+    actionManager()->actionContainer(Core::Id(Core::Constants::MENUBAR))->addMenu(pmenu, Core::Id(Core::Constants::G_PLUGINS));
 
     createTemplatesMenu();
     createConfigurationMenu();
@@ -326,14 +361,13 @@ bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
     connectHelpActions();
 
     // aClearPatient
-    aClearPatient = new QAction(this);
     aClearPatient->setObjectName("aClearPatient");
     aClearPatient->setIcon(theme()->icon(Core::Constants::ICONCLEAR));
-    Core::Command *cmd = actionManager()->registerAction(aClearPatient, "aClearPatient", QList<int>() << Core::Constants::C_GLOBAL_ID);
+    Core::Command *cmd = actionManager()->registerAction(aClearPatient, Core::Id("aClearPatient"), Core::Context(Core::Constants::C_GLOBAL));
     cmd->setTranslations(tkTr(Trans::Constants::CLEAR_PATIENT_INFOS));
     cmd->setDefaultKeySequence(QKeySequence(Qt::Key_C + Qt::CTRL + Qt::SHIFT));
-    Core::ActionContainer *menu = actionManager()->actionContainer(Core::Constants::M_FILE);
-    menu->addAction(cmd, Core::Constants::G_FILE_NEW);
+    Core::ActionContainer *menu = actionManager()->actionContainer(Core::Id(Core::Constants::M_FILE));
+    menu->addAction(cmd, Core::Id(Core::Constants::G_FILE_NEW));
     connect(aClearPatient, SIGNAL(triggered()), this, SLOT(clearPatientInfos()));
 
     return true;
@@ -374,7 +408,7 @@ void MainWindow::extensionsInitialized()
     /** end */
 
     // Creating MainWindow interface
-    m_ui = new Internal::Ui::MainWindow();
+    m_ui = new Ui::MainWindow;
     m_ui->setupUi(this);
     m_ui->centralLayout->setMargin(5);
     m_ui->sexCombo->addItems(genders());
@@ -414,19 +448,15 @@ void MainWindow::extensionsInitialized()
 
     // If needed read exchange file
     if (!d->readExchangeFile(tr("Reading exchange file..."))) {
-        LOG_ERROR(tkTr(Trans::Constants::FILE_1_ISNOT_READABLE).arg(commandLine()->value(Core::CommandLine::CL_ExchangeOutFile).toString()));
+        LOG_ERROR(tkTr(Trans::Constants::FILE_1_ISNOT_READABLE)
+                  .arg(commandLine()->value(Core::CommandLine::CL_ExchangeOutFile).toString()));
     }
 
     // Start the update checker
     if (updateChecker()->needsUpdateChecking(settings()->getQSettings())) {
-        messageSplash(tkTr(Trans::Constants::CHECKING_UPDATES));
-        LOG(tkTr(Trans::Constants::CHECKING_UPDATES));
-        statusBar()->addWidget(new QLabel(tkTr(Trans::Constants::CHECKING_UPDATES), this));
-        statusBar()->addWidget(updateChecker()->progressBar(this),1);
-        connect(updateChecker(), SIGNAL(updateFound()), this, SLOT(updateFound()));
-        connect(updateChecker(), SIGNAL(done(bool)), this, SLOT(updateCheckerEnd()));
-        updateChecker()->check(Utils::Constants::FREEDIAMS_UPDATE_URL);
-        settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
+        settings()->setPath(Core::ISettings::UpdateUrl, Utils::Constants::FREEDIAMS_UPDATE_URL);
+        if (checkUpdate())
+            settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
     }
 
     // Block patient datas
@@ -448,9 +478,6 @@ void MainWindow::extensionsInitialized()
         m_ui->clearPatient->setEnabled(false);
     }
 
-    show();
-    raise();
-
     // Connect post core initialization
     connect(Core::ICore::instance(), SIGNAL(coreOpened()), this, SLOT(postCoreInitialization()));
     connect(&drugsBase(), SIGNAL(drugsBaseHasChanged()), this, SLOT(refreshPatient()));
@@ -479,6 +506,9 @@ void MainWindow::postCoreInitialization()
     createDockWindows();
     readSettings();
 
+    show();
+    raise();
+
     contextManager()->updateContext();
     actionManager()->retranslateMenusAndActions();
     refreshPatient();
@@ -573,35 +603,30 @@ void MainWindow::closeEvent(QCloseEvent *event)
         QString format = commandLine()->value(Core::CommandLine::CL_ExchangeOutFileFormat).toString();
         QString tmp;
 
-        // create PDF cache printed docs XML lines
-        const QList<Core::PrintedDocumentTracer> &pdfs = printer()->printedDocs();
-        QString extraDatas = patient()->toXml();
-        foreach(const Core::PrintedDocumentTracer &t, pdfs) {
-            extraDatas += QString("<Printed file=\"%1\" docName=\"%2\" dateTime=\"%3\" userUid=\"%4\"/>\n")
-                           .arg(t.fileName())
-                           .arg(t.documentName())
-                           .arg(t.dateTime().toString(Qt::ISODate))
-                           .arg(t.userUid());
-        }
-
-        // Add date of creation
-        extraDatas.prepend(QString("<DateOfCreation>%1</DateOfCreation>").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
+        QString extraData = d->getXmlExtraData();
 
         // Manage specific MedinTux output exchange file format
-        DrugsDB::DrugsIO io;
         if (commandLine()->value(Core::CommandLine::CL_MedinTux).toBool() ||
-            commandLine()->value(Core::CommandLine::CL_EMR_Name).toString().compare("medintux",Qt::CaseInsensitive) == 0) {
+            commandLine()->value(Core::CommandLine::CL_EMR_Name).toString().compare("medintux", Qt::CaseInsensitive) == 0) {
             if (format=="html_xml" || format=="html") {
-                tmp = io.prescriptionToHtml(drugModel(), extraDatas, DrugsDB::DrugsIO::MedinTuxVersion);
+                DrugsDB::PrescriptionPrinterJob job;
+                job.readSettings();
+                job.addXmlExtraData(extraData);
+                job.setDrugsModel(drugModel());
+                tmp = prescriptionPrinter().prescriptionToHtml(job);
                 tmp.replace("font-weight:bold;", "font-weight:600;");
-                Utils::saveStringToFile(Utils::toHtmlAccent(tmp), exfile, Utils::Overwrite, Utils::DontWarnUser);
+                Utils::saveStringToFile(tmp, exfile, Utils::Overwrite, Utils::DontWarnUser);
             } else if (format=="xml") {
                 savePrescription(exfile);
             }
         } else {
             if (format=="html_xml" || format=="html") {
-                tmp = io.prescriptionToHtml(drugModel(), extraDatas, DrugsDB::DrugsIO::MedinTuxVersion);
-                Utils::saveStringToFile(Utils::toHtmlAccent(tmp), exfile, Utils::Overwrite, Utils::DontWarnUser);
+                DrugsDB::PrescriptionPrinterJob job;
+                job.readSettings();
+                job.addXmlExtraData(extraData);
+                job.setDrugsModel(drugModel());
+                tmp = prescriptionPrinter().prescriptionToHtml(job);
+                Utils::saveStringToFile(tmp, exfile, Utils::Overwrite, Utils::DontWarnUser);
             } else if (format=="xml") {
                 savePrescription(exfile);
             }
@@ -617,46 +642,47 @@ void MainWindow::closeEvent(QCloseEvent *event)
 void MainWindow::changeEvent(QEvent *event)
 {
     if (event->type()==QEvent::LanguageChange) {
-        QVariant sex = patient()->data(Core::IPatient::Gender);
-        bool state = true;
-        m_ui->centralwidget->blockSignals(state);
-        m_ui->crClUnit->blockSignals(state);
-        m_ui->creatinineUnit->blockSignals(state);
-        m_ui->dobDateEdit->blockSignals(state);
-        m_ui->m_CentralWidget->blockSignals(state);
-        m_ui->patientClCr->blockSignals(state);
-        m_ui->patientCreatinin->blockSignals(state);
-        m_ui->patientInformation->blockSignals(state);
-        m_ui->patientName->blockSignals(state);
-        m_ui->patientSize->blockSignals(state);
-        m_ui->patientFirstname->blockSignals(state);
-        m_ui->patientWeight->blockSignals(state);
-        m_ui->sexCombo->blockSignals(state);
-        m_ui->sizeUnit->blockSignals(state);
-        m_ui->weightUnit->blockSignals(state);
+        if (m_ui) {
+            QVariant sex = patient()->data(Core::IPatient::Gender);
+            bool state = true;
+            m_ui->centralwidget->blockSignals(state);
+            m_ui->crClUnit->blockSignals(state);
+            m_ui->creatinineUnit->blockSignals(state);
+            m_ui->dobDateEdit->blockSignals(state);
+            m_ui->m_CentralWidget->blockSignals(state);
+            m_ui->patientClCr->blockSignals(state);
+            m_ui->patientCreatinin->blockSignals(state);
+            m_ui->patientInformation->blockSignals(state);
+            m_ui->patientName->blockSignals(state);
+            m_ui->patientSize->blockSignals(state);
+            m_ui->patientFirstname->blockSignals(state);
+            m_ui->patientWeight->blockSignals(state);
+            m_ui->sexCombo->blockSignals(state);
+            m_ui->sizeUnit->blockSignals(state);
+            m_ui->weightUnit->blockSignals(state);
 
-	m_ui->retranslateUi(this);
+            m_ui->retranslateUi(this);
 
-        state = false;
-        m_ui->centralwidget->blockSignals(state);
-        m_ui->crClUnit->blockSignals(state);
-        m_ui->creatinineUnit->blockSignals(state);
-        m_ui->dobDateEdit->blockSignals(state);
-        m_ui->m_CentralWidget->blockSignals(state);
-        m_ui->patientClCr->blockSignals(state);
-        m_ui->patientCreatinin->blockSignals(state);
-        m_ui->patientInformation->blockSignals(state);
-        m_ui->patientName->blockSignals(state);
-        m_ui->patientSize->blockSignals(state);
-        m_ui->patientFirstname->blockSignals(state);
-        m_ui->patientWeight->blockSignals(state);
-        m_ui->sexCombo->blockSignals(state);
-        m_ui->sizeUnit->blockSignals(state);
-        m_ui->weightUnit->blockSignals(state);
-
-        actionManager()->retranslateMenusAndActions();
-        patient()->setData(patient()->index(0, Core::IPatient::Gender), sex);
-        refreshPatient();
+            state = false;
+            m_ui->centralwidget->blockSignals(state);
+            m_ui->crClUnit->blockSignals(state);
+            m_ui->creatinineUnit->blockSignals(state);
+            m_ui->dobDateEdit->blockSignals(state);
+            m_ui->m_CentralWidget->blockSignals(state);
+            m_ui->patientClCr->blockSignals(state);
+            m_ui->patientCreatinin->blockSignals(state);
+            m_ui->patientInformation->blockSignals(state);
+            m_ui->patientName->blockSignals(state);
+            m_ui->patientSize->blockSignals(state);
+            m_ui->patientFirstname->blockSignals(state);
+            m_ui->patientWeight->blockSignals(state);
+            m_ui->sexCombo->blockSignals(state);
+            m_ui->sizeUnit->blockSignals(state);
+            m_ui->weightUnit->blockSignals(state);
+            actionManager()->retranslateMenusAndActions();
+            patient()->setData(patient()->index(0, Core::IPatient::Gender), sex);
+            refreshPatient();
+        }
     }
 }
 
@@ -714,8 +740,9 @@ void MainWindow::togglePrecautions()
 {
 }
 
-void MainWindow::updateCheckerEnd()
+void MainWindow::updateCheckerEnd(bool error)
 {
+    Q_UNUSED(error);
     // this code avoid deletion of the resizer corner of the mainwindow
     delete statusBar();
     statusBar()->hide();
@@ -805,25 +832,7 @@ bool MainWindow::saveFile()
 */
 bool MainWindow::savePrescription(const QString &fileName)
 {
-    const QList<Core::PrintedDocumentTracer> &pdfs = printer()->printedDocs();
-    QString extraDatas = patient()->toXml();
-    foreach(const Core::PrintedDocumentTracer &t, pdfs) {
-        extraDatas += QString("<Printed file=\"%1\" docName=\"%2\" dateTime=\"%3\" userUid=\"%4\"/>\n")
-                       .arg(t.fileName())
-                       .arg(t.documentName())
-                       .arg(t.dateTime().toString(Qt::ISODate))
-                       .arg(t.userUid());
-    }
-    if (commandLine()->value(Core::CommandLine::CL_EMR_Name).isValid()) {
-        extraDatas.append(QString("<EMR name=\"%1\"").arg(commandLine()->value(Core::CommandLine::CL_EMR_Name).toString()));
-        if (commandLine()->value(Core::CommandLine::CL_EMR_Name).isValid()) {
-            extraDatas.append(QString(" uid=\"%1\"").arg(commandLine()->value(Core::CommandLine::CL_EMR_Uid).toString()));
-        }
-        extraDatas.append("/>");
-    }
-//    qWarning() << extraDatas;
-    DrugsDB::DrugsIO io;
-    return io.savePrescription(drugModel(), extraDatas, fileName);
+    return drugsIo().savePrescription(drugModel(), d->getXmlExtraData(), fileName);
 }
 
 /**
@@ -849,7 +858,6 @@ bool MainWindow::openFile()
 void MainWindow::readFile(const QString &file)
 {
     QString datas;
-    DrugsDB::DrugsIO io;
     if (drugModel()->rowCount() > 0) {
         int r = Utils::withButtonsMessageBox(
                 tr("Opening a prescription : merge or replace?"),
@@ -857,12 +865,12 @@ void MainWindow::readFile(const QString &file)
                 QString(), QStringList() << tr("Replace prescription") << tr("Add to prescription"),
                 tr("Open a prescription") + " - " + qApp->applicationName());
         if (r == 0) {
-            io.loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::ReplacePrescription);
+            drugsIo().loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::ReplacePrescription);
         } else if (r==1) {
-            io.loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::AppendPrescription);
+            drugsIo().loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::AppendPrescription);
         }
     } else {
-        io.loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::ReplacePrescription);
+        drugsIo().loadPrescription(drugModel(), file, datas, DrugsDB::DrugsIO::ReplacePrescription);
     }
     // Process patient's data only if not blockDatas and not using exchange-in file
     if ((!commandLine()->value(Core::CommandLine::CL_BlockPatientDatas).toBool()) &&
