@@ -1,3 +1,29 @@
+/***************************************************************************
+ *  The FreeMedForms project is a set of free, open source medical         *
+ *  applications.                                                          *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  All rights reserved.                                                   *
+ *                                                                         *
+ *  This program is free software: you can redistribute it and/or modify   *
+ *  it under the terms of the GNU General Public License as published by   *
+ *  the Free Software Foundation, either version 3 of the License, or      *
+ *  (at your option) any later version.                                    *
+ *                                                                         *
+ *  This program is distributed in the hope that it will be useful,        *
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
+ *  GNU General Public License for more details.                           *
+ *                                                                         *
+ *  You should have received a copy of the GNU General Public License      *
+ *  along with this program (COPYING.FREEMEDFORMS file).                   *
+ *  If not, see <http://www.gnu.org/licenses/>.                            *
+ ***************************************************************************/
+/***************************************************************************
+ *  Main Developer: Eric MAEKER, MD <eric.maeker@gmail.com>                *
+ *  Contributors:                                                          *
+ *       NAME <MAIL@ADDRESS.COM>                                           *
+ *       NAME <MAIL@ADDRESS.COM>                                           *
+ ***************************************************************************/
 #include "icdgrouplinker.h"
 #include "ui_icdgrouplinker.h"
 
@@ -5,10 +31,11 @@
 #include <coreplugin/imainwindow.h>
 #include <coreplugin/itheme.h>
 #include <coreplugin/constants_icons.h>
-#include <coreplugin/globaltools.h>
 #include <coreplugin/isettings.h>
 #include <coreplugin/ftb_constants.h>
 
+#include <drugsdb/tools.h>
+
 #include <icdplugin/icdcollectiondialog.h>
 #include <icdplugin/icdcollectionmodel.h>
 #include <icdplugin/icddatabase.h>
@@ -26,6 +53,7 @@
 #include <QUrl>
 #include <QClipboard>
 
+using namespace Icd10;
 
 //#define MODEL_PREFETCH 100
 
@@ -39,7 +67,7 @@ static inline ICD::IcdDatabase *icdBase() {return ICD::IcdDatabase::instance();}
 static inline QString workingPath()        {return QDir::cleanPath(settings()->value(Core::Constants::S_TMP_PATH).toString() + "/IcgGroupLinkerSources/") + QDir::separator();}
 //static inline QString iamDatabaseAbsPath() {return QDir::cleanPath(settings()->value(Core::Constants::S_DBOUTPUT_PATH).toString() + Core::Constants::IAM_DATABASE_FILENAME);}
 
-static inline QString icdGroupLinkerFile() {return QDir::cleanPath(settings()->value(Core::Constants::S_SVNFILES_PATH).toString() + Core::Constants::ICD_GROUP_LINKER_FILENAME);}
+static inline QString icdGroupLinkerFile() {return QDir::cleanPath(settings()->value(Core::Constants::S_GITFILES_PATH).toString() + Core::Constants::ICD_GROUP_LINKER_FILENAME);}
 
 QWidget *IcdGroupLinkerPage::createPage(QWidget *parent)
 {
@@ -205,6 +233,7 @@ private:
 
 }  // End anonymous namespace
 
+namespace Icd10 {
 class IcdGroupLinkerModelPrivate
 {
 public:
@@ -240,7 +269,7 @@ public:
 private:
     IcdGroupLinkerModel *q;
 };
-
+} // namespace Icd10
 
 IcdGroupLinkerModel *IcdGroupLinkerModel::m_Instance = 0;
 IcdGroupLinkerModel *IcdGroupLinkerModel::instance(QObject *parent)
@@ -264,6 +293,12 @@ IcdGroupLinkerModel::~IcdGroupLinkerModel()
     d = 0;
 }
 
+/** Return true is the ICD database is initialized and accessible */
+bool IcdGroupLinkerModel::databaseAvailable()
+{
+    return icdBase()->isInitialized();
+}
+
 bool IcdGroupLinkerModel::saveModel()
 {
     QFile file(icdGroupLinkerFile());
