@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,9 +19,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "packcategoriesmodel.h"
@@ -92,6 +92,8 @@ public:
                       << "forms/fullsets" << "forms/subforms"
                       << "documents"
                       << "zipcodes"
+                      << "alertpacks"
+                      << "binaries"
                       << "various";
 
         QStandardItem *main = new QStandardItem(vendor);
@@ -142,6 +144,8 @@ public:
         _categories.value(vendor+"/forms/subforms")->setData(Trans::Constants::FORMS_SUBFORMS, ::UnTranslatedNameRole);
         _categories.value(vendor+"/documents")->setData(Trans::Constants::USER_DOCUMENTS, ::UnTranslatedNameRole);
         _categories.value(vendor+"/zipcodes")->setData(Trans::Constants::ZIP_CODES, ::UnTranslatedNameRole);
+        _categories.value(vendor+"/alertpacks")->setData(Trans::Constants::ALERT_PACKS, ::UnTranslatedNameRole);
+        _categories.value(vendor+"/binaries")->setData(Trans::Constants::BINARY_PACKS, ::UnTranslatedNameRole);
         _categories.value(vendor+"/various")->setData(Trans::Constants::VARIOUS, ::UnTranslatedNameRole);
 
         _categories.value(vendor+"/account")->setIcon(icon("freeaccount.png", DataPackCore::SmallPixmaps));
@@ -150,6 +154,8 @@ public:
         _categories.value(vendor+"/forms")->setIcon(icon("forms.png", DataPackCore::SmallPixmaps));
         _categories.value(vendor+"/documents")->setIcon(icon("user.png", DataPackCore::SmallPixmaps));
         _categories.value(vendor+"/zipcodes")->setIcon(icon("package.png", DataPackCore::SmallPixmaps));
+        _categories.value(vendor+"/alertpacks")->setIcon(icon("package.png", DataPackCore::SmallPixmaps));
+        _categories.value(vendor+"/binaries")->setIcon(icon("package.png", DataPackCore::SmallPixmaps));
         _categories.value(vendor+"/various")->setIcon(icon("package.png", DataPackCore::SmallPixmaps));
 
         _categories.value(vendor+"/account")->setData(Pack::Accountancy, ::PackDataTypeRole);
@@ -160,6 +166,8 @@ public:
         _categories.value(vendor+"/forms/subforms")->setData(Pack::SubForms, ::PackDataTypeRole);
         _categories.value(vendor+"/documents")->setData(Pack::UserDocuments, ::PackDataTypeRole);
         _categories.value(vendor+"/zipcodes")->setData(Pack::ZipCodes, ::PackDataTypeRole);
+        _categories.value(vendor+"/alertpacks")->setData(Pack::AlertPacks, ::PackDataTypeRole);
+        _categories.value(vendor+"/binaries")->setData(Pack::Binaries, ::PackDataTypeRole);
         _categories.value(vendor+"/various")->setData(Pack::UnknownType, ::PackDataTypeRole);
 
         QFont bold;
@@ -170,6 +178,8 @@ public:
         _categories.value(vendor+"/forms")->setFont(bold);
         _categories.value(vendor+"/documents")->setFont(bold);
         _categories.value(vendor+"/zipcodes")->setFont(bold);
+        _categories.value(vendor+"/alertpacks")->setFont(bold);
+        _categories.value(vendor+"/binaries")->setFont(bold);
         _categories.value(vendor+"/various")->setFont(bold);
     }
 
@@ -198,6 +208,8 @@ public:
         case Pack::UserDocuments: vendor += "/documents"; break;
         case Pack::ZipCodes: vendor += "/zipcodes"; break;
         case Pack::ICD: vendor += "/diseases/icd10"; break;
+        case Pack::AlertPacks: vendor += "/alertpacks"; break;
+        case Pack::Binaries: vendor += "/binaries"; break;
         default: vendor += "/various"; break;
         }
         return _categories.value(vendor, q->invisibleRootItem());
@@ -233,16 +245,17 @@ public:
 
         // Keep only application compatible Packs
         int appId = PackDescription::FreeMedFormsCompatVersion;
-        if (qApp->applicationName().contains("freediams")) {
+        if (qApp->applicationName().contains("freediams", Qt::CaseInsensitive)) {
             appId = PackDescription::FreeDiamsCompatVersion;
-        } else if (qApp->applicationName().contains("freeaccount")) {
+        } else if (qApp->applicationName().contains("freeaccount", Qt::CaseInsensitive)) {
             appId = PackDescription::FreeAccountCompatVersion;
         }
         Utils::VersionNumber appVersion(qApp->applicationVersion());
         for(int i = m_AvailPacks.count()-1; i >= 0; --i) {
             const Pack &p = m_AvailPacks.at(i);
             Utils::VersionNumber packCompatVersion(p.description().data(appId).toString());
-            if (appVersion < packCompatVersion)
+            // qWarning() << "pack version" << packCompatVersion << "app" << appVersion << "keep" << p.uuid() << (appVersion >= packCompatVersion);
+            if (appVersion < packCompatVersion) // appVersion >= packVersion -> keep it
                 m_AvailPacks.removeAt(i);
         }
 
@@ -376,21 +389,24 @@ QString PackCategoriesModel::vendor(const QModelIndex &index) const
 /** Refresh the model. */
 void PackCategoriesModel::updateModel()
 {
+    beginResetModel();
     d->checkAvailablePack();
-    reset();
+    endResetModel();
 }
 
 /** Manage model when a server is added to the server manager */
 void PackCategoriesModel::onServerAdded(const int index)
 {
+    beginResetModel();
     d->serverAdded(index);
-    reset();
+    endResetModel();
 }
 
 /** Manage model when a server is about to be removed from the server manager */
 void PackCategoriesModel::onServerRemoved(const int index)
 {
+    beginResetModel();
     d->serverRemoved(index);
-    reset();
+    endResetModel();
 }
 
