@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,9 +19,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
@@ -52,7 +52,6 @@
 
 #include <QDebug>
 
-
 using namespace DataPack;
 using namespace Trans::ConstantTranslations;
 
@@ -81,6 +80,7 @@ struct PackItem {
     Qt::CheckState userCheckState;
 };
 
+// Return the HTML pack label
 static QString packToHtml(const PackItem &item)
 {
     QString inst, color;
@@ -117,23 +117,24 @@ static QString packToHtml(const PackItem &item)
             .arg(inst);
 }
 
+// Return the HTML pack tooltip
 static QString packTooltip(const PackItem &item)
 {
     const PackDescription &descr = item.pack.description();
     return QString("<p style=\"font-weight:bold;font-size:large;\">%1</p>"
-                      "<p style=\"font-size:small;margin-left:20px;color:gray\">"
-                      "%2: %3<br />"
-                      "%4: %5<br />"
-                      "%6: %7<br />"
-                      "%8: %9<br />"
-                      "%10: %11"
-                      "</p>"
-                       )
+                   "<p style=\"font-size:small;margin-left:20px;color:gray\">"
+                   "%2: %3<br />"
+                   "%4: %5<br />"
+                   "%6: %7<br />"
+                   "%8: %9<br />"
+                   "%10: %11"
+                   "</p>"
+                   )
             .arg(descr.data(PackDescription::Label).toString().replace(" ","&nbsp;"))
             .arg(tkTr(Trans::Constants::VERSION))
             .arg(descr.data(PackDescription::Version).toString())
             .arg(tkTr(Trans::Constants::LAST_MODIFICATION))
-            .arg(descr.data(PackDescription::LastModificationDate).toDate().toString("dd MM yyyy"))
+            .arg(QLocale().toString(descr.data(PackDescription::LastModificationDate).toDate(), QLocale::LongFormat).replace(" ","&nbsp;"))
             .arg(tkTr(Trans::Constants::AUTHOR))
             .arg(descr.data(PackDescription::Author).toString())
             .arg(tkTr(Trans::Constants::VENDOR))
@@ -168,16 +169,15 @@ public:
     // Get all packs available from a server (avoid duplicates) : populate m_AvailPacks list
     void scanServerPack(const int index)
     {
-//        qWarning() << "PackModel::Scanning server" << serverManager()->getServerAt(index).uuid();
+        // qWarning() << "PackModel::Scanning server" << serverManager()->getServerAt(index).uuid();
         foreach(const Pack &p, serverManager()->getPackForServer(serverManager()->getServerAt(index))) {
-//            qWarning() << "   ?? " << p.uuid() << p.version();
+            // qWarning() << "   ?? " << p.uuid() << p.version();
             // Add to the package list if not already included
             if (!p.isValid())
                 continue;
-//            qWarning() << "   valid " << p.uuid() << p.version();
             if (m_AvailPacks.contains(p))
                 continue;
-//            qWarning() << "   adding" << p.uuid() << p.version();
+            // qWarning() << "   adding" << p.uuid() << p.version();
             m_AvailPacks << p;
         }
     }
@@ -192,7 +192,7 @@ public:
             if (p.uuid() == packUuid) {
                 // keep only highest version
                 Utils::VersionNumber testing(p.version());
-//                qWarning() << "testing version" << packUuid << testing << highest;
+                //                qWarning() << "testing version" << packUuid << testing << highest;
                 if (testing > highest) {
                     highest = testing;
                     id = i;
@@ -202,10 +202,10 @@ public:
         return id;
     }
 
-    void createPackItem()
+    void createModelContent()
     {
         // Get all packages from servers
-        QList<Pack> installedPacks = packManager()->installedPack();
+        const QList<Pack> &installedPacks = packManager()->installedPack();
         for(int i=0; i < serverManager()->serverCount(); ++i) {
             scanServerPack(i);
         }
@@ -219,16 +219,17 @@ public:
 
         // Keep only application compatible Packs
         int appId = PackDescription::FreeMedFormsCompatVersion;
-        if (qApp->applicationName().contains("freediams")) {
+        if (qApp->applicationName().contains("freediams", Qt::CaseInsensitive)) {
             appId = PackDescription::FreeDiamsCompatVersion;
-        } else if (qApp->applicationName().contains("freeaccount")) {
+        } else if (qApp->applicationName().contains("freeaccount", Qt::CaseInsensitive)) {
             appId = PackDescription::FreeAccountCompatVersion;
         }
         Utils::VersionNumber appVersion(qApp->applicationVersion());
         for(int i = m_AvailPacks.count()-1; i >= 0; --i) {
             const Pack &p = m_AvailPacks.at(i);
             Utils::VersionNumber packCompatVersion(p.description().data(appId).toString());
-            if (appVersion < packCompatVersion)
+            // qWarning() << "pack version" << packCompatVersion << "app" << appVersion << "keep" << p.uuid() << (appVersion >= packCompatVersion);
+            if (appVersion < packCompatVersion) // appVersion >= packVersion -> keep it
                 m_AvailPacks.removeAt(i);
         }
 
@@ -243,8 +244,8 @@ public:
             processed << p.uuid();
         }
 
-//        qWarning() << idInUse;
-//        qWarning() << m_AvailPacks;
+        //        qWarning() << idInUse;
+        //        qWarning() << m_AvailPacks;
 
         // Create PackItem list
         foreach(int id, idInUse) {
@@ -273,7 +274,7 @@ public:
         Q_UNUSED(index);
         m_Items.clear();
         m_AvailPacks.clear();
-        createPackItem();
+        createModelContent();
     }
 
     void serverRemoved(const int index)
@@ -282,12 +283,12 @@ public:
         Q_UNUSED(index);
         m_Items.clear();
         m_AvailPacks.clear();
-        createPackItem();
+        createModelContent();
     }
 
 public:
     bool m_InstallChecking, m_PackCheckable;
-    QList<PackItem> m_Items;    // represent the non filtered model (all packages are shown)
+    QList<PackItem> m_Items;    // represents the non filtered model (all packages are shown)
     QList<Pack> m_AvailPacks;
     Pack m_InvalidPack;
     QList<int> rowToItem;       // when filtering the model, the list is populated with the item to show. If empty == not filtered
@@ -303,10 +304,12 @@ PackModel::PackModel(QObject *parent) :
     d(new Internal::PackModelPrivate)
 {
     setObjectName("DataPack::PackModel");
-    d->createPackItem();
-//    connect(serverManager(), SIGNAL(serverAdded(int)), this, SLOT(onServerAdded(int)));
+    d->createModelContent();
+    //    connect(serverManager(), SIGNAL(serverAdded(int)), this, SLOT(onServerAdded(int)));
     connect(serverManager(), SIGNAL(serverAboutToBeRemoved(int)), this, SLOT(onServerRemoved(int)));
     connect(serverManager(), SIGNAL(allServerDescriptionAvailable()), this, SLOT(updateModel()));
+    connect(packManager(), SIGNAL(packInstalled(DataPack::Pack)), this, SLOT(onPackInstalled(DataPack::Pack)));
+    connect(packManager(), SIGNAL(packRemoved(DataPack::Pack)), this, SLOT(onPackRemoved(DataPack::Pack)));
 }
 
 PackModel::~PackModel()
@@ -320,15 +323,17 @@ PackModel::~PackModel()
 /** When setting the installChecker feature to \e on, the model computes the packages dependencies. */
 void PackModel::setInstallChecker(const bool onOff)
 {
+    beginResetModel();
     d->m_InstallChecking = onOff;
-    reset();
+    endResetModel();
 }
 
 /** Allow user to check package (for installation/desinstallation). */
 void PackModel::setPackCheckable(const bool checkable)
 {
+    beginResetModel();
     d->m_PackCheckable = checkable;
-    reset();
+    endResetModel();
 }
 
 /** Return true is the model contains user modifications. */
@@ -348,8 +353,11 @@ bool PackModel::isDirty() const
     return false;
 }
 
-int PackModel::rowCount(const QModelIndex &) const
+int PackModel::rowCount(const QModelIndex &parent) const
 {
+    // prevent trees
+    if (parent.isValid()) return 0;
+
     if (!d->rowToItem.isEmpty())
         return d->rowToItem.count();
     return d->m_Items.count();
@@ -367,17 +375,17 @@ QVariant PackModel::data(const QModelIndex &index, int role) const
     if (row < 0 || row >= d->m_Items.count())
         return QVariant();
 
-    if (role==Qt::DisplayRole) {
+    if (role == Qt::DisplayRole) {
         switch (index.column()) {
         case Label: return packToHtml(d->m_Items.at(row));
         case IsInstalled: return d->m_Items.at(row).isInstalled;
         case IsAnUpdate: return d->m_Items.at(row).isAnUpdate;
         }
-    } else if (role==Qt::ToolTipRole && index.column()==Label) {
+    } else if (role == Qt::ToolTipRole && index.column()==Label) {
         return packTooltip(d->m_Items.at(row));
     } else if (d->m_PackCheckable && role==Qt::CheckStateRole) {
         return d->m_Items.at(row).userCheckState;
-    } else if (role==Qt::DecorationRole) {
+    } else if (role == Qt::DecorationRole) {
         QString iconFileName = d->m_Items.at(row).pack.description().data(PackDescription::GeneralIcon).toString();
         if (iconFileName.startsWith(Core::Constants::TAG_APPLICATION_THEME_PATH))
             iconFileName = iconFileName.remove(Core::Constants::TAG_APPLICATION_THEME_PATH);
@@ -408,7 +416,7 @@ bool PackModel::setData(const QModelIndex &index, const QVariant &value, int rol
             d->m_Items[row].userCheckState = Qt::CheckState(value.toInt());
         }
 
-//        Q_EMIT packStatusChanged(d->m_Items[index.row()].pack, save, d->m_Items[index.row()].userCheckState);
+        //        Q_EMIT packStatusChanged(d->m_Items[index.row()].pack, save, d->m_Items[index.row()].userCheckState);
         Q_EMIT dataChanged(index, index);
         return true;
     }
@@ -482,23 +490,29 @@ QList<Pack> PackModel::packageToRemove() const
 /** Refresh the model. */
 void PackModel::updateModel()
 {
+    beginResetModel();
     d->m_Items.clear();
     d->m_AvailPacks.clear();
-    d->createPackItem();
-    reset();
+    d->createModelContent();
+    filter(d->_filterVendor, d->_filterDataType);
+    endResetModel();
 }
 
 /** Filter the model using the \e vendor name and the Pack::DataType \e types. An empty /e vendor name and a empty \e types removes the filter. */
 void PackModel::filter(const QString &vendor, const QList<Pack::DataType> &types)
 {
+    // Begin reset
+    beginResetModel();
     d->rowToItem.clear();
+    // Clear filter ?
     if (types.isEmpty() && vendor.isEmpty()) {
         d->_filterVendor.clear();
         d->_filterDataType = types;
-        reset();
+        endResetModel();
         return;
     }
 
+    // Check rows/items
     for(int i=0; i < d->m_Items.count(); ++i) {
         const PackItem &item = d->m_Items.at(i);
         if (item.pack.vendor() == vendor && (types.contains(item.pack.dataType())))
@@ -506,7 +520,9 @@ void PackModel::filter(const QString &vendor, const QList<Pack::DataType> &types
     }
     d->_filterVendor = vendor;
     d->_filterDataType = types;
-    reset();
+
+    // Reset
+    endResetModel();
 }
 
 /** Manage model when a server is added to the server manager */
@@ -524,3 +540,27 @@ void PackModel::onServerRemoved(const int index)
     filter(d->_filterVendor, d->_filterDataType);
 }
 
+/** Manage pack installation: update the model */
+void PackModel::onPackInstalled(const DataPack::Pack &pack)
+{
+    for(int i=0; i < d->m_Items.count(); ++i) {
+        PackItem &item = d->m_Items[i];
+        if (item.pack != pack)
+            continue;
+        item.isInstalled = true;
+        item.isAnUpdate = false;
+        Q_EMIT dataChanged(index(i, Label), index(i, IsInstalled));
+        break;
+    }
+}
+
+/** Manage pack removal: remove it from the model */
+void PackModel::onPackRemoved(const DataPack::Pack &pack)
+{
+    for(int i=0; i < d->m_Items.count(); ++i) {
+        PackItem &item = d->m_Items[i];
+        if (item.pack != pack)
+            continue;
+        // removeRow(i);
+    }
+}
