@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,9 +19,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "httpserverengine.h"
@@ -38,14 +38,13 @@
 
 #include <QDir>
 #include <QFile>
-#include <QProgressBar>
 #include <QNetworkProxyQuery>
 #include <QAuthenticator>
 #include <QHash>
 
 #include <QDebug>
 
-// Zip Command Line: zip serverconf.zip server.conf.xml icd10/icd10.xml drugs/drugs.xml
+// Create datapacks with FreeToolBox
 
 using namespace DataPack;
 using namespace Internal;
@@ -68,20 +67,21 @@ namespace {
 
 }
 
-ReplyData::ReplyData(QNetworkReply *reply, Server *server, Server::FileRequested fileType, const Pack &pack, QProgressBar *progBar) {
-    this->reply = reply;
-    this->server = server;
-    this->fileType = fileType;
-    this->pack = pack;
-    this->bar = progBar;
+ReplyData::ReplyData(QNetworkReply *_reply, Server *_server, Server::FileRequested _fileType, const Pack &_pack, QProgressBar *_progBar) :
+    reply(_reply),
+    server(_server),
+    bar(_progBar),
+    pack(_pack),
+    fileType(_fileType)
+{
 }
 
-ReplyData::ReplyData(QNetworkReply *reply, Server *server, Server::FileRequested fileType, QProgressBar *progBar)
+ReplyData::ReplyData(QNetworkReply *_reply, Server *_server, Server::FileRequested _fileType, QProgressBar *_progBar) :
+    reply(_reply),
+    server(_server),
+    bar(_progBar),
+    fileType(_fileType)
 {
-    this->reply = reply;
-    this->server = server;
-    this->fileType = fileType;
-    this->bar = progBar;
 }
 
 
@@ -95,7 +95,7 @@ HttpServerEngine::HttpServerEngine(QObject *parent)  :
 {
     setObjectName("HttpServerEngine");
     m_NetworkAccessManager = new QNetworkAccessManager(this);
-//    const QString &id = Utils::testInternetConnexion();
+//    const QString &id = Utils::testInternetConnection();
 //    if (!id.isEmpty()) {
 //        LOG("Internet connection is enabled.");
 //        bool proxy = false;
@@ -127,33 +127,43 @@ HttpServerEngine::HttpServerEngine(QObject *parent)  :
     // Connect authentication request
     connect(m_NetworkAccessManager, SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), this, SLOT(authenticationRequired(QNetworkReply*,QAuthenticator*)));
     connect(m_NetworkAccessManager, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)), this, SLOT(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
+#if QT_VERSION >= 0x050000
+    m_NetworkAccessManager->clearAccessCache();
+#endif
 }
 
 HttpServerEngine::~HttpServerEngine()
 {}
 
+/** Returns true if this engine manages this server */
 bool HttpServerEngine::managesServer(const Server &server)
 {
-    if (core().isInternetConnexionAvailable())
+    if (core().isInternetConnectionAvailable())
         return server.nativeUrl().startsWith("http://");
     return false;
 }
 
+/** Add a dowload to the queue */
 void HttpServerEngine::addToDownloadQueue(const ServerEngineQuery &query)
 {
     m_queue.append(query);
 }
 
+/** Returns the number of downloads included in the queue */
 int HttpServerEngine::downloadQueueCount() const
 {
     return m_queue.count();
 }
 
+/**
+ * Start download
+ * \sa DataPack::IServerEngine::startDownloadQueue()
+ */
 bool HttpServerEngine::startDownloadQueue()
 {
     // Internet connection available ?
-    if (!core().isInternetConnexionAvailable()) {
-        LOG_ERROR("No internet connexion available.");
+    if (!core().isInternetConnectionAvailable()) {
+        LOG_ERROR("No internet connection available.");
         return false;
     }
     // Use a proxy ?
@@ -177,6 +187,7 @@ bool HttpServerEngine::startDownloadQueue()
             // Create a network request for the server config
             QNetworkRequest request = createRequest(s->url(Server::ServerConfigurationFile));
             ++m_DownloadCount_Server;
+            qWarning() << "Downloading server config" << s->url(Server::ServerConfigurationFile);
 
             // Create a status for the server
             ServerEngineStatus status;
@@ -188,6 +199,7 @@ bool HttpServerEngine::startDownloadQueue()
         } else if (query.downloadPackFile) {
             // Create a network request for the pack file
             QNetworkRequest request = createRequest(s->url(Server::PackFile, query.pack->serverFileName()));
+            qWarning() << "Downloading pack" << s->url(Server::PackFile, query.pack->serverFileName());
 
             // Create a status for the pack
             ServerEngineStatus status;
@@ -202,6 +214,7 @@ bool HttpServerEngine::startDownloadQueue()
         connect(reply, SIGNAL(finished()), this, SLOT(serverFinished()));
         connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(serverError(QNetworkReply::NetworkError)));
 
+        // TODO: Remove all direct access to progress bar -> use a signal instead
         QProgressBar *bar = query.progressBar;
         if (bar) {
             bar->setRange(0, 100);
@@ -212,6 +225,7 @@ bool HttpServerEngine::startDownloadQueue()
     return true;
 }
 
+/** Returns the number of running download */
 int HttpServerEngine::runningJobs() const
 {
     return m_replyToData.count();
@@ -238,6 +252,7 @@ bool HttpServerEngine::stopJobsAndClearQueue()
 /////////////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////  Specific Http code  ///////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////////////////
+/** Manage progress bar progress value */
 void HttpServerEngine::downloadProgress(qint64 bytesRead, qint64 totalBytes)
 {
     // Retreive progressBar
@@ -255,18 +270,34 @@ void HttpServerEngine::downloadProgress(qint64 bytesRead, qint64 totalBytes)
     }
 }
 
+/** Manage user authentication */
 void HttpServerEngine::authenticationRequired(QNetworkReply *reply, QAuthenticator *authenticator)
 {
     LOG("Server authentication required: " +  reply->url().toString());
+    // Already achieved the max tries ?
     const QString &host = reply->url().toString();
     m_AuthTimes.insert(host, m_AuthTimes.value(host, 0) + 1);
     if (m_AuthTimes.value(host) > MAX_AUTHENTIFICATION_TRIES) {
         LOG_ERROR("Server authentication max tries achieved. " +  host);
         return;
     }
+    // Get the related server
+    const ReplyData &data = m_replyToData.value(reply);
+    const Server *server = data.server;
+    Q_ASSERT(server);
+    if (!server)
+        return;
+    // Create the login dialog
     Utils::BasicLoginDialog dlg;
     dlg.setModal(true);
-    dlg.setTitle(tr("Server authentication required"));
+    QString serverName = server->url();
+    if (!server->name().isEmpty())
+        serverName += " - " + server->name();
+    dlg.setTitle(tr("Server %1\nrequires an authentication").arg(serverName));
+    QString html = QString("<p style=\"text-align: center\">Host: %1 <br /><span style=\"font-weight:bold; color:darkred\">%2</span></p>")
+            .arg(reply->url().host())
+            .arg(tr("If you don't have any login just cancel the dialog"));
+    dlg.setHtmlExtraInformation(html);
     dlg.setToggleViewIcon(core().icon(ICONEYES));
     if (dlg.exec()==QDialog::Accepted) {
         authenticator->setUser(dlg.login());
@@ -275,6 +306,7 @@ void HttpServerEngine::authenticationRequired(QNetworkReply *reply, QAuthenticat
     // TODO: manage ServerEngineStatus here
 }
 
+/** Manage proxy authentication */
 void HttpServerEngine::proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator)
 {
     LOG("Proxy authentication required: " +  proxy.hostName());
@@ -312,8 +344,8 @@ void HttpServerEngine::serverReadyRead()
 /** An error occured during the network access. */
 void HttpServerEngine::serverError(QNetworkReply::NetworkError error)
 {
-    WARN_FUNC << error;
     QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    qWarning() << "serverError" << reply->url().toString() << error;
     ReplyData &data = m_replyToData[reply];
     reply->deleteLater(); // we don't need reply anymore
     ServerEngineStatus *status = getStatus(data);
@@ -331,8 +363,8 @@ void HttpServerEngine::serverError(QNetworkReply::NetworkError error)
 void HttpServerEngine::serverFinished()
 {
     QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
-
     qWarning() << "HTTP: serverFinished" << reply->request().url() << reply->error();
+
     if (reply->error() != QNetworkReply::NoError) {
         reply->deleteLater(); // we don't need reply anymore
         m_replyToData.remove(reply);
@@ -386,6 +418,7 @@ void HttpServerEngine::serverFinished()
     }
 }
 
+/** Returns the engine status linked with a reply */
 ServerEngineStatus *HttpServerEngine::getStatus(const ReplyData &data)
 {
     if (data.server)
@@ -396,14 +429,24 @@ ServerEngineStatus *HttpServerEngine::getStatus(const ReplyData &data)
 /** Reads Server description XML file and start the dowloading of pack description if needed. */
 void HttpServerEngine::afterServerConfigurationDownload(const ReplyData &data)
 {
-//    qWarning() << "ServerConfigDone" << data.server->uuid();
+    qWarning() << "afterServerConfigurationDownload" << data.server->uuid() << data.response.size();
 
 //    bool downloadPackDescriptionNeeded = false;
     Server *server = data.server;
     ServerEngineStatus *status = getStatus(data);
     Q_ASSERT(status);
-    QStringList packDescriptionToDownload;
 
+    // Dowloaded content is empty -> return
+    if (data.response.size() == 0) {
+        status->hasError = true;
+        status->isSuccessful = false;
+        status->downloadCorrectlyFinished = true;
+        status->engineMessages << tkTr(Trans::Constants::FILE_1_ISEMPTY).arg(data.reply->url().toString());
+        LOG_ERROR(tkTr(Trans::Constants::FILE_1_ISEMPTY).arg(data.reply->url().toString()));
+        return;
+    }
+
+    QStringList packDescriptionToDownload;
     switch (server->urlStyle()) {
     case Server::Http:
     case Server::HttpPseudoSecuredNotZipped:
@@ -536,6 +579,10 @@ void HttpServerEngine::afterPackFileDownload(const ReplyData &data)
     Q_EMIT packDownloaded(pack, *status);
 }
 
+/**
+ * Creates an internal DataPack::Pack using the stored file \e pathToPackDescription
+ * from the DataPack::Server \e server
+ */
 void HttpServerEngine::createPackAndRegisterToServerManager(const Server &server, const QString &pathToPackDescription)
 {
     Pack p;
@@ -543,21 +590,30 @@ void HttpServerEngine::createPackAndRegisterToServerManager(const Server &server
     serverManager()->registerPack(server, p);
 }
 
+/** Creates the URL to query for this specific server engine */
 QNetworkRequest HttpServerEngine::createRequest(const QString &url)
 {
     QNetworkRequest request(url);
     request.setRawHeader("User-Agent", QString("FreeMedForms:%1;%2")
                          .arg(qApp->applicationName())
-                         .arg(qApp->applicationVersion()).toAscii());
+                         .arg(qApp->applicationVersion()).toUtf8());
     return request;
 }
 
+/**
+ * Returns the last DataPack::ServerEngineStatus associated with
+ * DataPack::Pack \e pack.
+ */
 const ServerEngineStatus &HttpServerEngine::lastStatus(const Pack &pack)
 {
     const QString &key = statusKey(pack);
     return m_PackStatus[key];
 }
 
+/**
+ * Returns the last DataPack::ServerEngineStatus associated with
+ * DataPack::Server \e server.
+ */
 const ServerEngineStatus &HttpServerEngine::lastStatus(const Server &server)
 {
     const QString &key = statusKey(server);
