@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,9 +19,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
@@ -30,7 +30,6 @@
   install, remove, update.
 
 */
-// FIXME: Crash when cancelling dialog if a download was started --> add core.stopJobsAndClearQueues()
 
 #include "packwizard.h"
 #include <datapackutils/datapackcore.h>
@@ -153,8 +152,10 @@ const QList<Pack> &PackWizard::updatePacks() const
     return d->m_UpdatePacks;
 }
 
-DataPack::PackWizard *PackPage::packWizard() const {return qobject_cast<DataPack::PackWizard*>(wizard());}
-
+DataPack::PackWizard *PackPage::packWizard() const
+{
+    return qobject_cast<DataPack::PackWizard*>(wizard());
+}
 
 /////////////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////  PackIntroPage  ///////////////////////////////////////
@@ -162,8 +163,7 @@ DataPack::PackWizard *PackPage::packWizard() const {return qobject_cast<DataPack
 PackIntroPage::PackIntroPage(QWidget *parent) :
     PackPage(parent)
 {
-    setTitle(tr("Pack processing"));
-    setSubTitle(tr("The following packs will be processed. Please verify all information."));
+    setObjectName("PackIntroPage");
     m_Browser = new QTextBrowser(this);
     QVBoxLayout *lay = new QVBoxLayout(this);
     setLayout(lay);
@@ -185,6 +185,8 @@ static QString toHtml(const QList<Pack> &packs, const QString &title)
 
 void PackIntroPage::initializePage()
 {
+    setTitle(tr("Pack processing"));
+    setSubTitle(tr("The following packs will be processed. Please verify all information."));
     m_Browser->clear();
     // Create the HTML output of processing packs
     QString html = "<p>";
@@ -228,11 +230,9 @@ PackLicensePage::PackLicensePage(QWidget *parent) :
     PackPage(parent)
 {
     setObjectName("PackLicensePage");
-    setTitle(tr("License agreement"));
-    setSubTitle(tr("The following packs need a license agreement."));
     m_Browser = new QTextBrowser(this);
     m_AgreeBox = new QCheckBox(this);
-    m_AgreeBox->setText(tr("Accept all license terms"));
+    m_AgreeBox->setText(tr("&Accept all license terms"));
     m_AgreeBox->setToolTip(m_AgreeBox->text());
     QVBoxLayout *lay = new QVBoxLayout(this);
     setLayout(lay);
@@ -287,6 +287,8 @@ static QString toHtml(const QList<Pack> &packs)
 
 void PackLicensePage::initializePage()
 {
+    setTitle(tr("License agreement"));
+    setSubTitle(tr("The following packs need a license agreement."));
     m_Browser->clear();
     // Create the HTML output of processing packs
     QString html = "<p>";
@@ -323,9 +325,6 @@ PackDownloadPage::PackDownloadPage(QWidget *parent) :
     PackPage(parent)
 {
     setObjectName("PackDownloadPage");
-    setTitle(tr("Downloading packs"));
-    // TODO: better would be "Please wait until..." (HIG), more professional
-    setSubTitle(tr("You just have to wait until all packs are downloaded."));
     m_Area = new QScrollArea(this);
     m_Grid = new QGridLayout(m_Area);
     m_Area->setLayout(m_Grid);
@@ -348,6 +347,8 @@ static inline QMovie *movie()
 
 void PackDownloadPage::initializePage()
 {
+    setTitle(tr("Downloading packs"));
+    setSubTitle(tr("Please wait until all packs are downloaded."));
     m_PackDownloaded.clear();
     m_DownloadPacks.clear();
     m_DownloadPacks << packWizard()->installPacks();
@@ -396,6 +397,8 @@ void PackDownloadPage::initializePage()
         packWizard()->button(QWizard::NextButton)->setEnabled(false);
         QTimer::singleShot(2, this, SLOT(startDownloads()));
     }
+    connect(packWizard()->button(QWizard::CancelButton), SIGNAL(clicked()), this, SLOT(cancelDownloads()));
+
 }
 
 void PackDownloadPage::startDownloads()
@@ -406,6 +409,11 @@ void PackDownloadPage::startDownloads()
     //        m_PackProcessing.value(k)->setMovie(movie());
 }
 
+void PackDownloadPage::cancelDownloads()
+{
+    core().stopJobsAndClearQueues();
+}
+
 //void PackDownloadPage::cleanupPage()
 //{
 //    // TODO
@@ -439,7 +447,6 @@ void PackDownloadPage::packDownloaded(const DataPack::Pack &pack, const DataPack
     QLabel *processLabel = m_PackProcessing.value(k);
     m_PackDownloaded.append(k);
 
-//    WARN_FUNC << k;
     // Correctly manage errors
     if (status.hasError || !status.isSuccessful) {
         processLabel->setPixmap(QPixmap(iconFullPath("warning.png", DataPackCore::SmallPixmaps)));
@@ -453,7 +460,7 @@ void PackDownloadPage::packDownloaded(const DataPack::Pack &pack, const DataPack
 
     // Start next download
     int id = m_DownloadPacks.indexOf(pack) + 1;
-    if (id!=m_DownloadPacks.count()) {
+    if (id != m_DownloadPacks.count()) {
         packManager()->downloadPack(m_DownloadPacks.at(id), m_PackBar.value(k));
     } else {
         allDownloadFinished();
@@ -474,11 +481,11 @@ void PackDownloadPage::allDownloadFinished()
 PackInstallPage::PackInstallPage(QWidget *parent) :
     PackPage(parent)
 {
-    setTitle(tr("Installing pack(s)"));
-    setSubTitle(tr("Please wait until all packs are installed."));
+    setObjectName("PackInstallPage");
     m_Area = new QScrollArea(this);
     m_Grid = new QGridLayout(m_Area);
     m_Area->setLayout(m_Grid);
+
     QVBoxLayout *lay = new QVBoxLayout(this);
     setLayout(lay);
     lay->addWidget(m_Area);
@@ -489,6 +496,8 @@ PackInstallPage::PackInstallPage(QWidget *parent) :
 
 void PackInstallPage::initializePage()
 {
+    setTitle(tr("%1 pack(s) to install").arg(packWizard()->installPacks().count()));
+    setSubTitle(tr("Please wait until all packs are installed."));
     m_PackInstalled.clear();
     m_InstallPacks.clear();
     m_InstallPacks << packWizard()->installPacks();
@@ -499,26 +508,26 @@ void PackInstallPage::initializePage()
         const QString &key = packKey(pack);
         if (!m_PackLabel.keys().contains(key)) {
             // Create a label
-            QLabel *processLabel = new QLabel(this);
-            processLabel->setPixmap(icon("package.png", DataPackCore::SmallPixmaps).pixmap(16,16));
-            processLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+            QLabel *processIconLabel = new QLabel(this);
+            processIconLabel->setPixmap(icon("package.png", DataPackCore::SmallPixmaps).pixmap(16,16));
+            processIconLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-            QLabel *label = new QLabel(pack.name() + " " + pack.version(), this);
-            label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+            QLabel *descriptionLabel = new QLabel(QString("%1 %2").arg(pack.name(),pack.version()), this);
+            descriptionLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
             // Create a progBar
             QProgressBar *bar = new QProgressBar(this);
 
             // Register both
-            m_PackProcessing.insert(key, processLabel);
-            m_PackLabel.insert(key, label);
+            m_PackProcessing.insert(key, processIconLabel);
+            m_PackLabel.insert(key, descriptionLabel);
             m_PackBar.insert(key, bar);
 
             // Put into ui
             int row = m_Grid->rowCount() + 1;
-            m_Grid->addWidget(processLabel, row, 0);
-            m_Grid->addWidget(label, row, 1);
-            QHBoxLayout *hb = new QHBoxLayout;
+            m_Grid->addWidget(processIconLabel, row, 0);
+            m_Grid->addWidget(descriptionLabel, row, 1);
+            QHBoxLayout *hb = new QHBoxLayout();
             QSpacerItem *sp = new QSpacerItem(20,20, QSizePolicy::Fixed, QSizePolicy::Fixed);
             hb->addSpacerItem(sp);
             hb->addWidget(bar);
@@ -602,10 +611,7 @@ int PackInstallPage::nextId() const
 PackRemovePage::PackRemovePage(QWidget *parent) :
     PackPage(parent)
 {
-    // TODO: better would be tr(" %1 Pack(s) to install/update/remove").arg(packWizard()->installPacks()) to reflect other languages.
-    setTitle(tr("Removing pack(s)"));
-    // TODO: better would be "Please wait until al packs are removed." - be nice to users ;-)
-    setSubTitle(tr("Wait until all removal are done."));
+    setObjectName("PackRemovePage");
     m_Area = new QScrollArea(this);
     m_Grid = new QGridLayout(m_Area);
     m_Area->setLayout(m_Grid);
@@ -616,6 +622,8 @@ PackRemovePage::PackRemovePage(QWidget *parent) :
 
 void PackRemovePage::initializePage()
 {
+    setTitle(tr("%1 pack(s) to remove").arg(packWizard()->removePacks().count()));
+    setSubTitle(tr("Please wait until all packs are removed."));
     for(int i = 0; i < packWizard()->removePacks().count(); ++i) {
         const Pack &pack = packWizard()->removePacks().at(i);
         // Create a label
@@ -679,6 +687,7 @@ int PackRemovePage::nextId() const
 PackEndPage::PackEndPage(QWidget *parent) :
     PackPage(parent)
 {
-    setTitle(tr("Pack(s) processed."));
+    setObjectName("PackEndPage");
+    setTitle(tr("All Pack(s) processed."));
     setFinalPage(true);
 }
