@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,14 +19,14 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
-  \class DataPack::ServerPackEditor
-  Widget used to modify the server list and the installed packs.
+ * \class DataPack::ServerPackEditor
+ * Widget used to modify the server list and the installed packs.
 */
 
 #include "serverpackeditor.h"
@@ -107,7 +107,7 @@ namespace Internal {
 class ServerPackEditorPrivate
 {
 public:
-    ServerPackEditorPrivate() :
+    ServerPackEditorPrivate(ServerPackEditor *parent) :
         ui(new Ui::ServerPackEditor),
         m_PackModel(0),
         m_PackCategoriesModel(0),
@@ -115,11 +115,78 @@ public:
         aServerRefresh(0), aServerEdit(0), aServerRemove(0), aServerAdd(0),
         aPackRefresh(0), aPackApply(0),
         m_ToolBarPacks(0),
+        bServer(0), bPack(0),
         m_ServerMapper(0),
         _segmented(0), _segPack(0), _segServer(0),
-        _toolbarCurrentMode(-1)
+        _toolbarCurrentMode(-1),
+        _progressDialog(0),
+        q(parent)
     {}
 
+    void createActions()
+    {
+        // Create server actions
+        QActionGroup *srvgr = new QActionGroup(q);
+        QAction *a = aServerRefresh = new QAction(q);
+        a->setObjectName("aServerRefresh");
+        a->setIcon(icon(::ICON_SERVER_REFRESH, DataPack::DataPackCore::MediumPixmaps));
+        a = aServerEdit = new QAction(q);
+        a->setObjectName("aServerEdit");
+        a->setIcon(icon(::ICON_SERVER_EDIT, DataPack::DataPackCore::MediumPixmaps));
+        a = aServerAdd = new QAction(q);
+        a->setObjectName("aInstall");
+        a->setIcon(icon(::ICON_SERVER_ADD, DataPack::DataPackCore::MediumPixmaps));
+        a = aServerRemove = new QAction(q);
+        a->setObjectName("aServerRemove");
+        a->setIcon(icon(::ICON_SERVER_REMOVE, DataPack::DataPackCore::MediumPixmaps));
+        QObject::connect(srvgr, SIGNAL(triggered(QAction*)), q, SLOT(serverActionTriggered(QAction *)));
+
+        // Create pack actions
+        a = aPackRefresh = new QAction(q);
+        a->setObjectName("aPackRefresh");
+        a->setIcon(icon(::ICON_SERVER_REFRESH, DataPack::DataPackCore::MediumPixmaps));
+        a = aPackApply = new QAction(q);
+        a->setObjectName("aPackApply");
+        a->setIcon(icon(::ICON_INSTALL, DataPack::DataPackCore::MediumPixmaps));
+        QObject::connect(aPackRefresh, SIGNAL(triggered()), q, SLOT(refreshPacks()));
+        QObject::connect(aPackApply, SIGNAL(triggered()), q, SLOT(processPacks()));
+    }
+
+    void createToolbar()
+    {
+        m_ToolBarPacks = new QToolBar(q);
+        m_ToolBarPacks->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
+
+        // Create the segmented button with server/pack
+        bServer = new QPushButton(q);
+        bServer->setCheckable(true);
+        bPack = new QPushButton(q);
+        bPack->setCheckable(true);
+        _segmented = new Utils::SegmentedButton(q);
+        _segmented->setFirstButton(bServer);
+        _segmented->setLastButton(bPack);
+        _segmented->setAutoExclusive(true);
+        QWidget *w1 = new QWidget(q);
+        w1->setMinimumSize(20,20);
+        w1->setMaximumSize(20,20);
+        m_ToolBarPacks->addWidget(w1);
+        m_ToolBarPacks->addWidget(_segmented);
+        w1 = new QWidget(q);
+        w1->setMinimumSize(20,20);
+        w1->setMaximumSize(20,20);
+        m_ToolBarPacks->addWidget(w1);
+        m_ToolBarPacks->addSeparator();
+
+        processToolBar(::PACK_MODE);
+
+        m_ToolBarPacks->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+        ui->toolbarLayout->addWidget(m_ToolBarPacks);
+
+        QObject::connect(m_ToolBarPacks, SIGNAL(actionTriggered(QAction*)), q, SLOT(serverActionTriggered(QAction*)));
+        QObject::connect(bPack, SIGNAL(clicked()), q, SLOT(switchToPackView()));
+        QObject::connect(bServer, SIGNAL(clicked()), q, SLOT(switchToServerView()));
+    }
+
     void processToolBar(int mode)
     {
         if (_toolbarCurrentMode==mode)
@@ -152,13 +219,17 @@ public:
     QDataWidgetMapper *m_ServerMapper;
     Utils::SegmentedButton *_segmented, *_segPack, *_segServer;
     int _toolbarCurrentMode;
+    QProgressDialog *_progressDialog; // used for server content refreshing
+
+private:
+    ServerPackEditor *q;
 };
 } // Internal
 } // DataPack
 
 ServerPackEditor::ServerPackEditor(QWidget *parent) :
     QWidget(parent),
-    d(new Internal::ServerPackEditorPrivate)
+    d(new Internal::ServerPackEditorPrivate(this))
 {
     setObjectName("ServerPackEditor");
     d->ui->setupUi(this);
@@ -193,19 +264,24 @@ ServerPackEditor::ServerPackEditor(QWidget *parent) :
     d->ui->packCategoriesView->setStyleSheet(::CSS);
     connect(d->ui->packCategoriesView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(onPackCategoriesChanged(QModelIndex, QModelIndex)));
 
-    Utils::HtmlDelegate *delegate = new Utils::HtmlDelegate;
-    d->ui->packView->setItemDelegate(delegate);
+    Utils::HtmlDelegate *packdelegate = new Utils::HtmlDelegate(this);
+    d->ui->packView->setItemDelegate(packdelegate);
     d->ui->packView->setStyleSheet(::CSS);
     //    d->ui->packView->header()->hide();
     d->ui->packView->setAlternatingRowColors(true);
+    d->ui->packView->setUniformItemSizes(false);
 
-//    // server page
+    // server view/model/delegate
+    Utils::HtmlDelegate *serverdelegate = new Utils::HtmlDelegate(this);
     d->m_serverModel = new ServerModel(this);
+    d->m_serverModel->initialize();
     d->ui->serverListView->setModel(d->m_serverModel);
     d->ui->serverListView->setModelColumn(ServerModel::HtmlLabel);
-    d->ui->serverListView->setItemDelegate(delegate);
+    d->ui->serverListView->setItemDelegate(serverdelegate);
     d->ui->serverListView->setStyleSheet(::CSS);
     d->ui->serverListView->setAlternatingRowColors(true);
+    d->ui->serverListView->setResizeMode(QListView::Adjust);
+    d->ui->serverListView->setUniformItemSizes(false);
 
     // Manage central view
     QFont bold;
@@ -216,9 +292,9 @@ ServerPackEditor::ServerPackEditor(QWidget *parent) :
     d->ui->serverName->setFont(bold);
     d->ui->serverName->setStyleSheet(::TITLE_CSS);
 
-    createActions();
-    createToolbar();
-    d->processToolBar(::PACK_MODE);
+    d->createActions();
+    d->createToolbar();
+    switchToPackView();
 //    createServerDataWidgetMapper();
     retranslate();
 
@@ -226,8 +302,11 @@ ServerPackEditor::ServerPackEditor(QWidget *parent) :
     connect(d->ui->packView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(onPackIndexActivated(QModelIndex,QModelIndex)));
     connect(d->ui->serverListView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(serverCurrentChanged(QModelIndex,QModelIndex)));
 
-    connect(serverManager(), SIGNAL(allServerDescriptionAvailable()), this, SLOT(selectedFirstRow()));
+    connect(serverManager(), SIGNAL(allServerDescriptionAvailable()), this, SLOT(selectFirstRow()));
     // file://Users/eric/Desktop/Programmation/freemedforms/global_resources/datapacks/default/
+
+    //refreshServerContent();
+    //selectFirstRow();
 }
 
 ServerPackEditor::~ServerPackEditor()
@@ -242,87 +321,42 @@ bool ServerPackEditor::refreshServerContent()
 {
     if (serverManager()->serverCount() == 0)
         return true;
-    QProgressDialog dlg(this);
-    connect(serverManager(), SIGNAL(allServerDescriptionAvailable()), &dlg, SLOT(close()));
-    QProgressBar *bar = new QProgressBar;
-    dlg.setLabelText(tr("Updating server information"));
-    dlg.setModal(true);
-    dlg.setBar(bar);
-    dlg.show();
-    connect(&dlg, SIGNAL(canceled()), &core(), SLOT(stopJobsAndClearQueues()));
+    if (d->_progressDialog) {
+        delete d->_progressDialog;
+        d->_progressDialog = 0;
+    }
+    d->_progressDialog = new QProgressDialog(this);
+    QProgressBar *bar = new QProgressBar(d->_progressDialog);
+    bar->setValue(0);
+    bar->setRange(0, 0);
+    d->_progressDialog->setBar(bar);
+    d->_progressDialog->setLabelText(tr("Updating server information"));
+    d->_progressDialog->setModal(true);
+    d->_progressDialog->show();
+    connect(serverManager(), SIGNAL(allServerDescriptionAvailable()), this, SLOT(onRefreshServerDone()), Qt::UniqueConnection);
     // TODO: Connect the cancel button
+    //connect(d->_progressDialog, SIGNAL(canceled()), &core(), SLOT(stopJobsAndClearQueues()));
+    //connect(d->_progressDialog, SIGNAL(canceled()), d->_progressDialog, SLOT(deleteLater()));
     serverManager()->getAllDescriptionFile(bar);
 //    dlg.exec();
     return true;
 }
 
+void ServerPackEditor::onRefreshServerDone()
+{
+    if (!d->_progressDialog)
+        return;
+    d->_progressDialog->setValue(d->_progressDialog->maximum());
+    delete d->_progressDialog;
+    d->_progressDialog = 0;
+    d->aServerRefresh->setEnabled(true);
+}
+
 bool ServerPackEditor::submitChanges()
 {
     return true;
 }
 
-void ServerPackEditor::createActions()
-{
-    // Create server actions
-    QActionGroup *srvgr = new QActionGroup(this);
-    QAction *a = d->aServerRefresh = new QAction(this);
-    a->setObjectName("aServerRefresh");
-    a->setIcon(icon(::ICON_SERVER_REFRESH, DataPack::DataPackCore::MediumPixmaps));
-    a = d->aServerEdit = new QAction(this);
-    a->setObjectName("aServerEdit");
-    a->setIcon(icon(::ICON_SERVER_EDIT, DataPack::DataPackCore::MediumPixmaps));
-    a = d->aServerAdd = new QAction(this);
-    a->setObjectName("aInstall");
-    a->setIcon(icon(::ICON_SERVER_ADD, DataPack::DataPackCore::MediumPixmaps));
-    a = d->aServerRemove = new QAction(this);
-    a->setObjectName("aServerRemove");
-    a->setIcon(icon(::ICON_SERVER_REMOVE, DataPack::DataPackCore::MediumPixmaps));
-    connect(srvgr, SIGNAL(triggered(QAction*)), this, SLOT(serverActionTriggered(QAction *)));
-
-    // Create pack actions
-    a = d->aPackRefresh = new QAction(this);
-    a->setObjectName("aPackRefresh");
-    a->setIcon(icon(::ICON_SERVER_REFRESH, DataPack::DataPackCore::MediumPixmaps));
-    a = d->aPackApply = new QAction(this);
-    a->setObjectName("aPackApply");
-    a->setIcon(icon(::ICON_INSTALL, DataPack::DataPackCore::MediumPixmaps));
-    connect(d->aPackRefresh, SIGNAL(triggered()), this, SLOT(refreshPacks()));
-    connect(d->aPackApply, SIGNAL(triggered()), this, SLOT(processPacks()));
-}
-
-void ServerPackEditor::createToolbar()
-{
-    d->m_ToolBarPacks = new QToolBar(this);
-
-    // Create the segmented button with server/pack
-    d->bServer = new QPushButton(this);
-    d->bServer->setCheckable(true);
-    d->bPack = new QPushButton(this);
-    d->bPack->setCheckable(true);
-    d->_segmented = new Utils::SegmentedButton(this);
-    d->_segmented->setFirstButton(d->bServer);
-    d->_segmented->setLastButton(d->bPack);
-    d->_segmented->setAutoExclusive(true);
-    QWidget *w1 = new QWidget(this);
-    w1->setMinimumSize(20,20);
-    w1->setMaximumSize(20,20);
-    d->m_ToolBarPacks->addWidget(w1);
-    d->m_ToolBarPacks->addWidget(d->_segmented);
-    w1 = new QWidget(this);
-    w1->setMinimumSize(20,20);
-    w1->setMaximumSize(20,20);
-    d->m_ToolBarPacks->addWidget(w1);
-    d->m_ToolBarPacks->addSeparator();
-
-    d->processToolBar(::PACK_MODE);
-
-    d->m_ToolBarPacks->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-    d->ui->toolbarLayout->addWidget(d->m_ToolBarPacks);
-
-    connect(d->m_ToolBarPacks, SIGNAL(actionTriggered(QAction*)), this, SLOT(serverActionTriggered(QAction*)));
-    connect(d->bPack, SIGNAL(clicked()), this, SLOT(swithToPackView()));
-    connect(d->bServer, SIGNAL(clicked()), this, SLOT(swithToServerView()));
-}
 
 //void ServerPackEditor::createServerDataWidgetMapper()
 //{
@@ -398,14 +432,18 @@ void ServerPackEditor::populatePackView(const int packId)
     QString zipPath = descr.data(PackDescription::UnzipToPath).toString();
     if (core().containsPathTag(zipPath))
         zipPath = core().replacePathTag(zipPath);
+    Server server = serverManager()->getServerForPack(pack);
     QString file = QString("<span style=\"font-weight:bold\">%1</span><br />"
                            "<table border=1 cellpadding=0 cellspacing=0 width=100%>"
                            "<tr><td>%2</td><td>%3</td></tr>"
-                           "<tr><td>MD5</td><td>%4</td></tr>"
-                           "<tr><td>SHA1</td><td>%5</td></tr>"
-                           "<tr><td>%6</td><td>%7</td></tr>"
+                           "<tr><td>%4</td><td>%5</td></tr>"
+                           "<tr><td>MD5</td><td>%6</td></tr>"
+                           "<tr><td>SHA1</td><td>%7</td></tr>"
+                           "<tr><td>%8</td><td>%9</td></tr>"
                            "</table>")
             .arg(tr("File specification"))
+            .arg(tkTr(Trans::Constants::SERVER))
+            .arg(server.url())
             .arg(tr("File name or URL:"))
             .arg(descr.data(PackDescription::AbsFileName).toString())
             .arg(descr.data(PackDescription::Md5).toString())
@@ -420,15 +458,17 @@ void ServerPackEditor::populatePackView(const int packId)
     d->ui->packSummary->setText(summary);
 }
 
-void ServerPackEditor::swithToPackView()
+void ServerPackEditor::switchToPackView()
 {
     d->ui->stackedWidget->setCurrentWidget(d->ui->packPage);
+    d->bPack->setChecked(true);
     d->processToolBar(::PACK_MODE);
 }
 
-void ServerPackEditor::swithToServerView()
+void ServerPackEditor::switchToServerView()
 {
     d->ui->stackedWidget->setCurrentWidget(d->ui->serverPage);
+    d->bServer->setChecked(true);
     d->processToolBar(::SERVER_MODE);
 }
 
@@ -436,9 +476,17 @@ void ServerPackEditor::onPackCategoriesChanged(const QModelIndex &index, const Q
 {
     if (!index.isValid())
         return;
+    // Filter the packmodel according to the selected category
     const QString &vendor = d->m_PackCategoriesModel->vendor(index);
     const QList<Pack::DataType> &type = d->m_PackCategoriesModel->datatype(index);
     d->m_PackModel->filter(vendor, type);
+    // Clear the view
+    d->ui->packName->clear();
+    d->ui->packSummary->clear();
+    // Select the first available row
+    d->ui->packView->setCurrentIndex(d->m_PackModel->index(0,0));
+    d->ui->packView->selectionModel()->select(d->m_PackModel->index(0,0), QItemSelectionModel::SelectCurrent);
+    onPackIndexActivated(d->m_PackModel->index(0,0), QModelIndex());
 }
 
 void ServerPackEditor::onPackIndexActivated(const QModelIndex &index, const QModelIndex &previous)
@@ -451,6 +499,7 @@ void ServerPackEditor::onPackIndexActivated(const QModelIndex &index, const QMod
 void ServerPackEditor::serverActionTriggered(QAction *a)
 {
     if (a==d->aServerRefresh) {
+        d->aServerRefresh->setEnabled(false);
         refreshServerContent();
     } if (a==d->aServerAdd) {
         AddServerDialog dlg(this);
@@ -474,7 +523,7 @@ void ServerPackEditor::serverActionTriggered(QAction *a)
         int row = d->ui->serverListView->selectionModel()->currentIndex().row();
         serverManager()->removeServerAt(row);
     } else if (a==d->aServerEdit) {
-        swithToServerView();
+        switchToServerView();
     }
 }
 
@@ -495,6 +544,7 @@ void ServerPackEditor::processPacks()
     if (dlg.exec()==QDialog::Rejected) {
         return;
     }
+    // Refresh the cached installed pack list from the packmanager
     packManager()->installedPack(true);
     d->m_PackModel->updateModel();
 }
@@ -506,30 +556,40 @@ void ServerPackEditor::populateServerView(const int serverId)
     QString summary;
 
     // short description, version date and author
-    summary = QString("<p style=\"font-weight:bold;font-size:large;\">%1</p>"
-                      "<p style=\"font-size:small;margin-left:20px;color:gray\">"
-                      "%2: %3<br />"
-                      "%4: %5<br />"
-                      "%6: %7<br />"
-                      "%8: %9<br />"
-                      "%10: %11<br />"
-                      "%12: %13<br />"
-                      "</p>"
-                       )
-            .arg(descr.data(ServerDescription::ShortDescription).toString())
-            .arg(tkTr(Trans::Constants::VERSION))
-            .arg(descr.data(ServerDescription::Version).toString())
-            .arg(tkTr(Trans::Constants::LAST_MODIFICATION))
-            .arg(descr.data(ServerDescription::LastModificationDate).toDate().toString("dd MM yyyy"))
-            .arg(tkTr(Trans::Constants::AUTHOR))
-            .arg(descr.data(ServerDescription::Author).toString())
-            .arg(tkTr(Trans::Constants::VENDOR))
-            .arg(descr.data(ServerDescription::Vendor).toString())
-            .arg(tr("Native URL"))
-            .arg(server.nativeUrl())
-            .arg(tkTr(Trans::Constants::RECOMMENDED_UPDATE_FREQUENCY))
-            .arg(Trans::ConstantTranslations::checkUpdateLabel(server.recommendedUpdateFrequency()))
-            ;
+    if (!server.uuid().isEmpty() && !server.version().isEmpty()) {
+        summary = QString("<p style=\"font-weight:bold;font-size:large;\">%1</p>"
+                          "<p style=\"font-size:small;margin-left:20px;color:gray\">"
+                          "%2: %3<br />"
+                          "%4: %5<br />"
+                          "%6: %7<br />"
+                          "%8: %9<br />"
+                          "%10: %11<br />"
+                          "%12: %13<br />"
+                          "</p>"
+                          )
+                .arg(descr.data(ServerDescription::ShortDescription).toString())
+                .arg(tkTr(Trans::Constants::VERSION))
+                .arg(descr.data(ServerDescription::Version).toString())
+                .arg(tkTr(Trans::Constants::LAST_MODIFICATION))
+                .arg(descr.data(ServerDescription::LastModificationDate).toDate().toString("dd MM yyyy"))
+                .arg(tkTr(Trans::Constants::AUTHOR))
+                .arg(descr.data(ServerDescription::Author).toString())
+                .arg(tkTr(Trans::Constants::VENDOR))
+                .arg(descr.data(ServerDescription::Vendor).toString())
+                .arg(tr("Native URL"))
+                .arg(server.nativeUrl())
+                .arg(tkTr(Trans::Constants::RECOMMENDED_UPDATE_FREQUENCY))
+                .arg(Trans::ConstantTranslations::checkUpdateLabel(server.recommendedUpdateFrequency()))
+                ;
+    } else {
+        summary = QString("<p style=\"font-weight:bold;\">%1</p>"
+                          "<p style=\"font-weight:bold;\">%2: %3</p>"
+                          )
+                .arg(tr("No information available. Please check the URL of the server."))
+                .arg(tr("Native URL"))
+                .arg(server.nativeUrl())
+                ;
+    }
 
     // Add description
     summary += descr.data(ServerDescription::HtmlDescription).toString();
@@ -575,7 +635,7 @@ void ServerPackEditor::serverCurrentChanged(const QModelIndex &c, const QModelIn
     populateServerView(c.row());
 }
 
-void ServerPackEditor::selectedFirstRow()
+void ServerPackEditor::selectFirstRow()
 {
     d->ui->packCategoriesView->setCurrentIndex(d->m_PackCategoriesModel->index(0,0));
     d->ui->packCategoriesView->selectionModel()->select(d->m_PackCategoriesModel->index(0,0), QItemSelectionModel::SelectCurrent);
@@ -588,6 +648,7 @@ void ServerPackEditor::selectedFirstRow()
 
     d->ui->serverListView->setCurrentIndex(d->m_serverModel->index(0,0));
     d->ui->serverListView->selectionModel()->select(d->m_serverModel->index(0,0), QItemSelectionModel::SelectCurrent);
+    populateServerView(0);
 }
 
 void ServerPackEditor::retranslate()
