@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,11 +19,17 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main Developers :                                                    *
+ *   Main Developers:                                                     *
  *       Eric MAEKER, MD <eric.maeker@gmail.com>                           *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
+/**
+ * \class Utils::GenericDescription
+ * \note Unit-test available (see: tests/auto/auto.pro)
+ */
+// TODO: write code documentation
+
 #include "genericdescription.h"
 
 #include <utils/log.h>
@@ -61,12 +67,15 @@ const char* const TAG_ICON             = "icon";
 const char* const TAG_TOOLTIP          = "tooltip";
 const char* const TAG_EXTRA            = "extra";
 const char* const TAG_AUTHORS          = "authors";
+const char* const TAG_COUNTRY          = "country";
+const char* const TAG_URL              = "url";
 const char* const TAG_VENDOR           = "vendor";
 const char* const TAG_VALIDITY         = "validity";
 const char* const TAG_CATEGORY         = "category";
 const char* const TAG_LICENSE_NAME     = "license";
 const char* const TAG_LICENSE_TERMS    = "licenseterms";
 const char* const TAG_SPECIALTIES      = "specialties";
+const char* const TAG_TYPENAME         = "typename";
 const char* const TAG_CREATIONDATE     = "cdate";
 const char* const TAG_LASTMODIFDATE    = "lmdate";
 const char* const TAG_BIBLIOGRAPHY     = "bibliography";
@@ -141,13 +150,29 @@ QVariant GenericDescription::data(const int ref, const QString &lang) const
     return var;
 }
 
-/** Define the data of the description. \sa addNonTranslatableExtraData(), addTranslatableExtraData() */
+/**
+ * Define the data of the description. \n
+ * All QDateTime milliseconds are set to zero to ensure a perfect compatibility with
+ * Qt::ISODate used in the XML transformation.
+ * \sa addNonTranslatableExtraData(), addTranslatableExtraData()
+ */
 bool GenericDescription::setData(const int ref, const QVariant &value, const QString &lang)
 {
     QString l = lang.toLower();
     if (lang.isEmpty())
         l = Trans::Constants::ALL_LANGUAGE;
-    m_Data[l].insert(ref, value);
+    // Remove ms to any QDateTime
+    if (value.canConvert(QVariant::DateTime)) {
+        QDateTime dt = value.toDateTime();
+        if (dt.time().msec() > 0) {
+            dt = QDateTime(dt.date(), QTime(dt.time().hour(), dt.time().minute(), dt.time().second()));
+            m_Data[l].insert(ref, dt);
+        } else {
+            m_Data[l].insert(ref, value);
+        }
+    } else {
+        m_Data[l].insert(ref, value);
+    }
     return true;
 }
 
@@ -229,7 +254,9 @@ QString GenericDescription::toXml() const
     // Create the main description tag
     QDomElement root = doc.createElement(m_RootTag);
     doc.appendChild(root);
-    toDomElement(&root, &doc);
+    if (!toDomElement(&root, &doc)) {
+        LOG_ERROR_FOR("GenericDescription", "Unable to generate XML code");
+    }
     return doc.toString(2);
 }
 
@@ -240,8 +267,14 @@ bool GenericDescription::toDomElement(QDomElement *root, QDomDocument *doc) cons
     Q_ASSERT(doc);
     if (!root || !doc)
         return false;
+    QDomElement toRoot = *root;
+    if (root->tagName().compare(m_RootTag, Qt::CaseInsensitive) != 0) {
+        QDomElement element = doc->createElement(m_RootTag);
+        root->appendChild(element);
+        toRoot = element;
+    }
     QDomComment comment = doc->createComment("Non translatable values");
-    root->appendChild(comment);
+    toRoot.appendChild(comment);
 
     // Set non translatable items
     QHash<int, QString> elements;
@@ -249,8 +282,10 @@ bool GenericDescription::toDomElement(QDomElement *root, QDomDocument *doc) cons
     QHashIterator<int, QString> i(elements);
     while (i.hasNext()) {
         i.next();
+        if (data(i.key()).toString().isEmpty())
+            continue;
         QDomElement e = doc->createElement(i.value());
-        root->appendChild(e);
+        toRoot.appendChild(e);
         e.setNodeValue(data(i.key()).toString());
         QDomText t = doc->createTextNode(data(i.key()).toString());
         e.appendChild(t);
@@ -258,7 +293,7 @@ bool GenericDescription::toDomElement(QDomElement *root, QDomDocument *doc) cons
 
     // Set translatable contents
     comment = doc->createComment("Translatable values");
-    root->appendChild(comment);
+    toRoot.appendChild(comment);
     elements.clear();
     elements = translatableTagsDataReference();
     i = elements;
@@ -270,7 +305,7 @@ bool GenericDescription::toDomElement(QDomElement *root, QDomDocument *doc) cons
                 continue;
             QDomElement e = doc->createElement(i.value());
             e.setAttribute(ATTRIB_LANG, l);
-            root->appendChild(e);
+            toRoot.appendChild(e);
             QDomText t = doc->createTextNode(tmp);
             e.appendChild(t);
         }
@@ -279,9 +314,9 @@ bool GenericDescription::toDomElement(QDomElement *root, QDomDocument *doc) cons
     // Add update information
     if (m_UpdateInfos.count() > 0) {
         comment = doc->createComment("Update information");
-        root->appendChild(comment);
+        toRoot.appendChild(comment);
         QDomElement e = doc->createElement(GenericUpdateInformation::xmlTagName());
-        root->appendChild(e);
+        toRoot.appendChild(e);
         for(int i=0; i < m_UpdateInfos.count(); ++i) {
             m_UpdateInfos.at(i).toDomElement(&e, doc);
         }
@@ -337,7 +372,7 @@ QString GenericDescription::htmlUpdateInformationForVersion(const Utils::Version
     return html;
 }
 
-/** You can add untranslatable extra-data. \e ref must be greated than GenericDescription::NonTranslatableExtraData and lower than GenericDescription::TranslatableExtraData. */
+/** You can add untranslatable extra-data. \e ref must be greater than NonTranslatableExtraData and lower than TranslatableExtraData. */
 void GenericDescription::addNonTranslatableExtraData(const int ref, const QString &tagName)
 {
     Q_ASSERT(ref>NonTranslatableExtraData && ref<TranslatableExtraData);
@@ -346,7 +381,7 @@ void GenericDescription::addNonTranslatableExtraData(const int ref, const QStrin
     }
 }
 
-/** You can add translatable extra-data. \e ref must be greated than GenericDescription::TranslatableExtraData. */
+/** You can add translatable extra-data. \e ref must be greater than TranslatableExtraData. */
 void GenericDescription::addTranslatableExtraData(const int ref, const QString &tagName)
 {
     Q_ASSERT(ref>TranslatableExtraData);
@@ -363,19 +398,64 @@ void GenericDescription::addTranslatableExtraData(const int ref, const QString &
 //    qWarning() << "Use the Utils::GenericDescription inheritance to access to these data";
 //}
 
-void GenericDescription::toTreeWidget(QTreeWidget *tree) const
+void GenericDescription::fillTreeWidget(QTreeWidget *tree) const
 {
     Q_UNUSED(tree);
 }
 
+/** Test equality between two descriptions. */
+bool GenericDescription::operator==(const GenericDescription &other) const
+{
+    // First basic tests
+    if (m_UpdateInfos.count() != other.m_UpdateInfos.count() ||
+            m_RootTag != other.m_RootTag ||
+            m_SourceFileName != other.m_SourceFileName ||
+            m_TranslatableExtra.count() != other.m_TranslatableExtra.count() ||
+            m_NonTranslatableExtra.count() != other.m_NonTranslatableExtra.count())
+        return false;
+
+    // Test extras
+    if (m_TranslatableExtra != other.m_TranslatableExtra)
+        return false;
+    if (m_NonTranslatableExtra != other.m_NonTranslatableExtra)
+        return false;
+
+    // Test datas
+    QStringList lang = m_Data.keys();
+    QStringList other_lang = other.m_Data.keys();
+    if (lang.count() != other_lang.count())
+        return false;
+    lang.sort();
+    other_lang.sort();
+    if (lang != other_lang)
+        return false;
+
+    foreach(const QString &l, lang) {
+        const QHash<int, QVariant> &first = m_Data.value(l);
+        const QHash<int, QVariant> &second = other.m_Data.value(l);
+//         foreach(int id, first.keys()) {
+//             if (first.value(id) != second.value(id))
+//                 qWarning() << id << l << first.value(id) << second.value(id);
+//         }
+        if (first != second)
+            return false;
+    }
+
+    // Test update infos /!\ lists must be sorted in the same order...
+    // TODO: bool updatesAreEqual = (m_UpdateInfos == other.m_UpdateInfos);
+    return true;
+}
+
 QHash<int, QString> GenericDescription::nonTranslatableTagsDataReference() const
 {
     QHash<int, QString> elements;
     // get non translatable items
     elements.insert(GenericDescription::Uuid, TAG_UUID);
     elements.insert(GenericDescription::Version, TAG_VERSION);
-    elements.insert(GenericDescription::AbsFileName, TAG_ABSFILENAME);
     elements.insert(GenericDescription::Author, TAG_AUTHORS);
+    elements.insert(GenericDescription::Country, TAG_COUNTRY);
+    elements.insert(GenericDescription::URL, TAG_URL);
+    elements.insert(GenericDescription::AbsFileName, TAG_ABSFILENAME);
     elements.insert(GenericDescription::Vendor, TAG_VENDOR);
     elements.insert(GenericDescription::Validity, TAG_VALIDITY);
     elements.insert(GenericDescription::FreeMedFormsCompatVersion, TAG_FMF_COMPAT_VERSION);
@@ -400,6 +480,7 @@ QHash<int, QString> GenericDescription::translatableTagsDataReference() const
     elements.insert(GenericDescription::Category, TAG_CATEGORY);
     elements.insert(GenericDescription::ToolTip, TAG_TOOLTIP);
     elements.insert(GenericDescription::Specialties, TAG_SPECIALTIES);
+    elements.insert(GenericDescription::TypeName, TAG_TYPENAME);
     elements.insert(GenericDescription::ShortDescription, TAG_DESCRIPTION);
     elements.insert(GenericDescription::HtmlDescription, TAG_HTMLDESCRIPTION);
     elements.insert(GenericDescription::HtmlSynthesis, TAG_HTMLSYNTHESIS);
