@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  The FreeAccount plugins are free, open source FreeMedForms' plugins.   *
@@ -24,9 +24,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *  Main Developers : Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
+ *  Main Developers:  Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
  *                     Eric MAEKER, <eric.maeker@gmail.com>                *
- *  Contributors :                                                         *
+ *  Contributors:                                                          *
  *      NAME <MAIL@ADDRESS.COM>                                            *
  ***************************************************************************/
 #include "accountbase.h"
@@ -68,6 +68,8 @@ using namespace AccountDB;
 using namespace AccountDB::Internal;
 using namespace Trans::ConstantTranslations;
 
+enum Warn {WarnDebugMessage = true};
+
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
 static inline Core::ICommandLine *commandLine()  { return Core::ICore::instance()->commandLine(); }
 
@@ -117,9 +119,7 @@ AccountBase *AccountBase::m_Instance = 0;
 /** \brief Returns the unique instance of AccountBase. If it does not exist, it is created */
 AccountBase *AccountBase::instance()
 {
-    if (!m_Instance) {
-        m_Instance = new AccountBase(qApp);
-    }
+    Q_ASSERT(m_Instance);
     return m_Instance;
 }
 
@@ -131,6 +131,7 @@ AccountBase::AccountBase(QObject *parent) :
     QObject(parent), Utils::Database(),
     d(0)
 {
+    m_Instance = this;
     d = new AccountBasePrivate(this);
     setObjectName("AccountBase");
 
@@ -169,6 +170,7 @@ AccountBase::AccountBase(QObject *parent) :
     addField(Table_MedicalProcedure, MP_REIMBOURSEMENT, "REIMBOURSEMENT", FieldIsReal);
     addField(Table_MedicalProcedure, MP_DATE,           "DATE",           FieldIsDate);
     addField(Table_MedicalProcedure, MP_OTHERS,         "OTHERS",         FieldIsBlob);
+    addField(Table_MedicalProcedure, MP_COUNTRY,        "COUNTRY",         FieldIsBlob);
 
 //    "CREATE TABLE 	actes_disponibles ("  --> medical_procedure
 //            "id_acte_dispo  int(10)  	UNSIGNED  	       		NOT NULL  	 auto_increment ,"
@@ -255,25 +257,25 @@ AccountBase::AccountBase(QObject *parent) :
     addField(Table_Account,  ACCOUNT_ID,              "ACCOUNT_ID",     FieldIsUniquePrimaryKey);
     addField(Table_Account,  ACCOUNT_UID,             "ACCOUNT_UID",    FieldIsUUID);
     addField(Table_Account,  ACCOUNT_USER_UID,        "USER_UID",       FieldIsUUID);
-    addField(Table_Account,  ACCOUNT_PATIENT_UID,     "PATIENT_UID",    FieldIsLongInteger);
-    addField(Table_Account,  ACCOUNT_PATIENT_NAME,    "PATIENT_NAME",   FieldIsLongInteger);
+    addField(Table_Account,  ACCOUNT_PATIENT_UID,     "PATIENT_UID",    FieldIsShortText);
+    addField(Table_Account,  ACCOUNT_PATIENT_NAME,    "PATIENT_NAME",   FieldIsLongText);
     addField(Table_Account,  ACCOUNT_SITE_ID,         "SITE_ID",        FieldIsLongInteger);
     addField(Table_Account,  ACCOUNT_INSURANCE_ID,    "INSURANCE_ID",   FieldIsLongInteger);
     addField(Table_Account,  ACCOUNT_DATE,            "DATE",           FieldIsDate, "CURRENT_DATE");
-    addField(Table_Account,  ACCOUNT_MEDICALPROCEDURE_XML,   "MP_XML", FieldIsLongText);
+    addField(Table_Account,  ACCOUNT_MEDICALPROCEDURE_XML,   "MP_XML",  FieldIsLongText);
     addField(Table_Account,  ACCOUNT_MEDICALPROCEDURE_TEXT,   "MP_TXT", FieldIsLongText);
     addField(Table_Account,  ACCOUNT_COMMENT,         "COMMENT",        FieldIsLongText);
-    addField(Table_Account,  ACCOUNT_CASHAMOUNT,      "CASH",         FieldIsReal);
+    addField(Table_Account,  ACCOUNT_CASHAMOUNT,      "CASH",           FieldIsReal);
     addField(Table_Account,  ACCOUNT_CHEQUEAMOUNT,    "CHEQUE",         FieldIsReal);
-    addField(Table_Account,  ACCOUNT_VISAAMOUNT,      "VISA",         FieldIsReal);
-    addField(Table_Account,  ACCOUNT_INSURANCEAMOUNT, "BANKING",         FieldIsReal);
-    addField(Table_Account,  ACCOUNT_OTHERAMOUNT,     "OTHER",         FieldIsReal);
-    addField(Table_Account,  ACCOUNT_DUEAMOUNT,       "DUE",         FieldIsReal);
-    addField(Table_Account,  ACCOUNT_DUEBY,           "DUE_BY", FieldIsLongText);
-    addField(Table_Account,  ACCOUNT_ISVALID,         "VALID",         FieldIsBoolean);
+    addField(Table_Account,  ACCOUNT_VISAAMOUNT,      "VISA",           FieldIsReal);
+    addField(Table_Account,  ACCOUNT_INSURANCEAMOUNT, "BANKING",        FieldIsReal);
+    addField(Table_Account,  ACCOUNT_OTHERAMOUNT,     "OTHER",          FieldIsReal);
+    addField(Table_Account,  ACCOUNT_DUEAMOUNT,       "DUE",            FieldIsReal);
+    addField(Table_Account,  ACCOUNT_DUEBY,           "DUE_BY",         FieldIsLongText);
+    addField(Table_Account,  ACCOUNT_ISVALID,         "VALID",          FieldIsBoolean);
     addField(Table_Account,  ACCOUNT_TRACE,           "TRACE",          FieldIsBlob);
-    
-    
+
+
     addField(Table_Acts,  ACTS_ID,              "ACTS_ID",     FieldIsUniquePrimaryKey);
     addField(Table_Acts,  ACTS_UID,             "ACTS_UID",    FieldIsUUID);
     addField(Table_Acts,  ACTS_USER_UID,        "USER_UID",       FieldIsUUID);
@@ -350,7 +352,7 @@ AccountBase::AccountBase(QObject *parent) :
     addField(Table_AssetsRates,  ASSETSRATES_YEARS,    "YEARS_RANGE",    FieldIsBlob);
     addField(Table_AssetsRates,  ASSETSRATES_RATES,    "RATE",           FieldIsReal);
     addField(Table_AssetsRates,  ASSETSRATES_DATE,     "DATE",           FieldIsDate,"CURRENT_DATE");
-    
+
 //    addTable(Table_Movement, "movement");
     addField(Table_Movement,  MOV_ID,             "MOV_ID",     FieldIsUniquePrimaryKey);
     addField(Table_Movement,  MOV_AV_MOVEMENT_ID, "AV_MOV_ID",  FieldIsLongInteger);
@@ -414,7 +416,7 @@ AccountBase::AccountBase(QObject *parent) :
     addField(Table_Insurance,  INSURANCE_MAIL,        "MAIL",   FieldIsShortText);
     addField(Table_Insurance,  INSURANCE_CONTACT,     "CONTACT",   FieldIsShortText);
     addField(Table_Insurance,  INSURANCE_PREF,        "PREFERRED",  FieldIsBoolean);
-    
+
     /** Create an IDENTITY table */
 
 //    "CREATE TABLE 	payeurs	("  --> insurance
@@ -483,8 +485,8 @@ AccountBase::AccountBase(QObject *parent) :
     addField(Table_Rules,  RULES_TYPE,    "TYPE",          FieldIsShortText);
     addField(Table_Rules,  RULES_VALUES,  "VALUES",        FieldIsShortText);
     addField(Table_Rules,  RULES_PREF,    "PREFERRED",      FieldIsBoolean);
-    
-    
+
+
     addField(Table_DistanceRules,  DISTRULES_ID,      "DISTRULES_ID",      FieldIsUniquePrimaryKey);
     addField(Table_DistanceRules,  DISTRULES_UID,     "DISTRULES_UID",     FieldIsUUID);
     addField(Table_DistanceRules,  DISTRULES_TYPE,    "TYPE",              FieldIsShortText);
@@ -549,7 +551,6 @@ AccountBase::AccountBase(QObject *parent) :
 //          "surname          varchar(50)               NULL,"
 //          "guid             varchar(6)                NOT NULL);";
 
-
     // Connect first run database creation requested
     connect(Core::ICore::instance(), SIGNAL(firstRunDatabaseCreation()), this, SLOT(onCoreFirstRunCreationRequested()));
 }
@@ -557,8 +558,10 @@ AccountBase::AccountBase(QObject *parent) :
 /** \brief Destructor. */
 AccountBase::~AccountBase()
 {
-    qWarning() << "AccountBase::~AccountBase()" ;
-    if (d) delete d;
+    if (Utils::Log::warnPluginsCreation())
+        WARN_FUNC;
+    if (d)
+        delete d;
     d=0;
 }
 
@@ -588,38 +591,43 @@ bool AccountBase::initialize()
     } else {
         LOG(tkTr(Trans::Constants::CONNECTED_TO_DATABASE_1_DRIVER_2).arg(database().connectionName()).arg(database().driverName()));
     }
-    if (!checkDatabaseScheme()) {
-       if(checkIfIsFirstVersion()){
-        qDebug() << __FILE__ << QString::number(__LINE__) << "ISFIRSTVERSION";             
-        if (fieldNamesSql(AccountDB::Constants::Table_MedicalProcedure).size()< AccountDB::Constants::MP_MaxParam)
-        {
-        	  if (!alterTableForNewField(AccountDB::Constants::Table_MedicalProcedure, AccountDB::Constants::MP_OTHERS,FieldIsBlob, QString("NULL")))
-        	  {
-        	  	  LOG_ERROR("Unable to add new field in table MP");
-        	  	  return false;
-        	      }
-        	  else
-        	  {
-        	      foreach(QString field,fieldNamesSql(AccountDB::Constants::Table_MedicalProcedure)){
-        	          qWarning() << __FILE__ << QString::number(__LINE__) << " field =" << field ;
-                          }
-                        LOG("New Version = "+checkAndReplaceVersionNumber());
-        	  	return true;
-        	      }
-                }           	
-                
-            }
-        else
-        {
-        	LOG_ERROR(tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(Constants::DB_ACCOUNTANCY));
-        	return false;
-            }
-        
-    }
 
-    //checkDatabaseScheme
+    // TODO: start a global transaction for all this code, remove the transactions inside the sub-members, close the transaction
+    if (!checkDatabaseScheme()) {
+        if(checkIfIsFirstVersion()){
+            qDebug() << __FILE__ << QString::number(__LINE__) << "ISFIRSTVERSION";
+            if (fieldNamesSql(AccountDB::Constants::Table_MedicalProcedure).size()< AccountDB::Constants::MP_MaxParam) {
+                if (!alterTableForNewField(AccountDB::Constants::Table_MedicalProcedure, AccountDB::Constants::MP_OTHERS,FieldIsBlob, QString("NULL"))
+                       && !alterTableForNewField(AccountDB::Constants::Table_MedicalProcedure, AccountDB::Constants::MP_COUNTRY,FieldIsBlob, QString("NULL"))) {
+                    LOG_ERROR("Unable to add new field in table MP");
+                    return false;
+                } else {
+                    foreach(QString field,fieldNamesSql(AccountDB::Constants::Table_MedicalProcedure)) {
+                        qWarning() << __FILE__ << QString::number(__LINE__) << " field =" << field ;
+                    }
+                    LOG("New Version = "+checkAndReplaceVersionNumber());
+                    return true;
+                }
+            }
+
+        } else {
+            LOG_ERROR(tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(Constants::DB_ACCOUNTANCY));
+            return false;
+        }
+    }  //checkDatabaseScheme
+
+    if (checkIfVersionBeforeThirdVersion() && driver() == Utils::Database::MySQL) {
+        if (alterFieldPatientNameIntToVarchar()) {
+            LOG("Field PATIENT_NAME has been changed to varchar");
+        } else {
+            LOG_ERROR(tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(Constants::DB_ACCOUNTANCY));
+            return false;
+        }
+    }
+    // End of todo commit transaction
+
     if (versionHasChanged()) {
-        LOG("Version has changed , new version = "+checkAndReplaceVersionNumber());
+        LOG("Version has changed , new version = " + checkAndReplaceVersionNumber());
     }
 
     connect(Core::ICore::instance(), SIGNAL(databaseServerChanged()), this, SLOT(onCoreDatabaseServerChanged()));
@@ -636,13 +644,16 @@ void AccountBase::logChronos(bool log)
     d->m_LogChrono = log;
 }
 
-bool AccountBase::createDatabase(const QString &connectionName , const QString &dbName,
-                    const QString &pathOrHostName,
-                    TypeOfAccess , AvailableDrivers driver,
-                    const QString & login, const QString & pass,
-                    const int port,
-                    CreationOption
-                   )
+bool AccountBase::createDatabase(const QString &connectionName ,
+                                 const QString &dbName,
+                                 const QString &pathOrHostName,
+                                 TypeOfAccess ,
+                                 AvailableDrivers driver,
+                                 const QString & login,
+                                 const QString & pass,
+                                 const int port,
+                                 CreationOption
+                                 )
 {
     // TODO: manage createOption
     if (connectionName != Constants::DB_ACCOUNTANCY)
@@ -657,7 +668,7 @@ bool AccountBase::createDatabase(const QString &connectionName , const QString &
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         DB.open();
         setDriver(Utils::Database::SQLite);
@@ -711,17 +722,26 @@ bool AccountBase::createDatabase(const QString &connectionName , const QString &
     }
 
     // Add version number
+    DB.transaction();
     QSqlQuery query(DB);
     query.prepare(prepareInsertQuery(Constants::Table_VERSION));
     query.bindValue(Constants::VERSION_ACTUAL, Constants::DB_VERSION_NUMBER);
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.commit();
         return false;
     }
-
+    query.finish();
+    DB.rollback();
     return true;
 }
 
+/*!
+ * \brief Queries the database to find an account with the given \uid.
+ * \return A pointer to a newly created \sa AccountData object, filled with
+ * all fields of this database record.
+ */
 AccountData *AccountBase::getAccountByUid(const QString &uid)
 {
     QSqlDatabase DB = QSqlDatabase::database(Constants::DB_ACCOUNTANCY);
@@ -734,21 +754,29 @@ AccountData *AccountBase::getAccountByUid(const QString &uid)
     QHash<int, QString> where;
     where.insert(Constants::ACCOUNT_UID, QString("=%1").arg(uid));
     QString req = select(Constants::Table_Account, where);
-    QSqlQuery q(req,DB);
-    if (q.isActive()) {
-        if (q.next()) {
+    DB.transaction();
+    QSqlQuery query(req,DB);
+    if (query.isActive()) {
+        if (query.next()) {
             AccountData *data = new AccountData();
-            for(int i = 0 ; i < Constants::ACCOUNT_MaxParam; ++i) {
-                data->setDatasFromDb(i, q.value(i));
+            // ref is a data representation of enum AccountDB::Constants::TableAcc_Fields
+            for(int ref = 0 ; ref < Constants::ACCOUNT_MaxParam; ++ref) {
+                data->setDataFromDb(ref, query.value(ref));
             }
+            query.finish();
+            DB.commit();
             return data;
         } else {
             LOG_ERROR("No account with an UID like " + uid);
+            query.finish();
+            DB.rollback();
             return 0;
         }
     } else {
         LOG_ERROR("No account with an UID like " + uid);
-        LOG_QUERY_ERROR(q);
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
         return 0;
     }
     return 0;
@@ -774,24 +802,26 @@ bool AccountBase::checkIfIsFirstVersion()
 {
     QSqlDatabase DB = QSqlDatabase::database(Constants::DB_ACCOUNTANCY);
     if (!connectDatabase(DB, __LINE__))
-        return 0;
+        return false;
     QVariant version;
-    QSqlQuery qy(database());
+    DB.transaction();
+    QSqlQuery qy(DB);
     QString req = select(Constants::Table_VERSION, Constants::VERSION_ACTUAL);//QString("SELECT %1 FROM %2").arg("ACTUAL","VERSION");
-    if (!qy.exec(req))
-    {
-    	  LOG_QUERY_ERROR(qy);
-    	  return false;
-        }
-    while (qy.next())
-    {
-    	version = qy.value(0);
-        }
-    if (version == QVariant("0.1"))
-    {
-    	  LOG(QString("VERSION == 0.1"));
-    	  return true;
-        }
+    if (!qy.exec(req)) {
+        LOG_QUERY_ERROR(qy);
+        qy.finish();
+        DB.rollback();
+        return false;
+    }
+    while (qy.next()) {
+        version = qy.value(0);
+    }
+    qy.finish();
+    DB.commit();
+    if (version == QVariant("0.1")) {
+        LOG(QString("VERSION == 0.1"));
+        return true;
+    }
     return false;
 }
 
@@ -800,22 +830,25 @@ QString AccountBase::checkAndReplaceVersionNumber()
     QSqlDatabase DB = QSqlDatabase::database(Constants::DB_ACCOUNTANCY);
     if (!connectDatabase(DB, __LINE__))
         return 0;
-   // if (versionHasChanged())
-   // {
-    	  QSqlQuery qy(database());
-    	  /*QString req = QString("INSERT INTO %1 (%2) VALUES ('%3')")
-    	               .arg("VERSION","ACTUAL",QString(Constants::VERSION_ACTUAL));*/
-    	  qy.prepare(prepareInsertQuery(Constants::Table_VERSION));
-          qy.bindValue(Constants::VERSION_ACTUAL, Constants::DB_VERSION_NUMBER);
-	  
-    	  if (!qy.exec())
-    	  {
-    	  	  LOG_QUERY_ERROR(qy);
-    	  	  return QString(qy.lastError().text());
-    	      }
-    	  return QString(Constants::DB_VERSION_NUMBER);
-     //   }
-     
+    // if (versionHasChanged())
+    // {
+    DB.transaction();
+    QSqlQuery qy(DB);
+    /*QString req = QString("INSERT INTO %1 (%2) VALUES ('%3')")
+                       .arg("VERSION","ACTUAL",QString(Constants::VERSION_ACTUAL));*/
+    qy.prepare(prepareInsertQuery(Constants::Table_VERSION));
+    qy.bindValue(Constants::VERSION_ACTUAL, Constants::DB_VERSION_NUMBER);
+
+    if (!qy.exec()) {
+        LOG_QUERY_ERROR(qy);
+        qy.finish();
+        DB.rollback();
+        return QString(qy.lastError().text());
+    }
+    qy.finish();
+    DB.commit();
+    return QString(Constants::DB_VERSION_NUMBER);
+    //   }
 }
 
 bool AccountBase::versionHasChanged()
@@ -824,20 +857,75 @@ bool AccountBase::versionHasChanged()
     if (!connectDatabase(DB, __LINE__))
         return 0;
     QString version;
-    QSqlQuery qy(database());
+    DB.transaction();
+    QSqlQuery qy(DB);
     QString req = select(Constants::Table_VERSION, Constants::VERSION_ACTUAL);//QString("SELECT %1 FROM %2").arg("ACTUAL","VERSION");
-    if (!qy.exec(req))
-    {
-    	  LOG_QUERY_ERROR(qy);
-    	  return false;
-        }
-    while (qy.next())
-    {
-    	version = qy.value(0).toString();
-        }
-    if (!version.contains(QString(Constants::DB_VERSION_NUMBER)))
-    {
-    	  return true;
-        }
+    if (!qy.exec(req)) {
+        LOG_QUERY_ERROR(qy);
+        return false;
+    }
+    while (qy.next()) {
+        version = qy.value(0).toString();
+    }
+    qy.finish();
+    DB.commit();
+    if (!version.contains(QString(Constants::DB_VERSION_NUMBER))) {
+        return true;
+    }
     return false;
 }
+
+bool AccountBase::checkIfVersionBeforeThirdVersion()
+{
+    QString version;
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_ACCOUNTANCY);
+    if (!connectDatabase(DB, __LINE__))
+        return false;
+    DB.transaction();
+    QSqlQuery qy(DB);
+    QString req = select(Constants::Table_VERSION, Constants::VERSION_ACTUAL);//QString("SELECT %1 FROM %2").arg("ACTUAL","VERSION");
+    if (!qy.exec(req))  {
+        LOG_QUERY_ERROR(qy);
+        DB.rollback();
+        return false;
+    }
+    while (qy.next())  {
+        version = qy.value(0).toString();
+    }
+    qy.finish();
+    DB.commit();
+    double versionDouble = version.toDouble();
+    if (versionDouble == 0.0) {
+        LOG("Alter field PATIENT_NAME: Error conversion of double");
+    }
+    if (versionDouble < 0.3) {
+        return true;
+    }
+    return false;
+}
+
+bool AccountBase::alterFieldPatientNameIntToVarchar()
+{
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_ACCOUNTANCY);
+    if (!connectDatabase(DB, __LINE__))
+        return false;
+    QString tableName = table(AccountDB::Constants::Table_Account);
+    QString fieldPatientName    = fieldName(AccountDB::Constants::Table_Account,AccountDB::Constants::ACCOUNT_PATIENT_NAME);
+    QString fieldPatientUuid = fieldName(AccountDB::Constants::Table_Account,AccountDB::Constants::ACCOUNT_PATIENT_UID);
+    if (WarnDebugMessage)
+    qDebug() << __FILE__ << QString::number(__LINE__) << " table + fielName ="
+    << tableName+" + "+fieldPatientName  ;
+    DB.transaction();
+    QSqlQuery qy(DB);
+    QString req = QString("ALTER TABLE %1 MODIFY %2 varchar(2000) NULL , MODIFY %3 varchar(200) NULL ;")
+                         .arg(tableName,fieldPatientName,fieldPatientUuid);
+    if (!qy.exec(req)) {
+        LOG_QUERY_ERROR(qy);
+        qy.finish();
+        DB.rollback();
+        return false;
+    }
+    qy.finish();
+    DB.commit();
+    return true;
+}
