@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  The FreeAccount plugins are free, open source FreeMedForms' plugins.   *
@@ -24,9 +24,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *  Main Developers : Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
+ *  Main Developers:  Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
  *                     Eric MAEKER, <eric.maeker@gmail.com>                *
- *  Contributors :                                                         *
+ *  Contributors:                                                          *
  *      NAME <MAIL@ADDRESS.COM>                                            *
  ***************************************************************************/
 #include "insurancepage.h"
@@ -43,7 +43,7 @@
 #include <coreplugin/isettings.h>
 #include <coreplugin/itheme.h>
 #include <coreplugin/iuser.h>
-
+#include <coreplugin/constants_menus.h>
 #include <coreplugin/constants_icons.h>
 
 #include "ui_insurancepage.h"
@@ -80,16 +80,20 @@ InsurancePage::~InsurancePage()
 }
 
 QString InsurancePage::id() const { return objectName(); }
-QString InsurancePage::name() const { return tkTr(Trans::Constants::INSURANCE); }
+QString InsurancePage::displayName() const { return tkTr(Trans::Constants::INSURANCE); }
 QString InsurancePage::category() const { return tkTr(Trans::Constants::ACCOUNTANCY); }
+int InsurancePage::sortIndex() const
+{
+    return Core::Constants::OPTIONINDEX_ACCOUNT + 60;
+}
 
 void InsurancePage::resetToDefaults()
 {
     m_Widget->writeDefaultSettings(settings());
-    m_Widget->setDatasToUi();
+    m_Widget->setDataToUi();
 }
 
-void InsurancePage::applyChanges()
+void InsurancePage::apply()
 {
     if (WarnDebugMessage)
         LOG("applyChanges");
@@ -165,17 +169,17 @@ InsuranceWidget::InsuranceWidget(QWidget *parent) :
     m_Mapper->addMapping(ui->cityEdit, AccountDB::Constants::INSURANCE_CITY,"text");
     m_Mapper->addMapping(ui->zip, AccountDB::Constants::INSURANCE_ZIPCODE, "currentText");
     m_Mapper->addMapping(ui->countryComboBox, AccountDB::Constants::INSURANCE_COUNTRY, "currentIsoCountry");
-    
+
     m_Mapper->addMapping(ui->phoneEdit, AccountDB::Constants::INSURANCE_TEL);
     m_Mapper->addMapping(ui->faxEdit, AccountDB::Constants::INSURANCE_FAX);
     m_Mapper->addMapping(ui->mailEdit, AccountDB::Constants::INSURANCE_MAIL);
-    
+
     m_Mapper->addMapping(ui->contactEdit, AccountDB::Constants::INSURANCE_CONTACT);
     m_Mapper->toFirst();
     ui->insuranceComboBox->setModel(m_Model);
     ui->insuranceComboBox->setModelColumn(AccountDB::Constants::INSURANCE_NAME);
-  
-    setDatasToUi();
+
+    setDataToUi();
 }
 
 InsuranceWidget::~InsuranceWidget()
@@ -208,7 +212,7 @@ InsuranceWidget::~InsuranceWidget()
 //    emit findCityFromZipCode(zipComboBox->currentText());
 //}
 
-void InsuranceWidget::setDatasToUi()
+void InsuranceWidget::setDataToUi()
 {
     if (WarnDebugMessage)
         LOG("index row  = " + QString::number(ui->insuranceComboBox->currentIndex()));
@@ -228,7 +232,7 @@ void InsuranceWidget::saveModel()
                 LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_SAVE_DATA_IN_DATABASE_1).
                           arg(tr("insurance")));
             }
-        } 
+        }
         else {
             m_Model->revert();
         }
@@ -265,7 +269,7 @@ void InsuranceWidget::on_addButton_clicked()
     cityEdit->setText("city");
     countryComboBox->addItem("country");
     countryComboBox->setFocus();
-    
+
     zipComboBox->setFocus();
     phoneEdit->setText("123457");
     faxEdit->setText("1245677");
@@ -288,7 +292,7 @@ void InsuranceWidget::saveToSettings(Core::ISettings *sets)
     if (!m_Model->submit()) {
         LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_SAVE_DATA_IN_DATABASE_1).arg(tr("insurance")));
         Utils::warningMessageBox(tr("Can not submit insurance to your personnal database."),
-                                 tr("An error occured during insurance saving. Datas are corrupted."));
+                                 tr("An error occured during insurance saving. Data are corrupted."));
     }
     connect(ui->nameEdit,SIGNAL(textEdited(const QString &)), ui->insuranceComboBox,SLOT(setEditText(const QString &)));
     update();
