@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  The FreeAccount plugins are free, open source FreeMedForms' plugins.   *
@@ -24,9 +24,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *  Main Developers : Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
+ *  Main Developers:  Pierre-Marie DESOMBRE <pm.desombre@medsyn.fr>,      *
  *                     Eric MAEKER, <eric.maeker@gmail.com>                *
- *  Contributors :                                                         *
+ *  Contributors:                                                          *
  *      NAME <MAIL@ADDRESS.COM>                                            *
  ***************************************************************************/
 #include "medicalprocedurepage.h"
@@ -43,11 +43,9 @@
 #include <coreplugin/isettings.h>
 #include <coreplugin/itheme.h>
 #include <coreplugin/iuser.h>
-
+#include <coreplugin/constants_menus.h>
 #include <coreplugin/constants_icons.h>
 
-#include <accountbaseplugin/constants.h>
-
 #include <QCompleter>
 #include <QDir>
 #include <QTimer>
@@ -78,16 +76,20 @@ MedicalProcedurePage::~MedicalProcedurePage()
 }
 
 QString MedicalProcedurePage::id() const { return objectName(); }
-QString MedicalProcedurePage::name() const { return tkTr(Trans::Constants::MEDICAL_PROCEDURES); }
+QString MedicalProcedurePage::displayName() const { return tkTr(Trans::Constants::MEDICAL_PROCEDURES); }
 QString MedicalProcedurePage::category() const { return tkTr(Trans::Constants::ACCOUNTANCY); }
+int MedicalProcedurePage::sortIndex() const
+{
+    return Core::Constants::OPTIONINDEX_ACCOUNT + 70;
+}
 
 void MedicalProcedurePage::resetToDefaults()
 {
     m_Widget->writeDefaultSettings(settings());
-    m_Widget->setDatasToUi();
+    m_Widget->setDataToUi();
 }
 
-void MedicalProcedurePage::applyChanges()
+void MedicalProcedurePage::apply()
 {
     if (WarnDebugMessage)
         LOG("applyChanges");
@@ -140,21 +142,21 @@ MedicalProcedureWidget::MedicalProcedureWidget(QWidget *parent) :
     dateEdit->setDate(QDate::currentDate());
     amountSpin->setRange(0.00,1000000000.00);
     rateSpin->setRange(0.00,100.00);
-    
+
     //fill alphabet combo
     QStringList listOfAlphabet;
-    //QString alphabet = trUtf8("abcdefghijklmnopqrstuvwxyz");
-    QString alphabet = trUtf8("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
+    //QString alphabet = "abcdefghijklmnopqrstuvwxyz";
+    QString alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
     for (int i = 0; i < alphabet.size(); i += 1)
     {
-    	QString letter = alphabet.mid(i,1);
-    	listOfAlphabet << letter;
+        QString letter = alphabet.mid(i,1);
+        listOfAlphabet << letter;
         }
     alphabetBox->addItems(listOfAlphabet);
-    
-    const QString insuranceInfo = trUtf8("Insurances are created in another preference widget.");
+
+    const QString insuranceInfo = tr("Insurances are created in another preference widget.");
     insuranceBox->setToolTip(insuranceInfo);
-    
+
     /*const QString filterMaxNumber = QString("%1 < '%2'").arg("MP_ID","10");
     modelMP->setFilter(filterMaxNumber);
     modelMP->select();*/
@@ -162,9 +164,9 @@ MedicalProcedureWidget::MedicalProcedureWidget(QWidget *parent) :
      //mpComboBox->setModel(modelMP);
     //mpComboBox->setMaxCount(20);
     //mpComboBox->setModelColumn(AccountDB::Constants::MP_NAME);
-    //setDatasToUi();
-     
-    
+    //setDataToUi();
+
+
 //    connect(createDefault, SIGNAL(clicked()), SLOT(createDefaultMedicalProcedures()));
 }
 
@@ -175,15 +177,15 @@ MedicalProcedureWidget::~MedicalProcedureWidget()
 }
 
 void MedicalProcedureWidget::fillHugeWidgets(){
-    
+
     fillTypeCompletionList();
     m_hashInsuranceBox = fillHashOfInsurances();
     QStringList listInsurances = m_hashInsuranceBox.values();
     insuranceBox->addItems(listInsurances);
-    fillMPCombo();       
+    fillMPCombo();
 }
 
-void MedicalProcedureWidget::setDatasToUi()
+void MedicalProcedureWidget::setDataToUi()
 {
     //m_Mapper->setCurrentIndex(900);
 }
@@ -205,7 +207,7 @@ void MedicalProcedureWidget::on_mpComboBox_currentIndexChanged(const QString & t
         LOG_QUERY_ERROR(q);
     }
     while (q.next()) {
-    	QString str = q.value(1).toString();
+        QString str = q.value(1).toString();
         if (str == text) {
             m_index = row;
             //ownersComboBox->setEditText(q.value(WN_MP_USER_UID).toString());
@@ -218,11 +220,11 @@ void MedicalProcedureWidget::on_mpComboBox_currentIndexChanged(const QString & t
             if (WarnDebugMessage)
                 qDebug() << __FILE__ << QString::number(__LINE__) << " index =" << QString::number(row) ;
         }
-    	++row;
+        ++row;
     }
-    
+
     //m_Mapper->setCurrentIndex(m_index);
-    
+
     if (WarnDebugMessage)
         qDebug() << __FILE__ << QString::number(__LINE__) << "index   =" << QString::number(m_index);
 }
@@ -268,7 +270,7 @@ void MedicalProcedureWidget::on_removeButton_clicked()
         fillMPCombo();
     }
 
-    
+
     /*if (!modelMP->removeRow(mpComboBox->currentIndex()))
     {
           LOG_ERROR("Unable to remove row");
@@ -285,7 +287,7 @@ void MedicalProcedureWidget::saveToSettings(Core::ISettings *sets)
     /*if (!modelMP->submit()) {
         LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_SAVE_DATA_IN_DATABASE_1).arg(tr("medical_procedures")));
         Utils::warningMessageBox(tr("Can not submit medical procedure to your personnal database."),
-                                 tr("An error occured during medical procedures saving. Datas are corrupted."));
+                                 tr("An error occured during medical procedures saving. Data are corrupted."));
     }
         QString typeText = type->text();
         setCompletionList(typeText);
@@ -364,10 +366,10 @@ void  MedicalProcedureWidget::fillMPCombo(){
         LOG_QUERY_ERROR(q);
     QStringList list;
     while (q.next()) {
-    	QString str = q.value(0).toString();
-    	if (WarnDebugMessage)
+        QString str = q.value(0).toString();
+        if (WarnDebugMessage)
             qDebug() << __FILE__ << QString::number(__LINE__) << " strItem =" << str ;
-    	list << str;
+        list << str;
     }
     mpComboBox->clear();
     mpComboBox->addItems(list);
@@ -381,15 +383,15 @@ void MedicalProcedureWidget::save(){
     if (!q.exec(req))
         LOG_QUERY_ERROR(q);
     while (q.next()) {
-    	QString str = q.value(0).toString();
+        QString str = q.value(0).toString();
         str = str.trimmed();
         if (str==nameStr) {
-    		  test = true;
+              test = true;
         }
     }
     if (test) {
-    	 QSqlQuery qInsert(m_db);
-         const QString medicalProcedure = tr("medical_procedure");
+         QSqlQuery qInsert(m_db);
+         const QString medicalProcedure = "medical_procedure";
          const QString abstractUpdate = QString("ABSTRACT = '%1'").arg(abstractEdit->text());
          const QString typeUpdate = QString("TYPE = '%1'").arg(type->text());
          const QString amountSpinUpdate = QString("AMOUNT = '%1'").arg(QString::number(amountSpin->value()));
@@ -420,7 +422,7 @@ void MedicalProcedureWidget::save(){
         int numberOfRows = modelMP->rowCount()  ;
         if (!modelMP->insertRows(numberOfRows,1,QModelIndex()))
         {
-        	  qWarning() << __FILE__ << QString::number(__LINE__) << "unable to insert row to MP" ;
+              qWarning() << __FILE__ << QString::number(__LINE__) << "unable to insert row to MP" ;
             }
         if (WarnDebugMessage)
             qDebug() << __FILE__ << QString::number(__LINE__) << " numberOfRows =" << QString::number(numberOfRows) ;
@@ -481,10 +483,10 @@ void MedicalProcedureWidget::fillTypeCompletionList(){
     if (!q.exec(req))
         LOG_QUERY_ERROR(q);
     while (q.next()) {
-    	QString str = q.value(0).toString();
+        QString str = q.value(0).toString();
         m_completionList << str;
     }
-    m_completionList.removeDuplicates(); 
+    m_completionList.removeDuplicates();
 }
 
 QHash<int,QString> MedicalProcedureWidget::fillHashOfInsurances(){
@@ -494,9 +496,9 @@ QHash<int,QString> MedicalProcedureWidget::fillHashOfInsurances(){
     if (!q.exec(req))
         LOG_QUERY_ERROR(q);
     while (q.next()) {
-    	int insuranceUid = q.value(0).toInt();
-    	QString name = q.value(1).toString();
-    	hash.insertMulti(insuranceUid,name);        
+        int insuranceUid = q.value(0).toInt();
+        QString name = q.value(1).toString();
+        hash.insertMulti(insuranceUid,name);
     }
     return hash;
 }
@@ -531,7 +533,7 @@ QHash<int,QString> MedicalProcedureWidget::fillHashOfInsurances(){
 //void MedicalProcedureWidget::createDefaultMedicalProcedures()
 //{
 //    int max = numberOfLinesForProgressBar();
-//    m_progressDialog = new QProgressDialog(trUtf8("Wait ..."),trUtf8("Abort"),0,max,this);
+//    m_progressDialog = new QProgressDialog(tr("Wait ..."), tr("Abort"),0,max,this);
 //    m_progressDialog->setWindowModality(Qt::WindowModal);
 //    m_progressDialog->setAutoReset(true);
 //    m_progressDialog->setAutoClose(true);
