@@ -1,7 +1,7 @@
 #ifndef FREETEXT_H
 #define FREETEXT_H
 
-#include "ui_FreeTextDialog.h"
+#include "ui_freetextdialog.h"
 #include <QDialog>
 #include <QString>
 
@@ -12,7 +12,6 @@ class FreeText:public QDialog, public Ui::FreeTextDialog
         FreeText(QWidget * parent = 0);
         ~FreeText();
         QString getFreeText();
-
 };
 
 #endif
