@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -44,22 +44,25 @@
 
 #include "agendabase.h"
 #include "constants.h"
-#include "appointement.h"
+#include "appointment.h"
 #include "calendaritemmodel.h"
 #include "nextavailabiliystepviewer.h"
 #include "usercalendar.h"
+#include "dayavailability.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/isettings.h>
 #include <coreplugin/constants_tokensandsettings.h>
-#include <printerplugin/textdocumentextra.h>
 #include <coreplugin/iuser.h>
 #include <coreplugin/icommandline.h>
 
 #include <patientbaseplugin/patientmodel.h>
 
+#include <usermanagerplugin/usercore.h>
 #include <usermanagerplugin/usermodel.h>
 
+#include <printerplugin/textdocumentextra.h>
+
 #include <utils/log.h>
 #include <utils/global.h>
 #include <utils/databaseconnector.h>
@@ -84,18 +87,19 @@ using namespace Internal;
 using namespace Agenda::Constants;
 using namespace Trans::ConstantTranslations;
 
+static inline Core::ICommandLine *commandLine()  { return Core::ICore::instance()->commandLine(); }
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
 static inline Core::IUser *user() {return Core::ICore::instance()->user();}
-static inline UserPlugin::UserModel *userModel() {return UserPlugin::UserModel::instance();}
-static inline Core::ICommandLine *commandLine()  { return Core::ICore::instance()->commandLine(); }
+static inline UserPlugin::UserCore &userCore() {return UserPlugin::UserCore::instance();}
+static inline UserPlugin::UserModel *userModel() {return userCore().userModel();}
 
-static inline bool connectDatabase(const QString &connectionName, const int line)
+static inline bool connectDatabase(QSqlDatabase &DB, const int line)
 {
-    QSqlDatabase db = QSqlDatabase::database(connectionName);
-    if (!db.isOpen()) {
-        if (!db.open()) {
+    if (!DB.isOpen()) {
+        if (!DB.open()) {
             Utils::Log::addError("AgendaBase", tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
-                      .arg(db.connectionName()).arg(db.lastError().text()), __FILE__, line);
+                                 .arg(DB.connectionName()).arg(DB.lastError().text()),
+                                 __FILE__, line);
             return false;
         }
     }
@@ -409,7 +413,7 @@ bool AgendaBase::createDatabase(const QString &connectionName, const QString &db
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         if (!DB.open())
             LOG(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(dbName).arg(DB.lastError().text()));
@@ -450,13 +454,18 @@ bool AgendaBase::createDatabase(const QString &connectionName, const QString &db
     }
 
     // Table INFORMATION
+    DB.transaction();
     QSqlQuery query(DB);
     query.prepare(prepareInsertQuery(Constants::Table_VERSION));
     query.bindValue(Constants::VERSION_ACTUAL, Constants::DB_VERSION);
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return false;
     }
     query.finish();
+    DB.commit();
 
     // database is readable/writable
     LOG(tkTr(Trans::Constants::DATABASE_1_CORRECTLY_CREATED).arg(pathOrHostName + QDir::separator() + dbName));
@@ -506,19 +515,29 @@ Agenda::UserCalendar *AgendaBase::createEmptyCalendar(const QString &userUid)
 /** Return true if the user \e userUid has recorded calendar(s) in the database. */
 bool AgendaBase::hasCalendar(const QString &userUuid)
 {
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return false;
+    }
+    DB.transaction();
     QString uid = userUuid;
     if (uid.isEmpty())
         uid = user()->uuid();
     Utils::Field get(Constants::Table_CALENDAR, Constants::CAL_ID);
     Utils::Join join(Constants::Table_CALENDAR, Constants::USERCAL_USER_UUID, Constants::Table_CALENDAR, Constants::CAL_ID);
     Utils::Field cond(Constants::Table_USERCALENDARS, Constants::USERCAL_USER_UUID, QString("='%1'").arg(uid));
-    QSqlQuery query(database());
+    QSqlQuery query(DB);
     if (query.exec(select(get, join, cond))) {
-        if (query.next())
+        if (query.next()) {
+            query.finish();
+            DB.commit();
             return true;
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
     }
+    query.finish();
+    DB.commit();
     return false;
 }
 
@@ -526,8 +545,10 @@ bool AgendaBase::hasCalendar(const QString &userUuid)
 QList<Agenda::UserCalendar *> AgendaBase::getUserCalendars(const QString &userUuid)
 {
     QList<Agenda::UserCalendar *> toReturn;
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return toReturn;
+    }
 
     QString uid = userUuid;
     if (userUuid.isEmpty())
@@ -554,12 +575,13 @@ QList<Agenda::UserCalendar *> AgendaBase::getUserCalendars(const QString &userUu
     get << Utils::Field(Constants::Table_USERCALENDARS, Constants::USERCAL_USER_UUID);
     reqs << select(get, joins, conds);
 
-    QSqlQuery query(database());
+    DB.transaction();
+    QSqlQuery query(DB);
     for(int i = 0; i < reqs.count(); ++i) {
         if (query.exec(reqs.at(i))) {
             while (query.next()) {
                 Agenda::UserCalendar *u = new Agenda::UserCalendar;
-                // set private datas
+                // set private data
                 //            u->setData(IUserCalendar::DbOnly_UserCalId, query.value());
                 u->setData(Constants::Db_CalId, query.value(Constants::CAL_ID));
                 u->setData(Agenda::UserCalendar::Uid, query.value(Constants::CAL_UID));
@@ -593,6 +615,9 @@ QList<Agenda::UserCalendar *> AgendaBase::getUserCalendars(const QString &userUu
             }
         } else {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return toReturn;
         }
         query.finish();
     }
@@ -650,11 +675,15 @@ QList<Agenda::UserCalendar *> AgendaBase::getUserCalendars(const QString &userUu
             tr.clear();
         } else {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return toReturn;
         }
     }
     query.finish();
+    DB.commit();
 
-    // Get Peoples
+    // Get People
     for(int i = 0; i < toReturn.count(); ++i) {
         Agenda::UserCalendar *u = toReturn[i];
         getRelatedPeoples(RelatedToCalendar, u->data(Constants::Db_CalId).toInt(), u);
@@ -666,9 +695,6 @@ QList<Agenda::UserCalendar *> AgendaBase::getUserCalendars(const QString &userUu
 /** Save the user's calendar availabilities for the specified \e calendar to database. The \e calendar is modified during this process (ids are set if needed). */
 bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
 {
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
-        return false;
-
     if (calendar->data(Constants::Db_CalId).isNull() ||
         !calendar->data(Constants::Db_CalId).isValid() ||
         calendar->data(Constants::Db_CalId).toInt()==-1) {
@@ -676,10 +702,15 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         return false;
     }
 
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return false;
+    }
+
     // delete all recorded availabilities of this calendar
     // get all availIds for the calId
-    database().transaction();
-    QSqlQuery query(database());
+    DB.transaction();
+    QSqlQuery query(DB);
     QHash<int, QString> where;
     QStringList availIds;
     // select avail_id from avail where cal_id=6
@@ -690,6 +721,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         database().rollback();
         return false;
     }
@@ -707,6 +739,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         } else {
             LOG_QUERY_ERROR(query);
             database().rollback();
+            query.finish();
             return false;
         }
         query.finish();
@@ -716,6 +749,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         where.insert(Constants::AVAIL_ID, QString("IN (%1)").arg(availIds.join(",")));
         if (!query.exec(this->prepareDeleteQuery(Constants::Table_AVAILABILITIES, where))) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             database().rollback();
             return false;
         }
@@ -724,6 +758,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         where.insert(Constants::AVTOTR_AVID, QString("IN (%1)").arg(availIds.join(",")));
         if (!query.exec(this->prepareDeleteQuery(Constants::Table_AVAIL_TO_TIMERANGE, where))) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             database().rollback();
             return false;
         }
@@ -732,6 +767,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         where.insert(Constants::TIMERANGE_ID, QString("IN (%1)").arg(trIds.join(",")));
         if (!query.exec(this->prepareDeleteQuery(Constants::Table_TIMERANGE, where))) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             database().rollback();
             return false;
         }
@@ -740,21 +776,22 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
 
     // No availabilities -> finished
     if (!calendar->hasAvailability()) {
+        DB.commit();
         return true;
     }
 
     // recreate availabilities
     QVector<Agenda::DayAvailability> av = calendar->availabilities();
     QHash<int, Agenda::DayAvailability> hashAv;
-    // fusion availabilties by days
-    for(int i = 0; i < av.count(); ++i) {
-        int weekDay = av.at(i).weekDay();
+    // fusion availabilities by days
+    for(int availabilityIndex = 0; availabilityIndex < av.count(); ++availabilityIndex) {
+        int weekDay = av.at(availabilityIndex).weekDay();
         if (hashAv.contains(weekDay)) {
-            for(int z = 0; z < av.at(i).timeRangeCount(); ++z) {
-                hashAv[weekDay].addTimeRange(av.at(i).timeRange(z));
+            for(int timeRangeIndex = 0; timeRangeIndex < av.at(availabilityIndex).timeRangeCount(); ++timeRangeIndex) {
+                hashAv[weekDay].addTimeRange(av.at(availabilityIndex).timeRangeAt(timeRangeIndex));
             }
         } else {
-            hashAv.insert(av.at(i).weekDay(), av.at(i));
+            hashAv.insert(av.at(availabilityIndex).weekDay(), av.at(availabilityIndex));
         }
     }
 
@@ -767,6 +804,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         query.bindValue(Constants::AVAIL_WEEKDAY, it.key());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             database().rollback();
             return false;
         }
@@ -776,13 +814,14 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         // save all timeRanges
         QVector<Agenda::TimeRange> newRanges;
         for(int z = 0; z < it.value().timeRangeCount(); ++z) {
-            Agenda::TimeRange range = it.value().timeRange(z);
+            Agenda::TimeRange range = it.value().timeRangeAt(z);
             query.prepare(prepareInsertQuery(Constants::Table_TIMERANGE));
             query.bindValue(Constants::TIMERANGE_ID, QVariant());
             query.bindValue(Constants::TIMERANGE_FROM, range.from.toString());
             query.bindValue(Constants::TIMERANGE_TO, range.to.toString());
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
+                query.finish();
                 database().rollback();
                 return false;
             }
@@ -796,6 +835,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
             query.bindValue(Constants::AVTOTR_TRID, trId);
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
+                query.finish();
                 database().rollback();
                 return false;
             }
@@ -805,7 +845,7 @@ bool AgendaBase::saveCalendarAvailabilities(Agenda::UserCalendar *calendar)
         hashAv[it.key()].setId(avId);
     }
     calendar->setAvailabilities(hashAv.values());
-    database().commit();
+    DB.commit();
     LOG("User agenda correctly saved");
     return true;
 }
@@ -815,13 +855,16 @@ bool AgendaBase::saveUserCalendar(Agenda::UserCalendar *calendar)
 {
     if (!calendar->isModified())
         return true;
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return false;
+    }
+    DB.transaction();
     if (calendar->data(Constants::Db_CalId).isNull() ||
         !calendar->data(Constants::Db_CalId).isValid() ||
         calendar->data(Constants::Db_CalId).toInt()==-1) {
         // save
-        QSqlQuery query(database());
+        QSqlQuery query(DB);
         query.prepare(prepareInsertQuery(Constants::Table_CALENDAR));
         query.bindValue(Constants::CAL_ID, QVariant());
         query.bindValue(Constants::CAL_UID, calendar->data(Agenda::UserCalendar::Uid).toString());
@@ -842,6 +885,8 @@ bool AgendaBase::saveUserCalendar(Agenda::UserCalendar *calendar)
         query.bindValue(Constants::CAL_XMLOPTIONS, calendar->xmlOptions());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         calendar->setData(Constants::Db_CalId, query.lastInsertId());
@@ -858,20 +903,26 @@ bool AgendaBase::saveUserCalendar(Agenda::UserCalendar *calendar)
         query.bindValue(Constants::USERCAL_USER_UUID, calendar->data(Agenda::UserCalendar::UserOwnerUid));
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         calendar->setData(Constants::Db_UserCalId, query.lastInsertId());
         calendar->setModified(false);
         query.finish();
 
-        if (!saveCalendarAvailabilities(calendar))
+        if (!saveCalendarAvailabilities(calendar)) {
+            DB.rollback();
             return false;
+        }
 
-        if (!saveRelatedPeoples(RelatedToCalendar, calendar->data(Constants::Db_CalId).toInt(), calendar))
+        if (!saveRelatedPeoples(RelatedToCalendar, calendar->data(Constants::Db_CalId).toInt(), calendar)) {
+            DB.rollback();
             return false;
+        }
     } else {
         // update
-        QSqlQuery query(database());
+        QSqlQuery query(DB);
         QHash<int, QString> where;
         where.insert(Constants::CAL_ID, "=" + calendar->data(Constants::Db_CalId).toString());
         query.prepare(prepareUpdateQuery(Constants::Table_CALENDAR,
@@ -909,29 +960,36 @@ bool AgendaBase::saveUserCalendar(Agenda::UserCalendar *calendar)
         query.bindValue(14, calendar->data(Constants::Db_IsVirtual).toInt());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         calendar->setModified(false);
         query.finish();
 
-        if (!saveCalendarAvailabilities(calendar))
+        if (!saveCalendarAvailabilities(calendar)) {
+            DB.rollback();
             return false;
+        }
 
-        if (!saveRelatedPeoples(RelatedToCalendar, calendar->data(Constants::Db_CalId).toInt(), calendar))
+        if (!saveRelatedPeoples(RelatedToCalendar, calendar->data(Constants::Db_CalId).toInt(), calendar)) {
+            DB.rollback();
             return false;
+        }
     }
+    DB.commit();
     return true;
 }
 
 /** Retrieve all events from database according to the Agenda::CalendarEventQuery \e calQuery. */
-QList<Appointement *> AgendaBase::getCalendarEvents(const CalendarEventQuery &calQuery)
+QList<Appointment *> AgendaBase::getCalendarEvents(const CalendarEventQuery &calQuery)
 {
-    QList<Appointement *> toReturn;
-    if (!connectDatabase(Constants::DB_NAME, __LINE__)) {
+    QList<Appointment *> toReturn;
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return toReturn;
     }
     QString req;
-    QSqlQuery query(database());
     // get events according to the query
     Utils::JoinList joins;
 //    joins << Utils::Join(Constants::Table_COMMON, Constants::COMMON_ID, Constants::Table_EVENTS, Constants::EVENT_COMMON_ID);
@@ -939,13 +997,13 @@ QList<Appointement *> AgendaBase::getCalendarEvents(const CalendarEventQuery &ca
     conds << Utils::Field(Constants::Table_CALENDAR, Constants::CAL_ISVALID, "=1");
     conds << Utils::Field(Constants::Table_EVENTS, Constants::EVENT_ISVALID, "=1");
     QString order;
-    // get a specific appointement ?
-    if (!calQuery.appointementId().isNull()) {
-        if (calQuery.appointementId().toInt() == -1)
+    // get a specific appointment ?
+    if (!calQuery.appointmentId().isNull()) {
+        if (calQuery.appointmentId().toInt() == -1)
             return toReturn;
-        conds << Utils::Field(Constants::Table_EVENTS, Constants::EVENT_ID, QString("='%1'").arg(calQuery.appointementId().toString()));
+        conds << Utils::Field(Constants::Table_EVENTS, Constants::EVENT_ID, QString("='%1'").arg(calQuery.appointmentId().toString()));
     } else {
-        // TODO: add virtual appointement filtering
+        // TODO: add virtual appointment filtering
         // Date conditions
         if (calQuery.hasDateRange()) {
             // TODO: code here: better management of dates in filters
@@ -993,9 +1051,11 @@ QList<Appointement *> AgendaBase::getCalendarEvents(const CalendarEventQuery &ca
 
 //    qWarning() << req;
 
+    DB.transaction();
+    QSqlQuery query(DB);
     if (query.exec(req)) {
         while (query.next()) {
-            Appointement *ev = new Appointement;
+            Appointment *ev = new Appointment;
             ev->setData(Constants::Db_EvId, query.value(Constants::EVENT_ID));
             ev->setData(Constants::Db_CalId, query.value(Constants::EVENT_CAL_ID));
             ev->setData(Constants::Db_ComId, query.value(Constants::EVENT_COMMON_ID));
@@ -1007,15 +1067,20 @@ QList<Appointement *> AgendaBase::getCalendarEvents(const CalendarEventQuery &ca
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return toReturn;
     }
     query.finish();
-    if (toReturn.isEmpty())
+    if (toReturn.isEmpty()) {
+        DB.commit();
         return toReturn;
+    }
 
     // Get common data
     QHash<int, QString> where;
     for(int i = 0; i < toReturn.count(); ++i) {
-        Appointement *ev = toReturn.at(i);
+        Appointment *ev = toReturn.at(i);
         where.insert(Constants::COMMON_ID, "=" + ev->data(Constants::Db_EvId).toString());
         req = select(Constants::Table_COMMON, where);
         if (query.exec(req)) {
@@ -1038,22 +1103,30 @@ QList<Appointement *> AgendaBase::getCalendarEvents(const CalendarEventQuery &ca
             }
         } else {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return toReturn;
         }
         query.finish();
 
         // Get peoples
-        getRelatedPeoples(RelatedToAppointement, ev->eventId(), ev);
+        getRelatedPeoples(RelatedToAppointment, ev->eventId(), ev);
     }
 
 //    getPatientNames(toReturn);
-
+    DB.commit();
     return toReturn;
 }
 
 /** Save or update the common part of events and set the commonId() in the event */
-bool AgendaBase::saveCommonEvent(Appointement *event)
+bool AgendaBase::saveCommonEvent(Appointment *event)
 {
-    QSqlQuery query(database());
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return false;
+    }
+    DB.transaction();
+    QSqlQuery query(DB);
     if (event->eventId() == -1) {
         // save
         query.prepare(prepareInsertQuery(Constants::Table_COMMON));
@@ -1075,6 +1148,8 @@ bool AgendaBase::saveCommonEvent(Appointement *event)
         query.bindValue(Constants::COMMON_XMLCALOPTIONS, event->data(Constants::Db_XmlOptions));
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         int commonId = query.lastInsertId().toInt();
@@ -1119,10 +1194,13 @@ bool AgendaBase::saveCommonEvent(Appointement *event)
         query.bindValue(14, event->data(Constants::Db_IsVirtual).toInt());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
     }
     query.finish();
+    DB.commit();
     return true;
 }
 
@@ -1133,12 +1211,14 @@ bool AgendaBase::saveRelatedPeoples(RelatedEventFor relatedTo, const int eventOr
         LOG_ERROR("No Event/Calendar ID");
         return false;
     }
-    if (!connectDatabase(Constants::DB_NAME, __LINE__)) {
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return false;
     }
 
     // Delete old entries
-    QSqlQuery query(database());
+    DB.transaction();
+    QSqlQuery query(DB);
     QHash<int, QString> where;
     if (relatedTo == RelatedToCalendar) {
         where.insert(PEOPLE_CAL_ID, QString("=%1").arg(eventOrCalendarId));
@@ -1147,6 +1227,8 @@ bool AgendaBase::saveRelatedPeoples(RelatedEventFor relatedTo, const int eventOr
     }
     if (!query.exec(prepareDeleteQuery(Table_PEOPLE, where))) {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
         return false;
     }
     query.finish();
@@ -1169,11 +1251,15 @@ bool AgendaBase::saveRelatedPeoples(RelatedEventFor relatedTo, const int eventOr
             query.bindValue(PEOPLE_TYPE, i);
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
+                query.finish();
+                DB.rollback();
                 return false;
             }
             query.finish();
         }
     }
+    query.finish();
+    DB.commit();
     return true;
 }
 
@@ -1184,11 +1270,12 @@ bool AgendaBase::getRelatedPeoples(RelatedEventFor relatedTo, const int eventOrC
         LOG_ERROR("No Event/Calendar ID");
         return false;
     }
-    if (!connectDatabase(Constants::DB_NAME, __LINE__)) {
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return false;
     }
-
-    QSqlQuery query(database());
+    DB.transaction();
+    QSqlQuery query(DB);
     QHash<int, QString> where;
     if (relatedTo == RelatedToCalendar) {
         where.insert(Constants::PEOPLE_CAL_ID, QString("=%1").arg(eventOrCalendarId));
@@ -1203,6 +1290,8 @@ bool AgendaBase::getRelatedPeoples(RelatedEventFor relatedTo, const int eventOrC
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
         return false;
     }
     query.finish();
@@ -1218,12 +1307,12 @@ bool AgendaBase::getRelatedPeoples(RelatedEventFor relatedTo, const int eventOrC
             }
         }
     }
-
+    DB.commit();
     return true;
 }
 
-/** Retrieve the patients name for the specified list of Appointements \e items */
-bool AgendaBase::getPatientNames(const QList<Appointement *> &items)
+/** Retrieve the patients name for the specified list of Appointments \e items */
+bool AgendaBase::getPatientNames(const QList<Appointment *> &items)
 {
     // get all patient uids
     QStringList uids;
@@ -1247,10 +1336,10 @@ bool AgendaBase::getPatientNames(const QList<Appointement *> &items)
     return true;
 }
 
-/** Retrieve the patients name for the specified Appointement \e item */
-bool AgendaBase::getPatientNames(Appointement *item)
+/** Retrieve the patients name for the specified Appointment \e item */
+bool AgendaBase::getPatientNames(Appointment *item)
 {
-    return getPatientNames(QList<Appointement *>() << item);
+    return getPatientNames(QList<Appointment *>() << item);
 }
 
 /** Create a virtual UserCalendar and return it. Pointer must be deleted. */
@@ -1260,13 +1349,14 @@ Agenda::UserCalendar *AgendaBase::createVirtualUserCalendar(const QString &owner
                                            const QString &password, const QString &iconPath,
                                            const QList<Calendar::People> &peoples)
 {
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     QHash<int, QString> where;
     where.insert(CAL_LABEL, QString("='%1'").arg(label));
     where.insert(CAL_FULLCONTENT, QString("='%1'").arg(description));
     where.insert(CAL_DEFAULTDURATION, QString("=%1").arg(defaultDurationInMinutes));
     int alreadyInBase = count(Table_CALENDAR, CAL_ID, getWhereClause(Table_CALENDAR, where));
     if (alreadyInBase!=0) {
-        LOG_ERROR("Calendar is alreday in base, virtual user calendar not created");
+        LOG("Calendar is already in base, virtual user calendar not created");
         return 0;
     }
     UserCalendar *u = new UserCalendar;
@@ -1292,15 +1382,17 @@ Agenda::UserCalendar *AgendaBase::createVirtualUserCalendar(const QString &owner
         av.addTimeRange(QTime(7,0,0), QTime(20,0,0));
         u->addAvailabilities(av);
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!saveUserCalendar(u)) {
         delete u;
         u = 0;
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     return u;
 }
 
 /** Save or update a Calendar::CalendarItem to the agenda database */
-bool AgendaBase::saveNonCyclingEvent(Appointement *event)
+bool AgendaBase::saveNonCyclingEvent(Appointment *event)
 {
 //    WARN_FUNC << event->eventId();
     if (!event->isModified())
@@ -1312,7 +1404,12 @@ bool AgendaBase::saveNonCyclingEvent(Appointement *event)
     if (!saveCommonEvent(event))
         return false;
 
-    QSqlQuery query(database());
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return false;
+    }
+    DB.transaction();
+    QSqlQuery query(DB);
     if (event->eventId() == -1) {
         // save
         query.prepare(prepareInsertQuery(Constants::Table_EVENTS));
@@ -1324,6 +1421,8 @@ bool AgendaBase::saveNonCyclingEvent(Appointement *event)
         query.bindValue(Constants::EVENT_DATEEND, event->ending());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         event->setData(Constants::Db_EvId, query.lastInsertId());
@@ -1331,8 +1430,10 @@ bool AgendaBase::saveNonCyclingEvent(Appointement *event)
 
         event->setModified(false);
 
-        if (!saveRelatedPeoples(RelatedToAppointement, event->eventId(), event))
+        if (!saveRelatedPeoples(RelatedToAppointment, event->eventId(), event)) {
+            DB.rollback();
             return false;
+        }
 
     } else {
         // update
@@ -1351,6 +1452,8 @@ bool AgendaBase::saveNonCyclingEvent(Appointement *event)
         query.bindValue(3, event->ending());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         query.finish();
@@ -1361,26 +1464,28 @@ bool AgendaBase::saveNonCyclingEvent(Appointement *event)
         query.exec(prepareDeleteQuery(Table_PEOPLE, where));
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
             return false;
         }
         query.finish();
 
-        if (!saveRelatedPeoples(RelatedToAppointement, event->eventId(), event))
+        if (!saveRelatedPeoples(RelatedToAppointment, event->eventId(), event)) {
+            DB.rollback();
             return false;
+        }
     }
+    DB.commit();
     return true;
 }
 
 /** Save or update events in the database, when saved events are set to non modified. Return false in case of an error. */
-bool AgendaBase::saveCalendarEvents(const QList<Appointement *> &events)
+bool AgendaBase::saveCalendarEvents(const QList<Appointment *> &events)
 {
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
-        return false;
-    // update or save ?
-    QList<Appointement *> save, update;
     bool ok = true;
     for(int i = 0; i < events.count(); ++i) {
-        Appointement *ev = events.at(i);
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+        Appointment *ev = events.at(i);
         if (!saveNonCyclingEvent(ev))
             ok = false;
     }
@@ -1388,18 +1493,18 @@ bool AgendaBase::saveCalendarEvents(const QList<Appointement *> &events)
 }
 
 /** Save or update one event in the database, when saved events are set to non modified. Return false in case of an error. */
-bool AgendaBase::saveCalendarEvent(Appointement *event)
+bool AgendaBase::saveCalendarEvent(Appointment *event)
 {
-    return saveCalendarEvents(QList<Appointement *>() << event);
+    return saveCalendarEvents(QList<Appointment *>() << event);
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
-/** Return the next available appointements by searching for
+/** Return the next available appointments by searching for
   - \e startSearch date time,
   - a duration of \e duration minutes
   - for the agenda \e calendar
-  - limiting search to \e numberOfDates appointements
+  - limiting search to \e numberOfDates appointments
 */
 QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, const int durationInMinutes, const Agenda::UserCalendar &calendar, const int numberOfDates)
 {
@@ -1420,8 +1525,10 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
     if (calendar.data(UserCalendar::DefaultDuration).toInt() == 0)
         return toReturn;
 
-    if (!connectDatabase(Constants::DB_NAME, __LINE__))
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
         return toReturn;
+    }
 
     // Here we can go on
     QDateTime start, currentStart, currentEnd;
@@ -1432,7 +1539,7 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
     get << Utils::Field(Table_EVENTS, EVENT_DATEEND);
     Utils::JoinList joins;
     // no joins
-    // TODO: code here : condition with the type/status of appointements
+    // TODO: code here : condition with the type/status of appointments
     Utils::FieldList conds;
     conds << Utils::Field(Table_EVENTS, EVENT_ISVALID, "=1");
     conds << Utils::Field(Table_EVENTS, EVENT_CAL_ID, QString("=%1").arg(calendar.data(Constants::Db_CalId).toString()));
@@ -1444,18 +1551,20 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
     QString limit = QString("\n LIMIT 0,1");
 
     // get the first event and make tests
-    QSqlDatabase DB = database();
     DB.transaction();
     QSqlQuery query(DB);
-    QRect nextAppointement;
+    QRect nextAppointment;
     if (query.exec(req+order+limit)) {
         if (query.next()) {
             currentStart = query.value(0).toDateTime();
             currentEnd = query.value(1).toDateTime();
-            nextAppointement = NextAvailabiliyManager::dateToRect(currentStart, currentEnd);
+            nextAppointment = NextAvailabiliyManager::dateToRect(currentStart, currentEnd);
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return toReturn;
     }
 
     // Transform availabilities to QRect
@@ -1463,7 +1572,7 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
     for(int i=0; i < calendar.availabilities().count(); ++i) {
         DayAvailability av = calendar.availabilities().at(i);
         for(int j = 0; j < av.timeRangeCount(); ++j) {
-            const TimeRange &tr = av.timeRange(j);
+            const TimeRange &tr = av.timeRangeAt(j);
             avs.append(NextAvailabiliyManager::simplifiedDateToRect(av.weekDay(), tr.from, tr.to));
         }
     }
@@ -1476,23 +1585,24 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
         --limitComputation;
         if (limitComputation <= 0)
             break;
-        QList<QDateTime> dates = m_Next->nextAvailableTime(start, durationInMinutes, defaultDuration, nextAppointement, numberOfDates-toReturn.count());
+        QList<QDateTime> dates = m_Next->nextAvailableTime(start, durationInMinutes, defaultDuration, nextAppointment, numberOfDates-toReturn.count());
         toReturn.append(dates);
 
-        if (m_Next->hasReachedNextAppointement()) {
-            start = m_Next->requestingNewAppointementLaterThan();
-            // Get next appointement
-            nextAppointement = QRect();
+        if (m_Next->hasReachedNextAppointment()) {
+            start = m_Next->requestingNewAppointmentLaterThan();
+            // Get next appointment
+            nextAppointment = QRect();
             conds.removeLast();
             conds << Utils::Field(Table_EVENTS, EVENT_DATESTART, QString(">= '%1'").arg(start.toString(Qt::ISODate)));
             if (query.exec(select(get, joins, conds)+order+limit)) {
                 if (query.next()) {
                     currentStart = query.value(0).toDateTime();
                     currentEnd = query.value(1).toDateTime();
-                    nextAppointement = NextAvailabiliyManager::dateToRect(currentStart, currentEnd);
+                    nextAppointment = NextAvailabiliyManager::dateToRect(currentStart, currentEnd);
                 }
             } else {
                 LOG_QUERY_ERROR(query);
+                query.finish();
                 DB.rollback();
                 return toReturn;
             }
@@ -1501,7 +1611,7 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
 
 
 
-//    QRect nextAppointement;
+//    QRect nextAppointemnt;
 
 //    int nbFound = 0;
 //    int durationInSeconds = durationInMinutes * 60;
@@ -1524,17 +1634,17 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
 
 //        QRect testDate = dateToRect(start, durationInMinutes);
 
-//        // rect does not intersect the next recorded appointement ? -> go next appointement
-//        while (testDate.intersect(nextAppointement).height() < 1) {
+//        // rect does not intersect the next recorded appointment ? -> go next appointment
+//        while (testDate.intersect(nextAppointment).height() < 1) {
 
 //            if (WarnNextAvailableTimeWarnings)
-//                qWarning() << "\ntest" << testDate << "next" << nextAppointement << testDate.intersect(nextAppointement).height();
+//                qWarning() << "\ntest" << testDate << "next" << nextAppointment << testDate.intersect(nextAppointment).height();
 
 //            if (nbFound == numberOfDates) {
 //                DB.commit();
 //                return toReturn;
 //            }
-//            if (nextAppointement.top() <= testDate.top())
+//            if (nextAppointment.top() <= testDate.top())
 //                break;
 
 //            // rect in availabilities ?
@@ -1563,11 +1673,11 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
 //            }
 //        }
 
-//        // go at the end of nextAppointement and retry
+//        // go at the end of nextAppointment and retry
 
 //        // go next week ?
 
-////        // add before the currentDate (enough time or no appointement) ?
+////        // add before the currentDate (enough time or no appointment) ?
 ////        if (start.secsTo(currentStart) >= durationInSeconds || currentStart.isNull()) {
 ////            // does it feet the userCalendar availability ?
 ////            if (calendar.canBeAvailable(start, durationInMinutes)) {
@@ -1580,7 +1690,7 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
 ////            }
 ////        }
 
-//        // get next appointement
+//        // get next appointment
 //        //    remove the date from the SQL conditions
 //        start = currentEnd;
 //        conds.removeLast();
@@ -1589,9 +1699,9 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
 //            if (query.next()) {
 //                currentStart = query.value(0).toDateTime();
 //                currentEnd = query.value(1).toDateTime();
-//                nextAppointement = dateToRect(currentStart, currentEnd);
+//                nextAppointment = dateToRect(currentStart, currentEnd);
 //            } else {
-//                // no next appointement -> continue with usercalendar only
+//                // no next appointment -> continue with usercalendar only
 //                if (WarnNextAvailableTimeWarnings)
 //                    qWarning() <<  "no next";
 //                start = Utils::roundDateTime(start, calendar.data(UserCalendar::DefaultDuration).toInt());
@@ -1614,7 +1724,7 @@ QList<QDateTime> AgendaBase::nextAvailableTime(const QDateTime &startSearch, con
     return toReturn;
 }
 
-/** Return the next available appointement by searching for
+/** Return the next available appointment by searching for
   - \e startSearch date time,
   - a duration of \e duration minutes
   - for the agenda \e calendar
