@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "agendaplugin.h"
@@ -31,7 +31,7 @@
 #include "agendacore.h"
 
 // TEST
-#include "appointement.h"
+#include "appointment.h"
 #include "calendaritemmodel.h"
 #include "eventeditorwidget.h"
 #include <utils/randomizer.h>
@@ -48,6 +48,7 @@
 // END TEST
 
 #include <utils/log.h>
+#include <agendaplugin/dayavailability.h>
 #include <calendar/calendar_theme.h>
 
 #include <coreplugin/icore.h>
@@ -85,7 +86,8 @@ AgendaPlugin::AgendaPlugin() :
     if (Utils::Log::warnPluginsCreation())
         qWarning() << "creating AgendaPlugin";
     // Add Translator to the Application
-    Core::ICore::instance()->translators()->addNewTranslator("agendaplugin");
+    Core::ICore::instance()->translators()->addNewTranslator("plugin_agenda");
+    Core::ICore::instance()->translators()->addNewTranslator("lib_calendar");
 
     // Create the core object
     m_Core = new AgendaCore(this);
@@ -93,9 +95,8 @@ AgendaPlugin::AgendaPlugin() :
 
 AgendaPlugin::~AgendaPlugin()
 {
-    if (m_Core)
-        delete m_Core;
-    m_Core = 0;
+    if (Utils::Log::warnPluginsCreation())
+        WARN_FUNC;
 }
 
 bool AgendaPlugin::initialize(const QStringList &arguments, QString *errorString)
@@ -146,26 +147,29 @@ void AgendaPlugin::extensionsInitialized()
     th->setIconFileName(Calendar::CalendarTheme::NavigationMonthViewMode, Constants::I_VIEWMODE);
     th->setIconFileName(Calendar::CalendarTheme::NavigationNext, Core::Constants::ICONNEXT);
     th->setIconFileName(Calendar::CalendarTheme::NavigationPrevious, Core::Constants::ICONPREVIOUS);
+    th->setIconFileName(Calendar::CalendarTheme::NavigationForceModelRefreshing, Core::Constants::ICONSOFTWAREUPDATEAVAILABLE);
 
     if (commandLine()->value(Core::ICommandLine::CreateVirtuals).toBool()) {
         QList<Calendar::People> peoples;
         UserCalendar *u = 0;
         // McCoy calendar (Uhura as delegate)
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         peoples.append(Calendar::People(Calendar::People::PeopleUserDelegate, "", "0f148ea3de6e47b8bbf9c2cedea47511"));
         u = base().createVirtualUserCalendar("d1f29ad4a4ea4dabbe40ec888d153228", "McCoy @Enterprise",
                                           "Virtual calendar for the virtual user McCoy",
                                           15, 0, 0, 0, true, false, "", "mccoy.png", peoples);
         if (!u)
             return;
-        createVirtualAppointements(u);
+        createVirtualAppointments(u);
 
         // Phlox calendar (no delegates)
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         peoples.clear();
         u = base().createVirtualUserCalendar("b5caead635a246a2a87ce676e9d2ef4d", "Phlox @Enterprise",
                                           "Virtual calendar for the virtual user Phlox",
                                           10, 0, 0, 0, true, false, "", "phlox.png", peoples);
         if (u)
-            createVirtualAppointements(u);
+            createVirtualAppointments(u);
     }
 }
 
@@ -206,9 +210,13 @@ static QString patientUid(const int row)
 //    return QString();
 //}
 
-void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
+void AgendaPlugin::createVirtualAppointments(UserCalendar *calendar)
 {
-    QList<Appointement *> list;
+//    QTime chr;
+//    chr.start();
+
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    QList<Appointment *> list;
     Utils::Randomizer r;
     r.setPathToFiles(settings()->path(Core::ISettings::BundleResourcesPath) + "/textfiles/");
     QDir pix(settings()->path(Core::ISettings::SmallPixmapPath));
@@ -219,9 +227,11 @@ void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
     int maxDb = numberOfPatients();
     int defaultDuration = calendar->data(UserCalendar::DefaultDuration).toInt();
     int calendarId = calendar->data(Constants::Db_CalId).toInt();
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
 
     for(int i = 0; i < nbEvents; ++i) {
-        Appointement *ev = 0;
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+        Appointment *ev = 0;
 
         // add a duration to last start date
         start.setTime(start.addSecs(60*defaultDuration*r.randomInt(0, 5)).time());
@@ -232,7 +242,7 @@ void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
         }
         QDateTime end = start.addSecs(60*defaultDuration);
 
-        ev = new Appointement;
+        ev = new Appointment;
         ev->setData(Constants::Db_CalId, calendarId);
         ev->setData(Constants::Db_IsValid, 1);
         ev->setData(Constants::Db_IsVirtual, 1);
@@ -242,7 +252,7 @@ void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
         ev->setData(Constants::Db_XmlOptions, "XmlOptions");
         //    ev->setData(Constants::DbOnly_ComId, );
         ev->setData(Constants::Db_CatId, -1);
-        //            ev->setData(Calendar::CalendarItem::PatientUid, r.getRandomString(45));
+        //            ev->setData(Calendar::CalendarItem::PatientUid, r.randomString(45));
         ev->setData(CalendarItemModel::DateStart, start);
         ev->setData(CalendarItemModel::DateEnd, end);
         ev->setData(CalendarItemModel::Type, 1);
@@ -254,9 +264,10 @@ void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
         ev->setData(CalendarItemModel::IsAGroupEvent, r.randomInt(0,1));
         ev->setData(CalendarItemModel::Label, r.randomWords(r.randomInt(2, 15)));
         ev->setData(CalendarItemModel::Description, r.randomWords(r.randomInt(10, 500)));
-        ev->setData(CalendarItemModel::Location, r.getRandomString(r.randomInt(1,145)));
+        ev->setData(CalendarItemModel::Location, r.randomString(r.randomInt(1,145)));
         ev->setData(CalendarItemModel::IconPath, r.randomFile(pix, QStringList() << "*.png").fileName());
 
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         // Add 1 to 3 patients
         for(int y = 0; y < r.randomInt(1, 3); ++y) {
             int zz = r.randomInt(0, maxDb);
@@ -267,6 +278,7 @@ void AgendaPlugin::createVirtualAppointements(UserCalendar *calendar)
     if (!base().saveCalendarEvents(list))
         ok = false;
 
+//    Utils::Log::logTimeElapsed(chr, "---", "creating virtual events");
     if (ok)
         qWarning() << nbEvents << "events successfully created and saved";
 }
@@ -288,7 +300,7 @@ void AgendaPlugin::testDatabase()
             ucal = new Agenda::UserCalendar();
             // Create a calendar for the current user
             ucal->setData(Constants::Db_IsValid, 1);
-            ucal->setData(Agenda::UserCalendar::Password, r.getRandomString(r.randomInt(0,10)));
+            ucal->setData(Agenda::UserCalendar::Password, r.randomString(r.randomInt(0,10)));
             ucal->setData(Agenda::UserCalendar::Label, r.randomWords(r.randomInt(2,5)));
             ucal->setData(Agenda::UserCalendar::IsPrivate, r.randomInt(0,1));
             ucal->setData(Agenda::UserCalendar::IsDefault, 0);
@@ -296,7 +308,7 @@ void AgendaPlugin::testDatabase()
             ucal->setData(Agenda::UserCalendar::Description, r.randomWords(r.randomInt(5,50)));
             ucal->setData(Agenda::UserCalendar::AbsPathIcon, r.randomFile(pix, QStringList() << "*.png").fileName());
             // Create availabilities
-            for(int i = 1; i < 6; ++i) {
+            for(int i = Qt::Monday; i <= Qt::Sunday; ++i) {
                 QTime from = r.randomTime(6, 12);
                 QTime to = r.randomTime(17, 20);
                 Agenda::DayAvailability av;
@@ -333,12 +345,12 @@ void AgendaPlugin::testDatabase()
     CalendarEventQuery q;
     q.setDateRangeForCurrentWeek();
     q.setCalendarId(ucal->data(Constants::Db_CalId).toInt());
-    QList<Appointement *> list = base().getCalendarEvents(q);
+    QList<Appointment *> list = base().getCalendarEvents(q);
     qWarning() << "Retreived" << list.count() << "events from the database for user" << ucal->data(Agenda::UserCalendar::UserOwnerUid).toString() << "dateRange" << q.dateStart().toString(Qt::ISODate)<< q.dateEnd().toString(Qt::ISODate) << "in" << chrono.elapsed() << "ms";
 
     qWarning() << "PatientBase count" << numberOfPatients();
 
-    Appointement *ev = 0;
+    Appointment *ev = 0;
     int maxStatus = Calendar::availableStatus().count() - 1;
     if (list.count()==0) {
         chrono.restart();
@@ -367,7 +379,7 @@ void AgendaPlugin::testDatabase()
             QDateTime end = start.addSecs(60*15);
             ucal = cals.at(r.randomInt(0, cals.count()-1));
 
-            ev = new Appointement;
+            ev = new Appointment;
             ev->setData(Constants::Db_CalId, ucal->data(Constants::Db_CalId));
             ev->setData(Constants::Db_IsValid, 1);
             ev->setData(Constants::Db_EvId, -1);
@@ -376,7 +388,7 @@ void AgendaPlugin::testDatabase()
             ev->setData(Constants::Db_XmlOptions, "XmlOptions");
             //    ev->setData(Constants::DbOnly_ComId, );
             ev->setData(Constants::Db_CatId, -1);
-//            ev->setData(Calendar::CalendarItem::PatientUid, r.getRandomString(45));
+//            ev->setData(Calendar::CalendarItem::PatientUid, r.randomString(45));
             ev->setData(CalendarItemModel::DateStart, start);
             ev->setData(CalendarItemModel::DateEnd, end);
             ev->setData(CalendarItemModel::Type, 1);
@@ -388,7 +400,7 @@ void AgendaPlugin::testDatabase()
             ev->setData(CalendarItemModel::IsAGroupEvent, r.randomInt(0,1));
             ev->setData(CalendarItemModel::Label, r.randomWords(r.randomInt(2, 15)));
             ev->setData(CalendarItemModel::Description, r.randomWords(r.randomInt(10, 500)));
-            ev->setData(CalendarItemModel::Location, r.getRandomString(r.randomInt(1,145)));
+            ev->setData(CalendarItemModel::Location, r.randomString(r.randomInt(1,145)));
             ev->setData(CalendarItemModel::IconPath, r.randomFile(pix, QStringList() << "*.png").fileName());
 
             // Add 1 to 3 patients
@@ -425,4 +437,18 @@ void AgendaPlugin::testDatabase()
     qWarning() << "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx AGENDA BASE TEST END\n\n\n";
 }
 
+ExtensionSystem::IPlugin::ShutdownFlag AgendaPlugin::aboutToShutdown()
+{
+    if (Utils::Log::warnPluginsCreation())
+        WARN_FUNC;
+    // Save settings
+    // Disconnect from signals that are not needed during shutdown
+    // Hide UI (if you add UI that is not in the main window directly)
+    // Remove preferences pages to plugins manager object pool
+    if (m_Core)
+        delete m_Core;
+    m_Core = 0;
+    return SynchronousShutdown;
+}
+
 Q_EXPORT_PLUGIN(AgendaPlugin)
