@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,15 +19,15 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "calendaritemmodel.h"
 #include "agendacore.h"
 #include "constants.h"
-#include "appointement.h"
+#include "appointment.h"
 #include "agendabase.h"
 #include <agendaplugin/usercalendar.h>
 
@@ -39,7 +39,7 @@
 
 #include <usermanagerplugin/usermodel.h>
 
-#include <calendar/calendar_item.h>
+#include <calendar/modelanditem/calendar_item.h>
 
 #include <utils/log.h>
 #include <translationutils/constants.h>
@@ -74,7 +74,7 @@ CalendarItemModel::~CalendarItemModel()
 
 Calendar::CalendarItem CalendarItemModel::getItemByUid(const QString &uid) const
 {
-    Appointement *item = getItemPointerByUid(uid.toInt());
+    Appointment *item = getItemPointerByUid(uid.toInt());
 
     if (!item) {
         Calendar::CalendarItem cal;
@@ -86,14 +86,20 @@ Calendar::CalendarItem CalendarItemModel::getItemByUid(const QString &uid) const
 }
 
 /** Return the filtred items between specified dates \e from and \e to. This function uses the default Agenda::UserCalendar or the first available. */
-QList<Calendar::CalendarItem> CalendarItemModel::getItemsBetween(const QDate &from, const QDate &to) const
+QList<Calendar::CalendarItem> CalendarItemModel::getItemsBetween(const QDate &from, const QDate &to)
 {
     Q_ASSERT_X(from <= to, "CalendarItemModel::getItemsBetween", "<from> is strictly greater than <to>");
 
+    QList<Calendar::CalendarItem> list;
+    if (from.isNull() || to.isNull()) {
+        return list;
+    }
+
+    // qWarning() << "getItemsBetween" << from << to;
+
     getItemFromDatabase(from, to, -1);
 
-    QList<Calendar::CalendarItem> list;
-    QMap<Appointement *, bool> added;
+    QMap<Appointment *, bool> added;
 
     int pivot = searchForIntersectedItem(m_sortedByBeginList, from, to, 0, m_sortedByBeginList.count() - 1);
 
@@ -154,7 +160,7 @@ Calendar::CalendarItem CalendarItemModel::insertItem(const QDateTime &beginning,
         beginInsertItem();
 
     // create the item once but insert it in two lists
-    Appointement *item = new Appointement;
+    Appointment *item = new Appointment;
     item->setModelUid(createUid());
     item->setData(DateStart, beginning);
     item->setData(DateEnd, ending);
@@ -173,7 +179,7 @@ Calendar::CalendarItem CalendarItemModel::insertItem(const QDateTime &beginning,
 Calendar::CalendarItem CalendarItemModel::addCalendarItem(const Calendar::CalendarItem &item)
 {
     // already in list ? -> update item
-    Appointement *oldItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *oldItem = getItemPointerByUid(item.uid().toInt());
     if (oldItem) {
         LOG_ERROR("Item already present");
 //        if (updatetoCalendarItem(item))
@@ -183,7 +189,7 @@ Calendar::CalendarItem CalendarItemModel::addCalendarItem(const Calendar::Calend
     beginInsertItem();
 
     // create the item once but insert it in two lists
-    Appointement *pItem = new Appointement;
+    Appointment *pItem = new Appointment;
     pItem->setModelUid(createUid());
     pItem->setData(DateStart, item.beginning());
     pItem->setData(DateEnd, item.ending());
@@ -197,7 +203,7 @@ Calendar::CalendarItem CalendarItemModel::addCalendarItem(const Calendar::Calend
     return toCalendarItem(pItem);
 }
 
-Calendar::CalendarItem CalendarItemModel::toCalendarItem(Appointement *item) const
+Calendar::CalendarItem CalendarItemModel::toCalendarItem(Appointment *item) const
 {
     Calendar::CalendarItem c(QString::number(item->modelUid()), item->beginning(), item->ending());
     setItemIsMine(&c);
@@ -207,7 +213,7 @@ Calendar::CalendarItem CalendarItemModel::toCalendarItem(Appointement *item) con
 void CalendarItemModel::setItemByUid(const QString &uid, const Calendar::CalendarItem &item)
 {
     // remove the old item
-    Appointement *oldItem = getItemPointerByUid(uid.toInt());
+    Appointment *oldItem = getItemPointerByUid(uid.toInt());
     if (!oldItem)
             return;
 
@@ -217,7 +223,7 @@ void CalendarItemModel::setItemByUid(const QString &uid, const Calendar::Calenda
     m_sortedByEndList.removeAt(m_sortedByEndList.indexOf(oldItem));
 
     // create the item once but insert it in two lists
-    Appointement *pItem = new Appointement;
+    Appointment *pItem = new Appointment;
     pItem->setData(DateStart, item.beginning());
     pItem->setData(DateEnd, item.ending());
     pItem->setData(Constants::Db_IsValid, 1);
@@ -236,7 +242,7 @@ void CalendarItemModel::setItemByUid(const QString &uid, const Calendar::Calenda
 void CalendarItemModel::removeItem(const QString &uid)
 {
     // remove the old item
-    Appointement *oldItem = getItemPointerByUid(uid.toInt());
+    Appointment *oldItem = getItemPointerByUid(uid.toInt());
     if (!oldItem)
             return;
     // remove from database
@@ -259,16 +265,24 @@ bool CalendarItemModel::moveItem(const Calendar::CalendarItem &from, Calendar::C
     if (!from.isValid())
         return false;
 
-    Appointement *item = getItemPointerByUid(from.uid().toInt());
+    Appointment *item = getItemPointerByUid(from.uid().toInt());
     if (!item) {
         return false;
     }
 
-    Q_EMIT itemRemoved(from);
     item->setData(DateStart, to.beginning());
     item->setData(DateEnd, to.ending());
-    base().saveCalendarEvent(item);
-    Q_EMIT itemInserted(toCalendarItem(item));
+
+    m_sortedByBeginList.removeAll(item);
+    m_sortedByEndList.removeAll(item);
+    m_sortedByBeginList.insert(getInsertionIndex(true, item->beginning(), m_sortedByBeginList, 0, m_sortedByBeginList.count() - 1), item);
+    m_sortedByEndList.insert(getInsertionIndex(false, item->ending(), m_sortedByEndList, 0, m_sortedByEndList.count() - 1), item);
+
+    if (!base().saveCalendarEvent(item)) {
+        LOG_ERROR("Unable to moveItem");
+        return false;
+    }
+    Q_EMIT itemModified(from, toCalendarItem(item));
     return true;
 }
 
@@ -277,7 +291,7 @@ QVariant CalendarItemModel::data(const Calendar::CalendarItem &item, int dataRef
     if (!item.isValid())
         return QVariant();
 
-    const Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    const Appointment *pItem = getItemPointerByUid(item.uid().toInt());
 
     if (!pItem)
         return QVariant();
@@ -305,7 +319,7 @@ QVariant CalendarItemModel::data(const Calendar::CalendarItem &item, int dataRef
         QString time = QString(tkTr(Trans::Constants::FROM_1_TO_2)
                                .arg(pItem->beginning().time().toString("hh:mm"))
                                .arg(pItem->ending().time().toString("hh:mm")));
-        getPeopleNames((Appointement *)pItem);
+        getPeopleNames((Appointment *)pItem);
         QStringList names = pItem->peopleNames(Calendar::People::PeopleAttendee);
         if (names.count()) {
             QString html;
@@ -332,7 +346,7 @@ bool CalendarItemModel::setData(const Calendar::CalendarItem &item, int dataRef,
     if (dataRef==Uid)
         return false;
 
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
 
     if (!pItem)
         return false;
@@ -368,7 +382,7 @@ bool CalendarItemModel::setPeopleList(const Calendar::CalendarItem &item, const
     if (!item.isValid())
         return false;
 
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
     pItem->setPeopleList(peoples);
     return true;
 }
@@ -378,7 +392,7 @@ bool CalendarItemModel::addPeople(const Calendar::CalendarItem &item, const Cale
     if (!item.isValid())
         return false;
 
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
     pItem->addPeople(people);
     return true;
 }
@@ -387,7 +401,7 @@ QList<Calendar::People> CalendarItemModel::peopleList(const Calendar::CalendarIt
 {
     if (!item.isValid())
         return QList<Calendar::People>();
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
     getPeopleNames(pItem);
     return pItem->peopleList();
 }
@@ -409,7 +423,7 @@ void CalendarItemModel::clearAll()
     m_sortedByEndList.clear();
     m_RetrievedDates.clear();
     if (m_propagateEvents)
-            Q_EMIT reset();
+        Q_EMIT reset();
 }
 
 bool CalendarItemModel::submitAll()
@@ -422,7 +436,7 @@ bool CalendarItemModel::submit(const Calendar::CalendarItem &item)
     if (!item.isValid())
         return false;
 
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
 
     if (!pItem)
         return false;
@@ -435,21 +449,21 @@ bool CalendarItemModel::revert(const Calendar::CalendarItem &item)
     if (!item.isValid())
         return false;
 
-    Appointement *pItem = getItemPointerByUid(item.uid().toInt());
+    Appointment *pItem = getItemPointerByUid(item.uid().toInt());
 
     if (!pItem)
         return false;
 
     CalendarEventQuery query;
-    query.setAppointementId(pItem->data(Constants::Db_EvId));
-    QList<Appointement *> a = base().getCalendarEvents(query);
+    query.setAppointmentId(pItem->data(Constants::Db_EvId));
+    QList<Appointment *> a = base().getCalendarEvents(query);
 
     if (a.count()==1) {
         beginModifyItem();
         m_sortedByBeginList.removeAt(m_sortedByBeginList.indexOf(pItem));
         m_sortedByEndList.removeAt(m_sortedByEndList.indexOf(pItem));
 
-        Appointement *newItem = a.at(0);
+        Appointment *newItem = a.at(0);
         m_sortedByBeginList.insert(getInsertionIndex(true, newItem->beginning(), m_sortedByBeginList, 0, m_sortedByBeginList.count() - 1), newItem);
         m_sortedByEndList.insert(getInsertionIndex(false, newItem->ending(), m_sortedByEndList, 0, m_sortedByEndList.count() - 1), newItem);
 
@@ -463,7 +477,7 @@ bool CalendarItemModel::revert(const Calendar::CalendarItem &item)
 
 
 ////// PRIVATE PART
-int CalendarItemModel::searchForIntersectedItem(const QList<Appointement *> &list, const QDate &from, const QDate &to, int first, int last) const
+int CalendarItemModel::searchForIntersectedItem(const QList<Appointment *> &list, const QDate &from, const QDate &to, int first, int last) const
 {
     if (last == -1) // 0 items
         return -1;
@@ -485,7 +499,7 @@ int CalendarItemModel::searchForIntersectedItem(const QList<Appointement *> &lis
     return middle; // intersection => found!
 }
 
-int CalendarItemModel::getInsertionIndex(bool begin, const QDateTime &dateTime, const QList<Appointement *> &list, int first, int last) const
+int CalendarItemModel::getInsertionIndex(bool begin, const QDateTime &dateTime, const QList<Appointment *> &list, int first, int last) const
 {
     if (last == - 1) // 0 items
         return 0;
@@ -506,10 +520,10 @@ int CalendarItemModel::getInsertionIndex(bool begin, const QDateTime &dateTime,
     return getInsertionIndex(begin, dateTime, list, middle + 1, last); // insertion index is in right part
 }
 
-Appointement *CalendarItemModel::getItemPointerByUid(const int uid) const
+Appointment *CalendarItemModel::getItemPointerByUid(const int uid) const
 {
     // TODO: optimize it
-    foreach (Appointement *item, m_sortedByBeginList)
+    foreach (Appointment *item, m_sortedByBeginList)
         if (item->modelUid() == uid)
             return item;
     return 0;
@@ -527,7 +541,7 @@ void CalendarItemModel::getItemFromDatabase(const QDate &from, const QDate &to,
 {
     Q_UNUSED(calendarId);
     // TODO: code here: manage calendarId ?
-    // appointements are already available ?
+    // appointments are already available ?
     QDate testDate = from;
     bool getFullRange = true;
     QVector<QDate> getDates;
@@ -547,8 +561,8 @@ void CalendarItemModel::getItemFromDatabase(const QDate &from, const QDate &to,
         return;
     }
 
-    // get from database needed appointements
-    QList<Appointement *> items;
+    // get from database needed appointments
+    QList<Appointment *> items;
 
     CalendarEventQuery query;
 //    if (m_UserCalendarFilterIndex>0 && m_UserCalendarFilterIndex<m_UserCalendar.count()) {
@@ -572,34 +586,34 @@ void CalendarItemModel::getItemFromDatabase(const QDate &from, const QDate &to,
     }
 
     for(int i = 0; i < items.count(); ++i) {
-        Appointement *item = items.at(i);
+        Appointment *item = items.at(i);
         item->setModelUid(createUid());
         m_sortedByBeginList.insert(getInsertionIndex(true, item->beginning(), m_sortedByBeginList, 0, m_sortedByBeginList.count() - 1), item);
         m_sortedByEndList.insert(getInsertionIndex(false, item->ending(), m_sortedByEndList, 0, m_sortedByEndList.count() - 1), item);
     }
 }
 
-void CalendarItemModel::getPeopleNames(Appointement *appointement) const
+void CalendarItemModel::getPeopleNames(Appointment *appointment) const
 {
     for(int i = 0; i < Calendar::People::PeopleCount; ++i) {
-        if (!appointement->peopleNamesPopulated(i)) {
+        if (!appointment->peopleNamesPopulated(i)) {
             // get names
             switch (i) {
             case Calendar::People::PeopleUser:
             case Calendar::People::PeopleUserDelegate:
             case Calendar::People::PeopleOwner:
             {
-                QHash<QString, QString> names = UserPlugin::UserModel::getUserNames(appointement->peopleUids(i, false));
+                QHash<QString, QString> names = UserPlugin::UserModel::getUserNames(appointment->peopleUids(i, false));
                 foreach(const QString &uid, names) {
-                    appointement->setPeopleName(i, uid, names.value(uid));
+                    appointment->setPeopleName(i, uid, names.value(uid));
                 }
                 break;
             }
             case Calendar::People::PeopleAttendee:
             {
-                QHash<QString, QString> names = Patients::PatientModel::patientName(appointement->peopleUids(i, false));
+                QHash<QString, QString> names = Patients::PatientModel::patientName(appointment->peopleUids(i, false));
                 foreach(const QString &uid, names.keys()) {
-                    appointement->setPeopleName(i, uid, names.value(uid));
+                    appointment->setPeopleName(i, uid, names.value(uid));
                 }
                 break;
             }
