@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,18 +19,19 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 
 /**
-  \class Agenda::UserCalendarViewer
-  Is a Agenda::UserCalendar widget viewer for FreeMedForms used in the Agenda mode.
-  This widget presents:
-  - a left bar with some usercalendar information, plus automated next availabilities calculation
-  - a central Calendar::CalendarViewer for users to edit appointements
-*/
+ * \class Agenda::UserCalendarViewer
+ * Is a Agenda::UserCalendar widget viewer for FreeMedForms used in the Agenda mode.
+ *
+ * This widget presents:
+ * - a left bar with some usercalendar information, plus automated next availabilities calculation
+ * - a central Calendar::CalendarViewer for users to edit appointments
+ */
 
 #include "usercalendarviewer.h"
 #include "agendabase.h"
@@ -39,15 +40,20 @@
 #include "usercalendar.h"
 #include "usercalendarmodel.h"
 #include "agendacore.h"
+#include "dayavailability.h"
 
+#include <coreplugin/modemanager/modemanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/iuser.h>
 #include <coreplugin/ipatient.h>
 #include <coreplugin/itheme.h>
+#include <coreplugin/constants_menus.h>
 #include <coreplugin/constants_icons.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 
-#include <calendar/basic_item_edition_dialog.h>
+#include <patientbaseplugin/patientcore.h>
+
+#include <calendar/modelanditem/basic_item_edition_dialog.h>
 #include <utils/log.h>
 #include <translationutils/constants.h>
 #include <translationutils/trans_agenda.h>
@@ -58,6 +64,7 @@
 #include <QStandardItemModel>
 #include <QStandardItem>
 #include <QHash>
+#include <QMenu>
 
 using namespace Agenda;
 using namespace Internal;
@@ -67,8 +74,10 @@ static inline Core::IUser *user() {return Core::ICore::instance()->user();}
 static inline Core::IPatient *patient() {return Core::ICore::instance()->patient();}
 static inline Agenda::Internal::AgendaBase &base() {return Agenda::AgendaCore::instance().agendaBase();}
 static inline Agenda::AgendaCore &agendaCore() {return Agenda::AgendaCore::instance();}
-inline static Core::ActionManager *actionManager() {return Core::ICore::instance()->actionManager();}
-inline static Core::ITheme *theme() {return Core::ICore::instance()->theme();}
+static inline Core::ActionManager *actionManager() {return Core::ICore::instance()->actionManager();}
+static inline Core::ITheme *theme() {return Core::ICore::instance()->theme();}
+static inline Core::ModeManager *modeManager() {return Core::ICore::instance()->modeManager();}
+static inline Patients::PatientCore *patientCore() {return Patients::PatientCore::instance();}
 
 namespace {
     const int S_NUMBEROFAVAILABILITIESTOSHOW = 10;
@@ -82,7 +91,11 @@ public:
     UserCalendarViewerPrivate(UserCalendarViewer *parent) :
         ui(new Ui::UserCalendarViewer),
         m_CalendarItemModel(0),
+        m_ItemContextMenu(0),
         m_UserCalendarModel(agendaCore().userCalendarModel(user()->uuid())),
+        aToday(0), aTomorrow(0), aNextWeek(0), aNextMonth(0),
+        aSwitchToPatient(0), aEditItem(0), aPrintItem(0), aDeleteItem(0),
+//        aRefreshView(0),
         m_AvailModel(0),
         q(parent)
     {
@@ -101,60 +114,92 @@ public:
 //        }
     }
 
+    void createActions()
+    {
+        aToday = new QAction(q);
+        aTomorrow = new QAction(q);
+        aNextWeek = new QAction(q);
+        aNextMonth = new QAction(q);
+//        aRefreshView = new QAction(q);
+        aToday->setIcon(theme()->icon(Core::Constants::ICONDATE));
+        aTomorrow->setIcon(theme()->icon(Core::Constants::ICONDATE));
+        aNextWeek->setIcon(theme()->icon(Core::Constants::ICONDATE));
+        aNextMonth->setIcon(theme()->icon(Core::Constants::ICONDATE));
+//        aRefreshView->setIcon(theme()->icon(Core::Constants::ICONSOFTWAREUPDATEAVAILABLE));
+        ui->startDateSelector->addAction(aToday);
+        ui->startDateSelector->addAction(aTomorrow);
+        ui->startDateSelector->addAction(aNextWeek);
+        ui->startDateSelector->addAction(aNextMonth);
+        ui->startDateSelector->setDefaultAction(aToday);
+        QObject::connect(ui->startDateSelector, SIGNAL(triggered(QAction*)), q, SLOT(quickDateSelection(QAction*)));
+    }
+
     void populateCalendarWithCurrentWeek(Agenda::UserCalendar *calendar)
     {
         Q_UNUSED(calendar);
         ui->calendarViewer->setModel(m_CalendarItemModel);
     }
 
+    void createCalendarItemContextMenu()
+    {
+        if (!m_ItemContextMenu) {
+            m_ItemContextMenu = new QMenu(q);
+            aSwitchToPatient = new QAction(q);
+            aEditItem = new QAction(q);
+            aPrintItem = new QAction(q);
+            aDeleteItem = new QAction(q);
+
+            m_ItemContextMenu->addAction(aSwitchToPatient);
+            m_ItemContextMenu->addAction(aEditItem);
+            // TODO: re add the print appointement action
+//            m_ItemContextMenu->addAction(aPrintItem);
+            m_ItemContextMenu->addAction(aDeleteItem);
+
+            ui->calendarViewer->setContextMenuForItems(m_ItemContextMenu);
+            QObject::connect(aSwitchToPatient, SIGNAL(triggered()), q, SLOT(onSwitchToPatientClicked()));
+            QObject::connect(aEditItem, SIGNAL(triggered()), q, SLOT(onEditAppointmentClicked()));
+            QObject::connect(aPrintItem, SIGNAL(triggered()), q, SLOT(onPrintAppointmentClicked()));
+            QObject::connect(aDeleteItem, SIGNAL(triggered()), q, SLOT(onDeleteAppointmentClicked()));
+        }
+    }
 
 public:
     Ui::UserCalendarViewer *ui;
     CalendarItemModel *m_CalendarItemModel;
+    QMenu *m_ItemContextMenu;
     UserCalendarModel *m_UserCalendarModel;
     QHash<QString, int> m_UidToListIndex;
     bool scrollOnShow;
 
     QAction *aToday, *aTomorrow, *aNextWeek, *aNextMonth;
+    QAction *aSwitchToPatient, *aEditItem, *aPrintItem, *aDeleteItem;
+//    QAction *aRefreshView;
     QStandardItemModel *m_AvailModel;
 
 private:
     UserCalendarViewer *q;
 };
-
 }  // End namespace Internal
 }  // End namespace Agenda
 
-
 UserCalendarViewer::UserCalendarViewer(QWidget *parent) :
     QWidget(parent),
     d(new UserCalendarViewerPrivate(this))
 {
+    setObjectName("UserCalendarViewer");
     d->ui->setupUi(this);
     this->layout()->setMargin(0);
     d->ui->startDate->setDate(QDate::currentDate());
     d->ui->startDate->setDisplayFormat(tkTr(Trans::Constants::DATEFORMAT_FOR_EDITOR));
-
-    d->aToday = new QAction(this);
-    d->aTomorrow = new QAction(this);
-    d->aNextWeek = new QAction(this);
-    d->aNextMonth = new QAction(this);
-    d->aToday->setIcon(theme()->icon(Core::Constants::ICONDATE));
-    d->aTomorrow->setIcon(theme()->icon(Core::Constants::ICONDATE));
-    d->aNextWeek->setIcon(theme()->icon(Core::Constants::ICONDATE));
-    d->aNextMonth->setIcon(theme()->icon(Core::Constants::ICONDATE));
-    d->ui->startDateSelector->addAction(d->aToday);
-    d->ui->startDateSelector->addAction(d->aTomorrow);
-    d->ui->startDateSelector->addAction(d->aNextWeek);
-    d->ui->startDateSelector->addAction(d->aNextMonth);
-    d->ui->startDateSelector->setDefaultAction(d->aToday);
-    connect(d->ui->startDateSelector, SIGNAL(triggered(QAction*)), this, SLOT(quickDateSelection(QAction*)));
+    d->createActions();
 
     d->ui->refreshAvailabilities->setIcon(theme()->icon(Core::Constants::ICONSOFTWAREUPDATEAVAILABLE));
+    d->ui->refreshAvailabilities->setToolTip(tr("Refresh Availabilities"));
     d->ui->calendarViewer->setDate(QDate::currentDate());
     d->ui->calendarViewer->setDayScaleHourDivider(2);
     d->ui->calendarViewer->setDayGranularity(5);
     d->ui->calendarViewer->setHourHeight(4*20); // 20pixels per minutes
+    d->createCalendarItemContextMenu();
 
     // populate the availabilities duration selector combo (every five minutes)
     for(int i = 1; i < 19; ++i) {
@@ -178,7 +223,7 @@ UserCalendarViewer::UserCalendarViewer(QWidget *parent) :
     connect(d->ui->availabilitiesView, SIGNAL(activated(QModelIndex)), this, SLOT(newEventAtAvailabity(QModelIndex)));
     connect(d->ui->availableAgendasCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(on_availableAgendasCombo_activated(int)));
     connect(d->ui->refreshAvailabilities, SIGNAL(clicked()), this, SLOT(refreshAvailabilities()));
-    connect(d->ui->defaultDurationButton,SIGNAL(clicked()), this, SLOT(resetDefaultDuration()));
+    connect(d->ui->defaultDurationButton, SIGNAL(clicked()), this, SLOT(resetDefaultDuration()));
     connect(d->ui->startDate, SIGNAL(dateChanged(QDate)), this, SLOT(onStartDateChanged(QDate)));
     userChanged();
 
@@ -216,7 +261,7 @@ void UserCalendarViewer::newEventAtAvailabity(const QModelIndex &index)
         idx = d->m_AvailModel->index(0,0);
         idx = d->m_AvailModel->index(0,0, idx);
     }
-    Calendar::BasicItemEditionDialog dlg(d->m_CalendarItemModel, this);
+    Calendar::BasicItemEditorDialog dlg(d->m_CalendarItemModel, this);
     QDateTime start = d->m_AvailModel->itemFromIndex(idx)->data().toDateTime();
     if (start.isNull())
         return;
@@ -234,7 +279,7 @@ void UserCalendarViewer::newEventAtAvailabity(const QModelIndex &index)
     if (dlg.exec() != QDialog::Accepted) {
         d->m_CalendarItemModel->removeItem(item.uid());
     }
-    recalculateAvailabilitiesWithDurationIndex((d->ui->availDurationCombo->currentIndex()+1)*5);
+    recalculateAvailabilitiesWithDurationIndex(d->ui->availDurationCombo->currentIndex());
 }
 
 void UserCalendarViewer::refreshAvailabilities()
@@ -243,7 +288,8 @@ void UserCalendarViewer::refreshAvailabilities()
 }
 
 /** Resets the duration in the ComboBox to the default duration set in the Agenda config. */
-void UserCalendarViewer::resetDefaultDuration() {
+void UserCalendarViewer::resetDefaultDuration()
+{
     int currentCalendarIndex = d->ui->availableAgendasCombo->currentIndex();
 
     QModelIndex index = d->m_UserCalendarModel->index(currentCalendarIndex, UserCalendarModel::DefaultDuration);
@@ -257,7 +303,7 @@ void UserCalendarViewer::quickDateSelection(QAction *a)
     if (a==d->aTomorrow)
         d->ui->startDate->setDate(QDate::currentDate().addDays(1));
     if (a==d->aNextWeek)
-        d->ui->startDate->setDate(QDate::currentDate().addDays(QDate::currentDate().dayOfWeek() - 7));
+        d->ui->startDate->setDate(QDate::currentDate().addDays(8 - QDate::currentDate().dayOfWeek()));
     if (a==d->aNextMonth)
         d->ui->startDate->setDate(QDate(QDate::currentDate().year(), QDate::currentDate().month(), 1).addMonths(1));
     onStartDateChanged(d->ui->startDate->date());
@@ -306,7 +352,7 @@ void UserCalendarViewer::recalculateAvailabilitiesWithDurationIndex(const int in
         QDate dt = dates.at(i).date();
         QStandardItem *parent = main.value(dt.toString(Qt::ISODate), 0);
         if (!parent) {
-            parent = new QStandardItem(dt.toString(QLocale().dateFormat(QLocale::LongFormat)));
+            parent = new QStandardItem((QLocale().toString(dt, QLocale::LongFormat)));
             parent->setFont(bold);
             d->m_AvailModel->invisibleRootItem()->appendRow(parent);
             main.insert(dt.toString(Qt::ISODate), parent);
@@ -362,9 +408,11 @@ void UserCalendarViewer::on_availableAgendasCombo_activated(const int index)
         d->ui->calendarViewer->setDayScaleHourDivider(defaultDuration/60);
         d->ui->calendarViewer->setDayItemDefaultDuration(defaultDuration);
 
-        //: default agenda duration time (in minutes)
-        d->ui->defaultDurationButton->setToolTip(tr("Set back to default: ") +
-                                                 QString::number(defaultDuration) + " " + tkTr(Trans::Constants::MINUTES));
+        //: default agenda duration time, %1 = time, %2 = minutes (fixed)
+        d->ui->defaultDurationButton->setToolTip(tr("Set back to default: %1 %2").arg(
+                                                     QString::number(defaultDuration),
+                                                     tkTr(Trans::Constants::MINUTES)));
+        resetDefaultDuration();
         d->ui->description->setHtml(d->m_UserCalendarModel->index(index, UserCalendarModel::Description).data().toString());
     }
 //    d->populateCalendarWithCurrentWeek(d->m_UserCals.at(index));
@@ -373,7 +421,6 @@ void UserCalendarViewer::on_availableAgendasCombo_activated(const int index)
 void UserCalendarViewer::userChanged()
 {
     // Update ui
-    d->ui->userNameLabel->setText(user()->value(Core::IUser::FullName).toString());
     if (d->m_UserCalendarModel) {
         disconnect(d->m_UserCalendarModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(updateCalendarData(QModelIndex,QModelIndex)));
     }
@@ -412,20 +459,20 @@ void UserCalendarViewer::userChanged()
 
     // Scroll the view to now
     if (isVisible()) {
-        d->ui->calendarViewer->scrollToTime(QTime(17,0,0));
+        d->ui->calendarViewer->scrollToTime(QTime::currentTime());
         d->scrollOnShow = false;
     } else {
         d->scrollOnShow = true;
     }
 }
 
-
 void UserCalendarViewer::updateCalendarData(const QModelIndex &top, const QModelIndex &bottom)
 {
     Q_UNUSED(bottom);
     if (top.column()==UserCalendarModel::DefaultDuration) {
         // get default duration of given calendar
-        int defaultDuration = d->m_UserCalendarModel->index(top.row(), UserCalendarModel::DefaultDuration, top.parent()).data().toInt();
+        int defaultDuration = d->m_UserCalendarModel->index(top.row(),
+                                                            UserCalendarModel::DefaultDuration, top.parent()).data().toInt();
         d->ui->calendarViewer->setDayScaleHourDivider(defaultDuration/60);
         d->ui->calendarViewer->setDayItemDefaultDuration(defaultDuration);
         //: default agenda duration time (in minutes)
@@ -440,6 +487,50 @@ void UserCalendarViewer::updateCalendarData(const QModelIndex &top, const QModel
     }
 }
 
+/** Reacts to the user selected action in the contextual menu */
+void UserCalendarViewer::onSwitchToPatientClicked()
+{
+    // Get the clicked calendar item
+    Calendar::CalendarItem item = d->ui->calendarViewer->getContextualCalendarItem();
+
+    // Find the first patient of the appointement
+    QList<Calendar::People> people = d->m_CalendarItemModel->peopleList(item);
+    foreach(const Calendar::People &guest, people) {
+        if (guest.type == Calendar::People::PeopleAttendee) {
+            if (!patientCore()->setCurrentPatientUuid(guest.uid)) {
+                LOG_ERROR("Unable to set current patient");
+            }
+            break;
+        }
+    }
+}
+
+/** Reacts to the user selected action in the contextual menu */
+void UserCalendarViewer::onEditAppointmentClicked()
+{
+    // Get the clicked calendar item
+    Calendar::CalendarItem item = d->ui->calendarViewer->getContextualCalendarItem();
+
+    // Starts the editor dialog
+    Calendar::BasicItemEditorDialog dialog(d->m_CalendarItemModel, this);
+    dialog.init(item);
+    dialog.exec();
+}
+
+/** Reacts to the user selected action in the contextual menu */
+void UserCalendarViewer::onPrintAppointmentClicked()
+{
+    // TODO
+}
+
+/** Reacts to the user selected action in the contextual menu */
+void UserCalendarViewer::onDeleteAppointmentClicked()
+{
+    // Get the clicked calendar item
+    Calendar::CalendarItem item = d->ui->calendarViewer->getContextualCalendarItem();
+    d->m_CalendarItemModel->removeItem(item.uid());
+}
+
 bool UserCalendarViewer::event(QEvent *e)
 {
     switch (e->type()) {
@@ -457,10 +548,20 @@ bool UserCalendarViewer::event(QEvent *e)
         d->aToday->setToolTip(d->aToday->text());
         d->aTomorrow->setText(tkTr(Trans::Constants::TOMORROW));
         d->aTomorrow->setToolTip(d->aTomorrow->text());
-        d->aNextWeek->setText(tkTr(Trans::Constants::NEXT_WEEK));
+        d->aNextWeek->setText(tr("Beginning of next week"));
         d->aNextWeek->setToolTip(d->aNextWeek->text());
-        d->aNextMonth->setText(tkTr(Trans::Constants::NEXT_MONTH));
+        d->aNextMonth->setText(tr("Beginning of next month"));
         d->aNextMonth->setToolTip(d->aNextMonth->text());
+
+        if (d->aSwitchToPatient)
+            d->aSwitchToPatient->setText(tr("Switch to patient"));
+        if (d->aEditItem)
+            d->aEditItem->setText(tr("Edit appointment"));
+        if (d->aPrintItem)
+                d->aPrintItem->setText(tr("Print appointment"));
+        if (d->aDeleteItem)
+                d->aDeleteItem->setText(tr("Delete appointment"));
+
         break;
     }
     case QEvent::Show:
