@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -20,7 +20,7 @@
  ***************************************************************************/
 /***************************************************************************
  *   Main developers : Eric MAEKER, MD <eric.maeker@gmail.com>              *
- *   Contributors :                                                        *
+ *  Contributors:                                                          *
  *       Christian A. Reiter <christian.a.reiter@gmail.com>                *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -47,13 +47,13 @@ static inline Core::IPatient *patient() {return Core::ICore::instance()->patient
 using namespace BaseWidgets;
 using namespace Internal;
 
-inline static QLabel *findLabel(Form::FormItem *item)
+static inline QLabel *findLabel(Form::FormItem *item)
 {
     QLabel *l = 0;
     // Find label
     const QString &lbl = item->spec()->value(Form::FormItemSpec::Spec_UiLabel).toString();
     if (!lbl.isEmpty()) {
-        l = qFindChild<QLabel*>(item->parentFormMain()->formWidget(), lbl);
+        l = item->parentFormMain()->formWidget()->findChild<QLabel*>(lbl);
         if (l) {
             l->setText(item->spec()->label());
         } else {
@@ -519,6 +519,12 @@ bool AustrianSocialNumberFormData::isModified() const
     return m_OriginalValue != m_Widget->number();
 }
 
+void AustrianSocialNumberFormData::setModified(bool modified)
+{
+    if (!modified)
+        m_OriginalValue == m_Widget->number();
+}
+
 bool AustrianSocialNumberFormData::setData(const int ref, const QVariant &data, const int role)
 {
     Q_UNUSED(ref);
