@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "calculationwidgets.h"
@@ -30,6 +30,7 @@
 #include <coreplugin/iscriptmanager.h>
 
 #include <formmanagerplugin/iformitem.h>
+#include <formmanagerplugin/iformitemdata.h>
 
 #include <utils/global.h>
 #include <utils/log.h>
@@ -71,28 +72,28 @@ enum ProvidedWidget {
 
 }  // End namespace Anonymous
 
-inline static Form::IFormWidget::LabelOptions labelAlignement(Form::FormItem *item, Form::IFormWidget::LabelOptions defaultValue = Form::IFormWidget::Label_OnLeft)
+static inline Form::IFormWidget::LabelOptions labelAlignement(Form::FormItem *item, Form::IFormWidget::LabelOptions defaultValue = Form::IFormWidget::OnLeft)
 {
     const QStringList &o = item->getOptions();
     if (o.contains(::LABEL_ALIGN_TOP, Qt::CaseInsensitive))
-        return Form::IFormWidget::Label_OnTop;
+        return Form::IFormWidget::OnTop;
     else if (o.contains(::LABEL_ALIGN_LEFT, Qt::CaseInsensitive))
-        return Form::IFormWidget::Label_OnLeft;
+        return Form::IFormWidget::OnLeft;
     return defaultValue;
 }
 
-inline static bool dontPrintEmptyValues(Form::FormItem *item)
+static inline bool dontPrintEmptyValues(Form::FormItem *item)
 {
     return item->getOptions().contains(::DONTPRINTEMPTYVALUES, Qt::CaseInsensitive);
 }
 
-inline static QLabel *findLabel(Form::FormItem *item)
+static inline QLabel *findLabel(Form::FormItem *item)
 {
     QLabel *l = 0;
     // Find label
     const QString &lbl = item->spec()->value(Form::FormItemSpec::Spec_UiLabel).toString();
     if (!lbl.isEmpty()) {
-        l = qFindChild<QLabel*>(item->parentFormMain()->formWidget(), lbl);
+        l = item->parentFormMain()->formWidget()->findChild<QLabel*>(lbl);
         if (l) {
             l->setText(item->spec()->label());
         } else {
@@ -106,6 +107,7 @@ inline static QLabel *findLabel(Form::FormItem *item)
 CalculationWidgetsFactory::CalculationWidgetsFactory(QObject *parent) :
     IFormWidgetFactory(parent)
 {
+    // TODO: in all the widgets, manage the readonly state.
 }
 
 CalculationWidgetsFactory::~CalculationWidgetsFactory()
@@ -176,7 +178,8 @@ static void addResultToEpisodeLabel(Form::FormMain *parent, QLabel *label, const
         episodeLabel.insert(begin, add);
         parent->itemData()->setData(Form::IFormItemData::ID_EpisodeLabel, episodeLabel);
     } else {
-        parent->itemData()->setData(Form::IFormItemData::ID_EpisodeLabel, episodeLabel + "; " + add);
+        QString tmp = episodeLabel + "; " + add;
+        parent->itemData()->setData(Form::IFormItemData::ID_EpisodeLabel, tmp);
     }
 }
 
@@ -192,7 +195,7 @@ SumWidget::SumWidget(Form::FormItem *formItem, QWidget *parent) :
     const QString &widget = formItem->spec()->value(Form::FormItemSpec::Spec_UiWidget).toString();
     if (!widget.isEmpty()) {
         // Find widget
-        QLineEdit *le = qFindChild<QLineEdit*>(formItem->parentFormMain()->formWidget(), widget);
+        QLineEdit *le = formItem->parentFormMain()->formWidget()->findChild<QLineEdit*>(widget);
         if (le) {
             line = le;
         } else {
@@ -203,7 +206,7 @@ SumWidget::SumWidget(Form::FormItem *formItem, QWidget *parent) :
         m_Label = findLabel(formItem);
     } else {
         // Prepare Widget Layout and label
-        QBoxLayout *hb = getBoxLayout(Label_OnLeft, m_FormItem->spec()->label(), this);
+        QBoxLayout *hb = getBoxLayout(OnLeft, m_FormItem->spec()->label(), this);
         hb->addWidget(m_Label);
 
         // Add LineEdit for the result
@@ -214,6 +217,8 @@ SumWidget::SumWidget(Form::FormItem *formItem, QWidget *parent) :
     }
     retranslate();
 
+    setFocusedWidget(line);
+
     // connect to parent FormMain
     Form::FormMain *p = formItem->parentFormMain();
     if (p) {
@@ -268,12 +273,14 @@ void SumWidget::retranslate()
 {
     if (m_Label)
         m_Label->setText(m_FormItem->spec()->label());
+    if (line)
+        line->setToolTip(m_FormItem->spec()->tooltip());
 }
 
 void SumWidget::connectFormItems()
 {
-//    qWarning() << "SUM requiered" << formItem()->extraDatas().value(::SUM_EXTRA_KEY) << formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY);
-    if (formItem()->extraDatas().value(::SUM_EXTRA_KEY).isEmpty() && formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
+//    qWarning() << "SUM requiered" << formItem()->extraData().value(::SUM_EXTRA_KEY) << formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY);
+    if (formItem()->extraData().value(::SUM_EXTRA_KEY).isEmpty() && formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
         LOG_ERROR("Sum widget: No sumof tag <sumof> or <sumof_regexp>.");
         return;
     }
@@ -283,10 +290,10 @@ void SumWidget::connectFormItems()
         return;
     }
 //            qWarning() << "Parent = " << p->uuid();
-    if (!formItem()->extraDatas().value(::SUM_EXTRA_KEY).isEmpty()) {
-        QStringList uuids = formItem()->extraDatas().value(::SUM_EXTRA_KEY).split(";");
+    if (!formItem()->extraData().value(::SUM_EXTRA_KEY).isEmpty()) {
+        QStringList uuids = formItem()->extraData().value(::SUM_EXTRA_KEY).split(";");
         // get all formitems and connect to the dataChanged signal
-        QList<Form::FormItem *> items = p->flattenFormItemChildren();
+        QList<Form::FormItem *> items = p->flattenedFormItemChildren();
         foreach(QString uid, uuids) {
             uid = uid.simplified();
             for(int i = 0; i < items.count(); ++i) {
@@ -301,9 +308,9 @@ void SumWidget::connectFormItems()
                 }
             }
         }
-    } else if (!formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
-        QRegExp reg(formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY), Qt::CaseInsensitive, QRegExp::Wildcard);
-        QList<Form::FormItem *> items = p->flattenFormItemChildren();
+    } else if (!formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
+        QRegExp reg(formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY), Qt::CaseInsensitive, QRegExp::Wildcard);
+        QList<Form::FormItem *> items = p->flattenedFormItemChildren();
         for(int i = 0; i < items.count(); ++i) {
             Form::FormItem *item = items.at(i);
             if (item==m_FormItem)
@@ -319,17 +326,17 @@ void SumWidget::connectFormItems()
 void SumWidget::recalculate(const int modifiedRef)
 {
     Q_UNUSED(modifiedRef);
-//    qWarning() << "SUM recalculate" << formItem()->extraDatas().value(::SUM_EXTRA_KEY);
+//    qWarning() << "SUM recalculate" << formItem()->extraData().value(::SUM_EXTRA_KEY);
     Form::FormMain *p = formItem()->parentFormMain();
     if (!p) {
         LOG_ERROR("No FormMain parent");
         return;
     }
     double sum = 0;
-    if (!formItem()->extraDatas().value(::SUM_EXTRA_KEY).isEmpty()) {
-        QStringList uuids = formItem()->extraDatas().value(::SUM_EXTRA_KEY).split(";");
+    if (!formItem()->extraData().value(::SUM_EXTRA_KEY).isEmpty()) {
+        QStringList uuids = formItem()->extraData().value(::SUM_EXTRA_KEY).split(";");
         // get all formitems and connect to the dataChanged signal
-        QList<Form::FormItem *> items = p->flattenFormItemChildren();
+        QList<Form::FormItem *> items = p->flattenedFormItemChildren();
         foreach(const QString &uid, uuids) {
             for(int i = 0; i < items.count(); ++i) {
                 Form::FormItem *item = items.at(i);
@@ -339,9 +346,9 @@ void SumWidget::recalculate(const int modifiedRef)
                 }
             }
         }
-    } else if (!formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
-        QRegExp reg(formItem()->extraDatas().value(::SUM_REGEXP_EXTRA_KEY), Qt::CaseInsensitive, QRegExp::Wildcard);
-        QList<Form::FormItem *> items = p->flattenFormItemChildren();
+    } else if (!formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY).isEmpty()) {
+        QRegExp reg(formItem()->extraData().value(::SUM_REGEXP_EXTRA_KEY), Qt::CaseInsensitive, QRegExp::Wildcard);
+        QList<Form::FormItem *> items = p->flattenedFormItemChildren();
         for(int i = 0; i < items.count(); ++i) {
             Form::FormItem *item = items.at(i);
             if (item->uuid().contains(reg) && item->itemData()) {
@@ -370,7 +377,7 @@ ScriptWidget::ScriptWidget(Form::FormItem *formItem, QWidget *parent) :
     const QString &widget = formItem->spec()->value(Form::FormItemSpec::Spec_UiWidget).toString();
     if (!widget.isEmpty()) {
         // Find widget
-        QLineEdit *le = qFindChild<QLineEdit*>(formItem->parentFormMain()->formWidget(), widget);
+        QLineEdit *le = formItem->parentFormMain()->formWidget()->findChild<QLineEdit*>(widget);
         if (le) {
             line = le;
         } else {
@@ -386,7 +393,7 @@ ScriptWidget::ScriptWidget(Form::FormItem *formItem, QWidget *parent) :
             m_Editor->setObjectName("ScriptWidgetTextEditor_" + m_FormItem->uuid());
             m_Editor->setSizePolicy(QSizePolicy::Expanding , QSizePolicy::Fixed);
             // Find widget
-            QLayout *lay = qFindChild<QLayout*>(formItem->parentFormMain()->formWidget(), layout);
+            QLayout *lay = formItem->parentFormMain()->formWidget()->findChild<QLayout*>(layout);
             if (lay) {
                 lay->addWidget(m_Editor);
             } else {
@@ -395,7 +402,7 @@ ScriptWidget::ScriptWidget(Form::FormItem *formItem, QWidget *parent) :
             m_Label = findLabel(formItem);
         } else {
             // Prepare Widget Layout and label
-            QBoxLayout *hb = getBoxLayout(Label_OnLeft, m_FormItem->spec()->label(), this);
+            QBoxLayout *hb = getBoxLayout(OnLeft, m_FormItem->spec()->label(), this);
             hb->addWidget(m_Label);
 
             // Add LineEdit for the result
@@ -413,6 +420,8 @@ ScriptWidget::ScriptWidget(Form::FormItem *formItem, QWidget *parent) :
         }
     }
 
+    setFocusedWidget(line);
+
     retranslate();
 
     // connect to parent FormMain
@@ -474,13 +483,17 @@ void ScriptWidget::retranslate()
 {
     if (m_Label)
         m_Label->setText(m_FormItem->spec()->label());
+    if (line)
+        line->setToolTip(m_FormItem->spec()->tooltip());
+    if (m_Editor)
+        m_Editor->setToolTip(m_FormItem->spec()->tooltip());
 }
 
 void ScriptWidget::connectFormItems()
 {
-//    qWarning() << "CalculationScript" << formItem()->extraDatas().value(::CONNECT_EXTRA_KEY) << formItem()->extraDatas().value(::CONNECT_REGEXP_EXTRA_KEY);
-    if (formItem()->extraDatas().value(::CONNECT_EXTRA_KEY).isEmpty() &&
-            formItem()->extraDatas().value(::CONNECT_REGEXP_EXTRA_KEY).isEmpty()) {
+//    qWarning() << "CalculationScript" << formItem()->extraData().value(::CONNECT_EXTRA_KEY) << formItem()->extraData().value(::CONNECT_REGEXP_EXTRA_KEY);
+    if (formItem()->extraData().value(::CONNECT_EXTRA_KEY).isEmpty() &&
+            formItem()->extraData().value(::CONNECT_REGEXP_EXTRA_KEY).isEmpty()) {
         LOG_ERROR("Calculation script widget: No <connect> tag.");
         return;
     }
@@ -490,11 +503,11 @@ void ScriptWidget::connectFormItems()
         return;
     }
 //    qWarning() << "Parent = " << p->uuid();
-//    QString ns = formItem()->extraDatas().value(::SCRIPT_NS_EXTRA_KEY);
-    QString itemList = formItem()->extraDatas().value(::CONNECT_EXTRA_KEY);
-    QString regexp = formItem()->extraDatas().value(::CONNECT_REGEXP_EXTRA_KEY);
+//    QString ns = formItem()->extraData().value(::SCRIPT_NS_EXTRA_KEY);
+    QString itemList = formItem()->extraData().value(::CONNECT_EXTRA_KEY);
+    QString regexp = formItem()->extraData().value(::CONNECT_REGEXP_EXTRA_KEY);
     // Find all items to connect
-    QList<Form::FormItem *> children = p->flattenFormItemChildren();
+    QList<Form::FormItem *> children = p->flattenedFormItemChildren();
     if (!itemList.isEmpty()) {
         QStringList items = itemList.split(";");
         for(int i = 0; i < children.count(); ++i) {
@@ -529,13 +542,13 @@ void ScriptWidget::connectFormItems()
 void ScriptWidget::recalculate(const int modifiedRef)
 {
     Q_UNUSED(modifiedRef);
-//    qWarning() << "CalculationScript recalculate" << formItem()->extraDatas().value(::SCRIPT_EXTRA_KEY);
+//    qWarning() << "CalculationScript recalculate" << formItem()->extraData().value(::SCRIPT_EXTRA_KEY);
     Form::FormMain *p = formItem()->parentFormMain();
     if (!p) {
         LOG_ERROR("No FormMain parent");
         return;
     }
-    QString script = formItem()->extraDatas().value(::SCRIPT_EXTRA_KEY);
+    QString script = formItem()->extraData().value(::SCRIPT_EXTRA_KEY);
 
     // evaluate the script using the Core::IScriptEngine
     QScriptValue val = scriptManager()->evaluate(script);
