@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "categorybase.h"
@@ -193,7 +193,7 @@ bool CategoryBase::createDatabase(const QString &connectionName , const QString
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         if (!DB.open())
             LOG(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(dbName).arg(DB.lastError().text()));
@@ -253,11 +253,14 @@ bool CategoryBase::createDatabase(const QString &connectionName , const QString
     }
 
     // Add version number
+    DB.commit();
     QSqlQuery query(DB);
     query.prepare(prepareInsertQuery(Constants::Table_VERSION));
     query.bindValue(Constants::VERSION_TEXT, Constants::DB_ACTUALVERSION);
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
         return false;
     }
 
@@ -270,9 +273,8 @@ QVector<CategoryItem *> CategoryBase::getCategories(const QString &mime, const Q
     QVector<CategoryItem *> cats;
     QMultiHash<CategoryItem *, QString> childrenUuid;
     QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
-    if (!connectDatabase(DB, __LINE__)) {
+    if (!connectDatabase(DB, __LINE__))
         return cats;
-    }
 
     QString req;
     DB.transaction();
@@ -429,33 +431,46 @@ bool CategoryBase::saveCategory(CategoryItem *category)
 */
 bool CategoryBase::saveCategories(const QVector<CategoryItem *> &categories, bool createTransaction)
 {
+//    QTime chr;
+//    chr.start();
     QSqlDatabase DB = QSqlDatabase::database(Constants::DB_NAME);
     if (!connectDatabase(DB, __LINE__)) {
         return false;
     }
     if (createTransaction)
         DB.transaction();
+
+    QSqlQuery query(DB);
+
+//    if (createTransaction)
+//        Utils::Log::logTimeElapsed(chr, "---", "create transaction");
     for(int i=0; i < categories.count(); ++i) {
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         CategoryItem *category = categories.at(i);
 
         // save or update ?
         if (categoryNeedsUpdate(category)) {
+            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+//            Utils::Log::logTimeElapsed(chr, "---", "needs update");
             if (!updateCategory(category)) {
                 if (createTransaction)
                     DB.rollback();
                 return false;
             }
+//            Utils::Log::logTimeElapsed(chr, "---", "update");
             continue;
         }
 
         // save labels
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         if (!saveCategoryLabels(category)) {
             if (createTransaction)
                 DB.rollback();
             return false;
         }
+//        Utils::Log::logTimeElapsed(chr, "---", "save labels");
+
         // save category itself
-        QSqlQuery query(DB);
         query.prepare(prepareInsertQuery(Constants::Table_CATEGORIES));
         query.bindValue(Constants::CATEGORY_ID, QVariant());
         query.bindValue(Constants::CATEGORY_UUID, category->data(CategoryItem::Uuid));
@@ -472,21 +487,31 @@ bool CategoryBase::saveCategories(const QVector<CategoryItem *> &categories, boo
             category->setData(CategoryItem::DbOnly_Id, query.lastInsertId());
         } else {
             LOG_QUERY_ERROR(query);
-            if (createTransaction)
+            query.finish();
+            if (createTransaction) {
                 DB.rollback();
+            }
             return false;
         }
+        query.finish();
         category->setDirty(false);
+//        Utils::Log::logTimeElapsed(chr, "---", "save");
+
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         for(int i=0; i < category->childCount(); ++i)
             category->child(i)->setData(CategoryItem::DbOnly_ParentId, category->id());
 
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         if (!saveCategories(category->children().toVector(), false)) {
             if (createTransaction)
                 DB.rollback();
             return false;
         }
     }
-    DB.commit();
+//    Utils::Log::logTimeElapsed(chr, "---", "commit");
+
+    if (createTransaction)
+        DB.commit();
     return true;
 }
 
@@ -514,6 +539,7 @@ bool CategoryBase::categoryNeedsUpdate(CategoryItem *category)
             }
         } else {
             LOG_QUERY_ERROR(query);
+            return false;
         }
         // id found --> return true
         return (id >= 0);
@@ -577,18 +603,30 @@ bool CategoryBase::updateCategory(CategoryItem *category)
 /** \brief Save or update categories labels. */
 bool CategoryBase::saveCategoryLabels(CategoryItem *category)
 {
+//    QTime chr;
+//    chr.start();
     // we are inside a transaction
     if (!category->isDirty())
         return true;
     // get label_id
     int labelId = -1;
+    bool deleteOldValues = true;
+    QSqlQuery query(database());
     if (category->data(CategoryItem::DbOnly_LabelId).isNull() ||
         category->data(CategoryItem::DbOnly_LabelId).toInt()==-1) {
-        labelId = max(Constants::Table_CATEGORY_LABEL, Constants::CATEGORYLABEL_LABEL_ID).toInt();
-        ++labelId;
+        deleteOldValues = false;
+        if (query.exec(maxSqlCommand(Constants::Table_CATEGORY_LABEL, Constants::CATEGORYLABEL_LABEL_ID))) {
+            if (query.next())
+                labelId = query.value(0).toInt() + 1;
+        } else {
+            LOG_QUERY_ERROR(query);
+            return false;
+        }
+        query.finish();
+//        Utils::Log::logTimeElapsed(chr, "--- label", "get max");
+
         category->setData(CategoryItem::DbOnly_LabelId, labelId);
         // create an empty label using this LabelId
-        QSqlQuery query(database());
         query.prepare(prepareInsertQuery(Constants::Table_CATEGORY_LABEL));
         query.bindValue(Constants::CATEGORYLABEL_ID, QVariant());
         query.bindValue(Constants::CATEGORYLABEL_LABEL_ID, labelId);
@@ -599,44 +637,51 @@ bool CategoryBase::saveCategoryLabels(CategoryItem *category)
             LOG_QUERY_ERROR(query);
             return false;
         }
-        query.finish();
+//        Utils::Log::logTimeElapsed(chr, "--- label", "create LabelLID:" + QString::number(labelId));
     } else {
         labelId = category->data(CategoryItem::DbOnly_LabelId).toInt();
     }
+    query.finish();
 
     // delete all labels related to this LabelId
     QHash<int, QString> where;
     where.clear();
     where.insert(Constants::CATEGORYLABEL_LABEL_ID, QString("=%1").arg(labelId));
-    QSqlQuery query(database());
     // TODO: improve this, no need to delete before adding
-    if (!category->allLanguagesForLabel().isEmpty())
+    if (deleteOldValues) {
         query.exec(prepareDeleteQuery(Constants::Table_CATEGORY_LABEL, where));
+        query.finish();
+//        Utils::Log::logTimeElapsed(chr, "--- label", "delete old values");
+    }
+
     // save labels
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     foreach(const QString &lang, category->allLanguagesForLabel()) {
-        where.clear();
-        where.insert(Constants::CATEGORYLABEL_LABEL_ID, QString("=%1").arg(labelId));
-        where.insert(Constants::CATEGORYLABEL_LANG, QString("='%1'").arg(lang));
-        bool update = count(Constants::Table_CATEGORY_LABEL, Constants::CATEGORYLABEL_ID, getWhereClause(Constants::Table_CATEGORY_LABEL, where));
-        if (update) {
-            query.prepare(prepareUpdateQuery(Constants::Table_CATEGORY_LABEL, QList<int>()
-                                             << Constants::CATEGORYLABEL_VALUE
-                                             << Constants::CATEGORYLABEL_ISVALID, where));
-            query.bindValue(0, category->label(lang));
-            query.bindValue(1, 1);
-        } else {
+//        where.clear();
+//        where.insert(Constants::CATEGORYLABEL_LABEL_ID, QString("=%1").arg(labelId));
+//        where.insert(Constants::CATEGORYLABEL_LANG, QString("='%1'").arg(lang));
+//        bool update = count(Constants::Table_CATEGORY_LABEL, Constants::CATEGORYLABEL_ID, getWhereClause(Constants::Table_CATEGORY_LABEL, where));
+//        if (update) {
+//            query.prepare(prepareUpdateQuery(Constants::Table_CATEGORY_LABEL, QList<int>()
+//                                             << Constants::CATEGORYLABEL_VALUE
+//                                             << Constants::CATEGORYLABEL_ISVALID, where));
+//            query.bindValue(0, category->label(lang));
+//            query.bindValue(1, 1);
+//        } else {
             query.prepare(prepareInsertQuery(Constants::Table_CATEGORY_LABEL));
             query.bindValue(Constants::CATEGORYLABEL_ID, QVariant());
             query.bindValue(Constants::CATEGORYLABEL_LABEL_ID, labelId);
             query.bindValue(Constants::CATEGORYLABEL_LANG, lang);
             query.bindValue(Constants::CATEGORYLABEL_VALUE, category->label(lang));
             query.bindValue(Constants::CATEGORYLABEL_ISVALID, 1);
-        }
+//        }
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
             return false;
         }
         query.finish();
+//        Utils::Log::logTimeElapsed(chr, "--- label", "saved one label");
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     }
     return true;
 }
