@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #ifndef CORE_CONSTANTS_MENUS_H
@@ -43,6 +43,7 @@ const char * const  MENUBAR              = "menuBar";           /*!< \brief Defa
 const char * const  M_GENERAL            = "menuGeneral";          /*!< \brief Default menu File object name  \ingroup constants_actionmanager */
 const char * const  M_GENERAL_NEW        = "menuGeneral.New";
 const char * const  M_FILE               = "menuFile";          /*!< \brief Default menu File object name  \ingroup constants_actionmanager */
+const char * const  M_FILE_NEW           = "menuFile.New";          /*!< \brief Default menu File object name  \ingroup constants_actionmanager */
 const char * const  M_FILE_RECENTFILES   = "menuFile.recents";  /*!< \brief Default menu File object name  \ingroup constants_actionmanager */
 const char * const  M_TEMPLATES          = "menuTemplates";          /*!< \brief Default menu Edit object name  \ingroup constants_actionmanager */
 const char * const  M_EDIT               = "mEdit";          /*!< \brief Default menu Edit object name  \ingroup constants_actionmanager */
@@ -147,9 +148,11 @@ const char * const G_FORMAT_OTHER        = "grFormat.Other";      /*!< \brief De
 // Plugins groups
 const char * const G_PLUGINS              =   "grPlugins";
 const char * const G_PLUGINS_USERMANAGER  =   "grPlugins.usermanager";
+const char * const G_PLUGINS_FORM         =   "grPlugins.forms";
 const char * const G_PLUGINS_DRUGS        =   "grPlugins.drugs";
 const char * const G_PLUGINS_ACCOUNT      =   "grPlugins.account";
 const char * const G_PLUGINS_CALENDAR     =   "grPlugins.calendar";
+const char * const G_PLUGINS_PADTOOLS     =   "grPlugins.padTools";
 const char * const G_PLUGINS_OTHERS       =   "grPlugins.others";
 
 // Interpretor groups
@@ -198,6 +201,7 @@ const char * const  A_FILE_PRINT             = "actionFilePrint";  /*!< \brief D
 const char * const  A_FILE_PRINTFORM         = "actionFilePrintForm";  /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FILE_PRINTPRESCRIPTION = "actionFilePrintPrescription";  /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FILE_PRINTPREVIEW      = "actionFilePrintPreview";  /*!< \brief Default action object name  \ingroup constants_actionmanager */
+const char * const  A_FILE_LOCKAPPLICATION   = "actionFileLockApplication";   /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FILE_EXIT              = "actionFileExit";   /*!< \brief Default action object name  \ingroup constants_actionmanager */
 
 // Patient actions
@@ -246,6 +250,7 @@ const char * const  A_FORMAT_UNDERLINE    = "actionFormatUnderline";  /*!< \brie
 const char * const  A_FORMAT_STRIKE       = "actionFormatStrike";     /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FORMAT_BIGGER       = "actionFormatBigger";     /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FORMAT_SMALLER      = "actionFormatSmaller";    /*!< \brief Default action object name  \ingroup constants_actionmanager */
+const char * const  A_FORMAT_TYPEWRITERFONT = "actionFormatTypeWriterFont";    /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FORMAT_FONT         = "actionFormatFont";       /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FORMAT_FONTCOLOR    = "actionFormatFontColor";  /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_FORMAT_TABLE        = "actionFormatTable";      /*!< \brief Default action object name  \ingroup constants_actionmanager */
@@ -284,6 +289,7 @@ const char * const  A_ABOUTQT            = "actionAboutQt";           /*!< \brie
 const char * const  A_ABOUTFORM          = "actionAboutThisForm";     /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_APPLICATIONHELP    = "actionApplicationHelp";   /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_DEBUGHELPER        = "actionDebugHelper";       /*!< \brief Default action object name  \ingroup constants_actionmanager */
+const char * const  A_BUGREPORT          = "actionBugReport";         /*!< \brief Default action object name  \ingroup constants_actionmanager */
 const char * const  A_HELPTEXT_TOGGLER   = "actionHelpTextToggler";   /*!< \brief Default action object name  \ingroup constants_actionmanager */
 
 // Configuration Menu Action's name
@@ -306,17 +312,32 @@ const char * const MODE_ACCOUNT           = "account";
 const char * const MODE_AGENDA            = "agenda";
 const char * const MODE_USERMANAGER       = "usermanager";
 
-const int          P_MODE_PATIENT_SEARCH  = 100;
-const int          P_MODE_AGENDA          = 90;
-const int          P_MODE_PATIENT_HISTORY = 80;
-const int          P_MODE_PATIENT_DRUGS   = 60;
-const int          P_MODE_PATIENT_FILE    = 40;
-const int          P_MODE_USERMANAGER     = 30;
-const int          P_MODE_ACCOUNT         = 20;
+// Mode priorities
+const int          P_MODE_PATIENT_SEARCH  = 1000;
+const int          P_MODE_AGENDA          = 900;
+const int          P_MODE_PATIENT_HISTORY = 800;
+const int          P_MODE_PATIENT_DRUGS   = 600;
+const int          P_MODE_PATIENT_FILE    = 400;
+const int          P_MODE_USERMANAGER     = 300;
+const int          P_MODE_TOOLS           = 200;
+const int          P_MODE_ACCOUNT         = 100;
+
+// Options page sortIndex (per categories)
+const int          OPTIONINDEX_MAIN         = 0;
+const int          OPTIONINDEX_PRINT        = 1000;
+const int          OPTIONINDEX_AGENDA       = 2000;
+const int          OPTIONINDEX_PATIENTS     = 3000;
+const int          OPTIONINDEX_ALERTS       = 4000;
+const int          OPTIONINDEX_PMHX         = 5000;
+const int          OPTIONINDEX_FORMS        = 6000;
+const int          OPTIONINDEX_DRUGS        = 7000;
+const int          OPTIONINDEX_TEMPLATES    = 8000;
+const int          OPTIONINDEX_WEBCAM       = 9000;
+const int          OPTIONINDEX_ACCOUNT      = 10000;
+const int          OPTIONINDEX_TOOLS        = 11000;
 
 // Contexts
 const char* const   C_GLOBAL             = "context.global";
-const int           C_GLOBAL_ID          = 0;
 const char* const   C_EDITOR_BASIC       = "context.Editor";
 const char* const   C_EDITOR_CHAR_FORMAT = "context.Editor.char";
 const char* const   C_EDITOR_PARAGRAPH   = "context.Editor.paragraph";
@@ -337,7 +358,8 @@ const char * const G_EDITOR_ADDTEXT      = "group.Editor.AddText";
 const char * const A_EDITOR_FILEOPEN     = "action.Editor.Open";
 const char * const A_EDITOR_FILESAVE     = "action.Editor.Save";
 const char * const A_EDITOR_FILEPRINT    = "action.Editor.Print";
-const char * const A_EDITOR_ADDDATE      = "action.Editor.AddDate";
+const char * const A_EDITOR_ADDDATE_LONG   = "action.Editor.AddDate.LongFormat";
+const char * const A_EDITOR_ADDDATE_SHORT  = "action.Editor.AddDate.ShortFormat";
 const char * const A_EDITOR_ADDUSERNAME    = "action.Editor.AddUserName";
 const char * const A_EDITOR_ADDPATIENTNAME = "action.Editor.AddPatientName";
 
