@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -267,7 +267,7 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QRegExp>
-
+#include <QDesktopWidget>
 
 /**
   \todo Manage user's settings stored into database
@@ -278,11 +278,6 @@ namespace {
     const char* const WEBSITE              = "http://www.freemedforms.org/";
 
     // BUNDLE RESOURCES  --> located inside the bundle. Location calculated from BundleRootPath
-#ifdef DEBUG_WITHOUT_INSTALL
-    const char* const BUNDLERESOURCE_PATH   = "";                    // resources are located into global_resources paths
-#else
-    const char* const BUNDLERESOURCE_PATH  = "/Resources";          // resources are located inside the bundle
-#endif
     const char* const READONLYRESOURCES    = "";
     const char* const READONLYDATABASE     = "/databases";
     const char* const TRANSLATIONS_PATH    = "/translations";
@@ -291,6 +286,7 @@ namespace {
     const char* const DEFAULTDATAPACK_APPINSTALLED               = "/datapacks/appinstalled";
     const char* const DEFAULTDATAPACK_APPINSTALLED_SUBFORMS      = "/datapacks/appinstalled/forms/subforms";
     const char* const DEFAULTDATAPACK_APPINSTALLED_COMPLETEFORMS = "/datapacks/appinstalled/forms/completeforms";
+    const char* const DEFAULT_BUNDLED_ALERTPACKS = "/alertpacks";
     const char* const DEFAULTTHEME_PATH    = "";
     const char* const DEFAULTTHEME_PIXMAP  = "/pixmap";
     const char* const DEFAULTTHEME_SPLASH  = "/pixmap/splashscreens";
@@ -301,11 +297,6 @@ namespace {
     const char* const USER_COMPLETEFORMSPATH  = "/forms/completeforms";
 
     // APPLICATIONS RESOURCES --> located next to the application binary
-#ifdef DEBUG_WITHOUT_INSTALL
-    const char* const MAC_PLUGINSPATH      = "/../../../plugins";
-#else
-    const char* const MAC_PLUGINSPATH      = "/../plugins";
-#endif
     const char* const NONMAC_PLUGINSPATH   = "/plugins";
     const char* const ALL_QTPLUGINSPATH    = "/qt";
     const char* const MAC_QTFRAMEWORKPATH  = "/../FrameWorks";
@@ -363,12 +354,16 @@ SettingsPrivate::SettingsPrivate(QObject *parent, const QString &appName, const
     Q_ASSERT(!applicationName.isEmpty());
 
     setPath(ApplicationPath, qApp->applicationDirPath());
-    setPath(ApplicationTempPath, QDir::tempPath());
-    setPath(SystemTempPath, QDir::tempPath());
+    setPath(ApplicationTempPath, QString("%1/%2-%3").arg(QDir::tempPath()).arg(qApp->applicationName()).arg(Utils::createUid()));
+    setPath(SystemTempPath, path(ApplicationTempPath));
     setPath(WebSiteUrl, WEBSITE);
     setPath(UserDocumentsPath, QDir::homePath() + QDir::separator() + applicationName.remove("-alpha").remove("_alpha").remove("_debug").remove("-debug") + QDir::separator() + "Documents");
 
-//    // Create UserDocumentsPath
+    // Create paths
+    if (!QDir(path(ApplicationTempPath)).exists()) {
+        QDir().mkpath(path(ApplicationTempPath));
+    }
+    //    // Create UserDocumentsPath
 //    if (!QDir(path(UserDocumentsPath)).exists()) {
 //        QDir().mkpath(path(UserDocumentsPath));
 //    }
@@ -377,6 +372,7 @@ SettingsPrivate::SettingsPrivate(QObject *parent, const QString &appName, const
 //        setPath(FMFPluginsPath, LIBRARY_BASENAME);
 
     if (Utils::isDebugWithoutInstallCompilation()) {
+        LOG("Compiled with the debug_and_release configuration");
         // DEBUG WITHOUT INSTALL BUILD
         QString res;
         if (Utils::isRunningOnMac())
@@ -395,15 +391,18 @@ SettingsPrivate::SettingsPrivate(QObject *parent, const QString &appName, const
         }
     } else {
         // RELEASE OR DEBUG INSTALLED BUILD
-#ifdef LINUX_INTEGRATED
-        setPath(BundleResourcesPath, QString("/usr/share/freemedforms"));
-#else
-        if (Utils::isRunningOnMac()) {
-            setPath(BundleResourcesPath, qApp->applicationDirPath() + "/../" + QString(BUNDLERESOURCE_PATH));
+        if (Utils::isLinuxIntegratedCompilation()) {
+            setPath(BundleResourcesPath, QString("/usr/share/freemedforms"));
         } else {
-            setPath(BundleResourcesPath, qApp->applicationDirPath() + QDir::separator() + QString(BUNDLERESOURCE_PATH));
+            QString bundleResourcesPath;
+            if (!Utils::isDebugWithoutInstallCompilation())
+                bundleResourcesPath = "Resources";  // resources are located into global_resources paths
+
+            if (Utils::isRunningOnMac())
+                setPath(BundleResourcesPath, qApp->applicationDirPath() + "/../" + bundleResourcesPath);
+            else
+                setPath(BundleResourcesPath, qApp->applicationDirPath() + QDir::separator() + bundleResourcesPath);
         }
-#endif
         m_FirstTime = value("FirstTimeRunning", true).toBool();
         setPath(UserResourcesPath, QFileInfo(file).absolutePath());//QDir::homePath() + "/." + applicationName);//resourcesPath);
     }
@@ -428,6 +427,10 @@ SettingsPrivate::~SettingsPrivate()
         delete m_UserSettings;
         m_UserSettings = 0;
     }
+    QString error;
+    Utils::removeDirRecursively(path(ApplicationTempPath), &error);
+    if (!error.isEmpty())
+        LOG_ERROR(QString("Unable to remove application temporary path: %1; %2").arg(path(ApplicationTempPath)).arg(error));
 }
 
 void SettingsPrivate::setUserSettings(const QString &content)
@@ -441,7 +444,8 @@ void SettingsPrivate::setUserSettings(const QString &content)
     }
 
     // populate the file with the content
-    Utils::saveStringToFile(content, fileName, Utils::Overwrite, Utils::DontWarnUser);
+    if (!Utils::saveStringToFile(content, fileName, Utils::Overwrite, Utils::DontWarnUser))
+        LOG_ERROR("Unable to save user preferences content");
 
     // change the fileName of the current QSettings
     if (m_UserSettings) {
@@ -559,11 +563,13 @@ void SettingsPrivate::setPath(const int type, const QString & absPath)
             m_Enum_Path.insert(MediumPixmapPath, bundlePath + DEFAULTTHEME_PIXMAP + "/32x32/");
             m_Enum_Path.insert(BigPixmapPath, bundlePath + DEFAULTTHEME_PIXMAP + "/64x64/");
             m_Enum_Path.insert(SvgPixmapPath, bundlePath + DEFAULTTHEME_PIXMAP + "/svg/");
+            m_Enum_Path.insert(SplashScreenPixmapPath, bundlePath + DEFAULTTHEME_PIXMAP + "/splashscreens/");
             m_Enum_Path.insert(SubFormsPath, bundlePath + DEFAULTFORMS + "/subforms");
             m_Enum_Path.insert(CompleteFormsPath, bundlePath + DEFAULTFORMS + "/completeforms");
             m_Enum_Path.insert(DataPackApplicationPath, bundlePath + DEFAULTDATAPACK_APPINSTALLED);
             m_Enum_Path.insert(DataPackCompleteFormsPath, bundlePath + DEFAULTDATAPACK_APPINSTALLED_COMPLETEFORMS);
             m_Enum_Path.insert(DataPackSubFormsPath, bundlePath + DEFAULTDATAPACK_APPINSTALLED_SUBFORMS);
+            m_Enum_Path.insert(BundledAlertPacks, bundlePath + DEFAULT_BUNDLED_ALERTPACKS);
             QString appname = qApp->applicationName().toLower();
             if (qApp->applicationName().contains(" ")) {
                 appname = appname.left(appname.indexOf(" "));
@@ -572,15 +578,15 @@ void SettingsPrivate::setPath(const int type, const QString & absPath)
                 if (appname.contains("_d"))
                     appname = appname.left(appname.indexOf("_d"));
             }
-#ifdef LINUX_INTEGRATED
-            if (QDir(QString(LINUX_USERMANUAL_PATH).arg(appname)).exists()) {
-                m_Enum_Path.insert(DocumentationPath, QString(LINUX_USERMANUAL_PATH).arg(appname));
+            if (Utils::isLinuxIntegratedCompilation()) {
+                if (QDir(QString(LINUX_USERMANUAL_PATH).arg(appname)).exists()) {
+                    m_Enum_Path.insert(DocumentationPath, QString(LINUX_USERMANUAL_PATH).arg(appname));
+                } else {
+                    m_Enum_Path.insert(DocumentationPath, bundlePath + QString(USERMANUAL_PATH).arg(appname));
+                }
             } else {
                 m_Enum_Path.insert(DocumentationPath, bundlePath + QString(USERMANUAL_PATH).arg(appname));
             }
-#else
-            m_Enum_Path.insert(DocumentationPath, bundlePath + QString(USERMANUAL_PATH).arg(appname));
-#endif
             break;
         }
         case ApplicationPath :
@@ -589,9 +595,14 @@ void SettingsPrivate::setPath(const int type, const QString & absPath)
                 break;
             m_Enum_Path.insert(ApplicationPath, absPath);
             if (Utils::isRunningOnMac()) {
+                QString pluginPath;
+                if (Utils::isDebugWithoutInstallCompilation())
+                    pluginPath = "/../../../plugins";
+                else
+                    pluginPath = "/../plugins";
                 m_Enum_Path.insert(QtFrameWorksPath, QDir::cleanPath(absPath + MAC_QTFRAMEWORKPATH));
-                m_Enum_Path.insert(FMFPlugInsPath, QDir::cleanPath(absPath + MAC_PLUGINSPATH));
-                m_Enum_Path.insert(QtPlugInsPath, QDir::cleanPath(absPath + MAC_PLUGINSPATH + ALL_QTPLUGINSPATH));
+                m_Enum_Path.insert(FMFPlugInsPath, QDir::cleanPath(absPath + pluginPath));
+                m_Enum_Path.insert(QtPlugInsPath, QDir::cleanPath(absPath + pluginPath + ALL_QTPLUGINSPATH));
                 m_Enum_Path.insert(BundleRootPath, QDir::cleanPath(absPath + MAC_TOBUNDLEROOTPATH));
             } else if (Utils::isRunningOnLinux()) {
 #ifndef LINUX_INTEGRATED
@@ -751,20 +762,19 @@ QString SettingsPrivate::getIniFile(const QString & appName, const QString & fil
             tmpAppName = tmpAppName.left(tmpAppName.indexOf(" "));
     }
     tmpAppName.prepend(".");
-    if (Utils::isAlpha()) {
+    if (Utils::isAlpha())
         tmpAppName.append("_alpha");
-    }
 
     if (fileName.isEmpty())
         tmpFileName = "config.ini";
 
     // if QApplication args contains "--config=iniFileName.ini" use it if possible
     // retreive command line arguments
-    // TODO: get the command line args through the Core::ICommandLine
+    // TODO: get the command line args through the Core::ICommandLine?
     QStringList list = QCoreApplication::arguments();
     int index = list.indexOf(QRegExp("--config=*", Qt::CaseSensitive, QRegExp::Wildcard));
-
     if (index != -1) {
+        // Config file passed
         QString iniFileName = list[index];
         iniFileName = iniFileName.mid(iniFileName.indexOf("=") + 1);
         LOG_FOR("Settings", tr("Passing command line ini file: %1").arg(iniFileName));
@@ -778,71 +788,80 @@ QString SettingsPrivate::getIniFile(const QString & appName, const QString & fil
             if (info.isWritable()) {
                 LOG_FOR("Settings", tr("Using ini file %1.").arg(iniFileName));
                 return iniFileName;
-            }
-            else
+            } else {
                 LOG_ERROR_FOR("Settings", tr("Ini file %1 is not writable. Can not use it.").arg(iniFileName));
+            }
         } else {
             // can we create and access to ini file ?
             QFile file(iniFileName);
             if (file.open(QIODevice::ReadWrite | QIODevice::Text)) {
                 LOG_FOR("Settings", tr("Using ini file %1").arg(iniFileName));
                 return iniFileName;
-            }
-            else
+            } else {
                 LOG_FOR("Settings", tr("WARNING: Ini file %1 can not be used.").arg(iniFileName));
+            }
         }
     }
 
-    // try to use read the 'pathtoconfig.ini' inside the bundle
+    // No config file was passed in the command line
+    // Or the param config file does not exists
+    // -> try to use read the 'pathtoconfig.ini' inside the bundle
     if (QFile(QString("%1/pathtoconfig.ini").arg(qApp->applicationDirPath())).exists()) {
         QString content = Utils::readTextFile(QString("%1/pathtoconfig.ini").arg(qApp->applicationDirPath()), Utils::DontWarnUser);
         content = content.trimmed();
         if (!content.isEmpty()) {
+            LOG_FOR("Settings", tr("Found a configuration file next to the binary. Reading file."));
+            if (content.startsWith("~/"))
+                content.replace("~/", QDir::homePath() + "/");
             if (QDir::isRelativePath(content))
                 content.prepend(qApp->applicationDirPath() + QDir::separator());
             content = QDir::cleanPath(content);
-
             QFileInfo info(content);
             if (info.exists() && info.isReadable()) {
                 if (info.isWritable()) {
                     LOG_FOR("Settings", tr("Using ini file %1.").arg(content));
                     return content;
-                }
-                else
+                } else {
                     LOG_ERROR_FOR("Settings", tr("Ini file %1 is not writable. Can not use it.").arg(content));
+                }
             } else {
                 // can we create and access to ini file ?
+                if (!info.absoluteDir().exists()) {
+                    if (!QDir().mkpath(info.absolutePath()))
+                        LOG_ERROR_FOR("Settings", tr("File does not exists and can not be created: %1").arg(content));
+                }
                 QFile file(content);
                 if (file.open(QIODevice::ReadWrite | QIODevice::Text)) {
                     LOG_FOR("Settings", tr("Using ini file %1").arg(content));
                     return content;
-                }
-                else
+                } else {
                     LOG_FOR("Settings", tr("WARNING: Ini file %1 can not be used.").arg(content));
+                }
             }
+            LOG_ERROR_FOR("Settings", tr("File does not exists and can not be created: %1").arg(content));
         }
     }
 
     QString iniFile;
-//    if (Utils::isRunningOnMac())
-//        iniFile = QDir::cleanPath(qApp->applicationDirPath() + "/../Resources") + QDir::separator() + tmpFileName;
-//    else
-//        iniFile = qApp->applicationDirPath() + QDir::separator() + tmpFileName;
-//
-//    // test iniFile
-//    Log::addMessage("Settings", tr("Trying ini file %1").arg(iniFile));
-//    QFile file(iniFile);
-//    QFileInfo info(iniFile);
-//    if (info.exists() && info.isReadable() && info.isWritable()) {
-//        Log::addMessage("Settings", tr("Using ini file %1").arg(iniFile));
-//        return iniFile;
-//    } else {
-//        if ((! info.exists()) &&  file.open(QIODevice::ReadWrite | QIODevice::Text)) {
-//            Log::addMessage("Settings", tr("Using ini file %1").arg(iniFile));
-//            return iniFile;
-//        }
-//        else Log::addError("Settings", tr("Ini file %1 can not be used.").arg(iniFile));
-//    }
+    //    if (Utils::isRunningOnMac())
+    //        iniFile = QDir::cleanPath(qApp->applicationDirPath() + "/../Resources") + QDir::separator() + tmpFileName;
+    //    else
+    //        iniFile = qApp->applicationDirPath() + QDir::separator() + tmpFileName;
+    //
+    //    // test iniFile
+    //    Log::addMessage("Settings", tr("Trying ini file %1").arg(iniFile));
+    //    QFile file(iniFile);
+    //    QFileInfo info(iniFile);
+    //    if (info.exists() && info.isReadable() && info.isWritable()) {
+    //        Log::addMessage("Settings", tr("Using ini file %1").arg(iniFile));
+    //        return iniFile;
+    //    } else {
+    //        if ((! info.exists()) &&  file.open(QIODevice::ReadWrite | QIODevice::Text)) {
+    //            Log::addMessage("Settings", tr("Using ini file %1").arg(iniFile));
+    //            return iniFile;
+    //        }
+    //        else Log::addError("Settings", tr("Ini file %1 can not be used.").arg(iniFile));
+    //    }
 
     // Now use the $HOME path
     iniFile = QString("%1/%2/%3").arg(QDir::homePath(), tmpAppName, tmpFileName);
@@ -865,7 +884,6 @@ QString SettingsPrivate::getIniFile(const QString & appName, const QString & fil
   \fn void Core::ISettings::restoreState(QMainWindow * window, const QString & prefix)
   \brief Main windows restore state. \e prefix can be used if you store multiple main window in the same settings
 */
-#include <QDesktopWidget>
 void SettingsPrivate::restoreState(QMainWindow *window, const QString & prefix)
 {
     if (!window)
@@ -942,7 +960,7 @@ void SettingsPrivate::appendToValue(const QString &key, const QString &value)
 /**
   \fn QTreeWidget* Core::ISettings::getTreeWidget(QWidget *parent) const
   \brief For debugging purpose.
-  If \e parent is a QTreeWidget, it will be populated with datas and returned. Otherwise a
+  If \e parent is a QTreeWidget, it will be populated with data and returned. Otherwise a
   QTreeWidget is created with \e parent as parent and returned.
 */
 QTreeWidget* SettingsPrivate::getTreeWidget(QWidget *parent) const
@@ -967,22 +985,22 @@ QTreeWidget* SettingsPrivate::getTreeWidget(QWidget *parent) const
 #endif
 
     // add compilation information
-    QTreeWidgetItem * compilItem = new QTreeWidgetItem(tree, QStringList() << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_INFORMATION));
-    compilItem->setFont(0,bold);
-    new QTreeWidgetItem(compilItem, QStringList() << tr("Compilation Date") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DATE_TIME_1_2).arg(__DATE__, __TIME__));
-    new QTreeWidgetItem(compilItem, QStringList() << tr("Compile Qt version") << QString("%1").arg(QT_VERSION_STR));
-    new QTreeWidgetItem(compilItem, QStringList() << tr("Actual Qt version") << QString("%1").arg(qVersion()));
-    new QTreeWidgetItem(compilItem, QStringList() << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_VERSION_1).arg("") << qApp->applicationVersion());
+    QTreeWidgetItem * compileItem = new QTreeWidgetItem(tree, QStringList() << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_INFORMATION));
+    compileItem->setFont(0,bold);
+    new QTreeWidgetItem(compileItem, QStringList() << tr("Compilation Date") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DATE_TIME_1_2).arg(__DATE__, __TIME__));
+    new QTreeWidgetItem(compileItem, QStringList() << tr("Compile Qt version") << QString("%1").arg(QT_VERSION_STR));
+    new QTreeWidgetItem(compileItem, QStringList() << tr("Actual Qt version") << QString("%1").arg(qVersion()));
+    new QTreeWidgetItem(compileItem, QStringList() << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_VERSION_1).arg("") << qApp->applicationVersion());
     if (Utils::isReleaseCompilation()) {
-        new QTreeWidgetItem(compilItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_RELEASE));
+        new QTreeWidgetItem(compileItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_RELEASE));
     } else {
         if (Utils::isDebugWithoutInstallCompilation()) {
-            new QTreeWidgetItem(compilItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DEBUG) + " (no install)");
+            new QTreeWidgetItem(compileItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DEBUG) + " - no install");
         } else {
-            new QTreeWidgetItem(compilItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DEBUG));
+            new QTreeWidgetItem(compileItem, QStringList() << tr("Compile mode") << Trans::ConstantTranslations::tkTr(Trans::Constants::BUILD_DEBUG));
         }
     }
-    new QTreeWidgetItem(compilItem, QStringList() << tr("GIT revision") << QString(GIT_REVISION_HASH));
+    new QTreeWidgetItem(compileItem, QStringList() << tr("GIT revision") << QString(GIT_REVISION_HASH));
 
 
     // add paths
