@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -33,6 +33,7 @@
 #include <drugsbaseplugin/druginteractionresult.h>
 #include <drugsbaseplugin/idruginteractionalert.h>
 #include <drugsbaseplugin/idrug.h>
+#include <drugsbaseplugin/drugsdatabaseselector.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/itheme.h>
@@ -101,6 +102,36 @@ namespace  {
                 "<tr><td>%2</td></tr>\n"
                 "</table>\n";
 
+static inline DrugDrugInteractionEngine::TypesOfIAM getLevels(const QString &t)
+{
+    DrugDrugInteractionEngine::TypesOfIAM r = DrugDrugInteractionEngine::NoIAM;
+    if (t.contains("A"))
+        r |= DrugDrugInteractionEngine::DrugDuplication;
+    if (t.contains("U"))
+        r |= DrugDrugInteractionEngine::InnDuplication;
+    if (t.contains("Z"))
+        r |= DrugDrugInteractionEngine::ClassDuplication;
+    if (t.contains("P"))
+        r |= DrugDrugInteractionEngine::Precaution;
+    if (t.contains("C"))
+        r |= DrugDrugInteractionEngine::ContreIndication;
+    if (t.contains("D"))
+        r |= DrugDrugInteractionEngine::Deconseille;
+    if (t.contains("T"))
+        r |= DrugDrugInteractionEngine::APrendreEnCompte;
+    if (t.contains("450"))
+        r |= DrugDrugInteractionEngine::P450;
+    if (t.contains("I"))
+        r |= DrugDrugInteractionEngine::Information;
+    if (t.contains("Y"))
+        r |= DrugDrugInteractionEngine::GPG;
+
+    if (r == DrugDrugInteractionEngine::NoIAM)
+        qWarning() << "DRUG DRUG INTERACTION LEVEL NOT RECOGNIZED" << t;
+
+    return r;
+}
+
 class DrugsInteraction : public IDrugInteraction
 {
 public:
@@ -133,27 +164,7 @@ public:
     {
         if (ref == DI_TypeId) {
             QString t = value.toString();
-            DrugDrugInteractionEngine::TypesOfIAM r = DrugDrugInteractionEngine::NoIAM;
-            if (t.contains("A"))
-                r |= DrugDrugInteractionEngine::DrugDuplication;
-            if (t.contains("U"))
-                r |= DrugDrugInteractionEngine::InnDuplication;
-            if (t.contains("Z"))
-                r |= DrugDrugInteractionEngine::ClassDuplication;
-            if (t.contains("P"))
-                r |= DrugDrugInteractionEngine::Precaution;
-            if (t.contains("C"))
-                r |= DrugDrugInteractionEngine::ContreIndication;
-            if (t.contains("D"))
-                r |= DrugDrugInteractionEngine::Deconseille;
-            if (t.contains("T"))
-                r |= DrugDrugInteractionEngine::APrendreEnCompte;
-            if (t.contains("450"))
-                r |= DrugDrugInteractionEngine::P450;
-            if (t.contains("I"))
-                r |= DrugDrugInteractionEngine::Information;
-            if (t.contains("Y"))
-                r |= DrugDrugInteractionEngine::GPG;
+            DrugDrugInteractionEngine::TypesOfIAM r = getLevels(t);
             m_Infos.insert(ref, int(r));
         } else {
             m_Infos.insert(ref, value);
@@ -381,6 +392,12 @@ public:
               tmp << tkTr(Trans::Constants::CLASS_DUPLICATION);
          if (r & DrugDrugInteractionEngine::Information)
               tmp << tkTr(Trans::Constants::INFORMATION);
+         if (r & DrugDrugInteractionEngine::Unknown)
+              tmp << tkTr(Trans::Constants::UNKNOWN);
+         if (tmp.isEmpty()) {
+             tmp << tkTr(Trans::Constants::NOT_DEFINED);
+             qWarning() << "DRUG DRUG INTERACTION LEVEL NOT RECOGNIZED" << t;
+         }
          return tmp.join(", ");
     }
 
@@ -826,7 +843,7 @@ class DrugDrugInteractionEnginePrivate
 public:
     QVector<IDrug *> m_TestedDrugs;
     QVector<IDrugInteraction *> m_FirstPassInteractions, m_Interactions;
-    QMap<int, int> m_InteractionsIDs;        /*!<  All possible interactions based on ATC IDs*/
+    QMultiMap<int, int> m_InteractionsIDs;        /*!<  All possible interactions based on ATC IDs*/
     QMultiMap<int, int> m_DDIFound;               /*!< modified by checkDrugInteraction() */
     QVector<int> m_DoNotWarnAtcDuplicates;
     bool m_LogChrono;
@@ -837,10 +854,9 @@ public:
 DrugDrugInteractionEngine::DrugDrugInteractionEngine(QObject *parent) :
         IDrugEngine(parent), d(new DrugDrugInteractionEnginePrivate)
 {
-    setObjectName("DrugDrugInteractionEngine");
+    setObjectName("DDIEngine");
     m_IsActive = settings()->value(Constants::S_ACTIVATED_INTERACTION_ENGINES).toStringList().contains(Constants::DDI_ENGINE_UID);
     d->m_LogChrono = false;
-    connect(&drugsBase(), SIGNAL(drugsBaseHasChanged()), this, SLOT(drugsBaseChanged()));
 }
 
 DrugDrugInteractionEngine::~DrugDrugInteractionEngine()
@@ -852,6 +868,7 @@ DrugDrugInteractionEngine::~DrugDrugInteractionEngine()
 
 void DrugDrugInteractionEngine::drugsBaseChanged()
 {
+    qWarning() << "DrugDrugInteractionEngine::drugsBaseChanged()";
     init();
 }
 
@@ -859,6 +876,7 @@ bool DrugDrugInteractionEngine::init()
 {
     // get all interactions ids and mol <-> atc links
     d->m_InteractionsIDs.clear();
+    d->m_DDIFound.clear();
     QList<int> fields;
     fields << Constants::INTERACTIONS_ATC_ID1 << Constants::INTERACTIONS_ATC_ID2;
     QString req = drugsBase().select(Constants::Table_INTERACTIONS, fields);
@@ -893,6 +911,8 @@ bool DrugDrugInteractionEngine::init()
         LOG_QUERY_ERROR(query);
     }
     db.commit();
+
+    connect(&drugsBase(), SIGNAL(drugsBaseHasChanged()), this, SLOT(drugsBaseChanged()), Qt::UniqueConnection);
     return true;
 }
 
@@ -1243,6 +1263,152 @@ QVector<IDrugInteractionAlert *> DrugDrugInteractionEngine::getAllAlerts(DrugInt
     return alerts;
 }
 
+QString DrugDrugInteractionEngine::engineDataReport() const
+{
+    return getDrugDrugInteractionLevelStatistics();
+}
+
+/**
+ * Check the content of the database. Extract all available drug-drug interactions (DDI)
+ * and compute their level repartition. The returned hash as for key the Readable Level
+ * of interaction and for value the percent of this DDI level.
+ */
+QString DrugDrugInteractionEngine::getDrugDrugInteractionLevelStatistics() const
+{
+    QTime chrono;
+    chrono.start();
+    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_DRUGS_NAME);
+    if (!DB.isOpen()) {
+        if (!DB.open()) {
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(DB.connectionName()).arg(DB.lastError().text()));
+            return QString::null;
+        }
+    }
+    DB.transaction();
+    QSqlQuery query(DB);
+
+    int total = 0;
+    QMap<DrugDrugInteractionEngine::TypeOfIAM, int> levelStatistics;
+    QList<DrugDrugInteractionEngine::TypeOfIAM> recordedLevels;
+    recordedLevels << NoIAM
+            << Unknown
+            << Information
+            << DrugDuplication
+            << InnDuplication
+            << ClassDuplication
+            << Precaution
+            << APrendreEnCompte
+            << P450
+            << GPG
+            << Deconseille
+            << ContreIndication;
+
+    // Get all DDI from database
+    using namespace DrugsDB::Constants;
+    Utils::Field get(Table_IAKNOWLEDGE, IAKNOWLEDGE_TYPE);
+    Utils::JoinList joins;
+    joins << Utils::Join(Table_INTERACTIONS, INTERACTIONS_IAID, Table_IA_IAK, INTERACTIONS_IAID);
+    joins << Utils::Join(Table_IA_IAK, IA_IAK_IAKID, Table_IAKNOWLEDGE, IAKNOWLEDGE_IAKID);
+    Utils::FieldList conds;
+    const QList<int> &ids = d->m_InteractionsIDs.uniqueKeys();
+    for(int i=0; i < ids.count(); ++i) {
+        int firstId = ids.at(i);
+        int firstCount = 1;
+        if (drugsBase().isInteractingClass(firstId))
+            firstCount = drugsBase().interactingClassSingleAtcCount(firstId);
+        const QList<int> &second = d->m_InteractionsIDs.values(firstId);
+        for(int j=0; j < second.count(); ++j) {
+            int secondId = second.at(j);
+            int n = 0; // n represents the total of computable interactions for the couple
+            int secondCount = 1;
+            if (drugsBase().isInteractingClass(secondId))
+                secondCount = drugsBase().interactingClassSingleAtcCount(secondId);
+            n = firstCount * secondCount;
+
+            // get levels of the ddi
+            conds.clear();
+            conds << Utils::Field(Table_INTERACTIONS, INTERACTIONS_ATC_ID1, QString("='%1'").arg(firstId));
+            conds << Utils::Field(Table_INTERACTIONS, INTERACTIONS_ATC_ID2, QString("='%1'").arg(secondId));
+            QString req = drugsBase().select(get, joins, conds);
+
+            QString level;
+            if (query.exec(req)) {
+                while (query.next()) {
+                    level += query.value(0).toString();
+                }
+            } else {
+                LOG_QUERY_ERROR(query);
+            }
+            query.finish();
+
+            // analyze levels (split)
+            DrugDrugInteractionEngine::TypesOfIAM r = getLevels(level);
+            foreach(DrugDrugInteractionEngine::TypeOfIAM type, recordedLevels) {
+                if (r.testFlag(type))
+                    levelStatistics.insert(type, levelStatistics.value(type, 0) + n);
+            }
+            total += n;
+        }
+    }
+
+    QString out = tr("Data extracted from database") + "\n";
+    QMapIterator<DrugDrugInteractionEngine::TypeOfIAM, int> it(levelStatistics);
+    it.toBack();
+    while (it.hasPrevious()) {
+        it.previous();
+        out += QString("%1 %2 %3%\n")
+                .arg(DrugsInteraction::typeToString(it.key()).leftJustified(50, '.'))
+                .arg(QString::number(it.value()).leftJustified(7))
+                .arg(QString::number(it.value()*100/total, 'G', 2));
+    }
+    out += QString("\n%1 %2 100%\n")
+            .arg(tr("Total DDI per INN").leftJustified(50, '.'))
+            .arg(QString::number(total).leftJustified(7));
+
+    get = Utils::Field(Table_MASTER, MASTER_DID);
+    Utils::Join j1(Table_COMPO, COMPO_DID, Table_MASTER, MASTER_DID);
+//    Utils::Join j2(Table_MOLS, MOLS_MID, Table_COMPO, COMPO_MID);
+    Utils::Join j3(Table_LK_MOL_ATC, LK_MID, Table_COMPO, COMPO_MID);
+    joins.clear();
+    joins << j1 << j3;
+
+    QHash<int, QString> where;
+    conds.clear();
+    if (drugsBase().actualDatabaseInformation()) {
+        conds << Utils::Field(Table_MASTER, MASTER_SID, QString("='%1'").arg(drugsBase().actualDatabaseInformation()->sid));
+        where.insert(MASTER_SID, QString("='%1'").arg(drugsBase().actualDatabaseInformation()->sid));
+    } else {
+        conds << Utils::Field(Table_MASTER, MASTER_SID, QString("='%1'").arg(1));
+        where.insert(MASTER_SID, QString("='%1'").arg(1));
+    }
+    conds << Utils::Field(Table_LK_MOL_ATC, LK_ATC_ID, QString("IS NOT NULL"));
+
+    int analyzableDrugs = drugsBase().countDistinct(Utils::FieldList() << get, joins, conds);
+    int totalDrugs = drugsBase().count(Table_MASTER, MASTER_DID, drugsBase().getWhereClause(Table_MASTER, where));
+
+    out += QString("\n%1 %2\n")
+            .arg(tr("Total number of drugs").leftJustified(50, '.'))
+            .arg(QString::number(totalDrugs).leftJustified(7));
+    out += QString("%1 %2 %3%\n")
+            .arg(tr("Analyzable drugs*").leftJustified(50, '.'))
+            .arg(QString::number(analyzableDrugs).leftJustified(7))
+            .arg(QString::number(analyzableDrugs*100/totalDrugs, 'G', 2));
+
+    out += "\n" + tr("Memory usage (partial data)") + "\n";
+    out += QString("%1 %2\n")
+            .arg(tr("Cached in memory (pairs of integers)").leftJustified(50, '.'))
+            .arg(QString::number(d->m_InteractionsIDs.count()).leftJustified(7));
+    out += QString("%1 %2\n")
+            .arg(tr("Cached in memory (unique first interactor)").leftJustified(50, '.'))
+            .arg(QString::number(d->m_InteractionsIDs.uniqueKeys().count()).leftJustified(7));
+
+    out += QString("\n%1 %2ms\n\n")
+            .arg(tr("Report generation time").leftJustified(50, '.'))
+            .arg(chrono.elapsed());
+    out += Utils::lineWrapString(tr("*: analyzable drugs have one or more of their components recognized by the engine. Warning: Some of these drugs can have one or more molecule not recognized by the engine."), 70);
+    return out;
+}
+
 void DrugDrugInteractionEngine::setActive(bool activate)
 {
     if (isActive()==activate)
