@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -29,15 +29,14 @@
 
 #include <drugsbaseplugin/constants_databaseschema.h>
 
-#include <QFlags>
 #include <QHash>
 #include <QStringList>
 
 /**
  * \file constants.h
  * \author Eric MAEKER <eric.maeker@gmail.com>
- * \version 0.6.0
- * \date 02 Mar 2011
+ * \version 0.8.6
+ * \date 12 May 2013
 */
 
 namespace DrugsDB {
@@ -64,9 +63,13 @@ namespace Constants {
     const char * const  S_WATERMARK_HTML         = "DrugsWidget/print/watermark/Html";         /*!< \brief Key for settings. */
 
     const char * const  S_PRINTDUPLICATAS        = "DrugsWidget/PrintDuplicatas";
+    const char * const  S_AUTOSORT_ON_PRINTING   = "DrugsWidget/PrintAutoSort";
+    const char * const  S_ADD_BIOMETRICS_ON_PRINTS = "DrugsWidget/PrintAddBiometrics";
 
     // DEFAULTS SETTINGS
     const char * const  S_DEF_WATEMARKHTML           = "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\np, li { white-space: pre-wrap; }\n</style></head><body>\n<p align=\"center\"  style=\" font-family:'Lucida Grande'; font-size:12pt; font-weight:bold; font-style:normal; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#9c9c9c;\">Duplicata ne permettant pas la d&eacute;livrance des m&eacute;dicaments</span></p>\n<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#9c9c9c;\">A adresser &agrave; votre centre de S&eacute;curit&eacute; Sociale.</span></p></body></html>";              /*!< \brief Default value for settings. */
+
+    // OBSOLETE
     const char * const  S_DEF_USERHEADER             = "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\np, li { white-space: pre-wrap; }\n</style></head><body>\n"
                                                                  "<table border=\"1\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\">\n<tr>\n"
                                                                  "<td width=\"33%\">\n<p align=\"center\"><span style=\" font-weight:600;\">Dr TRUCMUCHE Bidulle</span>\n<br /><span style=\" font-weight:600;\">M&eacute;decine Machin</span><\n<br /><br /><span style=\" font-size:10pt; font-style:italic;\">Ancien Assistant des h&ocirc;pitaux</span>\n<br /><span style=\" font-size:10pt; font-style:italic;\">Tous mes dipl&ocirc;mes</span>\n<br />----------<br />\n<span style=\" font-size:10pt;\">Mes Consultations sur rendez-vous</span>\n<br />----------\n<br /><span style=\" font-size:10pt; font-style:italic;\">Mon ADELI</span></p></td>\n<td width=\"33%\"></td>\n"
@@ -77,6 +80,14 @@ namespace Constants {
 
     const char * const S_PRESCRIPTIONFORMATTING_HTML      = "DrugsWidget/print/prescription/HtmlFormatting";      /*!< \brief Key for settings. */
     const char * const S_PRESCRIPTIONFORMATTING_PLAIN     = "DrugsWidget/print/prescription/PlainFormatting";      /*!< \brief Key for settings. */
+
+#ifdef WITH_PAD
+    const char * const S_DEF_PRESCRIPTION_TOKENFILE_1_LANG = "/textfiles/prescription/padtoolsstyle_%1.html";
+#else
+    const char * const S_DEF_PRESCRIPTION_TOKENFILE_1_LANG = "/textfiles/prescription/oldtokenstyle_%1.html";
+#endif
+
+    // OBSOLETE
     const char * const S_DEF_PRESCRIPTIONFORMATTING       = QT_TRANSLATE_NOOP("mfDrugsConstants",
                                                                               "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">"
                                                                               "[[DRUG]]</span><span style=\"font-style:italic;\">[<br />"
@@ -85,6 +96,7 @@ namespace Constants {
                                                                               "Daily Distribution: [DISTRIBUTED_DAILY_SCHEME]][<br />"
                                                                               "Minimum interval between intakes: [MIN_INTERVAL]][<br />"
                                                                               "[NOTE] ]</span></p>");
+    // OBSOLETE
     const char * const S_DEF_PRESCRIPTIONFORMATTING_PLAIN = QT_TRANSLATE_NOOP("mfDrugsConstants",
                                                                               "[[DRUG]][\n"
                                                                               "Take [Q_FROM]][ to [Q_TO]] [[Q_SCHEME]][ [REPEATED_DAILY_SCHEME]][ [MEAL]][ each [[PERIOD] ][PERIOD_SCHEME]][ for [D_FROM]][ to [D_TO]][ [D_SCHEME]][\n"
@@ -95,8 +107,6 @@ namespace Constants {
 
     const char * const S_ALD_PRE_HTML               = "DrugsWidget/print/ALDPreHtml";             /*!< \brief Key for settings. */
     const char * const S_ALD_POST_HTML              = "DrugsWidget/print/ALDPostHtml";            /*!< \brief Key for settings. */
-    const char * const S_DEF_ALD_PRE_HTML           = "<table border=\"1\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" style=\"border-style:solid;\">\n<tr>\n<td width=\"100%\">\n<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;font-family:Arial;font-size:10pt;color:black;\">Prescriptions relatives au traitement de l'affection de longue dur&eacute;e reconnue (liste ou hors liste)<br />(AFFECTION EXON&Eacute;RANTE)</span></p></td></tr></table>";
-    const char * const S_DEF_ALD_POST_HTML          = "<table border=\"1\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" style=\"border-style:solid;\">\n<tr>\n<td width=\"100%\">\n<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;font-family:Arial;font-size:10pt;color:black;\">Prescriptions SANS RAPPORT avec l'affection de longue dur&eacute;e<br />(MALADIES INTERCURRENTES)</span></p></td></tr></table>";
     const char * const S_PRINTLINEBREAKBETWEENDRUGS = "DrugsWidget/print/prescription/LineBreakBetweenDrugsWhenPrinting";      /*!< \brief Key for settings. */
 
     const char * const S_SELECTOR_SHOWDRUGSNAME   = "DrugsWidget/Selector/ShowDrugsName";
@@ -152,66 +162,6 @@ namespace Constants {
     // HELP PAGES
     const char * const  H_INTERACTION_SYNTHETISOR = "interactions.html";
 
-    // MASKS
-    const char * const  PROTOCOL_AUTOMATIC_LABEL_MASK =  "[[Q_FROM]][-[Q_TO]] [[Q_SCHEME]][ [REPEATED_DAILY_SCHEME]][ [MEAL]][ [[PERIOD] / ][PERIOD_SCHEME]][; [D_FROM]][-[D_TO]][ [D_SCHEME]][; [DISTRIBUTED_DAILY_SCHEME]]";
-    const char * const  ENCODEDHTML_FULLDOC_MEDINTUX =
-            "<html>\n"
-            "<head>\n"
-            "  <meta name=\"qrichtext\" content=\"1\" />\n"
-            "  <meta name=\"charset\" content=\"UTF-8\" />\n"
-            "  <meta name=\"generator\" content=\"{GENERATOR}\" />\n"
-            "  <title>FreeDiams: Prescription</title>\n"
-            "  <style type=\"text/css\">\n"
-            "    A:link {text-decoration: none}\n"
-            "    A:visited {text-decoration: none}\n"
-            "    A:active {text-decoration: none}\n"
-            "    A:hover {text-decoration: none;}\n"
-            "  </style>\n"
-            "</head>\n"
-            "<body style=\"font-family:Arial;font-size:12pt\">\n\n"
-            "<a href=\"{ENCODEDPRESCRIPTION}\">\n"
-            "{PRESCRIPTION}\n"
-            "</a>\n\n"
-            "</body>\n"
-            "</html>\n";
-
-        const char * const  ENCODEDHTML_FULLDOC_NON_MEDINTUX =
-                "<html>\n"
-                "<head>\n"
-                "  <meta name=\"qrichtext\" content=\"1\" />\n"
-                "  <meta name=\"charset\" content=\"UTF-8\" />\n"
-                "  <meta name=\"generator\" content=\"{GENERATOR}\" />\n"
-                "  <meta name=\"prescription\" content=\"{ENCODEDPRESCRIPTION}\" />\n"
-                "  <title>FreeDiams: Prescription</title>\n"
-                "  <style type=\"text/css\">\n"
-                "    A:link {text-decoration: none}\n"
-                "    A:visited {text-decoration: none}\n"
-                "    A:active {text-decoration: none}\n"
-                "    A:hover {text-decoration: none;}\n"
-                "  </style>\n"
-                "</head>\n"
-                "<body style=\"font-family:Arial;font-size:12pt\">\n\n"
-                "{PRESCRIPTION}\n"
-                "</body>\n"
-                "</html>\n";
-
-    const char * const  ENCODEDHTML_FULLPRESCRIPTION_MEDINTUX =
-            "<table border=0>\n"
-            " <tr>\n"
-            "   <td>\n"
-            "     <ol>\n"
-            "{FULLPRESCRIPTION}\n"
-            "     </ol>\n"
-            "   </td>\n"
-            " </tr>\n"
-            "</table>\n";
-
-    const char * const  ENCODEDHTML_FULLPRESCRIPTION_NON_MEDINTUX =
-            "<ol style=\"font-size:10pt;font-weight:bold;\">\n"
-            "{FULLPRESCRIPTION}\n"
-            "</ol>\n";
-
-    const char * const  ENCODEDHTML_FREEDIAMSTAG = "FreeDiamsEncodedPrescription:";          /*!< \brief Default tag prepend to serialized prescription when saving. */
 
     // LIST OF KNOWN LABORATORY FOR INN PRESCRIPTION
     const QStringList LABOS = QStringList()
@@ -256,6 +206,34 @@ namespace Constants {
     const char * const  MANAGEMENT = QT_TRANSLATE_NOOP("mfDrugsConstants", "Management");
     const char * const  DDI_TEXT = QT_TRANSLATE_NOOP("mfDrugsConstants", "Drug-drug interactions");
 
+    // Tokens translations
+    const char * const  TOKEN_TEXT_PRESCRIPTION     = QT_TRANSLATE_NOOP("mfDrugsConstants", "Prescription");
+    const char * const  TOKEN_TEXT_DRUG             = QT_TRANSLATE_NOOP("mfDrugsConstants", "Drug");
+    const char * const  TOKEN_TEXT_PROTOCOL         = QT_TRANSLATE_NOOP("mfDrugsConstants", "Protocol");
+    const char * const  TOKEN_TOOLTIP_PRESCRIPTION  = QT_TRANSLATE_NOOP("mfDrugsConstants", "This token namespace contains all prescription's' related tokens.\n"
+                                                                       "It includes the drug brand name and all the items of your protocol.");
+    const char * const  TOKEN_TEXT_DRUGNAME         = QT_TRANSLATE_NOOP("mfDrugsConstants", "Drug brand name");
+    const char * const  TOKEN_TEXT_Q_FROM           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Quantity (from)");
+    const char * const  TOKEN_TEXT_Q_TO             = QT_TRANSLATE_NOOP("mfDrugsConstants", "Quantity (to)");
+    const char * const  TOKEN_TEXT_Q_SCHEME         = QT_TRANSLATE_NOOP("mfDrugsConstants", "Quantity (scheme)");
+    const char * const  TOKEN_TEXT_Q_FULL           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Quantity (complete)");
+    const char * const  TOKEN_TEXT_MEAL             = QT_TRANSLATE_NOOP("mfDrugsConstants", "Meal relation");
+    const char * const  TOKEN_TEXT_P_VALUE          = QT_TRANSLATE_NOOP("mfDrugsConstants", "Period (value)");
+    const char * const  TOKEN_TEXT_P_SCHEME         = QT_TRANSLATE_NOOP("mfDrugsConstants", "Period (scheme)");
+    const char * const  TOKEN_TEXT_P_FULL           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Period (complete)");
+    const char * const  TOKEN_TEXT_D_FROM           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Duration (from)");
+    const char * const  TOKEN_TEXT_D_TO             = QT_TRANSLATE_NOOP("mfDrugsConstants", "Duration (to)");
+    const char * const  TOKEN_TEXT_D_SCHEME         = QT_TRANSLATE_NOOP("mfDrugsConstants", "Duration (scheme)");
+    const char * const  TOKEN_TEXT_D_FULL           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Duration (complete)");
+    const char * const  TOKEN_TEXT_ROUTE            = QT_TRANSLATE_NOOP("mfDrugsConstants", "Route");
+    const char * const  TOKEN_TEXT_DISTRIB          = QT_TRANSLATE_NOOP("mfDrugsConstants", "Daily distribution");
+    const char * const  TOKEN_TEXT_REPEAT           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Repeated ditribution");
+    const char * const  TOKEN_TEXT_MININTERV_FULL   = QT_TRANSLATE_NOOP("mfDrugsConstants", "Minimum interval between intakes (complete)");
+    const char * const  TOKEN_TEXT_MININTERV_VALUE  = QT_TRANSLATE_NOOP("mfDrugsConstants", "Minimum interval between intakes (value)");
+    const char * const  TOKEN_TEXT_MININTERV_SCHEME = QT_TRANSLATE_NOOP("mfDrugsConstants", "Minimum interval between intakes (scheme)");
+    const char * const  TOKEN_TEXT_REFILL           = QT_TRANSLATE_NOOP("mfDrugsConstants", "Refill");
+    const char * const  TOKEN_TEXT_NOTE             = QT_TRANSLATE_NOOP("mfDrugsConstants", "Note");
+
      /*: This is the short name of the drug-drug interactions engine. */
     const char * const  DDI_SHORT_TEXT = QT_TRANSLATE_NOOP("mfDrugsConstants", "DDI");
 
@@ -342,28 +320,32 @@ namespace Prescription
         UsedDosage,
         Pack_UID,
         OnlyForTest,
+        IntakesFullString,  // READ ONLY
         IntakesFrom,
         IntakesTo,
         IntakesScheme,
         IntakesUsesFromTo,
-        IntakesFullString,
         Route,            // Textual translated route
         RouteId,          // Route Id from drugs database
+        DurationFullString,  // READ ONLY
         DurationFrom,
         DurationTo,
         DurationScheme,
         DurationUsesFromTo,
+        PeriodFullString,  // READ ONLY
         Period,
         PeriodScheme,
-        DailyScheme,
+        SerializedDailyScheme,
         MealTimeSchemeIndex,
+        IntakesIntervalFullString,
         IntakesIntervalOfTime,
-        IntakesIntervalScheme,
+        IntakesIntervalSchemeIndex,
         Note,
         IsINNPrescription,
         SpecifyForm,
         SpecifyPresentation,
         IsALD,
+        Refill,
         ToHtml,
         MaxParam
     };
@@ -426,7 +408,7 @@ namespace Constants {
         Period,
         PeriodScheme,
         AdministrationScheme,
-        DailyScheme,            /*!< \brief  */
+        SerializedDailyScheme,
         MealScheme,             /*!< \brief Schéma de prise par rapport aux repas // cf enum mfDrugDosage::MealTime */
         IsALD,                  /*!< \brief Prescription en ALD */
         TypeOfTreatment,        /*!< \brief Type de traitement (chronique, aiguë...) enumerator à créer  \todo this */
@@ -449,12 +431,13 @@ namespace Constants {
         CIM10LimitsLinks,       /*!< \brief CIM10 codes that must not be used with the prescription. Codes are separated by a ';'.  \todo this */
         EDRCLinks,              /*!< \brief EDRC codes related to the prescription. Codes are separated by a ';'.  \todo this  */
 
-        Extras,                 /*!< \brief blob that contains extra datas XML formatted (not used for now */
+        Extras,                 /*!< \brief blob that contains extra data, XML formatted (not used by now) */
         UserValidatorName,      /*!< \brief nom de celui qui a validé la posologie */
-        CreationDate,           /*!< \brief Date création      --> date */
-        ModificationDate,       /*!< \brief Date modification  --> date */
+        CreationDate,           /*!< \brief Creation date --> date */
+        ModificationDate,       /*!< \brief Modification date --> date */
         Transmitted,            /*!< \brief Sera utilisé pour récupérer via internet les dosages référencés en local. date */
         Route,                  /*!< \brief Outside SQL */
+        Refill,                  /*!< \brief Outside SQL */
         MaxParam                /*!< \brief Nombre maximum de colonnes du modèle (et de la table SQL) */
     };
 
