@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -45,14 +45,20 @@
 #include <QDir>
 
 using namespace DrugsDB;
-using namespace DrugsDB::Internal;
+using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
 static inline QString databaseFileName() {return settings()->databasePath() + QDir::separator() + QString(Constants::DB_DRUGS_NAME) + QDir::separator() + QString(Constants::DB_DRUGS_FILENAME);}
 
 namespace {
+#if DRUGS_DATABASE_VERSION == 0x000804
+const char * const CURRENTVERSION = "0.8.4";
+#elif DRUGS_DATABASE_VERSION == 0x000604
 const char * const CURRENTVERSION = "0.6.4";
+#else
+const char * const CURRENTVERSION = "0.0.0";
+#endif
 
 struct ftype {
     ftype(int _f, Utils::Database::TypeOfField _t) : f(_f), t(_t) {}
@@ -76,8 +82,11 @@ static inline bool connectDatabase(QSqlDatabase &DB, const QString &file, const
 }
 
 DrugBaseEssentials::DrugBaseEssentials():
-    Utils::Database(), m_dbcore_initialized(false), m_isDefaultDb(false)
+    Utils::Database(),
+    m_dbcore_initialized(false),
+    m_isDefaultDb(false)
 {
+    setConnectionName(Constants::DB_DRUGS_NAME);
     using namespace Constants;
     QMultiHash<int, ftype> types;
     int i = Table_MASTER;
@@ -248,6 +257,27 @@ DrugBaseEssentials::DrugBaseEssentials():
     types.insertMulti(i, ftype(CURRENTVERSION_ID, FieldIsUniquePrimaryKey));
     types.insertMulti(i, ftype(CURRENTVERSION_NUMBER, FieldIsShortText));
 
+#if DRUGS_DATABASE_VERSION >= 0x000804
+    i = Table_DRUG_SPC;
+    types.insertMulti(i, ftype(DRUG_SPC_DID,FieldIsInteger));
+    types.insertMulti(i, ftype(DRUG_SPC_SPCCONTENT_ID,FieldIsInteger));
+    i = Table_SPC_CONTENT;
+    types.insertMulti(i, ftype(SPCCONTENT_ID,FieldIsUniquePrimaryKey));
+    types.insertMulti(i, ftype(SPCCONTENT_LABEL,FieldIsShortText));
+    types.insertMulti(i, ftype(SPCCONTENT_URL_SOURCE,FieldIsShortText));
+    types.insertMulti(i, ftype(SPCCONTENT_DATEOFDOWNLOAD,FieldIsDateTime));
+    types.insertMulti(i, ftype(SPCCONTENT_HTMLCONTENT,FieldIsBlob));
+    types.insertMulti(i, ftype(SPCCONTENT_SPCCONTENT_RESOURCES_LINK_ID,FieldIsInteger));
+    i = Table_SPC_CONTENTRESOURCE_LINK;
+    types.insertMulti(i, ftype(SPCCONTENT_RESOURCES_LINK_ID,FieldIsInteger));
+    types.insertMulti(i, ftype(SPCCONTENT_SPCCONTENTRESOURCES_ID,FieldIsInteger));
+    i = Table_SPC_CONTENTRESOURCE;
+    types.insertMulti(i, ftype(SPCCONTENTRESOURCES_ID,FieldIsUniquePrimaryKey));
+    types.insertMulti(i, ftype(SPCCONTENTRESOURCES_TYPE,FieldIsShortText));
+    types.insertMulti(i, ftype(SPCCONTENTRESOURCES_NAME,FieldIsShortText));
+    types.insertMulti(i, ftype(SPCCONTENTRESOURCES_CONTENT,FieldIsBlob));
+#endif
+
     for(int i=0; i < Table_MaxParam; ++i) {
         addTable(i, "t" + QString::number(i));
         const QList<ftype> &tp = types.values(i);
@@ -303,18 +333,30 @@ DrugBaseEssentials::DrugBaseEssentials():
     addIndex(Table_PIMS, PIMS_ID);
     addIndex(Table_PIM_TYPES, PIM_TYPES_TID);
     addIndex(Table_PIM_SOURCES, PIM_SOURCES_SID);
+
+#if DRUGS_DATABASE_VERSION >= 0x000804
+    addIndex(Table_DRUG_SPC, DRUG_SPC_DID);
+    addIndex(Table_DRUG_SPC, DRUG_SPC_SPCCONTENT_ID);
+    addIndex(Table_SPC_CONTENT, SPCCONTENT_ID);
+#endif
 }
 
+/** Force the re-initialization of the database. Call initialize() after this. */
 void DrugBaseEssentials::forceFullDatabaseRefreshing()
 {
     m_dbcore_initialized = false;
 }
 
+/**
+ * Initialize the database (create it is required). \n
+ * By default, the connection is defined to DrugsDB::Constants::DB_DRUGS_NAME.
+ * If you want your own connection name, you can set it with the
+ * setConnectionName() and you must set it \b before calling initialize().
+ */
 bool DrugBaseEssentials::initialize(const QString &pathToDb, bool createIfNotExists)
 {
     if (m_dbcore_initialized)
         return true;
-    setConnectionName(Constants::DB_DRUGS_NAME);
     setDriver(Utils::Database::SQLite);
 
     // test driver
@@ -339,21 +381,21 @@ bool DrugBaseEssentials::initialize(const QString &pathToDb, bool createIfNotExi
     drugConnector.setAccessMode(Utils::DatabaseConnector::ReadOnly);
     drugConnector.setDriver(Utils::Database::SQLite);
 
-    LOG_FOR("DrugBaseEssentials", tkTr(Trans::Constants::SEARCHING_DATABASE_1_IN_PATH_2).arg(Constants::DB_DRUGS_NAME).arg(pathToDb));
+    LOG_FOR("DrugBaseEssentials", tkTr(Trans::Constants::SEARCHING_DATABASE_1_IN_PATH_2).arg(connectionName()).arg(pathToDb));
 
     if (createIfNotExists) {
-        createConnection(Constants::DB_DRUGS_NAME, Constants::DB_DRUGS_FILENAME,
+        createConnection(connectionName(), Constants::DB_DRUGS_FILENAME,
                          drugConnector,
                          Utils::Database::CreateDatabase);
     } else {
-        createConnection(Constants::DB_DRUGS_NAME, Constants::DB_DRUGS_FILENAME,
+        createConnection(connectionName(), Constants::DB_DRUGS_FILENAME,
                          drugConnector,
                          Utils::Database::WarnOnly);
     }
 
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR_FOR("DrugBaseEssentials",tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_DRUGS_NAME).arg(database().lastError().text()));
+            LOG_ERROR_FOR("DrugBaseEssentials",tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(connectionName()).arg(database().lastError().text()));
         } else {
             LOG_FOR("DrugBaseEssentials",tkTr(Trans::Constants::CONNECTED_TO_DATABASE_1_DRIVER_2).arg(database().connectionName()).arg(database().driverName()));
         }
@@ -362,7 +404,7 @@ bool DrugBaseEssentials::initialize(const QString &pathToDb, bool createIfNotExi
     }
 
     if (!checkDatabaseScheme()) {
-        LOG_ERROR_FOR("DrugBaseEssentials",tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(Constants::DB_DRUGS_NAME));
+        LOG_ERROR_FOR("DrugBaseEssentials",tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(connectionName()));
         return false;
     }
 
@@ -373,15 +415,13 @@ bool DrugBaseEssentials::initialize(const QString &pathToDb, bool createIfNotExi
         LOG_FOR("DrugBaseEssentials", QString("Using drug database version " + version()));
     }
 
-    setConnectionName(Constants::DB_DRUGS_NAME);
-
     m_dbcore_initialized = true;
     return true;
 }
 
 void DrugBaseEssentials::setVersion(const QString &version)
 {
-    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_DRUGS_NAME);
+    QSqlDatabase DB = QSqlDatabase::database(connectionName());
     if (!connectDatabase(DB, __FILE__, __LINE__))
         return;
     executeSQL(prepareDeleteQuery(Constants::Table_CURRENTVERSION, QHash<int,QString>()), DB);
@@ -396,7 +436,7 @@ void DrugBaseEssentials::setVersion(const QString &version)
 
 QString DrugBaseEssentials::version() const
 {
-    QSqlDatabase DB = QSqlDatabase::database(Constants::DB_DRUGS_NAME);
+    QSqlDatabase DB = QSqlDatabase::database(connectionName());
     if (!connectDatabase(DB, __FILE__, __LINE__))
         return QString();
     QSqlQuery query(DB);
@@ -416,7 +456,26 @@ bool DrugBaseEssentials::checkDatabaseVersion() const
     return (version()==::CURRENTVERSION);
 }
 
-bool DrugBaseEssentials::createDatabase(const QString &connectionName , const QString &prefixedDbName,
+int DrugBaseEssentials::getSourceId(const QString &drugsDbUid)
+{
+    QSqlDatabase DB = QSqlDatabase::database(connectionName());
+    if (!connectDatabase(DB, __FILE__, __LINE__))
+        return -1;
+    QSqlQuery query(DB);
+    QHash<int, QString> where;
+    where.insert(Constants::SOURCES_DBUID, QString("='%1'").arg(drugsDbUid));
+    query.prepare(select(Constants::Table_SOURCES, Constants::SOURCES_SID, where));
+    if (query.exec()) {
+        if (query.next()) {
+            return query.value(0).toInt();
+        }
+    } else {
+        LOG_QUERY_ERROR_FOR("DrugBaseEssentials", query);
+    }
+    return -1;
+}
+
+bool DrugBaseEssentials::createDatabase(const QString &connection, const QString &prefixedDbName,
                                   const Utils::DatabaseConnector &connector,
                                   CreationOption createOption
                                   )
@@ -424,22 +483,22 @@ bool DrugBaseEssentials::createDatabase(const QString &connectionName , const QS
 //    WARN_FUNC << connectionName << prefixedDbName;
 //    qWarning() << connector;
 
-    if (connectionName != Constants::DB_DRUGS_NAME)
+    if (connection != connectionName())
         return false;
     if (connector.driver() != SQLite) {
         return false;
     }
     if (createOption!=Utils::Database::CreateDatabase)
         return false;
-    QString pathOrHostName = connector.absPathToSqliteReadOnlyDatabase() + QDir::separator() + QString(Constants::DB_DRUGS_NAME);
+    QString pathOrHostName = connector.absPathToSqliteReadOnlyDatabase() + QDir::separator() + QString(connectionName());
     LOG_FOR("DrugBaseEssentials", tkTr(Trans::Constants::TRYING_TO_CREATE_1_PLACE_2).arg(prefixedDbName).arg(pathOrHostName));
 
-    setConnectionName(connectionName);
+    setConnectionName(connectionName());
     setDriver(connector.driver());
 
     // create an empty database and connect
-    if (QSqlDatabase::connectionNames().contains(connectionName)) {
-        QSqlDatabase::removeDatabase(connectionName);
+    if (QSqlDatabase::connectionNames().contains(connectionName())) {
+        QSqlDatabase::removeDatabase(connectionName());
     }
 
     if (!Utils::checkDir(pathOrHostName, true, "DrugBaseEssentials")) {
@@ -448,7 +507,7 @@ bool DrugBaseEssentials::createDatabase(const QString &connectionName , const QS
     }
 
     QSqlDatabase DB;
-    DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
+    DB = QSqlDatabase::addDatabase("QSQLITE", connectionName());
     DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + prefixedDbName));
     if (!DB.open())
         LOG_FOR("DrugBaseEssentials", tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(prefixedDbName).arg(DB.lastError().text()));
