@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -38,7 +38,7 @@
      One UID code can be associated to many CIP, many Substances, many INNs, and many IamClasses.
 
   1. Initialization\n
-  This class is pure static, so you can not instanciate it. To initialize datas, just do once : init().
+  This class is a single instance object. To initialize data, just do once \e initialize().
   isInitialized() alerts you of the state of intialization.
   These two members returns true if all is ok.
 
@@ -92,7 +92,7 @@
 #include <QSet>
 #include <QCache>
 
-enum { WarnExtractedDrugs=false };
+enum { WarnExtractedDrugs = false };
 
 using namespace DrugsDB;
 using namespace DrugsDB::Internal;
@@ -121,7 +121,6 @@ namespace {
     };
 }
 
-
 namespace DrugsDB {
 namespace Internal {
 class DrugsBasePrivate
@@ -148,7 +147,7 @@ public:
 
     void retreiveLinkTables()
     {
-        if ((!m_AtcToMol.isEmpty()) && (!m_ClassToAtcs.isEmpty()))
+        if (!m_AtcToMol.isEmpty())
             return;
         QSqlQuery query(QSqlDatabase::database(Constants::DB_DRUGS_NAME));
         if (query.exec(q->select(Constants::Table_LK_MOL_ATC))) {
@@ -206,6 +205,8 @@ public:
 
     void getInteractingClassTree()
     {
+        m_ClassToAtcs.clear();
+        m_ClassesId.clear();
         // Retreive Interacting classes (1) ---> (n) ATC tree
         QString req = q->select(Constants::Table_IAM_TREE, QList<int>() << Constants::IAM_TREE_ID_CLASS << Constants::IAM_TREE_ID_ATC);
         QSqlQuery query(QSqlDatabase::database(Constants::DB_DRUGS_NAME));
@@ -216,6 +217,8 @@ public:
         } else {
             LOG_QUERY_ERROR_FOR(q, query);
         }
+        m_ClassesId = m_ClassToAtcs.uniqueKeys();
+        LOG_FOR(q, QString("Retrieving %1 interacting classes").arg(m_ClassToAtcs.uniqueKeys().count()));
     }
 
     void getSearchEngine()
@@ -300,7 +303,7 @@ public:
                 }
                 precedent = compo;
                 if (WarnExtractedDrugs)
-                    qWarning() << compo->data(IComponent::DebuggingText).toString();
+                    qWarning() << compo;
             }
         } else {
             LOG_QUERY_ERROR_FOR(q, query);
@@ -319,8 +322,8 @@ public:
                 route->setSystemicDatabaseTag(query.value(Constants::ROUTES_SYSTEMIC).toString());
                 route->setRouteId(query.value(Constants::ROUTES_RID).toInt());
                 QHash<QString, QString> labels = getAllLabels(query.value(Constants::ROUTES_MASTERLID).toInt());
-                foreach(const QString &lang, labels) {
-                    route->setLabel(lang, labels.value(lang));
+                foreach(const QString &lang, labels.keys()) {
+                    route->setLabel(labels.value(lang), lang);
                 }
             }
         } else {
@@ -335,8 +338,9 @@ public:
     DatabaseInfos *m_ActualDBInfos;
     bool m_initialized, m_LogChrono, m_RefreshDrugsBase, m_RefreshDosageBase, m_UseRoutes, m_IsDefaultDB;
 
-    QMultiHash<int, int> m_AtcToMol;   /*!< Link Iam_Id to Code_Subst */
-    QMultiHash<int, int> m_ClassToAtcs;   /*!< Link ClassIam_Id to Iam_Id */
+    QMultiHash<int, int> m_AtcToMol;      /*!< Link ATC code to Molecule Code */
+    QMultiHash<int, int> m_ClassToAtcs;   /*!< Link ATC Class to ATC Id (content of the class) */
+    QList<int> m_ClassesId;
     QCache<int, AtcLabel> m_AtcLabelCache;
     QHash<QString, int> m_DbUids;
     // TODO: improve memory usage here
@@ -386,7 +390,7 @@ static QString databaseFileName()
 }
 
 /** \brief Initializer for the database. Return the error state. */
-bool DrugsBase::init()
+bool DrugsBase::initialize()
 {
     // only one base can be initialized
     if (d->m_initialized)
@@ -448,7 +452,7 @@ bool DrugsBase::refreshAllDatabases()
     refreshDrugsBase();
 //    d->m_RefreshDosageBase = true;
     Q_EMIT drugsBaseIsAboutToChange();
-    bool r = init();
+    bool r = initialize();
     if (r) {
         Q_EMIT drugsBaseHasChanged();
     }
@@ -462,7 +466,7 @@ bool DrugsBase::datapackChanged()
     // Remove actual QSqlDatabase
     QSqlDatabase::removeDatabase(Constants::DB_DRUGS_NAME);
     // Re-init database
-    init();
+    initialize();
     return true;
 }
 
@@ -482,15 +486,17 @@ bool DrugsBase::refreshDrugsBase()
 
     d->m_ActualDBInfos = getDrugSourceInformation(drugSource);
     if (!d->m_ActualDBInfos) {
-//        Utils::warningMessageBox(tr("Drug database source does not exist."),
-//                                 tr("Switching to the default drugs database source."));
         d->m_ActualDBInfos = getDrugSourceInformation(DrugsDB::Constants::DB_DEFAULT_IDENTIFIANT);
         if (!d->m_ActualDBInfos) {
             // get the first available from the database
             d->m_ActualDBInfos = getDrugSourceInformation();
-            LOG(QString("%1 %2")
-                .arg(tr("Switching to the default drugs database source."))
-                .arg(d->m_ActualDBInfos->identifier));
+            if (d->m_ActualDBInfos) {
+                LOG(QString("%1 %2")
+                    .arg(tr("Switching to the default drugs database source."))
+                    .arg(d->m_ActualDBInfos->identifier));
+            } else {
+                LOG_ERROR(tr("No drug source detected."));
+            }
         }
         d->m_IsDefaultDB = true;
     }
@@ -621,385 +627,11 @@ DatabaseInfos *DrugsBase::getDrugSourceInformation(const QString &drugSourceUid)
     return info;
 }
 
-//QString DrugsBase::dosageCreateTableSqlQuery()
-//{
-//    // TODO: add UserUuid field or lkid
-//    return "CREATE TABLE IF NOT EXISTS `DOSAGE` ("
-//           "`POSO_ID`               INTEGER        PRIMARY KEY AUTOINCREMENT,"
-//           "`POSO_UUID`             varchar(40)    NULL,"    // put NOT NULL
-//           "`DRUGS_DATABASE_IDENTIFIANT` varchar(200) NULL,   "
-//           "`INN_LK`                int(11)        DEFAULT -1,"
-//           "`INN_DOSAGE`            varchar(100)   NULL,"    // contains the dosage of the SA INN
-//           "`DRUG_UID_LK`           varchar(20)    NULL,"
-//           "`CIP_LK`                int(11)        DEFAULT -1,"
-//           "`LABEL`                 varchar(300)   NULL,"    // put NOT NULL
-
-//           "`INTAKEFROM`            double         NULL,"    // put NOT NULL
-//           "`INTAKETO`              double         NULL,"
-//           "`INTAKEFROMTO`          bool           NULL,"
-//           "`INTAKESCHEME`          varchar(200)   NULL,"    // put NOT NULL
-//           "`INTAKESINTERVALOFTIME` int(10)        NULL,"
-//           "`INTAKESINTERVALSCHEME` varchar(200)   NULL,"
-//           "`ROUTE_ID`              integer        NULL,"
-
-//           "`DURATIONFROM`          double         NULL,"    // put NOT NULL
-//           "`DURATIONTO`            double         NULL,"
-//           "`DURATIONFROMTO`        bool           NULL,"
-//           "`DURATIONSCHEME`        varchar(200)   NULL,"    // put NOT NULL
-
-//           "`PERIOD`                int(10)        NULL,"    // put NOT NULL
-//           "`PERIODSCHEME`          varchar(200)   NULL,"    // put NOT NULL
-//           "`ADMINCHEME`            varchar(100)   NULL,"    // put NOT NULL
-//           "`DAILYSCHEME`           varchar(250)   NULL,"
-//           "`MEALSCHEME`            int(10)        NULL,"
-//           "`ISALD`                 bool           NULL,"
-//           "`TYPEOFTREATEMENT`      int(10)        NULL,"
-
-//           "`MINAGE`                int(10)        NULL,"
-//           "`MAXAGE`                int(10)        NULL,"
-//           "`MINAGEREFERENCE`       int(10)        NULL,"
-//           "`MAXAGEREFERENCE`       int(10)        NULL,"
-//           "`MINWEIGHT`             int(10)        NULL,"
-//           "`SEXLIMIT`              int(10)        NULL,"
-//           "`MINCLEARANCE`          int(10)        NULL,"
-//           "`MAXCLEARANCE`          int(10)        NULL,"
-//           "`PREGNANCYLIMITS`       int(10)        NULL,"
-//           "`BREASTFEEDINGLIMITS`   int(10)        NULL,"
-//           "`PHYSIOLOGICALLIMITS`   int(10)        NULL,"  // Is this really needed ?
-
-//           "`NOTE`                  varchar(500)   NULL,"
-
-//           "`CIM10_LK`              varchar(150)   NULL,"
-//           "`CIM10_LIMITS_LK`       varchar(150)   NULL,"
-//           "`EDRC_LK`               varchar(150)   NULL,"
-
-//           "`EXTRAS`                blob           NULL,"
-//           "`USERVALIDATOR`         varchar(200)   NULL,"
-//           "`CREATIONDATE`          date           NULL,"    // put NOT NULL
-//           "`MODIFICATIONDATE`      date           NULL,"
-//           "`TRANSMITTED`           date           NULL,"
-//           "`ORDER`                 int(10)        NULL"
-//           ");";
-//}
-
-///** \brief Create the protocols database if it does not exist. */
-//bool DrugsBase::createDatabase(const QString &connectionName , const QString &dbName,
-//                               const QString &pathOrHostName,
-//                               TypeOfAccess /*access*/, AvailableDrivers driver,
-//                               const QString & login, const QString & pass,
-//                               const int port,
-//                               CreationOption /*createOption*/
-//                              )
-//{
-//    // TODO:  ask user if he wants : 1. an empty dosage base ; 2. to retreive dosages from internet FMF website
-//    if (connectionName != Dosages::Constants::DB_DOSAGES_NAME)
-//        return false;
-
-//    LOG(tkTr(Trans::Constants::TRYING_TO_CREATE_1_PLACE_2).arg(dbName).arg(pathOrHostName));
-
-//    // create an empty database and connect
-//    QSqlDatabase DB;
-//    if (driver == SQLite) {
-//        DB = QSqlDatabase::addDatabase("QSQLITE" , connectionName);
-//        if (!QDir(pathOrHostName).exists())
-//            if (!QDir().mkpath(pathOrHostName))
-//                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
-//        DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
-//        DB.open();
-//        setDriver(Utils::Database::SQLite);
-//    }
-//    else if (driver == MySQL) {
-//        DB = QSqlDatabase::database(connectionName);
-//        if (!DB.open()) {
-//            QSqlDatabase d = QSqlDatabase::addDatabase("QMYSQL", "CREATOR");
-//            d.setHostName(pathOrHostName);
-//            // TODO: retreive log/pass
-//            d.setUserName(login);
-//            d.setPassword(pass);
-//            d.setPort(port);
-//            if (!d.open()) {
-//                Utils::warningMessageBox(tr("Unable to create the Protocol database."),tkTr(Trans::Constants::CONTACT_DEV_TEAM));
-//                return false;
-//            }
-//            QSqlQuery q(QString("CREATE DATABASE `%1`").arg(dbName), d);
-//            if (!q.isActive()) {
-//                LOG_QUERY_ERROR(q);
-//                Utils::warningMessageBox(tr("Unable to create the Protocol database."),tkTr(Trans::Constants::CONTACT_DEV_TEAM));
-//                return false;
-//            }
-//            DB.setDatabaseName(dbName);
-//            if (!DB.open()) {
-//                Utils::warningMessageBox(tr("Unable to create the Protocol database."),tkTr(Trans::Constants::CONTACT_DEV_TEAM));
-//                return false;
-//            }
-//            DB.setDatabaseName(dbName);
-//        }
-//        if (QSqlDatabase::connectionNames().contains("CREATOR"))
-//            QSqlDatabase::removeDatabase("CREATOR");
-//        DB.open();
-//        setDriver(Utils::Database::MySQL);
-//    }
-
-//    // create db structure
-//    // before we need to inform Utils::Database of the connectionName to use
-//    setConnectionName(connectionName);
-
-//    // The SQL scheme MUST BE synchronized with the Dosages::Constants Model Enumerator !!!
-//    // TODO: code here
-//    if (executeSQL(QStringList()
-//        << dosageCreateTableSqlQuery().remove("AUTOINCREMENT")
-//        << "CREATE TABLE IF NOT EXISTS `VERSION` ("
-//           "`ACTUAL`                varchar(10)    NULL"
-//           ");"
-//        << QString("INSERT INTO `VERSION` (`ACTUAL`) VALUES('%1');").arg(versionUpdater()->lastDosageDabaseVersion())
-//        , DB)) {
-//        LOG(tkTr(Trans::Constants::DATABASE_1_CORRECTLY_CREATED).arg(dbName));
-//        return true;
-//    } else {
-//        LOG_ERROR(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2)
-//                         .arg(dbName, DB.lastError().text()));
-//    }
-//    return false;
-//}
-
 void DrugsBase::onCoreDatabaseServerChanged()
 {
 //    refreshDosageBase();
 }
 
-/**
-  \brief Check the version of the doage database. Do the necessary updates for that database according to the application version number.
-  Added from freediams version 0.0.8 stable
-*/
-//void DrugsBase::checkDosageDatabaseVersion()
-//{
-//    if (!DrugsDB::VersionUpdater::instance()->isDosageDatabaseUpToDate())
-//        if (!DrugsDB::VersionUpdater::instance()->updateDosageDatabase())
-//            Utils::Log::addError(this, "Dosage database can not be successfully updated",
-//                                 __FILE__, __LINE__);
-//}
-
-///** \brief Returns hash that contains dosage uuid has key and the xml'd dosage to transmit as value */
-//QHash<QString, QString> DrugsBase::getDosageToTransmit()
-//{
-//    QHash<QString, QString> toReturn;
-//    QSqlDatabase DB = QSqlDatabase::database(Dosages::Constants::DB_DOSAGES_NAME);
-//    if (!connectDatabase(DB, __FILE__, __LINE__))
-//        return toReturn;
-
-//    QString req = QString("SELECT * FROM `DOSAGE` WHERE (`TRANSMITTED` IS NULL);");
-//    {
-//        QSqlQuery query(req,DB);
-//        if (query.isActive()) {
-//            while (query.next()) {
-//                QHash<QString,QString> toXml;
-//                int i=0;
-//                for (i=0;i<query.record().count();++i) {
-//                    // create a XML of the dosage
-//                    toXml.insert(query.record().field(i).name(), query.value(i).toString());
-//                }
-//                toReturn.insert(toXml.value("POSO_UUID"), Utils::createXml(Dosages::Constants::DB_DOSAGES_TABLE_NAME,toXml,4,false));
-//            }
-//        } else
-//            LOG_QUERY_ERROR(query);
-//    }
-
-//    req = QString("SELECT * FROM `DOSAGE` WHERE (`TRANSMITTED`<`MODIFICATIONDATE`);");
-//    {
-//        QSqlQuery query(req,DB);
-//        if (query.isActive()) {
-//            while (query.next()) {
-//                QHash<QString,QString> toXml;
-//                int i=0;
-//                for (i=0;i<query.record().count();++i) {
-//                    // create a XML of the dosage
-//                    toXml.insert(query.record().field(i).name(), query.value(i).toString());
-//                }
-//                toReturn.insert(toXml.value("POSO_UUID"), Utils::createXml(Dosages::Constants::DB_DOSAGES_TABLE_NAME,toXml,4,false));
-//            }
-//        } else {
-//            LOG_QUERY_ERROR(query);
-//        }
-//    }
-//    return toReturn;
-//}
-
-///** Marks all dosages as transmitted now. \e dosageUuids contains dosage uuid. */
-//bool DrugsBase::markAllDosageTransmitted(const QStringList &dosageUuids)
-//{
-//    if (dosageUuids.count()==0)
-//        return true;
-//    QSqlDatabase DB = QSqlDatabase::database(Dosages::Constants::DB_DOSAGES_NAME);
-//    if (!connectDatabase(DB, __FILE__, __LINE__))
-//        return false;
-
-//    QStringList reqs;
-//    foreach(const QString &s, dosageUuids) {
-//        QString req = QString("UPDATE `DOSAGE` SET `TRANSMITTED`='%1' WHERE %2")
-//                      .arg(QDateTime::currentDateTime().toString(Qt::ISODate))
-//                      .arg(QString("`POSO_UUID`='%1'").arg(s));
-//        reqs << req;
-//    }
-//    if (!executeSQL(reqs,DB)) {
-//        Utils::Log::addError(this, tr("Unable to update the protocol's transmission date."),
-//                             __FILE__, __LINE__);
-//        return false;
-//    }
-//    return true;
-//}
-
-//struct minimalCompo {
-//    bool isLike(const int _inn, const QString &_dosage) const {
-//        return (inn == _inn) && (dosage == _dosage);
-//    }
-//    int inn;
-//    QString dosage;
-//};
-
-///////////////////////////////////////////////////////////////////////////////////////////////
-///**
-//  \brief Returns all CIS that have a recorded dosage. Manages INN dosage type.
-//  \todo put this in a thread...
-//*/
-//QList<QVariant> DrugsBase::getAllUIDThatHaveRecordedDosages() const
-//{
-//    QList<QVariant> toReturn;
-//    // TODO: recode this
-
-//    if (!actualDatabaseInformation())
-//        return toReturn;
-
-//    QSqlDatabase DosageDB = QSqlDatabase::database(Dosages::Constants::DB_DOSAGES_NAME);
-//    if (!connectDatabase(DosageDB, __FILE__, __LINE__))
-//        return toReturn;
-
-//    QString req;
-//    if (d->m_IsDefaultDB) {
-//        req = QString("SELECT DISTINCT `DRUG_UID_LK` FROM `DOSAGE` WHERE `DRUGS_DATABASE_IDENTIFIANT` = \"%1\";")
-//              .arg(Constants::DB_DEFAULT_IDENTIFIANT);
-//    } else {
-//        req = QString("SELECT DISTINCT `DRUG_UID_LK` FROM `DOSAGE` WHERE `DRUGS_DATABASE_IDENTIFIANT` = \"%1\";")
-//              .arg(actualDatabaseInformation()->identifiant);
-//    }
-//    {
-//        QSqlQuery query(req,DosageDB);
-//        if (query.isActive()) {
-//            while (query.next()) {
-//                toReturn << query.value(0);
-//            }
-//        } else {
-//            LOG_QUERY_ERROR(query);
-//        }
-//    }
-
-//    // Get all CIS that contains INN that have available dosage
-//    QMultiHash<int, QString> inn_dosageRef = getAllINNThatHaveRecordedDosages();
-
-//    // get all code_subst from INNs
-//    QHash<int, QString> where;
-//    QString tmp;
-//    QList<int> code_subst;
-//    QSqlDatabase DrugsDB = QSqlDatabase::database(Constants::DB_DRUGS_NAME);
-//    if (!connectDatabase(DrugsDB, __FILE__, __LINE__))
-//        return toReturn;
-
-//    // get all needed datas from database
-//    QMultiHash<QString, minimalCompo> cis_compo;
-//    foreach(int inn, inn_dosageRef.keys()) {
-//        const QVector<int> &molIds = getLinkedMoleculeCodes(inn);
-//        for(int i = 0; i < molIds.count(); ++i) {
-//            int code = molIds.at(i);
-//            if (!code_subst.contains(code))
-//                code_subst << code;
-//        }
-//    }
-//    foreach(int code, code_subst) {
-//        tmp += QString::number(code) + ", ";
-//    }
-//    tmp.chop(2);
-
-//    // Get all Drugs UID that contains the substance + dosage
-//    where.clear();
-//    req.clear();
-//    where.insert(Constants::COMPO_MOL_CODE, QString("IN (%1)").arg(tmp));
-//    req = select(Constants::Table_COMPO,
-//                 QList<int>() << Constants::COMPO_UID
-//                 << Constants::COMPO_MOL_CODE
-//                 << Constants::COMPO_DOSAGE,
-//                 where);
-//    QSqlQuery query(req, DrugsDB);
-//    if (query.isActive()) {
-//        while (query.next()) {
-//            QString cis = query.value(0).toString();
-//            minimalCompo compo;
-//            compo.inn = getLinkedAtcIds(query.value(1).toInt()).at(0);
-//            compo.dosage = query.value(2).toString();
-//            cis_compo.insertMulti(cis, compo);
-//        }
-//    } else {
-//        LOG_QUERY_ERROR(query);
-//    }
-
-//    // now check every drugs
-//    foreach(const QString &uid, cis_compo.uniqueKeys()) {
-//        QList<int> innsOfThisDrug;
-//        foreach(const minimalCompo &compo, cis_compo.values(uid)) {
-//            if (!innsOfThisDrug.contains(compo.inn))
-//                innsOfThisDrug << compo.inn;
-//            QString d = compo.dosage;
-//            foreach(const QString &r, inn_dosageRef.values(compo.inn)) {
-//                // remove unneeded strings
-//                if (d == r)
-//                    toReturn << uid;
-//                else if (d.remove(",000") == r)
-//                    toReturn << uid;
-//                else if (d.remove(",00") == r)
-//                    toReturn << uid;
-
-//                // try unit conversion
-//                if (d.replace("000 mg", " g") == r) {
-//                    toReturn << uid;
-//                }
-//                QString t = r;
-//                if (t.replace("000 mg", " g") == d) {
-//                    toReturn << uid;
-//                }
-//            }
-//        }
-//        if (innsOfThisDrug.count() > 1)
-//            toReturn.removeAll(uid);
-//    }
-//    return toReturn;
-//}
-
-//QMultiHash<int,QString> DrugsBase::getAllINNThatHaveRecordedDosages() const
-//{
-//    QMultiHash<int,QString> toReturn;
-//    QSqlDatabase DB = QSqlDatabase::database(Dosages::Constants::DB_DOSAGES_NAME);
-//    if (!connectDatabase(DB, __FILE__, __LINE__))
-//        return toReturn;
-
-//    QString req;
-//    if (d->m_IsDefaultDB) {
-//        req = QString("SELECT DISTINCT `INN_LK`, `INN_DOSAGE` FROM `DOSAGE` WHERE `DRUGS_DATABASE_IDENTIFIANT` = \"%1\";")
-//              .arg(Constants::DB_DEFAULT_IDENTIFIANT);
-//    } else {
-//        req = QString("SELECT DISTINCT `INN_LK`, `INN_DOSAGE` FROM `DOSAGE` WHERE `DRUGS_DATABASE_IDENTIFIANT` = \"%1\";")
-//              .arg(actualDatabaseInformation()->identifiant);
-//    }
-//    QSqlQuery query(req,DB);
-//    if (query.isActive()) {
-//        while (query.next()) {
-//            toReturn.insertMulti(query.value(0).toInt(), query.value(1).toString());
-//        }
-//    } else {
-//        LOG_QUERY_ERROR(query);
-//    }
-//    return toReturn;
-//}
-
-///////////////////////////////////////////////////////////////////////////////////////////////
-
 //--------------------------------------------------------------------------------------------------------
 //-------------------------------- Retreive drugs from database ------------------------------------------
 //--------------------------------------------------------------------------------------------------------
@@ -1038,43 +670,22 @@ QString DrugsBase::getDrugName(const QString &uid1, const QString &uid2, const Q
     return QString();
 }
 
-/** \brief Retrieve and return the drug according to its uids. */
+/**
+ * Retrieve and return the drug according to its uids.
+ * You can get the first recorded drug using "-1" as first uid: \e uid1.
+ * If you want the Xth available drug from the database just use
+ * the second uid \e uid2
+ * \code
+    // Get the first available drug from the database
+    getDrugByUID("-1");
+    // Get the third
+    getDrugByUID("-1", "3");
+    // Get the tenth
+    getDrugByUID("-1", "10");
+ * \endcode
+ */
 IDrug *DrugsBase::getDrugByUID(const QVariant &uid1, const QVariant &uid2, const QVariant &uid3, const QVariant &oldUid, const QString &srcUid)
 {
-    // Before SID caching
-//    "Chrono - DrugsIO" "1 ms: xxxxx"
-//    "Chrono - DrugsBase" "0 ms: Reading drug"
-//    "Chrono - DrugsBase" "0 ms: Prepare query"
-//    "Chrono - DrugsBase" "60 ms: Query.next()"
-//    "Chrono - DrugsBase" "1 ms: Query.finish"
-//    "Chrono - DrugsBase" "22 ms: Get Compo"
-//    "Chrono - DrugsIO" "95 ms: Reading drug0.30% POTASSIUM CHLORIDE IN 5% DEXTROSE AND 0.45% SODIUM CHLORIDE INJECTION"
-//    "Chrono - DrugsIO" "0 ms: Reading dose0.30% POTASSIUM CHLORIDE IN 5% DEXTROSE AND 0.45% SODIUM CHLORIDE INJECTION"
-//    "Chrono - DrugsBase" "0 ms: Reading drug"
-//    "Chrono - DrugsBase" "1 ms: Prepare query"
-//    "Chrono - DrugsBase" "60 ms: Query.next()"
-//    "Chrono - DrugsBase" "0 ms: Query.finish"
-//    "Chrono - DrugsBase" "9 ms: Get Compo"
-//    "Chrono - DrugsIO" "71 ms: Reading drugALDACTAZIDE 25"
-//    "Chrono - DrugsIO" "0 ms: Reading doseALDACTAZIDE 25"
-
-    // After SID caching
-//    "Chrono - DrugsIO" "1 ms: xxxxx"
-//    "Chrono - DrugsBase" "0 ms: Reading drug"
-//    "Chrono - DrugsBase" "1 ms: Prepare query"
-//    "Chrono - DrugsBase" "20 ms: Query.next()"
-//    "Chrono - DrugsBase" "1 ms: Query.finish"
-//    "Chrono - DrugsBase" "39 ms: Get Compo"
-//    "Chrono - DrugsIO" "71 ms: Reading drug0.30% POTASSIUM CHLORIDE IN 5% DEXTROSE AND 0.45% SODIUM CHLORIDE INJECTION"
-//    "Chrono - DrugsIO" "0 ms: Reading dose0.30% POTASSIUM CHLORIDE IN 5% DEXTROSE AND 0.45% SODIUM CHLORIDE INJECTION"
-//    "Chrono - DrugsBase" "0 ms: Reading drug"
-//    "Chrono - DrugsBase" "0 ms: Prepare query"
-//    "Chrono - DrugsBase" "11 ms: Query.next()"
-//    "Chrono - DrugsBase" "0 ms: Query.finish"
-//    "Chrono - DrugsBase" "4 ms: Get Compo"
-//    "Chrono - DrugsIO" "15 ms: Reading drugALDACTAZIDE 25"
-//    "Chrono - DrugsIO" "0 ms: Reading doseALDACTAZIDE 25"
-
 //    QTime time;
 //    time.start();
 
@@ -1100,6 +711,8 @@ IDrug *DrugsBase::getDrugByUID(const QVariant &uid1, const QVariant &uid2, const
     QString newUid3 = uid3.toString();
     if ((newUid1 == "-1" || newUid1.isEmpty()) &&
             (oldUid.toString().isEmpty() || oldUid.toString() == "-1")) {
+        if (newUid2.isEmpty())
+            newUid2 = "1";
         LOG(tr("Asking for a drug without UID"));
         QHash<int, QString> where;
         where.insert(Constants::MASTER_SID, QString("=%1").arg(d->m_DbUids.value(sourceUid)));
@@ -1107,7 +720,7 @@ IDrug *DrugsBase::getDrugByUID(const QVariant &uid1, const QVariant &uid2, const
                              << Constants::MASTER_UID1
                              << Constants::MASTER_UID2
                              << Constants::MASTER_UID3
-                             , where) + " LIMIT 1";
+                             , where) + QString(" LIMIT %1, 1").arg(newUid2);
         QSqlQuery q(DB);
         if (q.exec(req)) {
             if (q.next()) {
@@ -1143,7 +756,7 @@ IDrug *DrugsBase::getDrugByUID(const QVariant &uid1, const QVariant &uid2, const
     condition << Utils::Field(Constants::Table_DRUGS, Constants::DRUGS_SID, QString("='%1'").arg(d->m_DbUids.value(sourceUid)));
     if (oldUid.toString().isEmpty()) {
         condition << Utils::Field(Constants::Table_MASTER, Constants::MASTER_UID1, QString("='%1'").arg(newUid1));
-        if (!uid2.isNull())
+        if (!newUid2.isNull())
             condition << Utils::Field(Constants::Table_MASTER, Constants::MASTER_UID2, QString("='%1'").arg(newUid2));
         if (!uid3.isNull())
             condition << Utils::Field(Constants::Table_MASTER, Constants::MASTER_UID3, QString("='%1'").arg(newUid3));
@@ -1212,8 +825,8 @@ IDrug *DrugsBase::getDrugByUID(const QVariant &uid1, const QVariant &uid2, const
     // get ROUTES
     d->getRoutes(toReturn);
 
-    if (WarnExtractedDrugs && toReturn)
-        qWarning() << toReturn->warnText();
+    if (WarnExtractedDrugs)
+        qWarning() << toReturn;
 
     return toReturn;
 }
@@ -1383,7 +996,9 @@ QStringList DrugsBase::getFormLabels(const QVariant &drugId, const QString &lang
     QSqlQuery query(req, DB);
     if (query.isActive()) {
         while (query.next()) {
-            toReturn << query.value(0).toString();
+            const QString &form = query.value(0).toString();
+            if (!form.isEmpty())
+                toReturn << form;
         }
     } else {
         LOG_QUERY_ERROR(query);
@@ -1681,6 +1296,39 @@ QVector<int> DrugsBase::getAllMoleculeCodeWithAtcStartingWith(const QString &cod
     return getLinkedMoleculeCodes(atcIds);
 }
 
+/** Return true if the ATC id correspond to an interacting class */
+bool DrugsBase::isInteractingClass(int atcId)
+{
+    return d->m_ClassesId.contains(atcId);
+}
+
+/**
+ * Return the interacting class content (take care that class content can
+ * include interacting classes).
+ */
+QList<int> DrugsBase::interactingClassContent(int classId)
+{
+    return d->m_ClassToAtcs.values(classId);
+}
+
+/**
+ * Return the interacting class content count in pure simple ATC only
+ * (all classes are expanded).
+ */
+int DrugsBase::interactingClassSingleAtcCount(int classId)
+{
+    int n = 0;
+    const QList<int> &content = d->m_ClassToAtcs.values(classId);
+    for(int i=0; i < content.count(); ++i) {
+        int id = content.at(i);
+        if (isInteractingClass(id))
+            n += interactingClassSingleAtcCount(id);
+        else
+            ++n;
+    }
+    return n;
+}
+
 QVector<MedicalUtils::EbmData *> DrugsBase::getAllBibliographyFromTree(const QList<int> &allInnAndInteractingClassesIds)
 {
     QVector<MedicalUtils::EbmData *> ret;
@@ -1705,8 +1353,6 @@ QVector<MedicalUtils::EbmData *> DrugsBase::getAllBibliographyFromTree(const QLi
 
     QString req = select(Constants::Table_BIB, join, where);
 
-    qWarning() << req;
-
     QStringList links;
     QSqlQuery query(req, QSqlDatabase::database(Constants::DB_DRUGS_NAME));
     if (query.isActive()) {
@@ -1727,3 +1373,107 @@ QVector<MedicalUtils::EbmData *> DrugsBase::getAllBibliographyFromTree(const QLi
     }
     return ret;
 }
+
+/**
+ * If available from the database, extracts and returns
+ * absolute filename of the Summary of Product Characteristics (SPC) of the drug.\n
+ * Returns an empty QString if the SPC is not available from the database.\n
+ * The file and all resources files are saved into a specific temporary directory.
+ * This directory has to be deleted. \n
+ * Available since drugs database version 0.8.4.
+ */
+QString DrugsBase::getDrugSpc(const QVariant &drugId)
+{
+#if DRUGS_DATABASE_VERSION < 0x000804
+    Q_UNUSED(drugId);
+    // If the drugs database version is lower than 0.8.4 -> nothing to build
+    return QString::null;
+#else
+    // If the drugs database version is >= than 0.8.4 -> build this part
+    using namespace DrugsDB::Constants;
+    QSqlDatabase DB = QSqlDatabase::database(DB_DRUGS_NAME);
+    if (!connectDatabase(DB, __FILE__, __LINE__))
+        return QString::null;
+
+    // Get SPC content
+    Utils::FieldList get;
+    get << Utils::Field(Table_SPC_CONTENT, SPCCONTENT_SPCCONTENT_RESOURCES_LINK_ID);
+    get << Utils::Field(Table_SPC_CONTENT, SPCCONTENT_HTMLCONTENT);
+
+    Utils::FieldList cond;
+    cond << Utils::Field(Table_DRUG_SPC, DRUG_SPC_DID, QString("='%1'").arg(drugId.toString()));
+
+    Utils::JoinList joins;
+    joins << Utils::Join(Table_SPC_CONTENT, SPCCONTENT_ID, Table_DRUG_SPC, DRUG_SPC_SPCCONTENT_ID);
+    QString req = select(get, joins, cond);
+
+    QSqlQuery query(DB);
+    QString spc;
+    int resourceLinkId = -1;
+    if (query.exec(req)) {
+        if (query.next()) {
+            resourceLinkId = query.value(0).toInt();
+            spc = QString::fromUtf8(qUncompress(query.value(1).toByteArray()));
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
+        return QString::null;
+    }
+    query.finish();
+    get.clear();
+    cond.clear();
+    joins.clear();
+
+    if (spc.isEmpty())
+        return QString::null;
+
+    // Create the temp dir
+    QString path = settings()->path(Core::ISettings::ApplicationTempPath) + "/drugs/spc/";
+    if (!QDir().mkpath(path))
+        LOG_ERROR(QString("Unable to create path: %1").arg(path));
+    QString spcFileName = path + Utils::createUid() + ".html";
+
+    if (resourceLinkId==-1) {
+        // Save SPC file
+        Utils::saveStringToFile(spc, spcFileName, Utils::Overwrite, Utils::DontWarnUser);
+        return spcFileName;
+    }
+
+    // Get Resources content
+    get << Utils::Field(Table_SPC_CONTENTRESOURCE, SPCCONTENTRESOURCES_CONTENT);
+    get << Utils::Field(Table_SPC_CONTENTRESOURCE, SPCCONTENTRESOURCES_NAME);
+
+    cond << Utils::Field(Table_SPC_CONTENTRESOURCE_LINK, SPCCONTENT_RESOURCES_LINK_ID, QString("='%1'").arg(resourceLinkId));
+
+    joins << Utils::Join(Table_SPC_CONTENTRESOURCE_LINK, SPCCONTENT_SPCCONTENTRESOURCES_ID, Table_SPC_CONTENTRESOURCE, SPCCONTENTRESOURCES_ID);
+    req = select(get, joins, cond);
+
+    QHash<QString, QString> resources; //K=name V=content
+    if (query.exec(req)) {
+        while (query.next()) {
+            resources.insert(query.value(1).toString(), query.value(0).toString());
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
+    }
+    query.finish();
+
+    // Save the resources
+    QHashIterator<QString, QString> i(resources);
+    while (i.hasNext()) {
+        i.next();
+        const QString &name = i.key();
+        const QString &content = i.value();
+        QString resourceFileName = path + Utils::createUid();
+
+        spc.replace(name, resourceFileName);
+        Utils::saveStringToFile(content, resourceFileName, Utils::Overwrite, Utils::DontWarnUser);
+    }
+
+    // Save the SPC content
+     Utils::saveStringToFile(spc, spcFileName, Utils::Overwrite, Utils::DontWarnUser);
+
+    return spcFileName;
+#endif
+}
+
