@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,21 +19,24 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
-  \class DrugsModel
-  Model asks DrugsBase to check interaction only while passing new *Drugs via addDrugs()
-  or while passing QDrugsList via setDrugsList().
-  activeModel() , setActiveModel()
-  \todo write code documentation
+ * \class DrugsDB::DrugsModel
+ * Model asks DrugsBase to check interaction only while passing new *Drugs via addDrugs()
+ * or while passing QDrugsList via setDrugsList().
+ * activeModel() , setActiveModel()
+ * \note Unit-test partially available (see: DrugsDB:Internal::DrugsBasePlugin)
 */
+// TODO: write code documentation
 
 #include "drugsmodel.h"
 
+#include <drugsbaseplugin/constants.h>
+#include <drugsbaseplugin/constants_html.h>
 #include <drugsbaseplugin/drugbasecore.h>
 #include <drugsbaseplugin/drugsbase.h>
 #include <drugsbaseplugin/protocolsbase.h>
@@ -43,7 +46,6 @@
 #include <drugsbaseplugin/idrug.h>
 #include <drugsbaseplugin/drugsio.h>
 #include <drugsbaseplugin/interactionmanager.h>
-#include <drugsbaseplugin/constants.h>
 #include <drugsbaseplugin/dailyschememodel.h>
 #include <drugsbaseplugin/globaldrugsmodel.h>
 #include <drugsbaseplugin/druginteractioninformationquery.h>
@@ -75,6 +77,7 @@
 #include <QDomDocument>
 #include <QDir>
 
+enum { WarnDrugPosologicSentencePlainTextCache = false };
 
 namespace {
     const char * const ALD_BACKGROUND_COLOR               = "khaki";
@@ -90,6 +93,7 @@ static inline Core::ITheme *theme()  { return Core::ICore::instance()->theme();
 static inline DrugsDB::InteractionManager &interactionManager() {return DrugsDB::DrugBaseCore::instance().interactionManager();}
 static inline DrugsDB::DrugsBase &drugsBase() {return DrugsDB::DrugBaseCore::instance().drugsBase();}
 static inline DrugsDB::ProtocolsBase &protocolsBase() {return DrugsDB::DrugBaseCore::instance().protocolsBase();}
+static inline DrugsDB::DrugsIO &drugsIo() {return DrugsDB::DrugBaseCore::instance().drugsIo();}
 
 DrugsDB::DrugsModel *DrugsDB::DrugsModel::m_ActiveModel = 0;
 
@@ -99,10 +103,11 @@ namespace Internal {
 class DrugsModelPrivate
 {
 public:
-    DrugsModelPrivate() :
+    DrugsModelPrivate(DrugsModel *parent) :
         m_LastDrugRequiered(0), m_ShowTestingDrugs(true),
         m_SelectionOnlyMode(false), m_IsDirty(false),
-        m_InteractionResult(0), m_AllergyEngine(0), m_ComputeInteraction(true)
+        m_InteractionResult(0), m_AllergyEngine(0), m_ComputeInteraction(true),
+        q(parent)
     {
     }
 
@@ -146,25 +151,32 @@ public:
         if (column == Constants::Drug::Denomination) {
             if (textualdrug) {
                 textualdrug->setDenomination(value.toString());
-                m_IsDirty = true;
+                if (WarnDrugPosologicSentencePlainTextCache)
+                    LOG_FOR(q, QString("[DrugCache] Removing textual drug %1").arg(drug->brandName()));
+                _plainTextPosologicSentenceCache.remove(drug);
                 return true;
             } else {
                 return false;
             }
         }
-        if ((column < Constants::Prescription::Id) || (column > Constants::Prescription::MaxParam))
+        if ((column < Constants::Prescription::Id) || (column > Constants::Prescription::MaxParam)) {
+            if (WarnDrugPosologicSentencePlainTextCache)
+                LOG_FOR(q, QString("[DrugCache] Removing drug %1").arg(drug->brandName()));
+            _plainTextPosologicSentenceCache.remove(drug);
             return false;
+        }
         if (column == Constants::Prescription::Note) {
             drug->setPrescriptionValue(column, value.toString().replace("[","{").replace("]","}"));
-            m_IsDirty = true;
         } else {
 //            // Avoid modification of IsTextualOnly tag on TextualDrugs
 //            if (textualdrug && column==Constants::Prescription::IsTextualOnly) {
 //                return false;
 //            }
             drug->setPrescriptionValue(column, value);
-            m_IsDirty = true;
         }
+        if (WarnDrugPosologicSentencePlainTextCache)
+            LOG_FOR(q, QString("[DrugCache] Removing %1").arg(drug->brandName()));
+        _plainTextPosologicSentenceCache.remove(drug);
         return true;
     }
 
@@ -202,7 +214,7 @@ public:
         {
         case Drug::Denomination :       return drug->brandName();
         case Drug::DrugId :             return drug->drugId();
-        case Drug::UIDs :               return drug->uids().join(";");
+        case Drug::UIDs :               return drug->uids();
         case Drug::UidName:             return drug->drugId();
         case Drug::Form :               return drug->forms().join(", ");
         case Drug::Route :              return drug->routes().join(", ");
@@ -242,7 +254,8 @@ public:
         case Drug::AvailableForms :
             {
                 QStringList toReturn;
-                toReturn << drug->forms();
+                if (!drug->forms().isEmpty())
+                    toReturn << drug->forms();
                 toReturn << tkTr(Trans::Constants::INTAKES);
                 if (drug->numberOfInn() == 1) {
                     toReturn << QApplication::translate("DrugsModel", "x %1 of %2")
@@ -266,10 +279,10 @@ public:
                     return QVariant();
                 if (pres->prescriptionValue(Prescription::OnlyForTest).toBool() || m_SelectionOnlyMode) {
                     if (pres->prescriptionValue(Prescription::IsINNPrescription).toBool())
-                        return pres->innComposition() + " [" + tkTr(Trans::Constants::INN) + "]";
+                        return QString("%1 [%2]").arg(pres->innComposition()).arg(tkTr(Trans::Constants::INN));
                     else return pres->brandName();
                 }
-                return ::DrugsDB::DrugsModel::getFullPrescription(drug,false);
+                return q->getFullPrescription(drug, false);
             }
         case Drug::OwnInteractionsSynthesis:
             {
@@ -287,7 +300,7 @@ public:
         if (!drug)
             return QVariant();
         if (column ==  Prescription::ToHtml) {
-            return ::DrugsDB::DrugsModel::getFullPrescription(drug, true);
+            return q->getFullPrescription(drug, true);
         } else {
             return drug->prescriptionValue(column);
         }
@@ -313,7 +326,15 @@ public:
                     return m_InteractionResult->alertMessagesToHtml(drug, query);
 //                    display.append(interactionManager().listToHtml(m_InteractionResult->getInteractions(drug), false));
                 } else if (drug->data(IDrug::AllInnsKnown).toBool()) {
-                    display = drug->listOfInnLabels().join("<br />") + "<br />" + drug->listOfInteractingClasses().join("<br />");
+                    display = QString("<b>%1:</b><br/>•&nbsp;%2<br/><br/><b>%3:</b>•&nbsp;%4")
+                            .arg(tkTr(Trans::Constants::INN))
+                            .arg(drug->listOfInnLabels().join("<br/>•&nbsp;").replace(" ", "&nbsp;"))
+                            .arg(tkTr(Trans::Constants::INTERACTING_CLASSES))
+                            .arg(drug->listOfInteractingClasses().join("<br/>•&nbsp;").replace(" ", "&nbsp;"));
+                    // display = QString("<ul %3><li %3>%1</ul><ul %3><li %3>%2</ul>")
+                    // .arg(drug->listOfInnLabels().join("<li>"))
+                    // .arg(drug->listOfInteractingClasses().join("<li>"))
+                    // .arg("style=\"padding:0px;margin:0px;}\"");
                 } else {
                     display = tkTr(Trans::Constants::NO_1_FOUND).arg(tkTr(Trans::Constants::INN));
                 }
@@ -368,6 +389,10 @@ public:
     DrugInteractionQuery *m_InteractionQuery;
     IDrugAllergyEngine *m_AllergyEngine;
     bool m_ComputeInteraction;
+    QHash<const IDrug*, QString> _plainTextPosologicSentenceCache;
+
+private:
+    DrugsModel *q;
 };
 }  // End Internal
 }  // End DrugsDB
@@ -377,7 +402,7 @@ using namespace DrugsDB;
 /** Constructor */
 DrugsModel::DrugsModel(QObject * parent) :
     QAbstractTableModel(parent),
-    d(new Internal::DrugsModelPrivate)
+    d(new Internal::DrugsModelPrivate(this))
 {
     static int handler = 0;
     ++handler;
@@ -419,7 +444,9 @@ void DrugsModel::dosageDatabaseChanged()
 /** count the number of selected drugs */
 int DrugsModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
+    // prevent trees
+    if (parent.isValid()) return 0;
+
     return d->m_DrugsList.count();
 }
 
@@ -434,7 +461,7 @@ QModelIndex DrugsModel::index(int row, int column, const QModelIndex &parent) co
 /**
   Defines the data of drugs.
   The drugs model is a read only model for all the namespace mfDrugsConstants::Drug enum values.\n
-  Dosages values are not accessible from here. Use the mfDosageModel.\n
+  Dosages values are not accessible from here. Use the DosageModel.\n
   Prescritions values are writables. Information are transmitted using DrugsDB::setPrescriptionValue().
 */
 bool DrugsModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -447,18 +474,17 @@ bool DrugsModel::setData(const QModelIndex &index, const QVariant &value, int ro
         return false;
     IDrug *drug = d->m_DrugsList.at(row);
     if (d->setDrugData(drug, index.column(), value)) {
+        d->m_IsDirty = true;
         Q_EMIT dataChanged(index, index);
-        QModelIndex fullPrescr = this->index(index.row(), Constants::Drug::FullPrescription);
-        Q_EMIT dataChanged(fullPrescr, fullPrescr);
-        Q_EMIT prescriptionResultChanged(getFullPrescription(drug,false));
+        return true;
     }
-    return true;
+    return false;
 }
 
 /**
   Set data for the specified drug with the corresponding \e CIS.
   Drug must be set into the model otherwise, this function returns false.\n
-  If you want the model to be refreshed call resetModel() after all datas were set.
+  If you want the model to be refreshed, call resetModel() after all data were set.
 */
 bool DrugsModel::setDrugData(const QVariant &drugId, const int column, const QVariant &value)
 {
@@ -466,25 +492,36 @@ bool DrugsModel::setDrugData(const QVariant &drugId, const int column, const QVa
     if (!drug)
         return false;
     if (d->setDrugData(drug, column, value)) {
-        Q_EMIT prescriptionResultChanged(getFullPrescription(drug,false));
+        d->m_IsDirty = true;
+        QModelIndex index = this->index(d->m_DrugsList.indexOf(drug), column);
+        Q_EMIT dataChanged(index, index);
         return true;
     }
     return false;
 }
 
+bool DrugsModel::submit()
+{
+    bool ok = QAbstractTableModel::submit();
+    return ok;
+}
+
 /** Reset the model */
 void DrugsModel::resetModel()
 {
-    reset();
+    beginResetModel();
+    d->_plainTextPosologicSentenceCache.clear();
+    d->m_IsDirty = false;
+    endResetModel();
 }
 
 /**
   Returns the values of drugs and prescription, you can not access to the dosage model this way.
   DosageModel regarding a specific drug is accessible using dosageModel().\n
-  Available datas index :
-  \li Drugs specific datas : see DrugsDB::Constants::Drug enumerator
-  \li Interactions specific datas : see DrugsDB::Constants::Interaction enumerator
-  \li Prescription datas : see DrugsDB::Constants::Prescription enumerator
+  Available data index:
+  \li Drugs specific data: see DrugsDB::Constants::Drug enumerator
+  \li Interactions specific data: see DrugsDB::Constants::Interaction enumerator
+  \li Prescription data: see DrugsDB::Constants::Prescription enumerator
 */
 QVariant DrugsModel::data(const QModelIndex &index, int role) const
 {
@@ -613,12 +650,13 @@ bool DrugsModel::removeRows(int row, int count, const QModelIndex &parent)
 /** Add a textual drug to the prescription. \sa DrugsWidget::TextualPrescriptionDialog, DrugsWidget::Internal::DrugSelector */
 int DrugsModel::addTextualPrescription(const QString &drugLabel, const QString &drugNote)
 {
+    beginResetModel();
     ITextualDrug *drug = new ITextualDrug();
     drug->setDenomination(drugLabel);
     drug->setPrescriptionValue(Constants::Prescription::Note, drugNote);
     d->m_DrugsList << drug;
-    reset();
     d->m_IsDirty = true;
+    endResetModel();
     Q_EMIT numberOfRowsChanged();
     return d->m_DrugsList.indexOf(drug);
 }
@@ -663,12 +701,14 @@ int DrugsModel::addDrugs(const QVector<IDrug *> &drugs, bool automaticInteractio
     d->m_InteractionQuery->setDrugsList(d->m_DrugsList.toVector());
     if (automaticInteractionChecking)
         checkInteractions();
+    d->m_IsDirty = true;
     return drugs.count();
 }
 
 /** Clear the prescription. Clear all interactions too. Calling this causes a model reset. */
 void DrugsModel::clearDrugsList()
 {
+    beginResetModel();
     d->m_LastDrugRequiered = 0;
     qDeleteAll(d->m_DrugsList);
     d->m_DrugsList.clear();
@@ -676,9 +716,12 @@ void DrugsModel::clearDrugsList()
     d->m_TestingDrugsList.clear();
     d->m_InteractionQuery->clearDrugsList();
     d->m_InteractionResult->clear();
+    d->_plainTextPosologicSentenceCache.clear();
+    if (WarnDrugPosologicSentencePlainTextCache)
+        LOG(QString("[DrugCache] Clear cache"));
     d->m_levelOfWarning = settings()->value(Constants::S_LEVELOFWARNING_STATICALERT).toInt();
-    reset();
     d->m_IsDirty = true;
+    endResetModel();
     Q_EMIT numberOfRowsChanged();
 }
 
@@ -755,8 +798,9 @@ bool DrugsModel::prescriptionHasAllergies()
 /** Sort the drugs inside prescription. \sa DrugsDB::lessThan(). Calling this causes a model reset. */
 void DrugsModel::sort(int, Qt::SortOrder)
 {
+    beginResetModel();
     qSort(d->m_DrugsList.begin(), d->m_DrugsList.end(), IDrug::lessThan);
-    reset();
+    endResetModel();
 }
 
 /** Moves a drug up. Calling this causes a model reset. */
@@ -766,8 +810,9 @@ bool DrugsModel::moveUp(const QModelIndex &item)
         return false;
 
     if (item.row() >= 1) {
+        beginResetModel();
         d->m_DrugsList.move(item.row(), item.row()-1);
-        reset();
+        endResetModel();
         return true;
     }
     return false;
@@ -780,8 +825,9 @@ bool DrugsModel::moveDown(const QModelIndex &item)
         return false;
 
     if (item.row() < (rowCount()-1)) {
+        beginResetModel();
         d->m_DrugsList.move(item.row(), item.row()+1);
-        reset();
+        endResetModel();
         return true;
     }
     return false;
@@ -817,8 +863,9 @@ bool DrugsModel::testingDrugsAreVisible() const
 
 void DrugsModel::setSelectionOnlyMode(bool b)
 {
+    beginResetModel();
     d->m_SelectionOnlyMode = b;
-    reset();
+    endResetModel();
 }
 
 bool DrugsModel::isSelectionOnlyMode() const
@@ -840,14 +887,6 @@ bool DrugsModel::isModified() const
 /** Returns the dosage model for the selected drug */
 Internal::DosageModel *DrugsModel::dosageModel(const QVariant &drugId)
 {
-//    if (! d->m_DosageModelList.keys().contains(uid)) {
-//        d->m_DosageModelList.insert(uid, new Internal::DosageModel(this));
-//        d->m_DosageModelList[uid]->setDrugUID(uid);
-//    } else if (! d->m_DosageModelList.value(uid)) {
-//        d->m_DosageModelList.insert(uid, new Internal::DosageModel(this));
-//        d->m_DosageModelList[uid]->setDrugUID(uid);
-//    }
-//    return d->m_DosageModelList.value(uid);
     Internal::DosageModel *m = new Internal::DosageModel(this);
     m->setDrugId(drugId);
     return m;
@@ -873,6 +912,9 @@ int DrugsModel::removeDrug(const QVariant &drugId)
     foreach(IDrug * drug, d->m_DrugsList) {
         if (drug->drugId() == drugId) {
             d->m_DrugsList.removeAt(d->m_DrugsList.indexOf(drug));
+            if (WarnDrugPosologicSentencePlainTextCache)
+                LOG(QString("[DrugCache] Removing drug %1 (drug removed)").arg(drug->brandName()));
+            d->_plainTextPosologicSentenceCache.remove(drug);
             delete drug;
             ++i;
         } else {
@@ -892,6 +934,9 @@ int DrugsModel::removeLastInsertedDrug()
     d->m_LastDrugRequiered = 0;
     if (d->m_DrugsList.count() == 0)
         return 0;
+    if (WarnDrugPosologicSentencePlainTextCache)
+        LOG(QString("[DrugCache] Removing drug %1 (drug removed)").arg(d->m_DrugsList.last()->brandName()));
+    d->_plainTextPosologicSentenceCache.remove(d->m_DrugsList.last());
     delete d->m_DrugsList.last();
     d->m_DrugsList.removeLast();
     d->m_InteractionQuery->setDrugsList(d->m_DrugsList.toVector());
@@ -905,143 +950,45 @@ int DrugsModel::removeLastInsertedDrug()
 void DrugsModel::checkInteractions()
 {
     if (!d->m_ComputeInteraction) {
-        reset();
+        beginResetModel();
+        endResetModel();
         return;
     }
     if (d->m_InteractionResult)
         delete d->m_InteractionResult;
+    beginResetModel();
     d->m_InteractionResult = interactionManager().checkInteractions(*d->m_InteractionQuery, this);
-    reset();
+    d->m_IsDirty = true;
+    endResetModel();
 }
 
+/** \brief Transform a prescription (one drug) to a readable output.
+ * Item is translated to HTML using the token manager and
+ * the drug posologic sentence passed as param (\e mask). If the \e mask is
+ * empty, a new one is extracted from the user settings (drug posologic sentence plaintext or HTML). \n
+ * You can choose to output an HTML string or a plain text string \e toHtml.\n
+ * \sa DrugDB::DrugsIO::getDrugPrescription()
+ */
 QString DrugsModel::getFullPrescription(const IDrug *drug, bool toHtml, const QString &mask)
 {
-    QString tmp;
-    if (mask.isEmpty()) {
-        if (!toHtml)
-            tmp = settings()->value(Constants::S_PRESCRIPTIONFORMATTING_PLAIN).toString();
-        else
-            tmp = settings()->value(Constants::S_PRESCRIPTIONFORMATTING_HTML).toString();
+    if (!toHtml && d->_plainTextPosologicSentenceCache.contains(drug)) {
+        if (WarnDrugPosologicSentencePlainTextCache)
+            LOG(QString("[DrugCache] Returning from cache: drug %1; \n  %2")
+                .arg(drug->brandName())
+                .arg(d->_plainTextPosologicSentenceCache.value(drug)));
+        return d->_plainTextPosologicSentenceCache.value(drug);
     }
-    else
-        tmp = mask;
-
-    // Insert here __all__ tokens to the hash
-    QHash<QString, QString> tokens_value;
-    tokens_value.insert("DRUG", "");
-    tokens_value.insert("Q_FROM", "");
-    tokens_value.insert("Q_TO", "");
-    tokens_value.insert("Q_SCHEME", "");
-    tokens_value.insert("DAILY_SCHEME", "");
-    tokens_value.insert("REPEATED_DAILY_SCHEME", "");
-    tokens_value.insert("DISTRIBUTED_DAILY_SCHEME", "");
-    tokens_value.insert("PERIOD", "");
-    tokens_value.insert("PERIOD_SCHEME", "");
-    tokens_value.insert("ROUTE", "");
-    tokens_value.insert("D_FROM", "");
-    tokens_value.insert("D_TO", "");
-    tokens_value.insert("D_SCHEME", "");
-    tokens_value.insert("MEAL", "");
-    tokens_value.insert("NOTE", "");
-    tokens_value.insert("MIN_INTERVAL", "");
-    tokens_value.insert("MIN_INTERVAL_SCHEME", "");
-
-    // Manage Textual drugs only
-    if (drug->prescriptionValue(Constants::Prescription::IsTextualOnly).toBool()) {
-        if (toHtml) {
-            tokens_value["DRUG"] = drug->brandName().replace("\n","<br />");
-            tokens_value["NOTE"] = drug->prescriptionValue(Constants::Prescription::Note).toString().replace("\n","<br />");
-        } else {
-            tokens_value["DRUG"] = drug->brandName();
-            tokens_value["NOTE"] = drug->prescriptionValue(Constants::Prescription::Note).toString();
-        }
-        Utils::replaceTokens(tmp, tokens_value);
-        return tmp;
+    QString result;
+    result = drugsIo().getDrugPrescription(this, d->m_DrugsList.indexOf((IDrug*)drug), toHtml, mask);
+    if (!toHtml &&
+            mask != Constants::PROTOCOL_AUTOMATIC_LABEL_MASK) {
+        if (WarnDrugPosologicSentencePlainTextCache)
+            LOG(QString("[DrugCache] Inserting in cache: drug %1; \n  %2")
+                .arg(drug->brandName())
+                .arg(result));
+        d->_plainTextPosologicSentenceCache.insert(drug, result);
     }
-
-    // Manage full prescriptions
-    if (drug->prescriptionValue(Constants::Prescription::IsINNPrescription).toBool()) {
-        tokens_value["DRUG"] = drug->innComposition() + " [" + tkTr(Trans::Constants::INN) + "]";
-    } else {
-        tokens_value["DRUG"] =  drug->brandName();
-    }
-
-    if (drug->prescriptionValue(Constants::Prescription::IntakesFrom).toDouble()) {
-        tokens_value["Q_FROM"] = QString::number(drug->prescriptionValue(Constants::Prescription::IntakesFrom).toDouble());
-        if (drug->prescriptionValue(Constants::Prescription::IntakesUsesFromTo).toBool())
-            tokens_value["Q_TO"] = QString::number(drug->prescriptionValue(Constants::Prescription::IntakesTo).toDouble());
-
-        tokens_value["Q_SCHEME"] = drug->prescriptionValue(Constants::Prescription::IntakesScheme).toString();
-    } else {
-        tokens_value["Q_FROM"] = tokens_value["Q_TO"] = tokens_value["Q_SCHEME"] = "";
-    }
-
-    // Manage Daily Scheme See DailySchemeModel::setSerializedContent
-    DrugsDB::DailySchemeModel *day = new DrugsDB::DailySchemeModel;
-    day->setSerializedContent(drug->prescriptionValue(Constants::Prescription::DailyScheme).toString());
-    tokens_value["REPEATED_DAILY_SCHEME"] = day->humanReadableRepeatedDailyScheme();
-    tokens_value["DISTRIBUTED_DAILY_SCHEME"] = day->humanReadableDistributedDailyScheme();
-    delete day;
-    if (tokens_value.value("REPEATED_DAILY_SCHEME").isEmpty()) {
-        tokens_value["DAILY_SCHEME"] = tokens_value.value("DISTRIBUTED_DAILY_SCHEME");
-    } else {
-        tokens_value["DAILY_SCHEME"] = tokens_value.value("REPEATED_DAILY_SCHEME");
-    }
-
-    // Duration
-    if (drug->prescriptionValue(Constants::Prescription::DurationFrom).toDouble()) {
-        tokens_value["D_FROM"] = QString::number(drug->prescriptionValue(Constants::Prescription::DurationFrom).toDouble());
-        if (drug->prescriptionValue(Constants::Prescription::DurationUsesFromTo).toBool())
-            tokens_value["D_TO"] = QString::number(drug->prescriptionValue(Constants::Prescription::DurationTo).toDouble());
-
-        // Manage plurial form
-        tokens_value["PERIOD_SCHEME"] = drug->prescriptionValue(Constants::Prescription::PeriodScheme).toString();
-        tokens_value["D_SCHEME"] = drug->prescriptionValue(Constants::Prescription::DurationScheme).toString();
-        int max = qMax(drug->prescriptionValue(Constants::Prescription::DurationFrom).toDouble(), drug->prescriptionValue(Constants::Prescription::DurationTo).toDouble());
-        if (periods().contains(tokens_value["D_SCHEME"])) {
-            tokens_value["D_SCHEME"] = periodPlurialForm(periods().indexOf(tokens_value["D_SCHEME"]), max, tokens_value["D_SCHEME"]);
-        }
-        max = drug->prescriptionValue(Constants::Prescription::Period).toDouble();
-        if (max==1 && QLocale().name().left(2)=="fr")
-            ++max;
-        if (periods().contains(tokens_value["PERIOD_SCHEME"])) {
-            tokens_value["PERIOD_SCHEME"] = periodPlurialForm(periods().indexOf(tokens_value["PERIOD_SCHEME"]), max, tokens_value["PERIOD_SCHEME"]);
-        }
-    } else {
-        tokens_value["PERIOD_SCHEME"] = tokens_value["D_FROM"] = tokens_value["D_TO"] = tokens_value["D_SCHEME"] = "";
-    }
-
-    tokens_value["MEAL"] = Trans::ConstantTranslations::mealTime(drug->prescriptionValue(Constants::Prescription::MealTimeSchemeIndex).toInt());
-    QString tmp2 = drug->prescriptionValue(Constants::Prescription::Period).toString();
-
-    // TODO: provide a better management of 'EACH DAY__S__' here .. \sa Translation::periodPlurialForm()
-    // Period management of 'EACH DAY__S'
-    if (tmp2 == "1") {
-        tmp2.clear();
-    }
-    tokens_value["PERIOD"] = tmp2;
-
-    if (toHtml) {
-        tokens_value["NOTE"] = drug->prescriptionValue(Constants::Prescription::Note).toString().replace("\n","<br />");
-    } else {
-        tokens_value["NOTE"] = drug->prescriptionValue(Constants::Prescription::Note).toString();
-    }
-
-    // Min interval
-    const QVariant &interval = drug->prescriptionValue(Constants::Prescription::IntakesIntervalOfTime);
-    const QVariant &intervalScheme = drug->prescriptionValue(Constants::Prescription::IntakesIntervalScheme);
-    if ((!interval.isNull() && !intervalScheme.isNull()) &&
-        interval.toInt() > 0) {
-        tokens_value["MIN_INTERVAL"] = interval.toString() + " " + periodPlurialForm(intervalScheme.toInt(), interval.toInt());
-    }
-
-    // Route
-    tokens_value["ROUTE"] = drug->prescriptionValue(Constants::Prescription::Route).toString();
-
-    Utils::replaceTokens(tmp, tokens_value);
-    if (toHtml)
-        tmp = Utils::toHtmlAccent(tmp);
-    return tmp;
+    return result;
 }
 
 Qt::DropActions DrugsModel::supportedDropActions() const
@@ -1094,10 +1041,9 @@ bool DrugsModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int
         if (model->isCategory(idx))
             continue;
         // add content to model
-        DrugsDB::DrugsIO io;
-        io.prescriptionFromXml(this, model->index(idx.row(), Templates::Constants::Data_Content, idx.parent()).data().toString(), DrugsDB::DrugsIO::AppendPrescription);
+        drugsIo().prescriptionFromXml(this, model->index(idx.row(), Templates::Constants::Data_Content, idx.parent()).data().toString(), DrugsDB::DrugsIO::AppendPrescription);
     }
-
+    d->m_IsDirty = true;
     // never move templates but copy them
     if (action == Qt::MoveAction)
         return false;
@@ -1105,15 +1051,15 @@ bool DrugsModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int
     return true;
 }
 
-QDebug operator<<(QDebug dbg, const DrugsDB::DrugsModel *c)
-{
-    if (!c) {
-        dbg.nospace() << "DrugsModel(0x0)";
-        return dbg.space();
-    }
-    dbg.nospace() << "DrugsModel("
-                  << "Memory: " << c->drugsList().count()
-                  << ")";
-    return dbg.space();
-}
+//QDebug operator<<(QDebug dbg, const DrugsDB::DrugsModel *c)
+//{
+//    if (!c) {
+//        dbg.nospace() << "DrugsModel(0x0)";
+//        return dbg.space();
+//    }
+//    dbg.nospace() << "DrugsModel("
+//                  << "Memory: " << c->drugsList().count()
+//                  << ")";
+//    return dbg.space();
+//}
 
