@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -59,7 +59,12 @@
 #include <QStandardItemModel>
 #include <QSqlQueryModel>
 
+#if QT_VERSION < 0x050000
 #include <qtconcurrentrun.h>
+#else
+#include <QtConcurrent/QtConcurrent>
+#endif
+
 #include <QFuture>
 #include <QFutureWatcher>
 
@@ -293,11 +298,8 @@ GlobalDrugsModel::GlobalDrugsModel(const SearchMode searchMode, QObject *parent)
     d->updateCachedAvailableDosage();
     d->setQueryModelSearchMode(searchMode);
 
-    refreshDrugsPrecautions(patient()->index(0, Core::IPatient::DrugsAllergiesWithoutPrecision), patient()->index(0, Core::IPatient::DrugsAllergiesWithoutPrecision));
-
     connect(&protocolsBase(), SIGNAL(protocolsBaseHasChanged()), this, SLOT(updateCachedAvailableDosage()));
     connect(&drugsBase(), SIGNAL(drugsBaseHasChanged()), this, SLOT(onDrugsDatabaseChanged()));
-    connect(patient(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(refreshDrugsPrecautions(QModelIndex, QModelIndex)));
     connect(translators(), SIGNAL(languageChanged()), this, SLOT(onDrugsDatabaseChanged()));
     if (d->m_AllergyEngine) {
         connect(d->m_AllergyEngine, SIGNAL(allergiesUpdated()), this, SLOT(updateAllergies()));
@@ -422,7 +424,7 @@ QVariant GlobalDrugsModel::data(const QModelIndex &item, int role) const
         //        }
 
         // Create tooltip
-        tmp += QString("<table border=1 cellpadding=2 cellspacing=2 width=100%>"
+        tmp += QString("<table border=0 cellpadding=2 cellspacing=2 width=100%>"
                         " <tr>"
                         "   <td colspan=2 rowspan=1 align=center>"
                         "       <span style=\"font-weight: bold;\">%1</span>"
@@ -507,5 +509,6 @@ void GlobalDrugsModel::updateCachedAvailableDosage()
 
 void GlobalDrugsModel::updateAllergies()
 {
-    reset();
+    beginResetModel();
+    endResetModel();
 }
