@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -115,7 +115,9 @@ namespace Internal {
 class ProtocolsBasePrivate
 {
 public:
-    ProtocolsBasePrivate() : m_initialized(false) {}
+    ProtocolsBasePrivate() :
+        m_initialized(false)
+    {}
 
 public:
     bool m_initialized;
@@ -213,10 +215,15 @@ ProtocolsBase::~ProtocolsBase()
     d = 0;
 }
 
+bool ProtocolsBase::isInitialized() const
+{
+    return d->m_initialized;
+}
+
 void ProtocolsBase::onCoreFirstRunCreationRequested()
 {
     disconnect(Core::ICore::instance(), SIGNAL(firstRunDatabaseCreation()), this, SLOT(onCoreFirstRunCreationRequested()));
-    init();
+    initialize();
 }
 
 QString ProtocolsBase::dosageCreateTableSqlQuery()
@@ -286,7 +293,7 @@ void ProtocolsBase::forceReinitialization()
 }
 
 /** \brief Initializer for the database. Return the error state. */
-bool ProtocolsBase::init()
+bool ProtocolsBase::initialize()
 {
     // only one base can be initialized
     if (d->m_initialized)
@@ -523,7 +530,7 @@ QList<QVariant> ProtocolsBase::getAllUIDThatHaveRecordedDosages() const
     if (!connectDatabase(DrugsDB, __FILE__, __LINE__))
         return toReturn;
 
-    // get all needed datas from database
+    // get all needed data from database
     foreach(int inn, inn_dosageRef.keys()) {
         const QVector<int> &molIds = drugsBase().getLinkedMoleculeCodes(inn);
         for(int i = 0; i < molIds.count(); ++i) {
@@ -620,7 +627,7 @@ bool ProtocolsBase::onCoreDatabaseServerChanged()
     d->m_initialized = false;
 //    d->m_RefreshDosageBase = true;
     Q_EMIT protocolsBaseIsAboutToChange();
-    bool r = init();
+    bool r = initialize();
     if (r)
         Q_EMIT protocolsBaseHasChanged();
     return r;
