@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -35,7 +35,6 @@
   - If one of these process updates the prescription, the version is set to the toVersion().
   - You can not update either from the XML content and from the Model.
   \sa DrugsDb::DrugsIO
-  \ingroup freediams drugswidget
 */
 
 #include "versionupdater.h"
@@ -1019,7 +1018,7 @@ bool VersionUpdater::updateDosageDatabase()
             return false;
         }
         if (!step->saveUpdatedValuesToDatabase()) {
-            Utils::Log::addError("VersionUpdater", QString("Error while updating %1 from %2 to %3 : %4")
+            Utils::Log::addError("VersionUpdater", QString("Error while updating %1 from %2 to %3: %4")
                                  .arg(Dosages::Constants::DB_DOSAGES_NAME, step->fromVersion(), step->toVersion()),
                                  __FILE__, __LINE__);
             return false;
