@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -70,7 +70,7 @@ static inline DrugsDB::DrugsBase &drugsBase() {return DrugsDB::DrugBaseCore::ins
 static inline Core::ISettings *settings() {return Core::ICore::instance()->settings();}
 static inline Core::ITheme *theme() {return Core::ICore::instance()->theme();}
 static inline Core::ActionManager *actionManager() { return Core::ICore::instance()->actionManager(); }
-inline static DrugsDB::DrugsModel *drugModel() { return DrugsWidget::DrugsWidgetManager::instance()->currentDrugsModel(); }
+static inline DrugsDB::DrugsModel *drugModel() { return DrugsWidget::DrugsWidgetManager::instance()->currentDrugsModel(); }
 static inline Core::IMainWindow *mainWindow() {return Core::ICore::instance()->mainWindow();}
 
 namespace DrugsWidget {
@@ -117,7 +117,7 @@ DrugSelector::DrugSelector(QWidget *parent) :
         QWidget(parent),
         m_GlobalDrugsModel(0), m_InnModel(0), m_AtcProxyModel(0),
         m_SearchToolButton(0),
-        m_DrugsHistoricButton(0),
+        m_DrugsHistoryButton(0),
         m_HistoryAct(0)
 {
     setupUi(this);
@@ -194,14 +194,14 @@ void DrugSelector::createToolButtons()
     m_SearchToolButton->setPopupMode(QToolButton::InstantPopup);
     m_SearchToolButton->setIcon(theme()->icon(Core::Constants::ICONSEARCH));
 
-    m_DrugsHistoricButton = new QToolButton(searchLine);
-    m_DrugsHistoricButton->setPopupMode(QToolButton::InstantPopup);
-    m_DrugsHistoricButton->setToolTip(tr("Selected drugs historic."));
-    m_DrugsHistoricButton->setIcon(theme()->icon(Core::Constants::ICONEDIT));
+    m_DrugsHistoryButton = new QToolButton(searchLine);
+    m_DrugsHistoryButton->setPopupMode(QToolButton::InstantPopup);
+    m_DrugsHistoryButton->setToolTip(tr("Selected drugs history"));
+    m_DrugsHistoryButton->setIcon(theme()->icon(Core::Constants::ICONEDIT));
 
     // add buttons to search line
     searchLine->setLeftButton(m_SearchToolButton);
-    searchLine->setRightButton(m_DrugsHistoricButton);
+    searchLine->setRightButton(m_DrugsHistoryButton);
 
     refreshSearchToolButton();
     refreshAvailableDrugsDatabaseButtons();
@@ -302,7 +302,24 @@ void DrugSelector::updateDrugsViewColumns()
     drugsView->setColumnHidden(DrugsDB::GlobalDrugsModel::Forms , !settings()->value(DrugsDB::Constants::S_SELECTOR_SHOWFORM).toBool());
     drugsView->setColumnHidden(DrugsDB::GlobalDrugsModel::Strength , !settings()->value(DrugsDB::Constants::S_SELECTOR_SHOWSTRENGTH).toBool());
     drugsView->horizontalHeader()->setStretchLastSection(false);
+#if QT_VERSION < 0x050000
     drugsView->horizontalHeader()->setResizeMode(DrugsDB::GlobalDrugsModel::BrandName, QHeaderView::Stretch);
+#else
+    // Qt5
+    drugsView->horizontalHeader()->setSectionResizeMode(DrugsDB::GlobalDrugsModel::BrandName, QHeaderView::Stretch);
+#endif
+}
+
+/**
+ * Clear the view:
+ * - clear the search line edit,
+ * - keep the search method (brand name, INN, molecule search)
+ * - reset the search filter
+ */
+void DrugSelector::clear()
+{
+    searchLine->clear();
+    updateModelFilter();
 }
 
 void DrugSelector::createINNModelView()
@@ -318,7 +335,12 @@ void DrugSelector::createINNModelView()
 
     //InnView->setModel(m_InnModel);
     InnView->header()->setStretchLastSection(false);
+#if QT_VERSION < 0x050000
     InnView->header()->setResizeMode(0, QHeaderView::Stretch);
+#else
+    // Qt5
+    InnView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
+#endif
     InnView->hide();
 }
 
@@ -337,7 +359,7 @@ void DrugSelector::createDrugsHistoryActions()
     connect(m_HistoryAct, SIGNAL(triggered(QAction *)),
              this, SLOT(historyAct_triggered(QAction *)));
 
-    m_DrugsHistoricButton->addActions(m_HistoryAct->actions());
+    m_DrugsHistoryButton->addActions(m_HistoryAct->actions());
 }
 
 /** \brief Connect the search line edit to the filter update */
