@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -36,6 +36,7 @@
 #include <coreplugin/iuser.h>
 #include <coreplugin/dialogs/serverpreferenceswidget.h>
 #include <fmfcoreplugin/commandlineparser.h>
+#include <coreplugin/dialogs/networkpreferences.h>
 
 #include <utils/log.h>
 #include <utils/global.h>
@@ -46,6 +47,8 @@
 
 #include <translationutils/constants.h>
 #include <translationutils/trans_database.h>
+#include <translationutils/trans_user.h>
+#include <translationutils/trans_current.h>
 
 #include <extensionsystem/pluginmanager.h>
 
@@ -58,6 +61,8 @@
 #include <QProgressBar>
 #include <QTimer>
 #include <QProgressDialog>
+#include <QNetworkProxyQuery>
+#include <QNetworkProxy>
 
 using namespace Core;
 using namespace Trans::ConstantTranslations;
@@ -77,6 +82,8 @@ namespace {
 //    wizard->button(QWizard::BackButton)->setEnabled(false);
 //}
 
+    const char* const FIELD_TYPEOFINSTALL = "typeOfInstall";
+
     class CoreFirstRunPage : public Core::IFirstConfigurationPage
     {
     public:
@@ -85,6 +92,14 @@ namespace {
         QWizardPage *createPage(QWidget *parent) {return new CoreConfigPage(parent);}
     };
 
+    class ProxyFirstRunPage : public Core::IFirstConfigurationPage
+    {
+    public:
+        ProxyFirstRunPage(QObject *parent) : IFirstConfigurationPage(parent) {}
+        int id() const {return IFirstConfigurationPage::ProxyConfig;}
+        QWizardPage *createPage(QWidget *parent) {return new ProxyPage(parent);}
+    };
+
     class CoreFirstRunDatabaseCreationPage : public Core::IFirstConfigurationPage
     {
     public:
@@ -121,9 +136,11 @@ namespace {
 AppConfigWizard::AppConfigWizard(QWidget *parent) :
     QWizard(parent)
 {
+    setWindowFlags(windowFlags() | Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint | Qt::WindowMaximizeButtonHint);
     // create pages
     QList<IFirstConfigurationPage *> pages = pluginManager()->getObjects<IFirstConfigurationPage>();
     pages << new ::CoreFirstRunPage(this);
+    pages << new ::ProxyFirstRunPage(this);
     pages << new ::CoreServerConfigPage(this);
     pages << new ::CoreClientConfigPage(this);
     pages << new ::CoreFirstRunDatabaseCreationPage(this);
@@ -175,12 +192,21 @@ void AppConfigWizard::resizeEvent(QResizeEvent *event)
     Utils::centerWidget(this, qApp->desktop());
 }
 
+void AppConfigWizard::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::LanguageChange) {
+        setWindowTitle(tr("Application Configurator Wizard"));
+    }
+    QWizard::changeEvent(event);
+}
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////  CoreConfigPage  /////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 CoreConfigPage::CoreConfigPage(QWidget *parent) :
-        QWizardPage(parent)//, m_Wizard(parent)
+    QWizardPage(parent),
+    _proxyDectectionDone(false), _proxyDetected(false)
 {
+    setObjectName("FirstRun::CoreConfigPage");
     langLabel = new QLabel(this);
     langLabel->setWordWrap(true);
 
@@ -192,15 +218,12 @@ CoreConfigPage::CoreConfigPage(QWidget *parent) :
     combo->setDisplayMode(Utils::LanguageComboBox::AvailableTranslations);
     combo->setTranslationsPath(settings()->path(Core::ISettings::TranslationsPath));
     combo->setFlagsIconPath(settings()->path(Core::ISettings::SmallPixmapPath));
-    // TODO: connection here to retranslate (changeEvent())...
     connect(combo, SIGNAL(currentLanguageChanged(QLocale::Language)), Core::Translators::instance(), SLOT(changeLanguage(QLocale::Language)));
     combo->setCurrentLanguage(QLocale().language());
 
     // combo install type
     installCombo = new QComboBox(this);
 
-    retranslate();
-
     QGridLayout *layout = new QGridLayout(this);
     layout->setVerticalSpacing(30);
     layout->addWidget(langLabel, 2, 0, 1, 2);
@@ -208,18 +231,18 @@ CoreConfigPage::CoreConfigPage(QWidget *parent) :
     layout->addWidget(typeLabel, 10, 0, 1, 2);
     layout->addWidget(installCombo, 11, 1);
     setLayout(layout);
+
+    registerField(::FIELD_TYPEOFINSTALL, installCombo, "currentIndex");
+
+    retranslate();
 }
 
 void CoreConfigPage::retranslate()
 {
     setTitle(tr("Welcome to %1").arg(qApp->applicationName() + " v" + qApp->applicationVersion()));
-    setSubTitle(tr("<b>Welcome to %1</b><br /><br />"
-                   "This wizard will help you to configure the base parameters "
-                   "of the application.<br />"
-                   "At any time, you can cancel this wizard, the default "
-                   "values will be activated "
-                   "for the undefined parameters.")
-                .arg(qApp->applicationName() + " v" + qApp->applicationVersion()));
+    setSubTitle(tr("This wizard will help you to configure the base parameters "
+                   "of the application.\n"
+                   "Select your preferred language and the installation type."));
 
     langLabel->setText(tr("Select your language"));
     typeLabel->setText(tr("Select the type of installation"));
@@ -234,6 +257,7 @@ void CoreConfigPage::retranslate()
 
 bool CoreConfigPage::validatePage()
 {
+    setField(::FIELD_TYPEOFINSTALL, installCombo->currentIndex());
     switch (installCombo->currentIndex()) {
     case 0: // SQLite
     {
@@ -256,6 +280,24 @@ bool CoreConfigPage::validatePage()
 
 int CoreConfigPage::nextId() const
 {
+    // If a proxy is found -> ProxyPage
+    if (!_proxyDectectionDone) {
+        _proxyDectectionDone = true;
+        LOG("Trying to find system proxy.");
+        QNetworkProxyQuery npq(QUrl("http://www.google.com"));
+        QList<QNetworkProxy> listOfProxies = QNetworkProxyFactory::systemProxyForQuery(npq);
+        foreach(const QNetworkProxy &p, listOfProxies) {
+            if (p.type()==QNetworkProxy::HttpProxy && !p.hostName().isEmpty()) {
+                LOG("Proxy detected: " + p.hostName());
+                _proxyDetected = true;
+            }
+        }
+    }
+
+    if (_proxyDetected)
+        return Core::IFirstConfigurationPage::ProxyConfig;
+
+    // Fallback to install procedure
     switch (installCombo->currentIndex())
     {
     case 0: // No server
@@ -280,6 +322,66 @@ void CoreConfigPage::changeEvent(QEvent *e)
     }
 }
 
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////  ProxyPage  ///////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////////////////////////
+ProxyPage::ProxyPage(QWidget *parent) :
+        QWizardPage(parent)
+{
+    _proxyWidget = new Internal::ProxyPreferencesWidget(this);
+    _proxyWidget->autoDetectProxy();
+
+    retranslate();
+
+    QVBoxLayout *layout = new QVBoxLayout(this);
+    layout->setMargin(0);
+    layout->setSpacing(0);
+    layout->addWidget(_proxyWidget);
+    setLayout(layout);
+}
+
+void ProxyPage::retranslate()
+{
+    setTitle(tr("Proxy definition"));
+    setSubTitle(tr("%1 has detected a network proxy on your computer. \n"
+                   "Please check and validate the proxy settings.")
+                .arg(qApp->applicationName()));
+}
+
+bool ProxyPage::validatePage()
+{
+    _proxyWidget->saveToSettings();
+    return true;
+}
+
+int ProxyPage::nextId() const
+{
+    switch (field(::FIELD_TYPEOFINSTALL).toInt())
+    {
+    case 0: // No server
+        return Core::IFirstConfigurationPage::DatabaseCreationPage;
+    case 1: // Network as client
+        return Core::IFirstConfigurationPage::ServerClientConfig;
+    case 2: // Network as server
+        return Core::IFirstConfigurationPage::ServerConfig;
+    }
+    return Core::IFirstConfigurationPage::FirstPage;
+}
+
+void ProxyPage::changeEvent(QEvent *e)
+{
+    QWidget::changeEvent(e);
+    switch (e->type()) {
+    case QEvent::LanguageChange:
+        retranslate();
+        break;
+    default:
+        break;
+    }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////  ClientConfigPage  ////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////////////////////
@@ -290,7 +392,7 @@ ClientConfigPage::ClientConfigPage(QWidget *parent) :
     QGridLayout *layout = new QGridLayout(this);
     layout->setVerticalSpacing(30);
     serverWidget = new Core::ServerPreferencesWidget(this);
-    serverWidget->setUserLoginGroupTitle(tr("Personal login and password"));
+    serverWidget->setUserLoginGroupTitle(Trans::Constants::CONSTANTS_TR_CONTEXT, Trans::Constants::ENTER_YOUR_PERSONAL_LOGIN);
     layout->addWidget(serverWidget, 0, 0);
     setLayout(layout);
 
@@ -312,9 +414,7 @@ void ClientConfigPage::initializePage()
 void ClientConfigPage::retranslate()
 {
     setTitle(tr("Network client configuration"));
-    setSubTitle(tr("There were no automatic server configuration file found.<br />"
-                   "You must configure the server manually.<br />"
-                   "Use your <b>personnal login and password</b> to connect the database."));
+    setSubTitle(tr("You must configure the server using your personnal login and password to connect the database."));
 }
 
 bool ClientConfigPage::isComplete() const
@@ -410,6 +510,8 @@ ServerConfigPage::ServerConfigPage(QWidget *parent) :
     QGridLayout *layout = new QGridLayout(this);
     layout->setVerticalSpacing(30);
     serverWidget = new Core::ServerPreferencesWidget(this);
+    serverWidget->showUseDefaultAdminLogCheckbox(false);
+    serverWidget->setUserLoginGroupTitle(Trans::Constants::CONSTANTS_TR_CONTEXT, Trans::Constants::ENTER_MYSQL_ADMIN_LOGIN);
     layout->addWidget(serverWidget, 0, 0);
     setLayout(layout);
 
@@ -423,13 +525,21 @@ ServerConfigPage::ServerConfigPage(QWidget *parent) :
     connect(serverWidget, SIGNAL(hostConnectionChanged(bool)), this, SIGNAL(completeChanged()));
 }
 
+void ServerConfigPage::initializePage()
+{
+    if (!QFileInfo(serverConfigurationSqlScript()).exists()) {
+        Utils::warningMessageBox(tr("Missing files."),
+                                 tr("The configuration script is missing. You can not "
+                                    "configure the server without this script.<br><br>"
+                                    "<b>%1</b>").arg(tkTr(Trans::Constants::CONTACT_DEV_TEAM)));
+    }
+}
+
+
 void ServerConfigPage::retranslate()
 {
     setTitle(tr("Network server configuration"));
-    setSubTitle(tr("There were no automatic server configuration file found.<br />"
-                   "You must configure the server manually.<br />"
-                   "Use the <b>server super-administrator login and password</b> to connect the database."));
-    serverWidget->setUserLoginGroupTitle(tr("Server super-administrator login and password"));
+    setSubTitle(tr("You must configure the server using the server super-administrator login and password to connect the database."));
 }
 
 bool ServerConfigPage::isComplete() const
@@ -482,15 +592,20 @@ bool ServerConfigPage::validatePage()
         QSqlQuery query(mysql);
         if (query.exec("SELECT * FROM `user` where User='fmf_admin';")) {
             if (query.next()) {
-                LOG("Server already configurated");
-                Utils::informativeMessageBox(tr("Server already configurated"), tr("The server is already configurated for FreeMedForms."));
+                LOG("Server already configured");
+                Utils::informativeMessageBox(tr("Server already configured"), tr("The server is already configured for FreeMedForms."));
             } else {
                 LOG("Executing server configuration SQL script");
-                if (!Utils::Database::executeSqlFile("__APP_CONNECTION_TESTER", serverConfigurationSqlScript())) {
+                QString error;
+                if (!Utils::Database::executeSqlFile("__APP_CONNECTION_TESTER", serverConfigurationSqlScript(), &error)) {
                     LOG_ERROR("Server configuration script not processed");
+                    Utils::warningMessageBox(tr("An error occured..."),
+                                             tr("An error occured when trying to execute the script configuration script.\n"
+                                                "Please check out the log files and contact your administrator."),
+                                             error);
                 } else {
-                    LOG("Server successfully configurated");
-                    Utils::informativeMessageBox(tr("Server configurated"), tr("The server was successfully configurated."));
+                    LOG("Server successfully configured");
+                    Utils::informativeMessageBox(tr("Server configured"), tr("The server was successfully configured."));
                 }
             }
         }
@@ -546,7 +661,9 @@ void CoreDatabaseCreationPage::startDbCreation()
     Core::ICore::instance()->requestFirstRunDatabaseCreation();
     _completed = true;
     Q_EMIT completeChanged();
+    qApp->processEvents();
     wizard()->next();
+    qApp->processEvents();
 }
 
 bool CoreDatabaseCreationPage::isComplete() const
@@ -693,7 +810,7 @@ void EndConfigPage::retranslate()
                    "If you have any question, you can ask them to "
                    "the mailing list."));
     if (lblDb)
-        lblDb->setText(tr("You can clean and recreate all your databases. Select the option above. If you select the clean option, all databases will be erased with <b>definitive data lose</b>."));
+        lblDb->setText(tr("You can clean and recreate all your databases. Select the option above. If you select the clean option, all databases will be erased with <b>definitive data loss</b>."));
     int current = 0;
     if (comboDb) {
         current = comboDb->currentIndex();
