@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,17 +19,17 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "mainwindow.h"
-#include "patientmodelwrapper.h"
 
 #include <translationutils/constanttranslations.h>
 #include <utils/log.h>
 #include <utils/global.h>
+#include <utils/stylehelper.h>
 #include <utils/updatechecker.h>
 #include <utils/widgets/fancyactionbar.h>
 #include <utils/widgets/fancytabwidget.h>
@@ -45,16 +45,17 @@
 #include <coreplugin/actionmanager/mainwindowactions.h>
 #include <coreplugin/actionmanager/mainwindowactionhandler.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/contextmanager/contextmanager.h>
 #include <coreplugin/dialogs/settingsdialog.h>
 #include <coreplugin/ipatient.h>
 #include <coreplugin/iuser.h>
 #include <coreplugin/theme.h>
 
+#include <formmanagerplugin/formcore.h>
 #include <formmanagerplugin/iformio.h>
 #include <formmanagerplugin/iformitem.h>
 #include <formmanagerplugin/iformwidgetfactory.h>
-#include <formmanagerplugin/formmanager.h>
 #include <formmanagerplugin/formplaceholder.h>
 #include <formmanagerplugin/episodemodel.h>
 
@@ -63,7 +64,7 @@
 #include <patientbaseplugin/patientbar.h>
 #include <patientbaseplugin/patientsearchmode.h>
 #include <patientbaseplugin/patientwidgetmanager.h>
-#include <patientbaseplugin/patientmodel.h>
+#include <patientbaseplugin/patientcore.h>
 #include <patientbaseplugin/patientcreatorwizard.h>
 
 #include <extensionsystem/pluginerrorview.h>
@@ -81,7 +82,7 @@
 #include <QHBoxLayout>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
-
+#include <QMenu>
 
 using namespace MainWin;
 using namespace MainWin::Internal;
@@ -102,16 +103,14 @@ static inline Core::IPatient *patient() {return Core::ICore::instance()->patient
 
 static inline ExtensionSystem::PluginManager *pluginManager() { return ExtensionSystem::PluginManager::instance(); }
 
-static inline Form::FormManager *formManager() {return Form::FormManager::instance();}
+static inline Form::FormCore &formCore() {return Form::FormCore::instance();}
 
-static inline Patients::PatientModel *patientModel() {return Patients::PatientModel::activeModel();}
+static inline Patients::PatientCore *patientCore() {return Patients::PatientCore::instance();}
 
 // SplashScreen Messagers
 static inline void messageSplash(const QString &s) {theme()->messageSplashScreen(s); }
 static inline void finishSplash(QMainWindow *w) {theme()->finishSplashScreen(w); }
 
-
-
 MainWindowUserListener::MainWindowUserListener(MainWindow *parent) :
     UserPlugin::IUserListener(parent), m_MainWindow(parent)
 {
@@ -133,31 +132,28 @@ bool MainWindowUserListener::currentUserAboutToDisconnect()
     return true;
 }
 
-
-//--------------------------------------------------------------------------------------------------------
-//--------------------------------------- Constructor / Destructor ---------------------------------------
-//--------------------------------------------------------------------------------------------------------
 MainWindow::MainWindow(QWidget *parent) :
     Core::IMainWindow(parent),
     m_modeStack(0),
-    m_RecentPatients(0),
-    m_PatientModelWrapper(0),
     m_UserListener(0)
 {
     setObjectName("MainWindow");
     messageSplash(tr("Creating Main Window"));
     setAttribute(Qt::WA_QuitOnClose);
-    m_RecentPatients = new Core::FileManager(this);
-    m_RecentPatients->setSettingsKey(Core::Constants::S_PATIENT_UUID_HISTORY);
+    // Connect post core initialization
+    connect(Core::ICore::instance(), SIGNAL(coreOpened()), this, SLOT(postCoreInitialization()));
 }
 
+/**
+ * Initialize the main window.\n
+ * Creates menus, actions and the modeWidget
+ */
 void MainWindow::init()
 {
     Q_ASSERT(actionManager());
     Q_ASSERT(contextManager());
 
     // create menus
-//    createFileMenu();
     createGeneralMenu();
     createPatientMenu();
     createEditMenu();
@@ -171,9 +167,6 @@ void MainWindow::init()
 //    Core::ActionContainer *fmenu = actionManager()->actionContainer(Core::Constants::M_FILE);
 //    Q_ASSERT(fmenu);
 //    connect(fmenu->menu(), SIGNAL(aboutToShow()),this, SLOT(aboutToShowRecentFiles()));
-    Core::ActionContainer *pmenu = actionManager()->actionContainer(Core::Constants::M_PATIENTS);
-    Q_ASSERT(pmenu);
-    connect(pmenu->menu(), SIGNAL(aboutToShow()),this, SLOT(aboutToShowRecentPatients()));
 
     Core::MainWindowActions actions;
 
@@ -205,8 +198,6 @@ void MainWindow::init()
             Core::MainWindowActions::A_DebugDialog |
             Core::MainWindowActions::A_CheckUpdate |
             Core::MainWindowActions::A_AppGoToWebSite
-                //|
-//            Core::MainWindowActions::A_QtAbout
             );
     actions.setTemplatesActions(Core::MainWindowActions::A_Templates_New);
 
@@ -218,13 +209,10 @@ void MainWindow::init()
     modeManager()->init(m_modeStack);
 }
 
-//bool MainWindow::initialize(const QStringList &arguments, QString *errorString)
-//{
-//    Q_UNUSED(arguments);
-//    Q_UNUSED(errorString);
-//    return true;
-//}
-
+/**
+ * Second initialization step after all dependencies are initialized.
+ * Set windowIcon, connect menus and actions, start the updatechecker,
+ */
 void MainWindow::extensionsInitialized()
 {
 //    qWarning() << Q_FUNC_INFO << "user ok" << user()->hasCurrentUser();
@@ -243,20 +231,12 @@ void MainWindow::extensionsInitialized()
 
     // Start the update checker
     if (updateChecker()->needsUpdateChecking(settings()->getQSettings())) {
-        LOG(tkTr(Trans::Constants::CHECKING_UPDATES));
-//        statusBar()->addWidget(new QLabel(tkTr(Trans::Constants::CHECKING_UPDATES), this));
-//        statusBar()->addWidget(updateChecker()->progressBar(this),1);
-        connect(updateChecker(), SIGNAL(updateFound()), this, SLOT(updateFound()));
-        connect(updateChecker(), SIGNAL(done(bool)), this, SLOT(updateCheckerEnd()));
-        updateChecker()->check(Utils::Constants::FREEMEDFORMS_UPDATE_URL);
-        settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
+        settings()->setPath(Core::ISettings::UpdateUrl, Utils::Constants::FREEMEDFORMS_UPDATE_URL);
+        if (checkUpdate())
+            settings()->setValue(Utils::Constants::S_LAST_CHECKUPDATE, QDate::currentDate());
     }
 
-//    // Create IPatient
-    m_PatientModelWrapper = new Internal::PatientModelWrapper(patientModel());
-    Core::ICore::instance()->setPatient(m_PatientModelWrapper);
-    m_PatientModelWrapper->init();
-    m_modeStack->insertTopWidget(0, Patients::PatientBar::instance(this));
+    m_modeStack->insertTopWidget(patientCore()->patientBar());
     m_modeStack->statusBar()->hide();
 
     setCentralWidget(m_modeStack);
@@ -269,9 +249,6 @@ void MainWindow::extensionsInitialized()
 //        }
 //        settings()->noMoreFirstTimeRunning();
 //    }
-
-    // Connect post core initialization
-    connect(Core::ICore::instance(), SIGNAL(coreOpened()), this, SLOT(postCoreInitialization()));
 }
 
 MainWindow::~MainWindow()
@@ -284,33 +261,33 @@ MainWindow::~MainWindow()
     // delete ui components
     delete m_modeStack;
     m_modeStack = 0;
-    if (m_PatientModelWrapper) {
-        delete m_PatientModelWrapper;
-        m_PatientModelWrapper = 0;
-    }
 }
 
-/** \brief Post core initialization of MainWindow. */
+/**
+ * Third step of initialization after the core gets fuly opened.
+ * Manages user, menus and actions translations, finish splashscreen
+ * show UI.
+ */
 void MainWindow::postCoreInitialization()
 {
     if (Utils::Log::warnPluginsCreation())
         qWarning() << Q_FUNC_INFO;
+
     // Manage current user and patient
     onCurrentUserChanged();
     pluginManager()->addObject(m_UserListener = new MainWindowUserListener(this));
-    connect(user(), SIGNAL(userChanged()), this, SLOT(onCurrentUserChanged()));
-    connect(patient(), SIGNAL(currentPatientChanged()), this, SLOT(onCurrentPatientChanged()));
+    connect(user(), SIGNAL(userChanged()), this, SLOT(onCurrentUserChanged()), Qt::UniqueConnection);
+    connect(user(), SIGNAL(userDataChanged(int)), this, SLOT(onUserDataChanged(int)), Qt::UniqueConnection);
+    connect(user(), SIGNAL(reset()), this, SLOT(onCurrentUserChanged()), Qt::UniqueConnection);
+    connect(patient(), SIGNAL(currentPatientChanged()), this, SLOT(onCurrentPatientChanged()), Qt::UniqueConnection);
 
     contextManager()->updateContext();
     actionManager()->retranslateMenusAndActions();
 
-    // Open Last Opened Forms is necessary
-//    openPatientFormsFile();  // TODO: code here: remove this and all sub-member
-    // Create the patient navigation menu (needed in Patient::PatientSelector)
-    aboutToShowRecentPatients();
-
     theme()->finishSplashScreen(this);
 
+    manageIModeEnabledState();
+    connect(modeManager(), SIGNAL(currentModeChanged(Core::IMode*)), this, SLOT(onCurrentModeChanged(Core::IMode*)), Qt::UniqueConnection);
     modeManager()->activateMode(Core::Constants::MODE_PATIENT_SEARCH);
 
     raise();
@@ -320,7 +297,22 @@ void MainWindow::postCoreInitialization()
     setFocus();
 }
 
-/** \brief Slot connected to Core::IUser::userChanged().*/
+/** When the current Core::IMode is beeing updated, check the patient visibility */
+void MainWindow::onCurrentModeChanged(Core::IMode *newMode)
+{
+    Q_ASSERT(newMode);
+    if (!newMode)
+        return;
+    if (newMode->patientBarVisibility())
+        patient()->showPatientBar();
+    else
+        patient()->hidePatientBar();
+}
+
+/**
+ * \internal
+ * Slot connected to Core::IUser::userChanged().
+ */
 void MainWindow::onCurrentUserChanged()
 {
     // Change window title
@@ -330,32 +322,62 @@ void MainWindow::onCurrentUserChanged()
     readSettings();
 }
 
-/** \brief  Define the current patient to different structure such as Form::FormManager, Form::EpisodeModel.*/
+/**
+ * \internal
+ * Slot connected to Core::IUser::userDataChanged(int).
+ * Reset the window title.
+ */
+void MainWindow::onUserDataChanged(int id)
+{
+    if (id == Core::IUser::FullName) {
+        // Change window title
+        setWindowTitle(qApp->applicationName() + " - " + qApp->applicationVersion() + " / " +
+                       user()->value(Core::IUser::FullName).toString());
+    }
+}
+
+/**
+ * \internal
+ * Define the current patient to different structure such as
+ * Form::FormManager, Form::EpisodeModel.
+ */
 void MainWindow::onCurrentPatientChanged()
 {
-    // Activate Patient files mode
-    formManager()->activateMode();
+    // Manage mode enabled state
+    manageIModeEnabledState();
 
-    // Store the uuids of the patient in the recent manager
-    const QString &uuid = patient()->uuid();
-    m_RecentPatients->setCurrentFile(uuid);
-    m_RecentPatients->addToRecentFiles(uuid);
-    // refresh the navigation menu
-    aboutToShowRecentPatients();
+    // Activate Patient files mode
+    formCore().activatePatientFileCentralMode();
 
     endProcessingSpinner();
 }
 
+/** Enable/Disable modes according to the Core::IMode::isEnabledOnlyWithCurrentPatient() state */
+void MainWindow::manageIModeEnabledState()
+{
+    QList<Core::IMode*> modes = pluginManager()->getObjects<Core::IMode>();
+    foreach(Core::IMode *mode, modes) {
+        if (mode->isEnabledOnlyWithCurrentPatient())
+            mode->setEnabled(patient()->currentPatientIndex().isValid());
+    }
+}
+
 /** \brief Close the main window and the application */
 void MainWindow::closeEvent(QCloseEvent *event)
 {
     LOG("Closing MainWindow");
-    // Set focus to enshure that QDataWidgetMapper will send their last changes to their models
+    // Set focus to ensure that QDataWidgetMapper will send their last changes to their models
     setFocus();
     writeSettings();
     QList<Core::ICoreListener *> listeners = pluginManager()->getObjects<Core::ICoreListener>();
     for(int i = 0; i < listeners.count(); ++i) {
         if (!listeners.at(i)->coreAboutToClose()) {
+            const QString &msg = listeners.at(i)->errorMessage();
+            if (!msg.isEmpty()) {
+                Utils::warningMessageBox(tr("Unable to close window"),
+                                         tr("Unable to close the application, error message: <br/><b>%1</b>").arg(msg),
+                                         "", tr("Unable to close window"));
+            }
             event->ignore();
             return;
         }
@@ -381,8 +403,9 @@ QStatusBar *MainWindow::statusBar()
     return QMainWindow::statusBar();
 }
 
-void MainWindow::updateCheckerEnd()
+void MainWindow::updateCheckerEnd(bool error)
 {
+    Q_UNUSED(error)
     delete statusBar();
 }
 
@@ -414,7 +437,7 @@ bool MainWindow::loadFile(const QString &absDirPath)
 
     // Get the PatientFile FormMain empty root from FormManager
 //    Form::FormMain *root = 0;
-//    if (root = formManager()->loadFile(filename, list)) {
+//    if (root = formManager().loadFile(filename, list)) {
 //        fileManager()->setCurrentFile(filename);
 //    } else {
 //        return false;
@@ -428,6 +451,7 @@ bool MainWindow::loadFile(const QString &absDirPath)
 /** \brief Create a new patient. \sa Patients::PatientCreatorWizard */
 bool MainWindow::createNewPatient()
 {
+    // TODO: move this in the patient plugin
     Patients::PatientCreatorWizard wiz(this);
     wiz.exec();
     return true;
@@ -466,30 +490,6 @@ void MainWindow::aboutToShowRecentFiles()
     recentsMenu->menu()->setEnabled(hasRecentFiles);
 }
 
-/** \brief Rebuild the patients' history menu */
-void MainWindow::aboutToShowRecentPatients()
-{
-    // update patient history
-    Core::ActionContainer *recentsMenu = actionManager()->actionContainer(Core::Constants::M_PATIENTS_NAVIGATION);
-    if (!recentsMenu)
-        return;
-    if (!recentsMenu->menu())
-        return;
-    recentsMenu->menu()->clear();
-
-    bool hasRecentFiles = false;
-    const QStringList &uuids = m_RecentPatients->recentFiles();
-    const QHash<QString, QString> &names = Patients::PatientModel::patientName(uuids);
-    for(int i = 0; i < uuids.count(); ++i) {
-        hasRecentFiles = true;
-        QAction *action = recentsMenu->menu()->addAction(names.value(uuids.at(i)));
-        action->setData(uuids.at(i));
-        connect(action, SIGNAL(triggered()), this, SLOT(openRecentPatient()));
-    }
-
-    recentsMenu->menu()->setEnabled(hasRecentFiles);
-}
-
 /** \brief Opens a recent file. This slot is called by a recent files' menu's action. */
 void MainWindow::openRecentFile()
 {
@@ -502,23 +502,6 @@ void MainWindow::openRecentFile()
     }
 }
 
-/** \brief Opens a recent patient selected from the patient history. This slot is called by a recent patients' menu's action. */
-void MainWindow::openRecentPatient()
-{
-    // get the uuid of the sender
-    QAction *a = qobject_cast<QAction*>(sender());
-    if (!a)
-        return;
-    const QString &uuid = a->data().toString();
-    if (uuid.isEmpty())
-        return;
-
-    // get the QModelIndex corresponding to the uuid
-    patientModel()->setFilter("", "", uuid, Patients::PatientModel::FilterOnUuid);
-    QModelIndex index = patientModel()->index(0,0);
-    patientModel()->setCurrentPatient(index);
-}
-
 /** \brief Reads main window's settings */
 void MainWindow::readSettings()
 {
@@ -527,13 +510,14 @@ void MainWindow::readSettings()
     // Main Application settings
     settings()->restoreState(this);
     fileManager()->getRecentFilesFromSettings();
-    m_RecentPatients->getRecentFilesFromSettings();
+    fileManager()->getMaximumRecentFilesFromSettings();
     m_AutomaticSaveInterval = settings()->value(Core::Constants::S_SAVEINTERVAL, 600).toUInt(); // Default = 10 minutes
     m_OpenLastOpenedForm = settings()->value(Core::Constants::S_OPENLAST, true).toBool();
 
     // Main Widget settings
     m_HelpTextShow = settings()->value(Core::Constants::S_SHOWHELPTEXT, true).toBool();
 
+    Utils::StyleHelper::setBaseColor(Utils::StyleHelper::DEFAULT_BASE_COLOR);
     // Notify
 //    statusBar()->showMessage(tkTr(Trans::Constants::SETTINGS_RECOVERED), 2000);
 }
@@ -544,7 +528,6 @@ void MainWindow::writeSettings()
     settings()->saveState(this);
     // Recent managers
     fileManager()->saveRecentFiles();
-    m_RecentPatients->saveRecentFiles();
     // Main Application settings
     settings()->setValue(Core::Constants::S_SAVEINTERVAL, m_AutomaticSaveInterval);
     settings()->setValue(Core::Constants::S_OPENLAST, m_OpenLastOpenedForm);
