@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -130,6 +130,7 @@ EpisodeBase::EpisodeBase(QObject *parent) :
     addField(Table_EPISODES, EPISODES_USERDATE, "USERDATE", FieldIsDate);
     addField(Table_EPISODES, EPISODES_DATEOFCREATION, "DATECREATION", FieldIsDate);
     addField(Table_EPISODES, EPISODES_USERCREATOR, "CREATOR", FieldIsUUID);
+    addField(Table_EPISODES, EPISODES_PRIORITY, "PRIOR", FieldIsInteger, "1"); // Medium
     addIndex(Table_EPISODES, EPISODES_ID);
     addIndex(Table_EPISODES, EPISODES_PATIENT_UID);
     addIndex(Table_EPISODES, EPISODES_FORM_PAGE_UID);
@@ -216,7 +217,10 @@ bool EpisodeBase::initialize()
         LOG(tkTr(Trans::Constants::CONNECTED_TO_DATABASE_1_DRIVER_2).arg(database().connectionName()).arg(database().driverName()));
     }
 
-    //    d->checkDatabaseVersion();
+    if (!checkDatabaseVersion()) {
+        LOG_ERROR(tr("Unable to update the database schema"));
+        return false;
+    }
 
     if (!checkDatabaseScheme()) {
         LOG_ERROR(tkTr(Trans::Constants::DATABASE_1_SCHEMA_ERROR).arg(DB_NAME));
@@ -249,7 +253,7 @@ bool EpisodeBase::createDatabase(const QString &connectionName , const QString &
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         if (!DB.open())
             LOG(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(dbName).arg(DB.lastError().text()));
@@ -309,14 +313,7 @@ bool EpisodeBase::createDatabase(const QString &connectionName , const QString &
     }
 
     // Add version number
-    QSqlQuery query(DB);
-    query.prepare(prepareInsertQuery(Table_VERSION));
-    query.bindValue(VERSION_TEXT, DB_ACTUALVERSION);
-    if (!query.exec()) {
-        LOG_QUERY_ERROR(query);
-        return false;
-    }
-
+    setCurrentDatabaseVersion(Constants::DB_ACTUALVERSION);
     populateWithDefaultValues();
 
     return true;
@@ -329,6 +326,73 @@ void EpisodeBase::populateWithDefaultValues()
     setGenericPatientFormFile(QString("%1/%2").arg(Core::Constants::TAG_APPLICATION_COMPLETEFORMS_PATH).arg(Core::Constants::S_DEF_PATIENTFORMS_FILENAME));
 }
 
+bool EpisodeBase::setCurrentDatabaseVersion(const QString &version)
+{
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__))
+        return false;
+    DB.transaction();
+    QSqlQuery query(DB);
+    query.prepare(prepareDeleteQuery(Table_VERSION));
+    if (!query.exec()) {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return false;
+    }
+    query.finish();
+    DB.commit();
+    query.prepare(prepareInsertQuery(Table_VERSION));
+    query.bindValue(VERSION_TEXT, version);
+    if (!query.exec()) {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return false;
+    }
+    query.finish();
+    DB.commit();
+    return true;
+}
+
+QString EpisodeBase::currentDatabaseVersion()
+{
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__))
+        return QString::null;
+    DB.transaction();
+    QString version;
+    QSqlQuery query(DB);
+    if (query.exec(select(Constants::Table_VERSION, Constants::VERSION_TEXT))) {
+        if (query.next()) {
+            version = query.value(0).toString();
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return QString::null;
+    }
+    query.finish();
+    DB.commit();
+    return version;
+}
+
+bool EpisodeBase::checkDatabaseVersion()
+{
+    QString currentVersion = currentDatabaseVersion();
+    // Updates from 0.1
+    if (currentVersion == "0.1") {
+        if (!alterTableForNewField(Constants::Table_EPISODES, Constants::EPISODES_PRIORITY))
+            return false;
+        LOG(tr("Episode database updated from version %1 to version: %2")
+            .arg("0.1")
+            .arg(Constants::DB_ACTUALVERSION));
+    }
+    // Update the database version
+    return setCurrentDatabaseVersion(Constants::DB_ACTUALVERSION);
+}
+
 void EpisodeBase::onCoreDatabaseServerChanged()
 {
     m_initialized = false;
@@ -353,9 +417,8 @@ void EpisodeBase::onCoreFirstRunCreationRequested()
 bool EpisodeBase::setGenericPatientFormFile(const QString &absPathOrUid)
 {
     QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
-    if (!connectDatabase(DB, __LINE__)) {
+    if (!connectDatabase(DB, __LINE__))
         return false;
-    }
     DB.transaction();
     QHash<int, QString> where;
     where.insert(FORM_GENERIC, QString("IS NOT NULL"));
@@ -367,6 +430,7 @@ bool EpisodeBase::setGenericPatientFormFile(const QString &absPathOrUid)
         query.bindValue(0, absPathOrUid);
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             DB.rollback();
             return false;
         }
@@ -386,6 +450,7 @@ bool EpisodeBase::setGenericPatientFormFile(const QString &absPathOrUid)
         query.bindValue(FORM_USER_RESTRICTION_ID, QVariant());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             DB.rollback();
             return false;
         }
@@ -417,10 +482,15 @@ QString EpisodeBase::getGenericFormFile()
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
         return QString();
     }
+    query.finish();
     DB.commit();
+    // Fix a bug introduced in 0.7.6
+    if (path.endsWith("/central.xml"))
+        path = path.remove("/central.xml");
     return path;
 }
 
@@ -435,32 +505,44 @@ QVector<Form::SubFormInsertionPoint> EpisodeBase::getSubFormFiles()
     if (!connectDatabase(DB, __LINE__)) {
         return toReturn;
     }
+    Utils::FieldList where;
+    where.append(Utils::Field(Table_FORM, FORM_PATIENTUID, QString("='%1'").arg(patient()->uuid())));
+    where.append(Utils::Field(Table_FORM, FORM_PATIENTUID, QString("IS NULL")));
+    QString whereClause = getWhereClause(where, Utils::Database::OR);
+    where.clear();
+    where.append(Utils::Field(Table_FORM, FORM_GENERIC, QString("IS NULL")));
+    where.append(Utils::Field(Table_FORM, FORM_VALID, QString("=1")));
+    whereClause = QString("(%1) AND (%2)").arg(whereClause).arg(getWhereClause(where, Utils::Database::AND));
+
     DB.transaction();
-    QHash<int, QString> where;
-    where.insert(FORM_GENERIC, QString("IS NULL"));
-    where.insert(FORM_VALID, QString("=1"));
-    where.insert(FORM_PATIENTUID, QString("='%1'").arg(patient()->uuid()));
     QSqlQuery query(DB);
     QString req = select(Table_FORM, QList<int>()
                          << FORM_SUBFORMUID
                          << FORM_INSERTIONPOINT
                          << FORM_INSERTASCHILD
-                         << FORM_APPEND, where);
+                         << FORM_APPEND) + " WHERE " + whereClause;
     if (query.exec(req)) {
         while (query.next()) {
             QString insertUid = query.value(1).toString();
             insertUid.replace(Core::Constants::TAG_APPLICATION_COMPLETEFORMS_PATH, settings()->path(Core::ISettings::CompleteFormsPath));
             insertUid.replace(Core::Constants::TAG_APPLICATION_SUBFORMS_PATH, settings()->path(Core::ISettings::SubFormsPath));
-            SubFormInsertionPoint point(insertUid, query.value(0).toString());
+            if (insertUid.endsWith("/central.xml"))
+                insertUid = insertUid.remove("/central.xml");
+            QString subFormUid = query.value(0).toString();
+            if (subFormUid.endsWith("/central.xml"))
+                subFormUid = subFormUid.remove("/central.xml");
+            SubFormInsertionPoint point(insertUid, subFormUid);
             point.setAddAsChild(query.value(2).toBool());
             point.setAppendToForm(query.value(3).toBool());
             toReturn << point;
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
         return toReturn;
     }
+    query.finish();
     DB.commit();
     return toReturn;
 }
@@ -472,30 +554,34 @@ QVector<Form::SubFormInsertionPoint> EpisodeBase::getSubFormFiles()
 bool EpisodeBase::addSubForms(const QVector<SubFormInsertionPoint> &insertions)
 {
     QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
-    if (!connectDatabase(DB, __LINE__)) {
+    if (!connectDatabase(DB, __LINE__))
         return false;
-    }
     DB.transaction();
-    // save
     QSqlQuery query(DB);
     for(int i = 0; i < insertions.count(); ++i) {
+        const SubFormInsertionPoint &ip = insertions.at(i);
         query.prepare(prepareInsertQuery(Table_FORM));
         query.bindValue(FORM_ID, QVariant());
         query.bindValue(FORM_VALID, 1);
         query.bindValue(FORM_GENERIC, QVariant());
-        query.bindValue(FORM_PATIENTUID, patient()->uuid());
-        query.bindValue(FORM_SUBFORMUID, insertions.at(i).subFormUid());
-        query.bindValue(FORM_INSERTIONPOINT, insertions.at(i).receiverUid());
-        query.bindValue(FORM_INSERTASCHILD, insertions.at(i).addAsChild());
-        query.bindValue(FORM_APPEND, insertions.at(i).appendToForm());
+        if (ip.isForAllPatients())
+            query.bindValue(FORM_PATIENTUID, QVariant());
+        else
+            query.bindValue(FORM_PATIENTUID, patient()->uuid());
+        query.bindValue(FORM_SUBFORMUID, ip.subFormUid());
+        query.bindValue(FORM_INSERTIONPOINT, ip.receiverUidForDatabase());
+        query.bindValue(FORM_INSERTASCHILD, int(ip.addAsChild()));
+        query.bindValue(FORM_APPEND, int(ip.appendToForm()));
         query.bindValue(FORM_USER_RESTRICTION_ID, QVariant());
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
+            query.finish();
             DB.rollback();
             return false;
         }
         query.finish();
     }
+    query.finish();
     DB.commit();
     return true;
 }
@@ -636,6 +722,11 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
         EpisodeData *episode = episodes.at(i);
         if (!episode)
             continue;
+        // No patient uid? -> error
+        if (episode->data(EpisodeData::PatientUuid).toString().isEmpty()) {
+            LOG_ERROR("No patient Uid defined in episode. Episode not saved.");
+            continue;
+        }
         if (!episode->isModified())
             continue;
         if (episode->episodeId()==-1) {
@@ -653,6 +744,7 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
             query.bindValue(EPISODES_USERCREATOR, episode->data(EpisodeData::UserCreatorUuid));
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
+                query.finish();
                 DB.rollback();
                 return false;
             } else {
@@ -668,6 +760,7 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
                 query.bindValue(EPISODE_CONTENT_XML, episode->data(EpisodeData::XmlContent));
                 if (!query.exec()) {
                     LOG_QUERY_ERROR(query);
+                    query.finish();
                     DB.rollback();
                     return false;
                 } else {
@@ -713,6 +806,7 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
             query.bindValue(6, episode->data(EpisodeData::UserCreatorUuid));
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
+                query.finish();
                 DB.rollback();
                 return false;
             }
@@ -728,6 +822,7 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
                 query.bindValue(0, episode->data(EpisodeData::XmlContent));
                 if (!query.exec()) {
                     LOG_QUERY_ERROR(query);
+                    query.finish();
                     DB.rollback();
                     return false;
                 }
@@ -753,6 +848,168 @@ bool EpisodeBase::saveEpisode(const QList<EpisodeData *> &episodes)
     return true;
 }
 
+/** Save the raw XML episode content. Does not manage modification traces. */
+bool EpisodeBase::saveEpisodeContent(const QVariant &uid, const QString &xml)
+{
+    qWarning() << "EpisodeBase::saveEpisodeContent" << uid.toString() << xml.length();
+    if (!uid.isValid())
+        return false;
+    if (uid.toInt() < 0)
+        return false;
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return false;
+    }
+    QHash<int, QString> where;
+    where.insert(Constants::EPISODE_CONTENT_EPISODE_ID, QString("=%1").arg(uid.toString()));
+    if (count(Constants::Table_EPISODE_CONTENT, Constants::EPISODE_CONTENT_EPISODE_ID, getWhereClause(Constants::Table_EPISODE_CONTENT, where)) == 0) {
+        // save
+        DB.transaction();
+        QSqlQuery query(DB);
+        query.prepare(prepareInsertQuery(Table_EPISODE_CONTENT));
+        query.bindValue(EPISODE_CONTENT_ID, QVariant());
+        query.bindValue(EPISODE_CONTENT_EPISODE_ID, uid.toInt());
+        query.bindValue(EPISODE_CONTENT_XML, xml);
+        if (!query.exec()) {
+            LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return false;
+        }
+        query.finish();
+        DB.commit();
+    } else {
+        // update
+        // TODO: manage modification traces
+        DB.transaction();
+        QSqlQuery query(DB);
+        query.prepare(prepareUpdateQuery(Table_EPISODE_CONTENT, EPISODE_CONTENT_XML, where));
+        query.bindValue(0, xml);
+        if (!query.exec()) {
+            LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return false;
+        }
+        query.finish();
+        DB.commit();
+    }
+    qWarning() << "     correctly saved";
+    return true;
+}
+
+/**
+ * Save a specific episode validation \e validation.
+ * If needed, the Form::EpisodeValidationData::ValidationId is defined in the validation data.
+ * This member manages its own transaction.
+ */
+bool EpisodeBase::saveEpisodeValidation(EpisodeValidationData *validation)
+{
+    if (!validation->isModified())
+        return false;
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return false;
+    }
+    DB.transaction();
+    QSqlQuery query(DB);
+    QHash<int, QString> where;
+    if (validation->validationId()!=-1) {
+        // update
+        where.insert(EP_MODIF_ID, QString("=%1").arg(validation->validationId()));
+        query.prepare(prepareUpdateQuery(Table_VALIDATION,
+                                         QList<int>()
+                                         << VALIDATION_DATEOFVALIDATION
+                                         << VALIDATION_USERUID
+                                         << VALIDATION_ISVALID
+                                         , where));
+        query.bindValue(0, validation->data(EpisodeValidationData::ValidationDate));
+        query.bindValue(1, validation->data(EpisodeValidationData::UserUid));
+        query.bindValue(2, validation->data(EpisodeValidationData::IsValid));
+        if (!query.exec()) {
+            LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return false;
+        } else {
+            validation->setModified(false);
+        }
+    } else {
+        // save
+        query.prepare(prepareInsertQuery(Table_VALIDATION));
+        query.bindValue(VALIDATION_ID, QVariant());
+        query.bindValue(VALIDATION_EPISODE_ID, validation->data(EpisodeValidationData::EpisodeId));
+        query.bindValue(VALIDATION_DATEOFVALIDATION, validation->data(EpisodeValidationData::ValidationDate));
+        query.bindValue(VALIDATION_USERUID, validation->data(EpisodeValidationData::UserUid));
+        query.bindValue(VALIDATION_ISVALID, validation->data(EpisodeValidationData::IsValid).toInt());
+        if (!query.exec()) {
+            LOG_QUERY_ERROR(query);
+            query.finish();
+            DB.rollback();
+            return false;
+        } else {
+            validation->setData(EpisodeValidationData::ValidationId, query.lastInsertId());
+            validation->setModified(false);
+        }
+    }
+    query.finish();
+    DB.commit();
+    return true;
+}
+
+/** Remove the episode from the database. The episode is marked as non-valid (IsValid field) */
+bool EpisodeBase::removeEpisode(const QVariant &uid)
+{
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__))
+        return false;
+    using namespace Constants;
+    QHash<int, QString> where;
+    where.insert(EPISODES_ID, QString("='%1'").arg(uid.toString()));
+    DB.transaction();
+    QSqlQuery query(DB);
+    query.prepare(prepareUpdateQuery(Table_EPISODES, EPISODES_ISVALID, where));
+    query.bindValue(0, "0");
+    if (!query.exec()) {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return false;
+    }
+    query.finish();
+    DB.commit();
+    return true;
+}
+
+/** Remove all the recorded episode for the form \e formUid and for the patient \e patientUid in the database */
+bool EpisodeBase::removeAllEpisodeForForm(const QVariant &formUid, const QString &patientUid)
+{
+    if (!formUid.isValid())
+        return false;
+    if (patientUid.isEmpty())
+        return false;
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__))
+        return false;
+    using namespace Constants;
+    QHash<int, QString> where;
+    where.insert(EPISODES_FORM_PAGE_UID, QString("='%1'").arg(formUid.toString()));
+    where.insert(EPISODES_PATIENT_UID, QString("='%1'").arg(patientUid));
+    DB.transaction();
+    QSqlQuery query(DB);
+    query.prepare(prepareUpdateQuery(Table_EPISODES, EPISODES_ISVALID, where));
+    query.bindValue(0, "0");
+    if (!query.exec()) {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return false;
+    }
+    query.finish();
+    DB.commit();
+    return true;
+}
+
 /** Return all recorded episodes form the database according to the Form::Internal::EpisodeBaseQuery \e baseQuery. Episodes are sorted by UserDate. */
 QList<EpisodeData *> EpisodeBase::getEpisodes(const EpisodeBaseQuery &baseQuery)
 {
@@ -762,7 +1019,6 @@ QList<EpisodeData *> EpisodeBase::getEpisodes(const EpisodeBaseQuery &baseQuery)
     if (!connectDatabase(DB, __LINE__)) {
         return toReturn;
     }
-    DB.transaction();
 
     // Manage filter
     QString order, limit, req;
@@ -809,6 +1065,7 @@ QList<EpisodeData *> EpisodeBase::getEpisodes(const EpisodeBaseQuery &baseQuery)
 
 //    qWarning() << req;
 
+    DB.transaction();
     QSqlQuery query(DB);
     if (query.exec(req)) {
         while (query.next()) {
@@ -845,6 +1102,9 @@ QList<EpisodeData *> EpisodeBase::getEpisodes(const EpisodeBaseQuery &baseQuery)
                 }
             } else {
                 LOG_QUERY_ERROR(query2);
+                query2.finish();
+                DB.rollback();
+                return toReturn;
             }
             query2.finish();
 
@@ -864,20 +1124,23 @@ QList<EpisodeData *> EpisodeBase::getEpisodes(const EpisodeBaseQuery &baseQuery)
                 }
             } else {
                 LOG_QUERY_ERROR(query2);
+                query2.finish();
+                DB.rollback();
+                return toReturn;
             }
             query2.finish();
 
             e->setModified(false);
-
-//            qWarning() << e;
-//            qWarning() << e->data(EpisodeData::Label);
             toReturn << e;
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return toReturn;
     }
+    query.finish();
     DB.commit();
-//    qWarning() << toReturn;
     return toReturn;
 }
 
@@ -903,6 +1166,7 @@ bool EpisodeBase::getEpisodeContent(EpisodeData *episode)
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
         return false;
     }
@@ -911,7 +1175,103 @@ bool EpisodeBase::getEpisodeContent(EpisodeData *episode)
     return true;
 }
 
-void EpisodeBase::toTreeWidget(QTreeWidget *tree)
+/** Return the raw xml content of a episode identified by its \e uid */
+QString EpisodeBase::getEpisodeContent(const QVariant &uid)
+{
+    if (!uid.isValid())
+        return QString::null;
+    if (uid.toInt() < 0)
+        return QString::null;
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return QString::null;
+    }
+    QHash<int, QString> where;
+    where.insert(Constants::EPISODE_CONTENT_EPISODE_ID, QString("=%1").arg(uid.toString()));
+    QString req = select(Constants::Table_EPISODE_CONTENT, Constants::EPISODE_CONTENT_XML, where);
+    DB.transaction();
+    QSqlQuery query(DB);
+    if (query.exec(req)) {
+        if (query.next()) {
+            const QString &xml = query.value(0).toString();
+            query.finish();
+            DB.commit();
+            return xml;
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return QString::null;
+    }
+    query.finish();
+    DB.commit();
+    return QString::null;
+}
+
+/** Return all the episode validation of an episode. */
+QList<EpisodeValidationData *> EpisodeBase::getEpisodeValidations(const QVariant &uid)
+{
+    QList<EpisodeValidationData*> toReturn;
+    if (!uid.isValid())
+        return toReturn;
+    if (uid.toInt() < 0)
+        return toReturn;
+    QSqlDatabase DB = QSqlDatabase::database(DB_NAME);
+    if (!connectDatabase(DB, __LINE__)) {
+        return toReturn;
+    }
+    DB.transaction();
+    QSqlQuery query(DB);
+    QHash<int, QString> where;
+    where.insert(VALIDATION_EPISODE_ID, QString("=%1").arg(uid.toInt()));
+    QString req = select(Table_VALIDATION, where);
+    if (query.exec(req)) {
+        while (query.next()) {
+            EpisodeValidationData *v = new EpisodeValidationData;
+            v->setData(EpisodeValidationData::ValidationId, query.value(VALIDATION_ID));
+            v->setData(EpisodeValidationData::ValidationDate, query.value(VALIDATION_DATEOFVALIDATION));
+            v->setData(EpisodeValidationData::UserUid, query.value(VALIDATION_USERUID));
+            v->setData(EpisodeValidationData::IsValid, query.value(VALIDATION_ISVALID));
+            v->setModified(false);
+            toReturn << v;
+        }
+    } else {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
+        return toReturn;
+    }
+    query.finish();
+    DB.commit();
+    return toReturn;
+}
+
+/**
+ * Returns the total number of episodes recorded for one Form identified by its \e formUid for the current patient.
+ * Counts also all form equivalents.
+ */
+int EpisodeBase::getNumberOfEpisodes(const QString &formUid, const QStringList &equivalents)
+{
+    // Filter the form uuid (and equivalents)
+    Utils::FieldList uuid;
+    uuid << Utils::Field(Constants::Table_EPISODES, Constants::EPISODES_FORM_PAGE_UID, QString("='%1'").arg(formUid));
+    if (!equivalents.isEmpty()) {
+        foreach(const QString &eq, equivalents)
+            uuid << Utils::Field(Constants::Table_EPISODES, Constants::EPISODES_FORM_PAGE_UID, QString("='%1'").arg(eq));
+    }
+
+    // Filter valid episodes
+    QHash<int, QString> where;
+    where.insert(Constants::EPISODES_ISVALID, "=1");
+    where.insert(Constants::EPISODES_PATIENT_UID, QString("='%1'").arg(patient()->uuid()));
+
+    QString filter = getWhereClause(Constants::Table_EPISODES, where).remove("WHERE") +
+            " AND (" + getWhereClause(uuid, Utils::Database::OR) + ")";
+    return count(Constants::Table_EPISODES, Constants::EPISODES_ID, filter);
+}
+
+void EpisodeBase::toTreeWidget(QTreeWidget *tree) const
 {
     Database::toTreeWidget(tree);
     QString uuid = user()->uuid();
