@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
@@ -79,13 +79,12 @@ static inline QString fullDatabasePath() {
 #ifdef FREETOOLBOX
     return QDir::cleanPath(settings()->value(Core::Constants::S_DBOUTPUT_PATH).toString() + "/icd10/");
 #else
-//    return settings()->path(Core::ISettings::ReadOnlyDatabasesPath) + QDir::separator() + QString(ICD::Constants::DB_ICD10);
-    QString dbRelPath = QString("/%1").arg(Constants::DB_ICD10);
+//    return settings()->path(Core::ISettings::ReadOnlyDatabasesPath) + QDir::separator() + QString(ICD::Constants::DB_NAME);
+    QString dbRelPath = QString("/%1").arg(Constants::DB_NAME);
     QString tmp;
     tmp = settings()->dataPackInstallPath() + dbRelPath;
     if (QFileInfo(tmp).exists())
         return settings()->dataPackInstallPath();
-    tmp = settings()->dataPackApplicationInstalledPath() + dbRelPath;
     return settings()->dataPackApplicationInstalledPath();
 #endif
 }
@@ -93,7 +92,6 @@ static inline QString fullDatabasePath() {
 namespace ICD {
 namespace Internal {
 
-
 struct Daget {
     int associatedSid;
     QString dag;
@@ -102,8 +100,9 @@ struct Daget {
 class IcdDatabasePrivate
 {
 public:
-    IcdDatabasePrivate(IcdDatabase *base) :
-            m_LogChrono(false), m_DownloadAndPopulate(false), q(base)
+    IcdDatabasePrivate(IcdDatabase */*base*/) :
+        m_LogChrono(false),
+        m_DownloadAndPopulate(false) // , q(base)
     {
         m_CachedCodes.setMaxCost(1000);
         m_CachedDaget.setMaxCost(1000);
@@ -126,12 +125,11 @@ public:
     QMultiHash<int, Daget *> m_CachedDependentDaget;
 
 private:
-    IcdDatabase *q;
+//    IcdDatabase *q;
 };
 }  // End Internal
 }  // End ICD
 
-
 //--------------------------------------------------------------------------------------------------------
 //--------------------------------- Initialization of static members -------------------------------------
 //--------------------------------------------------------------------------------------------------------
@@ -146,7 +144,6 @@ IcdDatabase *IcdDatabase::instance()
 {
     if (!m_Instance) {
         m_Instance = new IcdDatabase(qApp);
-        m_Instance->init();
     }
     return m_Instance;
 }
@@ -282,6 +279,8 @@ IcdDatabase::IcdDatabase(QObject *parent) :
     addField(Table_Version,  VERSION_DATE, "date");
     addField(Table_Version,  VERSION_COMMENT, "expl");
 
+    connect(Core::ICore::instance()->translators(), SIGNAL(languageChanged()), this, SLOT(refreshLanguageDependCache()));
+
     connect(packManager(), SIGNAL(packInstalled(DataPack::Pack)), this, SLOT(packChanged(DataPack::Pack)));
     connect(packManager(), SIGNAL(packRemoved(DataPack::Pack)), this, SLOT(packChanged(DataPack::Pack)));
 //    connect(packManager(), SIGNAL(packUpdated(DataPack::Pack)), this, SLOT(packChanged(DataPack::Pack)));
@@ -290,11 +289,12 @@ IcdDatabase::IcdDatabase(QObject *parent) :
 /** \brief Destructor. */
 IcdDatabase::~IcdDatabase()
 {
-    if (d) delete d;
+    if (d)
+        delete d;
     d=0;
 }
 
-bool IcdDatabase::init()
+bool IcdDatabase::initialize()
 {
     // only one base can be initialized
     if (m_initialized)
@@ -311,11 +311,11 @@ bool IcdDatabase::init()
 
      // log the path of the database
      QString pathToDb = fullDatabasePath();
-     LOG(tkTr(Trans::Constants::SEARCHING_DATABASE_1_IN_PATH_2).arg(Constants::DB_ICD10).arg(pathToDb));
+     LOG(tkTr(Trans::Constants::SEARCHING_DATABASE_1_IN_PATH_2).arg(Constants::DB_NAME).arg(pathToDb));
 
      // Removing existing old connections
-     if (QSqlDatabase::contains(Constants::DB_ICD10)) {
-         QSqlDatabase::removeDatabase(Constants::DB_ICD10);
+     if (QSqlDatabase::contains(Constants::DB_NAME)) {
+         QSqlDatabase::removeDatabase(Constants::DB_NAME);
      }
 
      // Connect normal Account Database
@@ -323,17 +323,17 @@ bool IcdDatabase::init()
      Utils::DatabaseConnector connector;
 //     connector.setAbsPathToReadOnlySqliteDatabase(settings()->path(Core::ISettings::ReadOnlyDatabasesPath));
      connector.setAbsPathToReadOnlySqliteDatabase(fullDatabasePath());
-     connector.setHost(QString(Constants::DB_ICD10) + ".db");
+     connector.setHost(QString(Constants::DB_NAME) + ".db");
      connector.setAccessMode(Utils::DatabaseConnector::ReadOnly);
      connector.setDriver(Utils::Database::SQLite);
 
-     if (!createConnection(Constants::DB_ICD10, QString(Constants::DB_ICD10) + ".db", connector)) {
+     if (!createConnection(Constants::DB_NAME, QString(Constants::DB_NAME) + ".db", connector)) {
          d->m_DownloadAndPopulate = true;
      }
 
      if (!database().isOpen()) {
          if (!database().open()) {
-             LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+             LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
          } else { // db successfully opened
              LOG(tkTr(Trans::Constants::CONNECTED_TO_DATABASE_1_DRIVER_2).arg(database().connectionName()).arg(database().driverName()));
 
@@ -349,8 +349,6 @@ bool IcdDatabase::init()
      if (!d->m_DownloadAndPopulate)
          m_initialized = true;
 
-     connect(Core::ICore::instance()->translators(), SIGNAL(languageChanged()), this, SLOT(refreshLanguageDependCache()));
-
      if (m_initialized) {
          Q_EMIT databaseInitialized();
      }
@@ -360,9 +358,12 @@ bool IcdDatabase::init()
 
 bool IcdDatabase::refreshDatabase()
 {
+    WARN_FUNC;
     m_initialized = false;
-    QSqlDatabase::removeDatabase(Constants::DB_ICD10);
-    return init();
+    d->m_DownloadAndPopulate = false;
+    if (QSqlDatabase::connectionNames().contains(Constants::DB_NAME))
+        QSqlDatabase::removeDatabase(Constants::DB_NAME);
+    return initialize();
 }
 
 void IcdDatabase::packChanged(const DataPack::Pack &pack)
@@ -391,7 +392,7 @@ QString IcdDatabase::getDatabaseVersion()
         return toReturn;
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return toReturn;
         }
     }
@@ -459,7 +460,7 @@ QList<int> IcdDatabase::getHeadersSID(const QVariant &SID)
     QList<int> toReturn;
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return toReturn;
         }
     }
@@ -496,7 +497,7 @@ QVariant IcdDatabase::getSid(const QString &code)
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QVariant();
         }
     }
@@ -523,7 +524,7 @@ QVariant IcdDatabase::getIcdCode(const QVariant &SID)
     }
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QVariant();
         }
     }
@@ -551,7 +552,7 @@ QString IcdDatabase::getDagStarCode(const QVariant &SID)
     }
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QString();
         }
     }
@@ -605,14 +606,15 @@ QString IcdDatabase::getHumanReadableIcdDaget(const QVariant &SID)
 
 QVariant IcdDatabase::getIcdCodeWithDagStar(const QVariant &SID)
 {
-    return getIcdCode(SID).toString() + getHumanReadableIcdDaget(SID);
+    QString tmp = getIcdCode(SID).toString() + getHumanReadableIcdDaget(SID);
+    return QVariant(tmp);
 }
 
 QVector<int> IcdDatabase::getDagStarDependencies(const QVariant &SID)
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QVector<int>();
         }
     }
@@ -645,7 +647,7 @@ Internal::IcdAssociation IcdDatabase::getAssociation(const QVariant &mainSID, co
     }
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return Internal::IcdAssociation();
         }
     }
@@ -693,7 +695,7 @@ bool IcdDatabase::codeCanBeUsedAlone(const QVariant &SID)
 //    }
 //    if (!database().isOpen()) {
 //        if (!database().open()) {
-//            LOG_ERROR(this, tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+//            LOG_ERROR(this, tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
 //            return QString();
 //        }
 //    }
@@ -730,7 +732,7 @@ QString IcdDatabase::getLabelFromLid(const QVariant &LID)
     }
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QString();
         }
     }
@@ -758,7 +760,7 @@ QString IcdDatabase::getSystemLabel(const QVariant &SID)
     }
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QString();
         }
     }
@@ -788,7 +790,7 @@ QStringList IcdDatabase::getAllLabels(const QVariant &SID, const int libelleFiel
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QStringList();
         }
     }
@@ -819,7 +821,7 @@ QString IcdDatabase::getAssociatedLabel(const QVariant &mainSID, const QVariant
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QString();
         }
     }
@@ -847,7 +849,7 @@ QStringList IcdDatabase::getIncludedLabels(const QVariant &SID)
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QStringList();
         }
     }
@@ -879,7 +881,7 @@ QVector<int> IcdDatabase::getExclusions(const QVariant &SID)
 {
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QVector<int>();
         }
     }
@@ -912,7 +914,7 @@ QString IcdDatabase::getMemo(const QVariant &SID)
     // get the Note
     if (!database().isOpen()) {
         if (!database().open()) {
-            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_ICD10).arg(database().lastError().text()));
+            LOG_ERROR(tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2).arg(Constants::DB_NAME).arg(database().lastError().text()));
             return QString();
         }
     }
