@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,27 +19,32 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 #include "icdwidgetmanager.h"
 #include "constants.h"
 #include "icddownloader.h"
 #include "icdcentralwidget.h"
+#include "icddatabase.h"
 
 #include <utils/log.h>
 #include <utils/global.h>
-#include <translationutils/constanttranslations.h>
+#include <utils/widgets/databaseinformationdialog.h>
+#include <translationutils/constants.h>
+#include <translationutils/trans_menu.h>
+#include <translationutils/trans_database.h>
 
+#include <coreplugin/icore.h>
+#include <coreplugin/itheme.h>
+#include <coreplugin/isettings.h>
+#include <coreplugin/imainwindow.h>
 #include <coreplugin/constants_icons.h>
 #include <coreplugin/constants_menus.h>
-#include <coreplugin/icore.h>
-#include <coreplugin/isettings.h>
-#include <coreplugin/itheme.h>
 #include <coreplugin/contextmanager/contextmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/uniqueidmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 
 #include <QGridLayout>
 #include <QDialog>
@@ -50,7 +55,7 @@ using namespace ICD;
 using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
-inline static Core::ActionManager *actionManager() {return Core::ICore::instance()->actionManager();}
+static inline Core::ActionManager *actionManager() {return Core::ICore::instance()->actionManager();}
 static inline Core::ContextManager *contextManager() { return Core::ICore::instance()->contextManager(); }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -67,14 +72,15 @@ IcdWidgetManager *IcdWidgetManager::instance()
 
 IcdWidgetManager::IcdWidgetManager(QObject *parent) : IcdActionHandler(parent)
 {
-    connect(Core::ICore::instance()->contextManager(), SIGNAL(contextChanged(Core::IContext*)),
-            this, SLOT(updateContext(Core::IContext*)));
+    connect(Core::ICore::instance()->contextManager(), SIGNAL(contextChanged(Core::IContext*,Core::Context)),
+            this, SLOT(updateContext(Core::IContext*,Core::Context)));
     setObjectName("IcdWidgetManager");
 //    Utils::Log::addMessage(this, "Instance created");
 }
 
-void IcdWidgetManager::updateContext(Core::IContext *object)
+void IcdWidgetManager::updateContext(Core::IContext *object, const Core::Context &additionalContexts)
 {
+    Q_UNUSED(additionalContexts);
     IcdCentralWidget *view = 0;
     do {
         if (!object) {
@@ -125,15 +131,11 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
         m_CurrentView(0)
 {
     setObjectName("IcdActionHandler");
-//    Utils::Log::addMessage(this, "Instance created");
-
-    Core::UniqueIDManager *uid = Core::ICore::instance()->uniqueIDManager();
     Core::ITheme *th = Core::ICore::instance()->theme();
-
     QAction *a = 0;
     Core::Command *cmd = 0;
-    QList<int> ctx = QList<int>() << uid->uniqueIdentifier(ICD::Constants::C_ICD_PLUGINS);
-    QList<int> globalcontext = QList<int>() << Core::Constants::C_GLOBAL_ID;
+    Core::Context ctx(ICD::Constants::C_ICD_PLUGINS);
+    Core::Context globalcontext(Core::Constants::C_GLOBAL);
 
 //    Core::ActionContainer *menu = actionManager()->actionContainer(Constants::M_PLUGINS_ICD);
 //    if (!menu) {
@@ -149,34 +151,34 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
 //#endif
 
     // Create ICD10 database
-    Core::ActionContainer *hmenu = actionManager()->actionContainer(Core::Constants::M_HELP_DATABASES);
+    Core::ActionContainer *hmenu = actionManager()->actionContainer(Core::Id(Core::Constants::M_HELP_DATABASES));
     if (!Utils::isReleaseCompilation()) {
         a = aRecreateDatabase = new QAction(this);
         a->setObjectName("aRecreateDatabase");
-        cmd = actionManager()->registerAction(a, Constants::A_RECREATE_ICD_DB, QList<int>() << Core::Constants::C_GLOBAL_ID);
+        cmd = actionManager()->registerAction(a, Core::Id(Constants::A_RECREATE_ICD_DB), globalcontext);
         cmd->setTranslations(Constants::RECREATE_DATABASE_TEXT, Constants::RECREATE_DATABASE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-        hmenu->addAction(cmd, Core::Constants::G_HELP_DATABASES);
+        hmenu->addAction(cmd, Core::Id(Core::Constants::G_HELP_DATABASES));
         connect(a, SIGNAL(triggered()), this, SLOT(recreateDatabase()));
     }
 
     // Show Databases information
     a = aShowDatabaseInformation = new QAction(this);
     a->setIcon(th->icon(Core::Constants::ICONHELP));
-    cmd = actionManager()->registerAction(a, Constants::A_DATABASE_INFOS, QList<int>() << Core::Constants::C_GLOBAL_ID);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_DATABASE_INFOS), globalcontext);
     cmd->setTranslations(Constants::DATABASE_INFOS_TEXT, Constants::DATABASE_INFOS_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
     cmd->retranslate();
-    hmenu->addAction(cmd, Core::Constants::G_HELP_DATABASES);
+    hmenu->addAction(cmd, Core::Id(Core::Constants::G_HELP_DATABASES));
     connect(aShowDatabaseInformation,SIGNAL(triggered()), this, SLOT(showDatabaseInformation()));
 
     // Search method menu
-    Core::ActionContainer *searchmenu = actionManager()->actionContainer(Constants::M_ICD_SEARCH);
+    Core::ActionContainer *searchmenu = actionManager()->actionContainer(Core::Id(Constants::M_ICD_SEARCH));
     if (!searchmenu) {
         searchmenu = actionManager()->createMenu(Constants::M_ICD_SEARCH);
-        searchmenu->appendGroup(Constants::G_ICD_SEARCH);
+        searchmenu->appendGroup(Core::Id(Constants::G_ICD_SEARCH));
         searchmenu->setTranslations(Constants::SEARCHMENU_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
 #ifndef FREETOOLBOX
-        Core::ActionContainer *menu = actionManager()->actionContainer(Core::Constants::M_EDIT);
-        menu->addMenu(searchmenu, Core::Constants::G_EDIT_FIND);
+        Core::ActionContainer *menu = actionManager()->actionContainer(Core::Id(Core::Constants::M_EDIT));
+        menu->addMenu(searchmenu, Core::Id(Core::Constants::G_EDIT_FIND));
 #endif
     }
     Q_ASSERT(searchmenu);
@@ -186,18 +188,18 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
     a->setCheckable(true);
     a->setChecked(false);
     a->setIcon(th->icon(Constants::I_SEARCH_LABEL));
-    cmd = actionManager()->registerAction(a, Constants::A_SEARCH_LABEL, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_SEARCH_LABEL), ctx);
     cmd->setTranslations(Constants::SEARCHLABEL_TEXT, Constants::SEARCHLABEL_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    searchmenu->addAction(cmd, Constants::G_ICD_SEARCH);
+    searchmenu->addAction(cmd, Core::Id(Constants::G_ICD_SEARCH));
     gSearchMethod->addAction(a);
 
     a = aSearchByCode = new QAction(this);
     a->setCheckable(true);
     a->setChecked(false);
     a->setIcon(th->icon(Constants::I_SEARCH_CODE));
-    cmd = actionManager()->registerAction(a, Constants::A_SEARCH_CODE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_SEARCH_CODE), ctx);
     cmd->setTranslations(Constants::SEARCHCODE_TEXT, Constants::SEARCHCODE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    searchmenu->addAction(cmd, Constants::G_ICD_SEARCH);
+    searchmenu->addAction(cmd, Core::Id(Constants::G_ICD_SEARCH));
     gSearchMethod->addAction(a);
     connect(gSearchMethod,SIGNAL(triggered(QAction*)),this,SLOT(searchActionChanged(QAction*)));
 
@@ -205,12 +207,12 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
     Core::ActionContainer *modesmenu = actionManager()->actionContainer(Constants::M_ICD_MODES);
     if (!modesmenu) {
         modesmenu = actionManager()->createMenu(Constants::M_ICD_MODES);
-        modesmenu->appendGroup(Constants::G_ICD_SELECTORMODE);
-        modesmenu->appendGroup(Constants::G_ICD_COLLECTIONMODE);
+        modesmenu->appendGroup(Core::Id(Constants::G_ICD_SELECTORMODE));
+        modesmenu->appendGroup(Core::Id(Constants::G_ICD_COLLECTIONMODE));
         modesmenu->setTranslations(Constants::MODESMENU_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
 #ifndef FREETOOLBOX
         Core::ActionContainer *menu = actionManager()->actionContainer(Core::Constants::M_EDIT);
-        menu->addMenu(modesmenu, Core::Constants::G_EDIT_OTHER);
+        menu->addMenu(modesmenu, Core::Id(Core::Constants::G_EDIT_OTHER));
 #endif
     }
     Q_ASSERT(modesmenu);
@@ -220,36 +222,36 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
     a->setObjectName("aSelectorSimpleMode");
     a->setCheckable(true);
     a->setChecked(false);
-    cmd = actionManager()->registerAction(a, Constants::A_SELECTOR_SIMPLEMODE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_SELECTOR_SIMPLEMODE), ctx);
     cmd->setTranslations(Constants::SELECTORSIMPLEMODE_TEXT, Constants::SELECTORSIMPLEMODE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    modesmenu->addAction(cmd, Constants::G_ICD_SELECTORMODE);
+    modesmenu->addAction(cmd, Core::Id(Constants::G_ICD_SELECTORMODE));
     gModes->addAction(a);
 
     a = aSelectorFullMode = new QAction(this);
     a->setObjectName("aSelectorFullMode");
     a->setCheckable(true);
     a->setChecked(false);
-    cmd = actionManager()->registerAction(a, Constants::A_SELECTOR_FULLMODE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_SELECTOR_FULLMODE), ctx);
     cmd->setTranslations(Constants::SELECTORFULLMODE_TEXT, Constants::SELECTORFULLMODE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    modesmenu->addAction(cmd, Constants::G_ICD_SELECTORMODE);
+    modesmenu->addAction(cmd, Core::Id(Constants::G_ICD_SELECTORMODE));
     gModes->addAction(a);
 
     a = aCollectionModelSimpleMode = new QAction(this);
     a->setObjectName("aCollectionModelSimpleMode");
     a->setCheckable(true);
     a->setChecked(false);
-    cmd = actionManager()->registerAction(a, Constants::A_COLLECTION_SIMPLEMODE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_COLLECTION_SIMPLEMODE), ctx);
     cmd->setTranslations(Constants::COLLECTIONSIMPLEMODE_TEXT, Constants::COLLECTIONSIMPLEMODE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    modesmenu->addAction(cmd, Constants::G_ICD_COLLECTIONMODE);
+    modesmenu->addAction(cmd, Core::Id(Constants::G_ICD_COLLECTIONMODE));
     gModes->addAction(a);
 
     a = aCollectionModelFullMode = new QAction(this);
     a->setObjectName("aCollectionModelFullMode");
     a->setCheckable(true);
     a->setChecked(false);
-    cmd = actionManager()->registerAction(a, Constants::A_COLLECTION_FULLMODE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_COLLECTION_FULLMODE), ctx);
     cmd->setTranslations(Constants::COLLECTIONFULLMODE_TEXT, Constants::COLLECTIONFULLMODE_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-    modesmenu->addAction(cmd, Constants::G_ICD_COLLECTIONMODE);
+    modesmenu->addAction(cmd, Core::Id(Constants::G_ICD_COLLECTIONMODE));
     gModes->addAction(a);
     connect(gModes, SIGNAL(triggered(QAction*)), this, SLOT(modeActionChanged(QAction*)));
 
@@ -257,21 +259,21 @@ IcdActionHandler::IcdActionHandler(QObject *parent) :
     a = aToggleSelector = new QAction(this);
     a->setObjectName("aToggleSelector");
     a->setIcon(th->icon(Constants::I_TOGGLEICDSELECTOR));
-    cmd = actionManager()->registerAction(a, Constants::A_TOGGLE_ICDSELECTOR, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Constants::A_TOGGLE_ICDSELECTOR), ctx);
     cmd->setTranslations(Constants::TOGGLEICDSELECTOR_TEXT, Constants::TOGGLEICDSELECTOR_TEXT, Constants::ICDCONSTANTS_TR_CONTEXT);
-//    menu->addAction(cmd, DrugsWidget::Constants::G_PLUGINS_VIEWS);
+//    menu->addAction(cmd, Core::Id(DrugsWidget::Constants::G_PLUGINS_VIEWS));
     connect(a, SIGNAL(triggered()), this, SLOT(toggleSelector()));
 
     a = aClear = new QAction(this);
     a->setIcon(th->icon(Core::Constants::ICONCLEAR));
-    cmd = actionManager()->registerAction(a, Core::Constants::A_LIST_CLEAR, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Core::Constants::A_LIST_CLEAR), ctx);
     cmd->setTranslations(Trans::Constants::LISTCLEAR_TEXT);
 //    menu->addAction(cmd, DrugsWidget::Constants::G_PLUGINS_DRUGS);
     connect(a, SIGNAL(triggered()), this, SLOT(clear()));
 
     a = aRemoveRow = new QAction(this);
     a->setIcon(th->icon(Core::Constants::ICONREMOVE));
-    cmd = actionManager()->registerAction(a, Core::Constants::A_LIST_REMOVE, ctx);
+    cmd = actionManager()->registerAction(a, Core::Id(Core::Constants::A_LIST_REMOVE), ctx);
     cmd->setTranslations(Trans::Constants::LISTREMOVE_TEXT);
 //    menu->addAction(cmd, DrugsWidget::Constants::G_PLUGINS_DRUGS);
     connect(a, SIGNAL(triggered()), this, SLOT(removeItem()));
@@ -382,13 +384,9 @@ void IcdActionHandler::recreateDatabase()
 
 void IcdActionHandler::showDatabaseInformation()
 {
-    QDialog dlg(qApp->activeWindow(), Qt::Window | Qt::CustomizeWindowHint | Qt::WindowSystemMenuHint | Qt::WindowCloseButtonHint | Qt::WindowMinMaxButtonsHint);
-    QGridLayout lay(&dlg);
-    QTreeWidget tree(&dlg);
-    tree.setColumnCount(2);
-    tree.header()->hide();
-
-    lay.addWidget(&tree);
+    Utils::DatabaseInformationDialog dlg(Core::ICore::instance()->mainWindow());
+    dlg.setTitle(tkTr(Trans::Constants::ICD_DATABASE_INFORMATION));
+    dlg.setDatabase(*IcdDatabase::instance());
     Utils::resizeAndCenter(&dlg);
     dlg.exec();
 }
