@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,9 +19,9 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *  Main Developers : Eric Maeker <eric.maeker@gmail.com>,                *
+ *  Main Developers:  Eric Maeker <eric.maeker@gmail.com>,                *
  *                    Guillaume Denry <guillaume.denry@gmail.com>          *
- *  Contributors :                                                         *
+ *  Contributors:                                                          *
  *      NAME <MAIL@ADDRESS.COM>                                            *
  ***************************************************************************/
 /**
@@ -45,6 +45,7 @@
 #include <QDebug>
 
 using namespace PadTools;
+using namespace Internal;
 
 PadConditionnalSubItem::PadConditionnalSubItem(TokenCoreCondition cond, Place place, PadFragment *parent) :
     PadFragment(parent), _coreCond(cond), _place(place)
@@ -72,7 +73,7 @@ void PadConditionnalSubItem::debug(int indent) const
 
 void PadConditionnalSubItem::run(QMap<QString,QVariant> &tokens, PadDocument *document)
 {
-//    qWarning() << "run PadConditionnalSubItem";
+//qWarning() << "run PadConditionnalSubItem";
     // Get parent PadItem
     PadFragment *f = parent();
     PadItem *item = 0;
@@ -87,9 +88,9 @@ void PadConditionnalSubItem::run(QMap<QString,QVariant> &tokens, PadDocument *do
         return;
     }
 
-    const QString &value = tokens.value(item->getCore()->name()).toString();
+    const QString &value = tokens.value(item->getCore()->uid()).toString();
 
-//    const QString &value = tokens[_name].toString();
+//const QString &value = tokens[_name].toString();
     bool removeMe = false;
     if (value.isEmpty()) {
         // Keep Type::Undefined / Before && After
@@ -112,10 +113,10 @@ void PadConditionnalSubItem::run(QMap<QString,QVariant> &tokens, PadDocument *do
         document->positionTranslator().addOutputTranslation(outputStart(), -rawLength());
         return;
     } else {
-//         Remove only delimiters
+// Remove only delimiters
         foreach(const PadDelimiter &delim, _delimiters) {
 
-//            qWarning() << "SUBITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
+//qWarning() << "SUBITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
 
             QTextCursor cursor(document->outputDocument());
             int pos = document->positionTranslator().rawToOutput(delim.rawPos);
@@ -133,9 +134,9 @@ void PadConditionnalSubItem::run(QMap<QString,QVariant> &tokens, PadDocument *do
     setOutputEnd(document->positionTranslator().rawToOutput(end()));
 }
 
-void PadConditionnalSubItem::toOutput(Core::ITokenPool *pool, PadDocument *document)
+void PadConditionnalSubItem::toOutput(Core::ITokenPool *pool, PadDocument *document, TokenReplacementMethod method)
 {
-//    qWarning() << "run PadConditionnalSubItem";
+//qWarning() << "run PadConditionnalSubItem";
     // Get parent PadItem
     PadFragment *f = parent();
     PadItem *item = 0;
@@ -149,11 +150,19 @@ void PadConditionnalSubItem::toOutput(Core::ITokenPool *pool, PadDocument *docum
         LOG_ERROR_FOR("PadConditionnalSubItem", "No PadItem parent");
         return;
     }
+    if (!item->getCore()) {
+        LOG_ERROR_FOR("PadConditionnalSubItem", "No PadCore parent");
+        return;
+    }
 
-//    const QString &value = tokens.value(item->getCore()->name()).toString();
-    const QString &value = pool->tokenCurrentValue(item->getCore()->name()).toString();
-
-//    const QString &value = tokens[_name].toString();
+    QString value;
+    switch (method) {
+    case ReplaceWithTokenDisplayName: value = pool->token(item->getCore()->uid())->humanReadableName(); break;
+    case ReplaceWithTokenTestingValue: value = pool->token(item->getCore()->uid())->testValue().toString(); break;
+    case ReplaceWithTokenUuid: value = item->getCore()->uid();  break;
+    default: // ReplaceWithTokenValue
+        value = pool->token(item->getCore()->uid())->value().toString();
+    }
     bool removeMe = false;
     if (value.isEmpty()) {
         // Keep Type::Undefined / Before && After
@@ -176,10 +185,10 @@ void PadConditionnalSubItem::toOutput(Core::ITokenPool *pool, PadDocument *docum
         document->positionTranslator().addOutputTranslation(outputStart(), -rawLength());
         return;
     } else {
-//         Remove only delimiters
+// Remove only delimiters
         foreach(const PadDelimiter &delim, _delimiters) {
 
-//            qWarning() << "SUBITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
+            // qWarning() << "SUBITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
 
             QTextCursor cursor(document->outputDocument());
             int pos = document->positionTranslator().rawToOutput(delim.rawPos);
@@ -192,7 +201,7 @@ void PadConditionnalSubItem::toOutput(Core::ITokenPool *pool, PadDocument *docum
 
         // Run nested fragments
         foreach(PadFragment *frag, _fragments)
-            frag->toOutput(pool, document);
+            frag->toOutput(pool, document, method);
     }
     setOutputEnd(document->positionTranslator().rawToOutput(end()));
 }
@@ -222,6 +231,18 @@ void PadConditionnalSubItem::toRaw(PadDocument *doc)
     Q_UNUSED(doc);
 }
 
+/** Returns the UUID of the PadTools::PadCore fragment */
+const QString &PadCore::uid() const
+{
+    return _uid;
+}
+
+/** Defines the UUID of the PadTools::PadCore fragment */
+void PadCore::setUid(const QString &uid)
+{
+    _uid = uid;
+}
+
 /** Overwrite the position tester, by default a PadTools::PadCore does not include the outputStart() and outputEnd() position. */
 bool PadCore::containsOutputPosition(const int pos) const
 {
@@ -247,15 +268,15 @@ void PadCore::debug(int indent) const
     str += QString("[padCore:Source(%1;%2);Output(%3;%4)]: %5")
             .arg(start()).arg(end())
             .arg(outputStart()).arg(outputEnd())
-            .arg(_name);
+            .arg(_uid);
     qDebug("%s", qPrintable(str));
 }
 
 void PadCore::run(QMap<QString,QVariant> &tokens, PadDocument *document)
 {
-//    qWarning() << "run Core";
+//qWarning() << "run Core";
     // PadItem calls run on the Core only if core value is defined
-    const QString &value = tokens[_name].toString();
+    const QString &value = tokens[_uid].toString();
     if (value.isEmpty()) {
         // Define output range
         LOG_ERROR_FOR("PadCore", "token run without value? Check PadItem.");
@@ -286,13 +307,13 @@ void PadCore::run(QMap<QString,QVariant> &tokens, PadDocument *document)
     document->positionTranslator().addOutputTranslation(outputStart(), delta);
 }
 
-void PadCore::toOutput(Core::ITokenPool *pool, PadDocument *document)
+void PadCore::toOutput(Core::ITokenPool *pool, PadDocument *document, TokenReplacementMethod method)
 {
-//    qWarning() << "run Core";
+    Q_UNUSED(method);
+    // qWarning() << "run Core";
     // PadItem calls run on the Core only if core value is defined
-//    const QString &value = tokens[_name].toString();
-    const QString &value = pool->tokenCurrentValue(_name).toString();
-    if (value.isEmpty()) {
+    const QString &coreValue = tokenValue(pool, method);
+    if (coreValue.isEmpty()) {
         // Define output range
         LOG_ERROR_FOR("PadCore", "token run without value? Check PadItem.");
         return;
@@ -300,21 +321,25 @@ void PadCore::toOutput(Core::ITokenPool *pool, PadDocument *document)
     // Compute output positions
     setOutputStart(document->positionTranslator().rawToOutput(start()));
 
-    // Replace core source
+    // Replace the core token by its value
     QTextCursor cursor(document->outputDocument());
     cursor.setPosition(outputStart());
     cursor.setPosition(outputStart() + rawLength(), QTextCursor::KeepAnchor);
     QTextCharFormat format = cursor.charFormat();
     cursor.removeSelectedText();
-    if (Qt::mightBeRichText(value)) {
-        cursor.insertHtml(value);
+
+    // Add core value in the output document according to the contentType
+    if ((document->contentType() == PadDocument::ContentAutoType
+            && Qt::mightBeRichText(coreValue))
+            || document->contentType() == PadDocument::ContentIsHtml) {
+        cursor.insertHtml(coreValue);
         setOutputEnd(cursor.selectionEnd());
         cursor.setPosition(outputStart());
         cursor.setPosition(outputEnd(), QTextCursor::KeepAnchor);
         cursor.mergeCharFormat(format);
     } else {
-        cursor.insertText(value, format);
-        setOutputEnd(outputStart() + value.size());
+        cursor.insertText(coreValue, format);
+        setOutputEnd(outputStart() + coreValue.size());
     }
 
     // Add translation to document
@@ -324,8 +349,8 @@ void PadCore::toOutput(Core::ITokenPool *pool, PadDocument *document)
 
 void PadCore::toRaw(PadDocument *doc)
 {
-//    qWarning() << "core start";
-//    debug(2);
+    // qWarning() << "core start";
+    // debug(2);
     QTextCursor raw(doc->rawSourceDocument());
     int oldLength = _end - _start;
 
@@ -338,8 +363,8 @@ void PadCore::toRaw(PadDocument *doc)
     raw.setPosition(_start);
     raw.setPosition(_end, QTextCursor::KeepAnchor);
     raw.removeSelectedText();
-    raw.insertText(_name);
-    int newLength = _name.size();
+    raw.insertText(_uid);
+    int newLength = _uid.size();
     doc->positionTranslator().addRawTranslation(_start, newLength-oldLength);
     doc->positionTranslator().addRawTranslation(_start, s);
 
@@ -352,12 +377,26 @@ void PadCore::toRaw(PadDocument *doc)
     raw.setPosition(_end);
     raw.insertText(Constants::TOKEN_CORE_DELIMITER);
     setEnd(_end + s);
-//    qWarning() << "core end";
-//    debug(2);
-//    doc->positionTranslator().debug();
+    // qWarning() << "core end";
+    // debug(2);
+    // doc->positionTranslator().debug();
 }
 
-
+/** Return the PadCore value according to the token replacement method */
+QString PadCore::tokenValue(Core::ITokenPool *pool, TokenReplacementMethod method) const
+{
+    switch (method) {
+    case ReplaceWithTokenDisplayName: return pool->token(uid())->humanReadableName();
+    case ReplaceWithTokenTestingValue: return pool->token(uid())->testValue().toString();
+    case ReplaceWithTokenUuid: return uid();
+    default: // ReplaceWithTokenValue
+        if (pool->token(uid()))
+            return pool->token(uid())->value().toString();
+        else
+            qWarning() << "**** Missing token "<<uid();
+    }
+    return QString::null;
+}
 
 /**
  * \class PadTools::PadItem
@@ -448,13 +487,13 @@ PadConditionnalSubItem *PadItem::subItem(const PadConditionnalSubItem::TokenCore
 
 void PadItem::run(QMap<QString,QVariant> &tokens, PadDocument *document)
 {
-//    qWarning() << "run Item: rawStart" << start() << "outputStart" << document->positionTranslator().rawToOutput(start()) << document->positionTranslator().deltaForSourcePosition(start());
+//qWarning() << "run Item: rawStart" << start() << "outputStart" << document->positionTranslator().rawToOutput(start()) << document->positionTranslator().deltaForSourcePosition(start());
     PadCore *core = getCore();
     QString coreValue;
 
     // if core value is null -> remove it from the output
     if (core) {
-        coreValue = tokens.value(core->name()).toString();
+        coreValue = tokens.value(core->uid()).toString();
 
         if (coreValue.isEmpty()) {
             // No value -> Remove the entire PadItem from the text output and add a translation
@@ -468,11 +507,11 @@ void PadItem::run(QMap<QString,QVariant> &tokens, PadDocument *document)
         } else {
             // Remove delimiters before the core
             foreach(const PadDelimiter &delim, _delimiters) {
-//                qWarning() << delim.rawPos << core->start();
+//qWarning() << delim.rawPos << core->start();
                 if (delim.rawPos >= core->start())
                     continue;
 
-//                qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
+//qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
 
                 QTextCursor cursor(document->outputDocument());
                 int pos = document->positionTranslator().rawToOutput(delim.rawPos);
@@ -492,7 +531,7 @@ void PadItem::run(QMap<QString,QVariant> &tokens, PadDocument *document)
                 if (delim.rawPos < core->end())
                     continue;
 
-//                qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
+//qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
 
                 QTextCursor cursor(document->outputDocument());
                 int pos = document->positionTranslator().rawToOutput(delim.rawPos);
@@ -508,66 +547,64 @@ void PadItem::run(QMap<QString,QVariant> &tokens, PadDocument *document)
     }
 }
 
-void PadItem::toOutput(Core::ITokenPool *pool, PadDocument *document)
+void PadItem::toOutput(Core::ITokenPool *pool, PadDocument *document, TokenReplacementMethod method)
 {
-//    qWarning() << "run Item: rawStart" << start() << "outputStart" << document->positionTranslator().rawToOutput(start()) << document->positionTranslator().deltaForSourcePosition(start());
+    // qWarning() << "run Item: rawStart" << start() << "outputStart" << document->positionTranslator().rawToOutput(start()) << document->positionTranslator().deltaForSourcePosition(start());
     PadCore *core = getCore();
-    QString coreValue;
+    if (!core) {
+        LOG_ERROR_FOR("PadItem", "No Core.");
+        return;
+    }
+    // Replace core by its value
+    const QString &coreValue = core->tokenValue(pool, method);
+    if (coreValue.isEmpty()) {
+        // No value -> Remove the entire PadItem from the text output and add a translation
+        QTextCursor cursor(document->outputDocument());
+        setOutputStart(document->positionTranslator().rawToOutput(start()));
+        cursor.setPosition(outputStart());
+        cursor.setPosition(outputStart() + rawLength(), QTextCursor::KeepAnchor);
+        cursor.removeSelectedText();
+        setOutputEnd(outputStart());
+        document->positionTranslator().addOutputTranslation(outputStart(), -rawLength());
+    } else {
+        // Remove delimiters before the core
+        foreach(const PadDelimiter &delim, _delimiters) {
+            // qWarning() << delim.rawPos << core->start();
+            if (delim.rawPos >= core->start())
+                continue;
 
-    // if core value is null -> remove it from the output
-    if (core) {
-//        coreValue = tokens.value(core->name()).toString();
-        coreValue = pool->tokenCurrentValue(core->name()).toString();
+            // qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
 
-        if (coreValue.isEmpty()) {
-            // No value -> Remove the entire PadItem from the text output and add a translation
             QTextCursor cursor(document->outputDocument());
-            setOutputStart(document->positionTranslator().rawToOutput(start()));
-            cursor.setPosition(outputStart());
-            cursor.setPosition(outputStart() + rawLength(), QTextCursor::KeepAnchor);
+            int pos = document->positionTranslator().rawToOutput(delim.rawPos);
+            cursor.setPosition(pos);
+            cursor.setPosition(pos + delim.size, QTextCursor::KeepAnchor);
             cursor.removeSelectedText();
-            setOutputEnd(outputStart());
-            document->positionTranslator().addOutputTranslation(outputStart(), -rawLength());
-        } else {
-            // Remove delimiters before the core
-            foreach(const PadDelimiter &delim, _delimiters) {
-//                qWarning() << delim.rawPos << core->start();
-                if (delim.rawPos >= core->start())
-                    continue;
-
-//                qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
-
-                QTextCursor cursor(document->outputDocument());
-                int pos = document->positionTranslator().rawToOutput(delim.rawPos);
-                cursor.setPosition(pos);
-                cursor.setPosition(pos + delim.size, QTextCursor::KeepAnchor);
-                cursor.removeSelectedText();
-                document->positionTranslator().addOutputTranslation(pos, -delim.size);
-            }
-
-            // Value -> run fragments
-            foreach(PadFragment *f, _fragments) {
-                f->toOutput(pool, document);
-            }
-
-            // Remove delimiters after the core
-            foreach(const PadDelimiter &delim, _delimiters) {
-                if (delim.rawPos < core->end())
-                    continue;
-
-//                qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
-
-                QTextCursor cursor(document->outputDocument());
-                int pos = document->positionTranslator().rawToOutput(delim.rawPos);
-                cursor.setPosition(pos);
-                cursor.setPosition(pos + delim.size, QTextCursor::KeepAnchor);
-                cursor.removeSelectedText();
-                document->positionTranslator().addOutputTranslation(pos, -delim.size);
-            }
-
-            setOutputStart(document->positionTranslator().rawToOutput(start()));
-            setOutputEnd(document->positionTranslator().rawToOutput(end()));
+            document->positionTranslator().addOutputTranslation(pos, -delim.size);
         }
+
+        // Token has a value -> run its children fragments
+        foreach(PadFragment *f, _fragments) {
+            f->toOutput(pool, document, method);
+        }
+
+        // Remove delimiters after the core
+        foreach(const PadDelimiter &delim, _delimiters) {
+            if (delim.rawPos < core->end())
+                continue;
+
+            // qWarning() << "ITEM DELIM raw" << delim.rawPos << "size" << delim.size << "output" << (document->positionTranslator().rawToOutput(delim.rawPos));
+
+            QTextCursor cursor(document->outputDocument());
+            int pos = document->positionTranslator().rawToOutput(delim.rawPos);
+            cursor.setPosition(pos);
+            cursor.setPosition(pos + delim.size, QTextCursor::KeepAnchor);
+            cursor.removeSelectedText();
+            document->positionTranslator().addOutputTranslation(pos, -delim.size);
+        }
+
+        setOutputStart(document->positionTranslator().rawToOutput(start()));
+        setOutputEnd(document->positionTranslator().rawToOutput(end()));
     }
 }
 
@@ -581,7 +618,7 @@ void PadItem::toRaw(PadDocument *doc)
     int s = QString(Constants::TOKEN_OPEN_DELIMITER).size();
     doc->positionTranslator().addRawTranslation(_start, s);
 
-    doc->positionTranslator().debug();
+    // doc->positionTranslator().debug();
 
     PadCore *core = getCore();
     Q_ASSERT(core);
@@ -589,7 +626,7 @@ void PadItem::toRaw(PadDocument *doc)
         return;
     core->toRaw(doc);
 
-    doc->positionTranslator().debug();
+    // doc->positionTranslator().debug();
 
     setEnd(doc->positionTranslator().outputToRaw(_outputEnd));
     raw.setPosition(_end);
@@ -601,7 +638,7 @@ void PadItem::toRaw(PadDocument *doc)
         if (f!=core)
             f->toRaw(doc);
     }
-//    qWarning() << "end";
-//    doc->positionTranslator().debug();
-//    debug();
+    // qWarning() << "end";
+    // doc->positionTranslator().debug();
+    // debug();
 }
