@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -76,7 +76,7 @@ static inline bool connectDatabase(QSqlDatabase &DB, const int line)
 {
     if (!DB.isOpen()) {
         if (!DB.open()) {
-            Utils::Log::addError("EpisodeBase", tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
+            Utils::Log::addError("PmhBase", tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
                                  .arg(DB.connectionName()).arg(DB.lastError().text()),
                                  __FILE__, line);
             return false;
@@ -218,7 +218,7 @@ bool PmhBase::createDatabase(const QString &connectionName , const QString &dbNa
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         DB.open();
         setDriver(Utils::Database::SQLite);
@@ -318,6 +318,7 @@ QVector<PmhData *> PmhBase::getPmh(const QString &patientUid) const
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
         return pmhs;
     }
@@ -345,6 +346,7 @@ QVector<PmhData *> PmhBase::getPmh(const QString &patientUid) const
             }
         } else {
             LOG_QUERY_ERROR(query);
+            query.finish();
             DB.rollback();
             return pmhs;
         }
@@ -352,6 +354,7 @@ QVector<PmhData *> PmhBase::getPmh(const QString &patientUid) const
 
     // Get Contacts
     where.clear();
+    query.finish();
     DB.commit();
     return pmhs;
 }
@@ -416,6 +419,7 @@ bool PmhBase::savePmhData(PmhData *pmh)
         pmh->setData(PmhData::Uid, query.lastInsertId());
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
     }
 
@@ -423,6 +427,7 @@ bool PmhBase::savePmhData(PmhData *pmh)
         savePmhEpisodeData(ep);
     }
 
+    query.finish();
     DB.commit();
     return true;
 }
@@ -469,6 +474,7 @@ bool PmhBase::updatePmhData(PmhData *pmh)
     query.bindValue(9, pmh->data(PmhData::IsPrivate).toInt());
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
         return false;
     }
