@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,16 +19,16 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 
 /**
-   \class PMH::PmhCategoryModel
-   General model for the categorized PMHx.
-   Unique instance is available in the PMH::PmhCore.
+ * \class PMH::PmhCategoryModel
+ *  General model for the categorized PMHx.
+ *  Its unique instance is available in the PMH::PmhCore.
  */
 
 #include "pmhcategorymodel.h"
@@ -46,13 +46,17 @@
 #include <coreplugin/translators.h>
 #include <coreplugin/constants_icons.h>
 
+#include <formmanagerplugin/formcore.h>
 #include <formmanagerplugin/iformitem.h>
 #include <formmanagerplugin/formmanager.h>
+#include <formmanagerplugin/episodemanager.h>
+#include <formmanagerplugin/formtreemodel.h>
 #include <formmanagerplugin/episodemodel.h>
 
 #include <categoryplugin/categorycore.h>
 
 #include <utils/log.h>
+#include <utils/global.h>
 #include <translationutils/constanttranslations.h>
 
 #include <QSqlQuery>
@@ -74,11 +78,13 @@ static inline QString currentUserUuid() {return Core::ICore::instance()->user()-
 static inline Core::ISettings *settings() {return Core::ICore::instance()->settings();}
 static inline Core::ITheme *theme() {return Core::ICore::instance()->theme();}
 static inline Core::Translators *translators() {return Core::ICore::instance()->translators();}
-static inline Form::FormManager *formManager() { return Form::FormManager::instance(); }
+static inline Form::FormManager &formManager() {return Form::FormCore::instance().formManager();}
+static inline Form::EpisodeManager &episodeManager() {return Form::FormCore::instance().episodeManager();}
 static inline Category::CategoryCore *categoryCore() {return Category::CategoryCore::instance();}
 
 namespace {
 
+    // FIXME: use a QStandardItemModel
     class TreeItem
     {
     public:
@@ -88,11 +94,10 @@ namespace {
 
         TreeItem(TreeItem *parent = 0) :
                 m_Parent(parent),
-                m_IsModified(false),
                 m_Cat(0),
                 m_Pmh(0),
                 m_Form(0),
-                m_EpisodeModel(0)
+                m_FormEpisodeModel(0)
         {
             if (m_Parent)
                 m_Parent->addChildren(this);
@@ -103,13 +108,11 @@ namespace {
             m_Label.clear();
             qDeleteAll(m_Children);
             m_Children.clear();
-//            delete m_Form;
-//            delete m_EpisodeModel;
             m_Pmh=0;
             m_Cat=0;
             m_Parent=0;
             m_Form=0;
-            m_EpisodeModel=0;
+            m_FormEpisodeModel=0;
         }
 
 
@@ -138,10 +141,6 @@ namespace {
                 return m_Parent->m_Children.indexOf(const_cast<TreeItem*>(this));
             return 0;
         }
-        //    void sortChildren()
-        //    {
-        //        qSort(m_Children.begin(), m_Children.end(), TreeItem::lessThan);
-        //    }
         bool removeChild(TreeItem *child)
         {
             if (m_Children.contains(child)) {
@@ -180,23 +179,12 @@ namespace {
         void setForm(Form::FormMain *form, Form::EpisodeModel *model)
         {
             m_Form = form;
-            m_EpisodeModel = model;
+            m_FormEpisodeModel = model;
         }
         Form::FormMain *form() const {return m_Form;}
-        Form::EpisodeModel *episodeModel() const {return m_EpisodeModel;}
+        Form::EpisodeModel *formEpisodeModel() const {return m_FormEpisodeModel;}
 
-        bool isForm() const {return ((m_Form) && (m_EpisodeModel));}
-
-        // For sort functions
-        //    static bool lessThan(TreeItem *item1, TreeItem *item2)
-        //    {
-        //        // category goes first
-        //        // then sort by name
-        //        bool sameType = (((item1->isTemplate()) && (item2->isTemplate())) || ((!item1->isTemplate()) && (!item2->isTemplate())));
-        //        if (sameType)
-        //            return item1->data(Constants::Data_Label).toString() < item2->data(Constants::Data_Label).toString();
-        //        return item2->isTemplate();
-        //    }
+        bool isForm() const {return ((m_Form) && (m_FormEpisodeModel));}
 
         void warn(int indent = 0)
         {
@@ -216,25 +204,21 @@ namespace {
         QString m_Label;
         QIcon m_Icon;
         QVector<int> m_DirtyRows;
-        bool m_IsModified;
         Category::CategoryItem *m_Cat;
         PmhData *m_Pmh;
         Form::FormMain *m_Form;
-        Form::EpisodeModel *m_EpisodeModel;
+        Form::EpisodeModel *m_FormEpisodeModel;
     };
-
 }
 
-
 namespace PMH {
 namespace Internal {
 class PmhCategoryModelPrivate
 {
 public:
-    PmhCategoryModelPrivate(PmhCategoryModel *parent) :
+    PmhCategoryModelPrivate(PmhCategoryModel */*parent*/) :
         _rootItem(0),
-        _synthesis(0),
-        q(parent)
+        _overview(0) // ,q(parent)
     {
         clearTree();
     }
@@ -275,20 +259,16 @@ public:
         return _rootItem;
     }
 
-    void episodeModelToTreeItem(Form::FormMain *form, TreeItem *parentItem, Form::EpisodeModel *model, const QModelIndex &index = QModelIndex())
+    void formModelToTreeItem(Form::FormMain *form, TreeItem *parentItem, Form::FormTreeModel *formTreeModel, const QModelIndex &index = QModelIndex())
     {
-        for(int i = 0; i < model->rowCount(index); ++i) {
-            QModelIndex idx = model->index(i, Form::EpisodeModel::Label, index);
-            // Do not include the episodes
-            if (!model->isForm(idx))
-                continue;
-            if (model->isLastEpisodeIndex(idx))
-                continue;
+        // Create an item for each form
+        for(int i = 0; i < formTreeModel->rowCount(index); ++i) {
+            QModelIndex idx = formTreeModel->index(i, Form::FormTreeModel::Label, index);
             TreeItem *newItem = new TreeItem(parentItem);
-            newItem->setLabel(idx.data().toString());
-            newItem->setForm(model->formForIndex(idx), model);
+            newItem->setLabel(formTreeModel->data(idx).toString());
+            newItem->setForm(formTreeModel->formForIndex(idx), episodeManager().episodeModel(form));
             // Read all its children
-            episodeModelToTreeItem(form, newItem, model, idx);
+            formModelToTreeItem(form, newItem, formTreeModel, idx);
         }
     }
 
@@ -301,22 +281,16 @@ public:
         // Category has forms ?
         const QString &xml = cat->data(Category::CategoryItem::ExtraXml).toString();
         if (!xml.isEmpty()) {
-            // TODO: improve this +++ this should be part of the XmlFormIO plugin
             // Check the addfile tag
             QDomDocument doc;
             doc.setContent(xml);
             QDomElement addFile = doc.documentElement();
             addFile = addFile.firstChildElement("file");
             if (!addFile.isNull()) {
-                // Load the form
-                QList<Form::FormMain*> forms = formManager()->loadFormFile(addFile.text());
-                if (!forms.isEmpty()) {
-                    // Create the EpisodeModel with the form
-                    Form::EpisodeModel *model = new Form::EpisodeModel(forms.at(0), q);
-                    model->init(false);
-                    // Translate all modelindex to TreeItem
-                    episodeModelToTreeItem(forms.at(0), item, model);
-                }
+                // Get the FormTreeModel with the form
+                Form::FormTreeModel *formModel = formManager().formTreeModelForSubForm(addFile.text());
+                // Translate all modelindex to TreeItem
+                formModelToTreeItem(formModel->formForIndex(formModel->index(0,0)), item, formModel);
             }
         }
 
@@ -428,26 +402,24 @@ public:
 
     void getCategories(bool getFromDatabase = false)
     {
-        QVector<Category::CategoryItem *> cats;
         if (getFromDatabase) {
             qDeleteAll(_categoryTree);
             _categoryTree.clear();
             // Get all categories from database
             _categoryToItem.clear();
-            if (!_synthesis) {
-                _synthesis = new Category::CategoryItem;
-                _synthesis->setData(Category::CategoryItem::DbOnly_Id, -2);
-                _synthesis->setData(Category::CategoryItem::DbOnly_LabelId, -1);
-                _synthesis->setData(Category::CategoryItem::DbOnly_ParentId, -3);
-                _synthesis->setData(Category::CategoryItem::DbOnly_Mime, -1);
-                _synthesis->setData(Category::CategoryItem::ThemedIcon, Core::Constants::ICONPATIENTSYNTHESIS);
-                _synthesis->setData(Category::CategoryItem::SortId, -1);
-                _synthesis->setLabel("Synthesis", Trans::Constants::ALL_LANGUAGE);
+            if (!_overview) {
+                _overview = new Category::CategoryItem;
+                _overview->setData(Category::CategoryItem::DbOnly_Id, -2);
+                _overview->setData(Category::CategoryItem::DbOnly_LabelId, -1);
+                _overview->setData(Category::CategoryItem::DbOnly_ParentId, -3);
+                _overview->setData(Category::CategoryItem::DbOnly_Mime, -1);
+                _overview->setData(Category::CategoryItem::ThemedIcon, Core::Constants::ICONPATIENTOVERVIEW);
+                _overview->setData(Category::CategoryItem::SortId, -1);
+                _overview->setLabel(tkTr(Trans::Constants::OVERVIEW), Trans::Constants::ALL_LANGUAGE);
             }
-            _categoryTree << _synthesis;
+            _categoryTree << _overview;
             _categoryTree << base()->getPmhCategory(_rootUid);
         }
-
         // Recreate the category tree
         foreach(Category::CategoryItem *cat, _categoryTree) { ///base()->createCategoryTree(_categoryTree)) {
             _rootItem->pmhCategory()->addChild(cat);
@@ -478,12 +450,12 @@ public:
     QHash<Category::CategoryItem *, TreeItem *> _categoryToItem;
     QHash<PmhData *, TreeItem *> _pmhToItems;
     QMultiHash<Category::CategoryItem *, PmhData *> _categoryToMultiPmh;
-    Category::CategoryItem *_synthesis;
+    Category::CategoryItem *_overview;
     QString _htmlSynthesis;
     QString _rootUid;
 
 private:
-    PmhCategoryModel *q;
+//    PmhCategoryModel *q;
 };
 
 }  // End namespace Internal
@@ -493,7 +465,7 @@ private:
 PmhCategoryModel::PmhCategoryModel(QObject *parent) :
         Category::ICategoryModelHelper(parent), d(new Internal::PmhCategoryModelPrivate(this))
 {
-    connect(patient(), SIGNAL(currentPatientChanged()), this, SLOT(patientChanged()));
+    connect(patient(), SIGNAL(currentPatientChanged()), this, SLOT(onCurrentPatientChanged()));
     connect(translators(), SIGNAL(languageChanged()), this, SLOT(retranslate()));
 }
 
@@ -522,8 +494,9 @@ QString PmhCategoryModel::rootFormUid() const
 /** Clear and refresh the whole model */
 void PmhCategoryModel::refreshFromDatabase()
 {
-    if (!patient())
+    if (patient()->uuid().isEmpty())
         return;
+    beginResetModel();
     qDeleteAll(d->_pmh);
     d->_pmh.clear();
     d->_categoryToMultiPmh.clear();
@@ -534,7 +507,7 @@ void PmhCategoryModel::refreshFromDatabase()
     d->_htmlSynthesis.clear();
     d->clearTree();
     d->getDataFromDatabase();
-    reset();
+    endResetModel();
 }
 
 QModelIndex PmhCategoryModel::index(int row, int column, const QModelIndex &parent) const
@@ -642,8 +615,8 @@ QVariant PmhCategoryModel::data(const QModelIndex &index, int role) const
         {
             if (index.column()==Label) {
                 if (it->isCategory()) {
-                    if (it->pmhCategory() == d->_synthesis)
-                        return tkTr(Trans::Constants::PATIENT_SYNTHESIS);
+                    if (it->pmhCategory() == d->_overview)
+                        return tkTr(Trans::Constants::PATIENT_OVERVIEW);
                     else
                         return it->label();// + " " + QString::number(it->pmhCategory()->sortId());
                 }
@@ -1013,7 +986,7 @@ bool PmhCategoryModel::isSynthesis(const QModelIndex &item) const
         return false; // root is a category
     TreeItem *it = d->getItem(item);
     if (it->isCategory())
-        return (it->pmhCategory() == d->_synthesis);
+        return (it->pmhCategory() == d->_overview);
     return false;
 }
 
@@ -1075,7 +1048,11 @@ Form::FormMain *PmhCategoryModel::formForIndex(const QModelIndex &item) const
     return 0;
 }
 
-bool PmhCategoryModel::activateFormEpisode(const QModelIndex &formIndex)
+/**
+ * Activate the last episode of the form \e formIndex
+ * \sa Form::EpisodeModel::populateFormWithEpisodeContent()
+ */
+bool PmhCategoryModel::activateFormEpisode(const QModelIndex &formIndex) const
 {
     if (!formIndex.isValid())
         return false;
@@ -1087,21 +1064,16 @@ bool PmhCategoryModel::activateFormEpisode(const QModelIndex &formIndex)
         return false;
 
     // get the Form::FormMain index
-    Form::EpisodeModel *model = it->episodeModel();
+    Form::EpisodeModel *model = it->formEpisodeModel();
     if (!model)
         return false;
-    QModelIndex idx = model->indexForForm(it->form()->uuid());
-    if (!idx.isValid())
-        return false;
 
-    if (!model->hasChildren(idx)) {
+    if (model->rowCount() == 0) {
         // Create the unique episode
-        model->insertRow(0, idx);
+        model->insertRow(0);
     }
 
-    model->activateEpisode(model->index(0, 0, idx), model->index(idx.row(), Form::EpisodeModel::FormUuid, idx.parent()).data().toString());
-
-    return true;
+    return model->populateFormWithEpisodeContent(model->index(0, 0), true);
 }
 
 /**
@@ -1209,13 +1181,27 @@ QString PmhCategoryModel::indexToHtml(const QModelIndex &index, int indent) cons
             id += "&nbsp;&nbsp;";
         html += QString("•&nbsp;%1<br />").arg(index.data(Qt::ToolTipRole).toString().replace("<br />","; "));
     } else if (isForm(index)) {
-        html = QString("<p style=\"margin:0px 0px 0px %1px\">%2<br />")
-                .arg(indent*10)
-                .arg(formForIndex(index)->printableHtml());
+        // populate form
+        activateFormEpisode(index);
+        Form::FormMain *form = formForIndex(index);
+        // get print mask
+        html = formManager().formPrintHtmlOutput(form);
+        QString css = Utils::htmlTakeAllCssContent(html);
+        html = Utils::htmlBodyContent(html);
+        html.prepend(css);
+        // clear form
+        if (form)
+            form->clear();
     }
     return html;
 }
 
+void PmhCategoryModel::refreshSynthesis()
+{
+    d->_htmlSynthesis.clear();
+}
+
+/** Return the synthesis of the whole model of a specific category \e index */
 QString PmhCategoryModel::synthesis(const QModelIndex &parent) const
 {
     if (parent==QModelIndex() || isSynthesis(parent)) {
@@ -1243,8 +1229,9 @@ QString PmhCategoryModel::synthesis(const QModelIndex &parent) const
 }
 
 /** Update the model when the current patient changes. */
-void PmhCategoryModel::patientChanged()
+void PmhCategoryModel::onCurrentPatientChanged()
 {
+    beginResetModel();
     qDeleteAll(d->_pmh);
     d->_pmh.clear();
     d->_categoryToMultiPmh.clear();
@@ -1264,9 +1251,10 @@ void PmhCategoryModel::patientChanged()
         d->getCategories(false);
         d->getPmh();
     }
-    reset();
+    endResetModel();
 }
 
+/** Update the category label (retranslate for eg) */
 void PmhCategoryModel::updateCategoryLabel(const Category::CategoryItem *category)
 {
     QModelIndex cat = indexForCategory(category);
