@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       Guillaume Denry <guillaume.denry@gmail.com>                       *
  *       Filipe Azevedo <>                                                 *
  *       Jourdan Romain <>                                                 *
@@ -43,10 +43,6 @@
 #include <QSizeF>
 #include <QRectF>
 #include <QRect>
-#include <QPrinter>
-#include <QPrinterInfo>
-#include <QPrintDialog>
-#include <QPrintPreviewDialog>
 #include <QSizeF>
 #include <QTextBlock>
 #include <QAbstractTextDocumentLayout>
@@ -56,6 +52,18 @@
 #include <QPointer>
 #include <QFileInfo>
 
+#if QT_VERSION < 0x050000
+#include <QPrinterInfo>
+#include <QPrinter>
+#include <QPrintDialog>
+#include <QPrintPreviewDialog>
+#else
+#include <QtPrintSupport/QPrinterInfo>
+#include <QtPrintSupport/QPrinter>
+#include <QtPrintSupport/QPrintDialog>
+#include <QtPrintSupport/QPrintPreviewDialog>
+#endif
+
 // For test
 #include <QTextBrowser>
 #include <QDialog>
@@ -141,13 +149,17 @@ static inline Core::ISettings *settings() {return Core::ICore::instance()->setti
 namespace Print {
 namespace Internal {
 
-/** \brief Only keeps some private members and private datas */
+/** \brief Only keeps some private members and private data */
 class PrinterPrivate
 {
 public:
-    PrinterPrivate(Printer *parent) :
-            m_TwoNUp(false), m_Printer(0), m_Content(0), m_WithDuplicata(false), m_PrintingDuplicata(false),
-            q(parent)
+    PrinterPrivate(Printer */*parent*/) :
+        m_WatermarkPresence(-1),
+        m_TwoNUp(false),
+        m_Printer(0),
+        m_Content(0),
+        m_WithDuplicata(false),
+        m_PrintingDuplicata(false)  //, q(parent)
     {
         m_TwoNUp = settings()->value(Print::Constants::S_TWONUP).toBool();
     }
@@ -308,7 +320,7 @@ public:
     /** \brief Return true if the document must be added at page \e page assuming that document as a Presence like \e presence. */
     bool presenceIsRequiredAtPage(const int page, const int presence)
     {
-        if ((presence == Printer::DuplicataOnly) && (m_PrintingDuplicata))
+        if ((presence == Printer::DuplicatesOnly) && (m_PrintingDuplicata))
             return true;
         if (presence == Printer::EachPages)
             return true;
@@ -449,7 +461,7 @@ public:
     QList<QPicture *> m_Pages;
 
 private:
-    Printer *q;
+//    Printer *q;
 };
 
 }  // End Internal
@@ -522,17 +534,13 @@ bool PrinterPrivate::complexDraw()
             // need new page ?
             if ((drawnedSize.height() + blockRect.size().height()) > pageSize.height()) {
 
-                int i = 0;
                 QTextLayout *layout = block.layout();
                 if (layout->lineCount() > 1) {
-
-
                     // TODO --> draw line by line
-
-
 //                    qWarning() << "lines in block" << block.layout()->lineCount();
                     int heightSave = drawnedSize.height();
                     // draw the maximum lines into the page before creating a new one
+                    int i = 0;
                     while (layout->lineAt(i).height() + drawnedSize.height() < pageSize.height()) {
                         //                        layout->lineAt(i).draw(&painter, layout->lineAt(i).position());
                         drawnedSize.setHeight(drawnedSize.height() + layout->lineAt(i).height());
@@ -540,10 +548,7 @@ bool PrinterPrivate::complexDraw()
                         ++i;
                     }
                     drawnedSize.setHeight(heightSave);
-
-
                     // END TODO
-
                 }
                 pageNumber = complexDrawNewPage(painter, headerSize, footerSize, pageSize,
                                                  correctedY, drawnedSize, pageNumber);
@@ -646,8 +651,7 @@ int PrinterPrivate::complexDrawNewPage(QPainter &p, QSizeF & headerSize, QSizeF
 bool PrinterPrivate::simpleDraw()
 {
     if (!m_Content) {
-        Utils::Log::addError("Printer", QCoreApplication::translate("tkPrinter", "No content to preview (simpleDraw)."),
-                             __FILE__, __LINE__);
+        LOG_ERROR_FOR("Printer", QCoreApplication::translate("tkPrinter", "No content to preview (simpleDraw)."));
         return false;
     }
     m_PrintingDuplicata = false;
@@ -717,7 +721,7 @@ bool PrinterPrivate::simpleDrawPreparePages(QRect &contentRect)
                 currentRect.translate(0, currentRect.height());
                 pageNumber++;
             }
-            m_PrintingDuplicata=!m_PrintingDuplicata;
+            m_PrintingDuplicata = !m_PrintingDuplicata;
         } else {
             drawnHeight += currentRect.height();
             // translate the currentRect to the beginning of the next page
@@ -728,10 +732,11 @@ bool PrinterPrivate::simpleDrawPreparePages(QRect &contentRect)
         // if there is still something to print --> create a newpage to the printer
         if (currentRect.intersects(contentRect)) {
             if (fromToPage) {
-                if ((pageNumber>=fromToPage) && (pageNumber<=toPage))
+                if ((pageNumber>=fromPage) && (pageNumber<=toPage))
                     m_Printer->newPage();
-            } else
+            } else {
                     m_Printer->newPage();
+            }
         }
         m_Pages.append(pic);
     }
@@ -818,7 +823,7 @@ bool Printer::askForPrinter(QWidget *parent)
 {
     d->renewPrinter();
     QPrintDialog dialog(d->m_Printer, parent);
-    dialog.setWindowTitle(tr("Print Document"));
+    dialog.setWindowTitle(tr("Print document"));
     if (dialog.exec() == QDialog::Accepted)
         return true;
     return false;
@@ -874,6 +879,53 @@ void Printer::setContent(const QTextDocument & docToPrint)
     d->m_Content = docToPrint.clone();
 }
 
+/** Return the html content of the printer. This does not includes header, footer and watermark. */
+QString Printer::htmlContent() const
+{
+    return d->m_Content->toHtml();
+}
+
+QString Printer::toHtml() const
+{
+    QString html, content, css;
+    QTextDocument *doc = 0;
+    // Add Each page header
+    doc = d->header(EachPages);
+    if (doc) {
+        content = doc->toHtml("UTF-8");
+        css = Utils::htmlTakeAllCssContent(content);
+        content = Utils::htmlBodyContent(content);
+        content.prepend(css);
+        html += content;
+    }
+    // Add first page header
+    QTextDocument *doc2 = 0;
+    doc2 = d->header(FirstPageOnly);
+    if (doc2 && doc2 != doc) {
+        content = doc->toHtml("UTF-8");
+        css = Utils::htmlTakeAllCssContent(content);
+        content = Utils::htmlBodyContent(content);
+        content.prepend(css);
+        html += content;
+    }
+    // Add content
+    content = d->m_Content->toHtml("UTF-8");
+    css = Utils::htmlTakeAllCssContent(content);
+    content = Utils::htmlBodyContent(content);
+    content.prepend(css);
+    html += content;
+    // Add Footer
+    doc = d->footer(EachPages);
+    if (doc) {
+        content = doc->toHtml("UTF-8");
+        css = Utils::htmlTakeAllCssContent(content);
+        content = Utils::htmlBodyContent(content);
+        content.prepend(css);
+        html += content;
+    }
+    return html;
+}
+
 /** \brief Try to find the default system printer and use it without annoying user. */
 bool Printer::useDefaultPrinter()
 {
@@ -923,11 +975,11 @@ bool Printer::preparePages()
         return false;
 
     if (d->isSimple()) {
-        Utils::Log::addMessage(this, "Printing using simpleDraw method.");
+        LOG("Printing using simpleDraw method.");
         return d->simpleDraw();
     }
     else {
-        Utils::Log::addMessage(this, "WARNING: Printing using complexDraw method (should be buggy).");
+        LOG("WARNING: Printing using complexDraw method (should be buggy).");
         return d->complexDraw();
     }
 }
