@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -56,6 +56,19 @@ using namespace Trans::ConstantTranslations;
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
 static inline Core::ICommandLine *commandLine()  { return Core::ICore::instance()->commandLine(); }
 
+static inline bool connectDatabase(QSqlDatabase &DB, const int line)
+{
+    if (!DB.isOpen()) {
+        if (!DB.open()) {
+            Utils::Log::addError("TemplateBase", tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
+                                 .arg(DB.connectionName()).arg(DB.lastError().text()),
+                                 __FILE__, line);
+            return false;
+        }
+    }
+    return true;
+}
+
 namespace Templates {
 namespace Internal {
 /**
@@ -75,13 +88,21 @@ public:
 
     void checkDatabaseVersion()
     {
+        QSqlDatabase DB = QSqlDatabase::database(Constants::DB_TEMPLATES_NAME);
+        if (!connectDatabase(DB, __LINE__)) {
+            return;
+        }
+        DB.transaction();
+        QSqlQuery query(DB);
         QString version;
-        QSqlQuery query(q->select(Constants::Table_Version, QList<int>() << Constants::VERSION_ACTUAL), QSqlDatabase::database(Constants::DB_TEMPLATES_NAME));
-        if (query.isActive()) {
+        if (query.exec(q->select(Constants::Table_Version, QList<int>() << Constants::VERSION_ACTUAL))) {
             if (query.next())
                 version = query.value(0).toString();
         } else {
             LOG_QUERY_ERROR_FOR(q, query);
+            query.finish();
+            DB.rollback();
+            return;
         }
         query.finish();
         bool updateVersionNumber = false;
@@ -100,7 +121,6 @@ public:
                     << "ALTER TABLE `CATEGORIES` RENAME TO `OLD_CATEGORIES`;"
                     << "ALTER TABLE `TEMPLATES` RENAME TO `OLD_TEMPLATES`;";
 
-            QSqlDatabase DB = q->database();
             if (!q->executeSQL(reqs, DB))
                 LOG_ERROR_FOR(q, "Unable to recreate template database during update (0.3.0 to 0.4.0)");
 
@@ -138,7 +158,7 @@ public:
                     << "DROP TABLE `OLD_TEMPLATES`;";
 
 
-            // Reinsert datas to new tables
+            // Reinsert data to new tables
             if (!q->executeSQL(reqs, DB))
                 LOG_ERROR_FOR(q, "Unable to recreate template database during update (0.3.0 to 0.4.0)");
 
@@ -150,12 +170,16 @@ public:
         if (updateVersionNumber) {
             query.prepare(q->prepareUpdateQuery(Constants::Table_Version, Constants::VERSION_ACTUAL));
             query.bindValue(0, Constants::DB_ACTUAL_VERSION);
-            query.exec();
-            if (!query.isActive()) {
+            if (!query.exec()) {
                 LOG_QUERY_ERROR_FOR(q, query);
+                query.finish();
+                DB.rollback();
+                return;
             }
             query.finish();
         }
+        query.finish();
+        DB.commit();
     }
 
 public:
@@ -301,7 +325,7 @@ bool TemplateBase::createDatabase(const QString &connectionName , const QString
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         if (!DB.open())
             LOG(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(dbName).arg(DB.lastError().text()));
@@ -372,14 +396,18 @@ bool TemplateBase::createDatabase(const QString &connectionName , const QString
     }
 
     // inform the version
-    QSqlQuery query(database());
+    DB.transaction();
+    QSqlQuery query(DB);
     query.prepare(prepareInsertQuery(Constants::Table_Version));
     query.bindValue(Constants::VERSION_ACTUAL, Constants::DB_ACTUAL_VERSION);
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        query.finish();
+        DB.rollback();
         return false;
     }
-
+    query.finish();
+    DB.commit();
     return true;
 }
 
