@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,15 +19,15 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
   \class Templates::TemplatesModel
   Manages the templates database in/out and link to views.
-  Datas are statically stored so that you can instanciate how many requiered models
+  Data are statically stored so that you can instanciate how many requiered models
   as you want without consuming RAM.
 */
 
@@ -62,7 +62,7 @@
   \todo Add a UUID to templates and categories
   \todo Add user filter, MimeType filter
   \todo Don't get the contents of templates to memory
-  \todo Remove the static datas. instead use a new member getDatas() to be called after setCategoryOnly(bool)
+  \todo Remove the static data. instead use a new member getData() to be called after setCategoryOnly(bool)
   \todo Detect corrupted templates database --> ask user what to do
   \todo Add a coreListener --> save templates database before closing application
   \todo Manage transmission date
@@ -72,7 +72,7 @@
 using namespace Templates;
 using namespace Trans::ConstantTranslations;
 
-enum { base64MimeDatas = true  };
+enum { base64MimeData = true  };
 
 #ifdef DEBUG
 enum {
@@ -127,18 +127,18 @@ private:
     Templates::TemplatesModel *m_Model;
 };
 
-
+// FIXME: use a QStandardItemModel instead of an internal specific pointer model
 class TreeItem : public Templates::ITemplate
 {
 public:
-    TreeItem(const QHash<int, QVariant> &datas, TreeItem *parent = 0) :
-            ITemplate(datas),
+    TreeItem(const QHash<int, QVariant> &hashData, TreeItem *parent = 0) :
+            ITemplate(hashData),
             m_Parent(parent),
             m_IsTemplate(false),
             m_IsModified(false)
     {
         setData(Constants::Data_UserUuid, "FreeDiams");
-        setIsTemplate(datas.value(Constants::Data_IsTemplate).toBool());
+        setIsTemplate(hashData.value(Constants::Data_IsTemplate).toBool());
     }
     ~TreeItem() { qDeleteAll(m_Children); }
 
@@ -275,7 +275,6 @@ public:
             m_ShowOnlyCategories(false),
             m_ReadOnly(false)
     {
-        q->setObjectName("TemplatesModel");
         m_Handles.insert(this);
         m_RootItem = m_Tree;
     }
@@ -289,14 +288,17 @@ public:
                 m_Tree = 0;
                 m_RootItem = 0;
             }
-            m_ModelDatasRetreived = false;
+            m_ModelDataRetreived = false;
         }
     }
 
+    // FIXME: create a Core to manage all these members
+    // FIXME: reset -> split into beginResetModel && endResetModel
     void allInstancesReset() const
     {
         foreach(TemplatesModelPrivate *pr, m_Handles) {
-            pr->q->reset();
+            pr->q->beginResetModel();
+            pr->q->endResetModel();
         }
     }
 
@@ -356,7 +358,7 @@ public:
 
     void setupModelData()
     {
-        if (m_ModelDatasRetreived)
+        if (m_ModelDataRetreived)
             return;
 
         if (m_RootItem) {
@@ -364,44 +366,50 @@ public:
         }
 
         // create root item
-        QHash<int, QVariant> datas;
-        datas.insert(Constants::Data_Label, "ROOT");
-        datas.insert(Constants::Data_ParentId, -1);
-        m_RootItem = new TreeItem(datas, 0);
+        QHash<int, QVariant> hashData;
+        hashData.insert(Constants::Data_Label, "ROOT");
+        hashData.insert(Constants::Data_ParentId, -1);
+        m_RootItem = new TreeItem(hashData, 0);
         m_RootItem->setIsTemplate(false);
         m_Tree = m_RootItem;
 
         // getting categories
-        Utils::Log::addMessage(q, "Getting Templates Categories");
+        LOG_FOR(q, "Getting Templates Categories");
         QSqlDatabase DB = QSqlDatabase::database(Constants::DB_TEMPLATES_NAME);
-        if (!DB.open()) {
-            Utils::Log::addError(q, tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
-                                 .arg(Constants::DB_TEMPLATES_NAME)
-                                 .arg(DB.lastError().text()),
-                                 __FILE__, __LINE__);
-            return;
+        if (!DB.isOpen()) {
+            if (!DB.open()) {
+                Utils::Log::addError(q, tkTr(Trans::Constants::UNABLE_TO_OPEN_DATABASE_1_ERROR_2)
+                                     .arg(Constants::DB_TEMPLATES_NAME)
+                                     .arg(DB.lastError().text()),
+                                     __FILE__, __LINE__);
+                return;
+            }
         }
 
         // get categories
         m_IdToCategory.clear();
         // TODO: Filter user's templates
-        QSqlQuery query(templateBase()->select(Templates::Constants::Table_Categories), DB);
-        if (query.isActive()) {
-            QHash<int, QVariant> datas;
+        DB.transaction();
+        QSqlQuery query(DB);
+        if (query.exec(templateBase()->select(Templates::Constants::Table_Categories))) {
+            QHash<int, QVariant> hashData;
             while (query.next()) {
-                datas.insert(Constants::Data_Id, query.value(Constants::CATEGORIES_ID));
-                datas.insert(Constants::Data_Uuid, query.value(Constants::CATEGORIES_UUID));
-                datas.insert(Constants::Data_UserUuid, query.value(Constants::CATEGORIES_USER_UID));
-                datas.insert(Constants::Data_ParentId, query.value(Constants::CATEGORIES_PARENT_ID));
-                datas.insert(Constants::Data_Label, query.value(Constants::CATEGORIES_LABEL));
-                datas.insert(Constants::Data_Summary, query.value(Constants::CATEGORIES_SUMMARY));
-                datas.insert(Constants::Data_CreationDate, query.value(Constants::CATEGORIES_DATECREATION));
-                datas.insert(Constants::Data_ModifDate, query.value(Constants::CATEGORIES_DATEMODIF));
-                m_IdToCategory.insert(datas.value(Constants::Data_Id).toInt(), new TreeItem(datas,0));
-                datas.clear();
+                hashData.insert(Constants::Data_Id, query.value(Constants::CATEGORIES_ID));
+                hashData.insert(Constants::Data_Uuid, query.value(Constants::CATEGORIES_UUID));
+                hashData.insert(Constants::Data_UserUuid, query.value(Constants::CATEGORIES_USER_UID));
+                hashData.insert(Constants::Data_ParentId, query.value(Constants::CATEGORIES_PARENT_ID));
+                hashData.insert(Constants::Data_Label, query.value(Constants::CATEGORIES_LABEL));
+                hashData.insert(Constants::Data_Summary, query.value(Constants::CATEGORIES_SUMMARY));
+                hashData.insert(Constants::Data_CreationDate, query.value(Constants::CATEGORIES_DATECREATION));
+                hashData.insert(Constants::Data_ModifDate, query.value(Constants::CATEGORIES_DATEMODIF));
+                m_IdToCategory.insert(hashData.value(Constants::Data_Id).toInt(), new TreeItem(hashData,0));
+                hashData.clear();
             }
         } else {
-            Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+            LOG_QUERY_ERROR_FOR(q, query);
+            query.finish();
+            DB.rollback();
+            return;
         }
         query.finish();
         foreach(TreeItem *item, m_IdToCategory.values()) {
@@ -413,33 +421,37 @@ public:
         }
 
         // get templates
-        Utils::Log::addMessage(q, "Getting Templates");
+        LOG_FOR(q, "Getting Templates");
         QList<TreeItem *> templates;
         // TODO: filter user's uuid
-        query.exec(templateBase()->select(Templates::Constants::Table_Templates));
-        if (query.isActive()) {
-            QHash<int, QVariant> datas;
+        if (query.exec(templateBase()->select(Templates::Constants::Table_Templates))) {
+            QHash<int, QVariant> hashData;
             while (query.next()) {
-                datas.insert(Constants::Data_Id, query.value(Constants::TEMPLATE_ID));
-                datas.insert(Constants::Data_Uuid, query.value(Constants::TEMPLATE_UUID));
-                datas.insert(Constants::Data_UserUuid, query.value(Constants::TEMPLATE_USER_UID));
-                datas.insert(Constants::Data_ParentId, query.value(Constants::TEMPLATE_ID_CATEGORY));
-                datas.insert(Constants::Data_Label, query.value(Constants::TEMPLATE_LABEL));
-                datas.insert(Constants::Data_Summary, query.value(Constants::TEMPLATE_SUMMARY));
-                datas.insert(Constants::Data_Content, query.value(Constants::TEMPLATE_CONTENT));
-                datas.insert(Constants::Data_ContentMimeTypes, query.value(Constants::TEMPLATE_CONTENTMIMETYPES));
-                datas.insert(Constants::Data_CreationDate, query.value(Constants::TEMPLATE_DATECREATION));
-                datas.insert(Constants::Data_ModifDate, query.value(Constants::TEMPLATE_DATEMODIF));
-                datas.insert(Constants::Data_ThemedIcon, query.value(Constants::TEMPLATE_THEMEDICON));
-                TreeItem *it = new TreeItem(datas,0);
+                hashData.insert(Constants::Data_Id, query.value(Constants::TEMPLATE_ID));
+                hashData.insert(Constants::Data_Uuid, query.value(Constants::TEMPLATE_UUID));
+                hashData.insert(Constants::Data_UserUuid, query.value(Constants::TEMPLATE_USER_UID));
+                hashData.insert(Constants::Data_ParentId, query.value(Constants::TEMPLATE_ID_CATEGORY));
+                hashData.insert(Constants::Data_Label, query.value(Constants::TEMPLATE_LABEL));
+                hashData.insert(Constants::Data_Summary, query.value(Constants::TEMPLATE_SUMMARY));
+                hashData.insert(Constants::Data_Content, query.value(Constants::TEMPLATE_CONTENT));
+                hashData.insert(Constants::Data_ContentMimeTypes, query.value(Constants::TEMPLATE_CONTENTMIMETYPES));
+                hashData.insert(Constants::Data_CreationDate, query.value(Constants::TEMPLATE_DATECREATION));
+                hashData.insert(Constants::Data_ModifDate, query.value(Constants::TEMPLATE_DATEMODIF));
+                hashData.insert(Constants::Data_ThemedIcon, query.value(Constants::TEMPLATE_THEMEDICON));
+                TreeItem *it = new TreeItem(hashData,0);
                 it->setIsTemplate(true);
-                templates.insert(datas.value(Constants::Data_Id).toInt(), it);
-                datas.clear();
+                templates.insert(hashData.value(Constants::Data_Id).toInt(), it);
+                hashData.clear();
             }
         } else {
-            Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+            LOG_QUERY_ERROR_FOR(q, query);
+            query.finish();
+            DB.rollback();
+            return;
         }
         query.finish();
+        DB.commit();
+
         // add templates to categories
         foreach(TreeItem *item, templates) {
             // need to be reparented
@@ -450,13 +462,13 @@ public:
         }
         sortItems();
 
-        m_ModelDatasRetreived = true;
+        m_ModelDataRetreived = true;
     }
 
-    void saveModelDatas(const QModelIndex &start = QModelIndex())
+    void saveModelData(const QModelIndex &start = QModelIndex())
     {
         if (WarnDatabaseSaving)
-            qWarning() << "saveModelDatas: " << start.data().toString();
+            qWarning() << "saveModelData: " << start.data().toString();
 
         QSqlDatabase DB = QSqlDatabase::database(Constants::DB_TEMPLATES_NAME);
         if (!DB.isOpen()) {
@@ -469,6 +481,8 @@ public:
         }
 
         QModelIndex idx = start;
+        DB.transaction();
+        QSqlQuery query(DB);
         for(int i = 0; i< q->rowCount(start); ++i) {
             idx = q->index(i, 0, start);
             TreeItem *t = getItem(idx);
@@ -476,7 +490,6 @@ public:
                 qWarning() << "   saving" << t->label().leftJustified(50,' ') << "user" << t->ownerUuid().leftJustified(50,' ') << "parent" << t->parent()->label().leftJustified(50,' ');
                 qWarning() << "   newly" << t->isNewlyCreated() << "modified" << t->isModified();
             }
-            QSqlQuery query(DB);
             QString req;
             if (t->isNewlyCreated()) {
                 if (t->isTemplate()) {
@@ -492,9 +505,11 @@ public:
                     query.bindValue(Constants::TEMPLATE_DATEMODIF, t->data(Constants::Data_ModifDate).toDate().toString(Qt::ISODate));
                     query.bindValue(Constants::TEMPLATE_THEMEDICON, t->data(Constants::Data_ThemedIcon).toString());
                     query.bindValue(Constants::TEMPLATE_TRANSMISSIONDATE, QVariant());
-                    query.exec();
-                    if (!query.isActive()) {
-                        Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+                    if (!query.exec()) {
+                        LOG_QUERY_ERROR_FOR(q, query);
+                        query.finish();
+                        DB.rollback();
+                        return;
                     } else {
                         t->setNewlyCreated(false);
                         t->setModified(false);
@@ -514,9 +529,11 @@ public:
                     query.bindValue(Constants::CATEGORIES_THEMEDICON, t->data(Constants::Data_ThemedIcon).toString());
                     query.bindValue(Constants::CATEGORIES_TRANSMISSIONDATE, QVariant());
                     // save category
-                    query.exec();
-                    if (!query.isActive()) {
-                        Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+                    if (!query.exec()) {
+                        LOG_QUERY_ERROR_FOR(q, query);
+                        query.finish();
+                        DB.rollback();
+                        return;
                     } else {
                         t->setNewlyCreated(false);
                         t->setModified(false);
@@ -547,9 +564,11 @@ public:
                     query.bindValue(Constants::TEMPLATE_DATEMODIF, t->data(Constants::Data_ModifDate).toDate().toString(Qt::ISODate));
                     query.bindValue(Constants::TEMPLATE_THEMEDICON, t->data(Constants::Data_ThemedIcon).toString());
                     query.bindValue(Constants::TEMPLATE_TRANSMISSIONDATE, QVariant());
-                    query.exec();
-                    if (!query.isActive()) {
-                        Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+                    if (!query.exec()) {
+                        LOG_QUERY_ERROR_FOR(q, query);
+                        query.finish();
+                        DB.rollback();
+                        return;
                     }
                     req.clear();
                     query.finish();
@@ -569,9 +588,11 @@ public:
                     query.bindValue(Constants::CATEGORIES_DATEMODIF, t->data(Constants::Data_ModifDate).toDate().toString(Qt::ISODate));
                     query.bindValue(Constants::CATEGORIES_THEMEDICON, t->data(Constants::Data_ThemedIcon).toString());
                     query.bindValue(Constants::CATEGORIES_TRANSMISSIONDATE, QVariant());
-                    query.exec();
-                    if (!query.isActive()) {
-                        Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
+                    if (!query.exec()) {
+                        LOG_QUERY_ERROR_FOR(q, query);
+                        query.finish();
+                        DB.rollback();
+                        return;
                     }
                     req.clear();
                     query.finish();
@@ -580,9 +601,12 @@ public:
                 }
             }
         }
+        query.finish();
+        DB.commit();
+
         // save all its children
         for(int i = 0; i < q->rowCount(start); ++i) {
-            saveModelDatas(q->index(i, 0, start));
+            saveModelData(q->index(i, 0, start));
         }
     }
 
@@ -599,12 +623,16 @@ public:
                 return toReturn;
             }
         }
+        DB.transaction();
         QHash<int, QString> where;
         where.insert(Constants::CATEGORIES_PARENT_ID, QString("=%1").arg(idCategory));
         req = templateBase()->select(Constants::Table_Categories, Constants::CATEGORIES_ID, where);
-        QSqlQuery query(req, DB);
-        if (!query.isActive()) {
+        QSqlQuery query(DB);
+        if (!query.exec(req)) {
             LOG_QUERY_ERROR_FOR(q, query);
+            query.finish();
+            DB.rollback();
+            return toReturn;
         } else {
             while (query.next()) {
                 toReturn << query.value(0).toInt();
@@ -612,6 +640,7 @@ public:
             }
         }
         query.finish();
+        DB.commit();
         return toReturn;
     }
 
@@ -629,6 +658,8 @@ public:
                 return;
             }
         }
+        DB.transaction();
+        QSqlQuery query(DB);
         QString req;
 
         if (m_CategoriesToDelete.count()) {
@@ -643,18 +674,21 @@ public:
                 req += QString::number(children.at(i)) + " , ";
             }
             req.chop(3);
-            
+
             // The Trigger cleans all children of each categories
 //            req = "DELETE FROM `CATEGORIES` WHERE `CATEGORY_ID` IN ( " + req + " )";
             QHash<int, QString> where;
             where.insert(Constants::CATEGORIES_ID, QString(" IN (%1)").arg(req));
             req = templateBase()->prepareDeleteQuery(Constants::Table_Categories, where);
 
-            QSqlQuery query(req, DB);
-            if (!query.isActive())
-                Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
-            else
+            if (!query.exec(req)) {
+                LOG_QUERY_ERROR_FOR(q, query);
+                query.finish();
+                DB.rollback();
+                return;
+            } else {
                 m_CategoriesToDelete.clear();
+            }
         }
 
         req.clear();
@@ -667,12 +701,17 @@ public:
             QHash<int, QString> where;
             where.insert(Constants::TEMPLATE_ID, QString(" IN (%1)").arg(req));
             req = templateBase()->prepareDeleteQuery(Constants::Table_Templates, where);
-            QSqlQuery query(req, DB);
-            if (!query.isActive())
-                Utils::Log::addQueryError(q, query, __FILE__, __LINE__);
-            else
+            if (!query.exec(req)) {
+                LOG_QUERY_ERROR_FOR(q, query);
+                query.finish();
+                DB.rollback();
+                return;
+            } else {
                 m_TemplatesToDelete.clear();
+            }
         }
+        query.finish();
+        DB.commit();
     }
 
     void sortItems(TreeItem *root = 0)
@@ -734,14 +773,14 @@ public:
     bool m_ShowOnlyCategories;
     bool m_ReadOnly;
     static TreeItem *m_Tree;
-    static bool m_ModelDatasRetreived;
+    static bool m_ModelDataRetreived;
     static QSet<TemplatesModelPrivate *> m_Handles;
     static QHash<int, TreeItem *> m_IdToCategory;
     static QVector<int> m_TemplatesToDelete, m_CategoriesToDelete;
 };
 
 TreeItem *TemplatesModelPrivate::m_Tree = 0;
-bool TemplatesModelPrivate::m_ModelDatasRetreived = false;
+bool TemplatesModelPrivate::m_ModelDataRetreived = false;
 QSet<TemplatesModelPrivate *> TemplatesModelPrivate::m_Handles;
 QHash<int, TreeItem *> TemplatesModelPrivate::m_IdToCategory;
 QVector<int> TemplatesModelPrivate::m_TemplatesToDelete;
@@ -757,8 +796,10 @@ TemplatesModel::TemplatesModel(QObject *parent) :
 {
     setObjectName("TemplatesModel");
     d->setupModelData();
+    // FIXME: Qt5 porting: CHECK THIS -> setSupportedDragActions(Qt::CopyAction | Qt::MoveAction);
+#if QT_VERSION < 0x050000
     setSupportedDragActions(Qt::CopyAction | Qt::MoveAction);
-
+#endif
     connect(Core::ICore::instance(), SIGNAL(databaseServerChanged()), this, SLOT(onCoreDatabaseServerChanged()));
 }
 
@@ -775,9 +816,10 @@ TemplatesModel::~TemplatesModel()
 
 void TemplatesModel::onCoreDatabaseServerChanged()
 {
-    d->m_ModelDatasRetreived = false;
+    beginResetModel();
+    d->m_ModelDataRetreived = false;
     d->setupModelData();
-    reset();
+    endResetModel();
 }
 
 bool TemplatesModel::isDirty() const
@@ -984,6 +1026,7 @@ Qt::DropActions TemplatesModel::supportedDropActions() const
 
 bool TemplatesModel::insertTemplate(const Templates::ITemplate *t)
 {
+    // TODO: wrong code
     if (d->m_ReadOnly)
         return false;
 
@@ -992,11 +1035,11 @@ bool TemplatesModel::insertTemplate(const Templates::ITemplate *t)
     if (!parent)
         return false;
     // insertRow in parentIndex
-    QModelIndex parentIndex = d->findIndex(parent->id());
-    // setDatas of newly created row
+//    QModelIndex parentIndex = d->findIndex(parent->id());
+    // setData of newly created row
 //    Internal::TreeItem *parentItem = d->getItem(item)->parent();
 //    // TODO: manage user
-//    Internal::TreeItem *newItem = new Internal::TreeItem(t.datas(),parentItem);
+//    Internal::TreeItem *newItem = new Internal::TreeItem(t.data(),parentItem);
 //    // TODO: reset all instances of the model
 //    d->resetAllInstances();
     return true;
@@ -1016,14 +1059,14 @@ bool TemplatesModel::insertRows(int row, int count, const QModelIndex &parent)
 //    if (!parent.isValid())
 //        return false;
 //    Internal::TreeItem *parentItem = d->getItem(parent);
-    QHash<int, QVariant> datas;
-    datas.insert(Constants::Data_Label, tr("New"));
-    datas.insert(Constants::Data_ParentId, parentItem->data(Constants::Data_Id));
-    datas.insert(Constants::Data_CreationDate, QDateTime::currentDateTime());
-    datas.insert(Constants::Data_IsTemplate, false);
+    QHash<int, QVariant> hashData;
+    hashData.insert(Constants::Data_Label, tr("New"));
+    hashData.insert(Constants::Data_ParentId, parentItem->data(Constants::Data_Id));
+    hashData.insert(Constants::Data_CreationDate, QDateTime::currentDateTime());
+    hashData.insert(Constants::Data_IsTemplate, false);
     d->allInstancesBeginInsertRows(parent, row, row+count-1);
     for(int i=0; i<count; ++i) {
-        Internal::TreeItem *item = new Internal::TreeItem(datas, parentItem);
+        Internal::TreeItem *item = new Internal::TreeItem(hashData, parentItem);
         if (!parentItem->insertChild(row+i, item))
             return false;
     }
@@ -1111,7 +1154,7 @@ bool TemplatesModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 
     if (!data->hasFormat(mimeTypes().at(0)))
         return false;
-    
+
     QModelIndex parentIndex = parent;
     while (isTemplate(parentIndex)) {
         parentIndex = parentIndex.parent();
@@ -1145,14 +1188,14 @@ bool TemplatesModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
             Internal::TreeItem *item = d->getItem(idx);
             Internal::TreeItem *newItem = d->getItem(index(row,0,parent));
             int id = newItem->id();
-            newItem->setDatas(item->datas());
+            newItem->replaceData(item->data());
             newItem->setData(Constants::Data_ParentId, parentId);
             newItem->setIsTemplate(item->isTemplate());
             newItem->setId(id);
         }
     }
 
-//    d->saveModelDatas();
+//    d->saveModelData();
 
     return true;
 }
@@ -1219,8 +1262,9 @@ bool TemplatesModel::isTemplate(const QModelIndex &index) const
 void TemplatesModel::categoriesOnly()
 {
     if (!d->m_ShowOnlyCategories) {
+        beginResetModel();
         d->m_ShowOnlyCategories = true;
-        reset();
+        endResetModel();
     }
 }
 
@@ -1245,7 +1289,7 @@ bool TemplatesModel::submit()
     if (d->m_ReadOnly)
         return false;
 
-    d->saveModelDatas();
+    d->saveModelData();
     d->deleteRowsInDatabase();
     return true;
 }
