@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -33,15 +33,13 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/itheme.h>
-#include <coreplugin/uniqueidmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/constants_tokensandsettings.h>
 #include <coreplugin/constants_menus.h>
 #include <coreplugin/constants_icons.h>
 #include <coreplugin/iuser.h>
-#include <coreplugin/idocumentprinter.h>
 
-#include <extensionsystem/pluginmanager.h>
 #include <utils/log.h>
 
 #include <QAction>
@@ -50,28 +48,24 @@
 
 #include <QDebug>
 
-
 using namespace Editor;
-using namespace Editor::Internal;
+using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
-static inline Core::UniqueIDManager *uid() { return Core::ICore::instance()->uniqueIDManager(); }
 static inline Core::ActionManager *actionManager() { return Core::ICore::instance()->actionManager(); }
 static inline Core::ITheme *theme() { return Core::ICore::instance()->theme(); }
-static inline Core::IDocumentPrinter *printer() {return ExtensionSystem::PluginManager::instance()->getObject<Core::IDocumentPrinter>();}
 
-
-static QAction *registerAction(const QString &id, const QList<int> &ctx, QObject *parent)
+static QAction *registerAction(const QString &id, const Core::Context &ctx, QObject *parent)
 {
     QAction *a = new QAction(parent);
-    Core::Command *cmd = Core::ICore::instance()->actionManager()->registerAction(a, id, ctx);
+    Core::Command *cmd = Core::ICore::instance()->actionManager()->registerAction(a, Core::Id(id), ctx);
     Q_UNUSED(cmd);
     return a;
 }
 
 static inline QAction *createAction(QObject *parent, const QString &name, const QString &icon,
                                     const QString &actionName,
-                                    const QList<int> &context, const QString &trans,
+                                    const Core::Context &context, const QString &trans,
                                     Core::Command *cmd,
                                     Core::ActionContainer *menu,
                                     const QString &group,
@@ -86,17 +80,17 @@ static inline QAction *createAction(QObject *parent, const QString &name, const
         a->setCheckable(true);
         a->setChecked(false);
     }
-    cmd = actionManager()->registerAction(a, actionName, context);
-    cmd->setTranslations(trans);
+    cmd = actionManager()->registerAction(a, Core::Id(actionName), context);
+    cmd->setTranslations(trans, trans); // use the Trans::Constants tr context (automatic)
     if (key != QKeySequence::UnknownKey)
         cmd->setDefaultKeySequence(key);
-    menu->addAction(cmd, group);
+    menu->addAction(cmd, Core::Id(group));
     return a;
 }
 
 static inline QAction *createAction(QObject *parent, const QString &name, const QString &icon,
                                     const QString &actionName,
-                                    const QList<int> &context, const QString &trans,
+                                    const Core::Context &context, const QString &trans,
                                     Core::Command *cmd,
                                     Core::ActionContainer *menu,
                                     const QString &group,
@@ -111,11 +105,11 @@ static inline QAction *createAction(QObject *parent, const QString &name, const
         a->setCheckable(true);
         a->setChecked(false);
     }
-    cmd = actionManager()->registerAction(a, actionName, context);
-    cmd->setTranslations(trans);
+    cmd = actionManager()->registerAction(a, Core::Id(actionName), context);
+    cmd->setTranslations(trans, trans); // use the Trans::Constants tr context (automatic)
     if (!key.isEmpty())
-        cmd->setDefaultKeySequence(QKeySequence::fromString(tkTr(key.toAscii())));
-    menu->addAction(cmd, group);
+        cmd->setDefaultKeySequence(QKeySequence::fromString(tkTr(key.toUtf8())));
+    menu->addAction(cmd, Core::Id(group));
     return a;
 }
 
@@ -126,13 +120,13 @@ EditorActionHandler::EditorActionHandler(QObject *parent) :
     aCut(0), aCopy(0), aPaste(0),
     aBold(0), aUnderline(0), aStrike(0),
     aItalic(0), aColor(0),
-    aFontFormat(0), aFontBigger(0), aFontSmaller(0),
+    aFontFormat(0), aFontBigger(0), aFontSmaller(0), aTypeWriterFont(0),
     aLeft(0), aCenter(0), aRight(0), aJustify(0),
     aAddTable(0), aTableProperties(0),
     aAddRow(0),aAddCol(0),
     aRemoveRow(0), aRemoveCol(0),
     aMergeCells(0), aSplitCells(0),
-    aAddDate(0),
+    aAddDateLong(0), aAddDateShort(0),
     aAddUserName(0),
     aAddPatientName(0),
     m_CurrentEditor(0)
@@ -149,21 +143,25 @@ EditorActionHandler::~EditorActionHandler()
 
 void EditorActionHandler::createContexts()
 {
-    charContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_CHAR_FORMAT);
-    paragraphContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_PARAGRAPH);
-    clipboardContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_CLIPBOARD);
-    basicContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_BASIC) << charContext << paragraphContext << clipboardContext;
-    ioContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_IO);
-    tableContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_TABLE);
-    textAdderContext = QList<int>() << uid()->uniqueIdentifier(Core::Constants::C_EDITOR_ADDTEXT);
-    allContexts = QList<int>() << basicContext << ioContext << tableContext;
+    charContext = Core::Context(Core::Constants::C_EDITOR_CHAR_FORMAT);
+    paragraphContext = Core::Context(Core::Constants::C_EDITOR_PARAGRAPH);
+    clipboardContext = Core::Context(Core::Constants::C_EDITOR_CLIPBOARD);
+    ioContext = Core::Context(Core::Constants::C_EDITOR_IO);
+    tableContext = Core::Context(Core::Constants::C_EDITOR_TABLE);
+    textAdderContext = Core::Context(Core::Constants::C_EDITOR_ADDTEXT);
+    allContexts.add(charContext);
+    allContexts.add(paragraphContext);
+    allContexts.add(clipboardContext);
+    allContexts.add(ioContext);
+    allContexts.add(tableContext);
+    allContexts.add(textAdderContext);
 }
 
 void EditorActionHandler::createMenus()
 {
     // Editor's Contextual Menu
     m_ContextualMenu = actionManager()->createMenu(Core::Constants::M_EDITOR_CONTEXT);
-    m_ContextualMenu->appendGroup(Core::Constants::G_EDITOR_CONTEXT);
+    m_ContextualMenu->appendGroup(Core::Id(Core::Constants::G_EDITOR_CONTEXT));
     m_ContextualMenu->setTranslations(Trans::Constants::EDITORMENU_TEXT);
 
     // Menu structure -- rootMenu is menubar if exists otherwise it is a specific editor's menu Core::Constants::M_EDITOR
@@ -187,11 +185,11 @@ void EditorActionHandler::createMenus()
         m_FormatMenu = actionManager()->createMenu(Core::Constants::M_FORMAT);
         rootMenu->addMenu(m_FormatMenu, Core::Constants::G_FORMAT);
         m_FormatMenu->setTranslations(Trans::Constants::M_FORMAT_TEXT);
-        m_FormatMenu->appendGroup(Core::Constants::G_FORMAT_FONT);
-        m_FormatMenu->appendGroup(Core::Constants::G_FORMAT_PARAGRAPH);
-        m_FormatMenu->appendGroup(Core::Constants::G_FORMAT_TABLE);
-        m_FormatMenu->appendGroup(Core::Constants::G_FORMAT_IMAGE);
-        m_FormatMenu->appendGroup(Core::Constants::G_FORMAT_OTHER);
+        m_FormatMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_FONT));
+        m_FormatMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_PARAGRAPH));
+        m_FormatMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_TABLE));
+        m_FormatMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_IMAGE));
+        m_FormatMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_OTHER));
     }
 
     m_FontMenu = actionManager()->actionContainer(Core::Constants::M_FORMAT_FONT);
@@ -199,9 +197,9 @@ void EditorActionHandler::createMenus()
         m_FontMenu = actionManager()->createMenu(Core::Constants::M_FORMAT_FONT);
         m_FormatMenu->addMenu(m_FontMenu, Core::Constants::G_FORMAT_FONT);
         m_FontMenu->setTranslations(Trans::Constants::M_FORMAT_FONT_TEXT);
-        m_FontMenu->appendGroup(Core::Constants::G_FORMAT_FONT_BASE);
-        m_FontMenu->appendGroup(Core::Constants::G_FORMAT_FONT_SIZE);
-        m_FontMenu->appendGroup(Core::Constants::G_FORMAT_FONT_EXTRAS);
+        m_FontMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_FONT_BASE));
+        m_FontMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_FONT_SIZE));
+        m_FontMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_FONT_EXTRAS));
     }
 
     m_ParagraphMenu = actionManager()->actionContainer(Core::Constants::M_FORMAT_PARAGRAPH);
@@ -209,7 +207,7 @@ void EditorActionHandler::createMenus()
         m_ParagraphMenu = actionManager()->createMenu(Core::Constants::M_FORMAT_PARAGRAPH);
         m_FormatMenu->addMenu(m_ParagraphMenu, Core::Constants::G_FORMAT_PARAGRAPH);
         m_ParagraphMenu->setTranslations(Trans::Constants::M_FORMAT_PARAGRAPH_TEXT);
-        m_ParagraphMenu->appendGroup(Core::Constants::G_FORMAT_PARAGRAPH);
+        m_ParagraphMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_PARAGRAPH));
     }
 
     m_TableMenu = actionManager()->actionContainer(Core::Constants::M_FORMAT_TABLE);
@@ -217,7 +215,7 @@ void EditorActionHandler::createMenus()
         m_TableMenu = actionManager()->createMenu(Core::Constants::M_FORMAT_TABLE);
         m_FormatMenu->addMenu(m_TableMenu, Core::Constants::G_FORMAT_TABLE);
         m_TableMenu->setTranslations(Trans::Constants::M_FORMAT_TABLE_TEXT);
-        m_TableMenu->appendGroup(Core::Constants::G_FORMAT_TABLE);
+        m_TableMenu->appendGroup(Core::Id(Core::Constants::G_FORMAT_TABLE));
 //        m_TableMenu->setEmptyAction(ActionContainer::EA_Hide);
     }
 
@@ -240,7 +238,7 @@ void EditorActionHandler::createActions()
 
     // Print
     if (!actionManager()->command(Core::Constants::A_FILE_PRINT)) {
-        aPrint = createAction(this, "aPrinte", ICONPRINT, A_FILE_PRINT, allContexts, FILEPRINT_TEXT, cmd, m_EditMenu, G_FILE_PRINT, QKeySequence::Print);
+        aPrint = createAction(this, "aPrint", ICONPRINT, A_FILE_PRINT, allContexts, FILEPRINT_TEXT, cmd, m_EditMenu, G_FILE_PRINT, QKeySequence::Print);
         aPrint->setEnabled(false);
     } else {
         aPrint = registerAction(Core::Constants::A_FILE_PRINT, allContexts, this);
@@ -285,6 +283,7 @@ void EditorActionHandler::createActions()
     aStrike = createAction(this, "aStrike", ICONSTRIKE, A_FORMAT_STRIKE, charContext, FORMATSTRIKE_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_BASE, QKeySequence::UnknownKey, true);
     aFontBigger = createAction(this, "aFontBigger", ICONFONTBIGGER, A_FORMAT_BIGGER, charContext, FORMATBIGGER_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_SIZE);
     aFontSmaller = createAction(this, "aFontSmaller", ICONFONTSMALLER, A_FORMAT_SMALLER, charContext, FORMATSMALLER_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_SIZE);
+    aTypeWriterFont = createAction(this, "aTypeWriterFont", ICONTYPEWRITER, A_FORMAT_TYPEWRITERFONT, charContext, FORMATTYPEWRITER_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_SIZE);
     aFontFormat = createAction(this, "aFontFormat", ICONFONTFORMAT, A_FORMAT_FONT, charContext, FORMATFONT_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_EXTRAS);
     aColor = createAction(this, "aColor", "", A_FORMAT_FONTCOLOR, charContext, FORMATFONTCOLOR_TEXT, cmd, m_FontMenu, G_FORMAT_FONT_EXTRAS);
     actionManager()->command(A_FORMAT_FONTCOLOR)->setAttribute(Core::Command::CA_UpdateIcon);
@@ -347,7 +346,8 @@ void EditorActionHandler::createActions()
     actionManager()->command(A_FORMAT_FONTCOLOR)->setAttribute(Core::Command::CA_UpdateText);
 
     // Text autocompletion
-    aAddDate = createAction(this, "aAddDate", "", A_EDITOR_ADDDATE, textAdderContext, EDITOR_ADDDATE_TEXT, cmd, m_AddTextMenu, Core::Constants::G_DEFAULT_ONE);
+    aAddDateLong = createAction(this, "aAddDateLong", "", A_EDITOR_ADDDATE_LONG, textAdderContext, EDITOR_ADDDATE_LONG_TEXT, cmd, m_AddTextMenu, Core::Constants::G_DEFAULT_ONE);
+    aAddDateShort = createAction(this, "aAddDateShort", "", A_EDITOR_ADDDATE_SHORT, textAdderContext, EDITOR_ADDDATE_SHORT_TEXT, cmd, m_AddTextMenu, Core::Constants::G_DEFAULT_ONE);
     aAddUserName = createAction(this, "aAddUserName", "", A_EDITOR_ADDUSERNAME, textAdderContext, EDITOR_ADDUSERNAME_TEXT, cmd, m_AddTextMenu, Core::Constants::G_DEFAULT_ONE);
     aAddPatientName = createAction(this, "aAddPatientName", "", A_EDITOR_ADDPATIENTNAME, textAdderContext, EDITOR_ADDPATIENTNAME_TEXT, cmd, m_AddTextMenu, Core::Constants::G_DEFAULT_ONE);
 
@@ -369,6 +369,7 @@ void EditorActionHandler::connectActions()
     connect(aStrike, SIGNAL(triggered()), this, SLOT(textStrike()));
     connect(aFontBigger, SIGNAL(triggered()), this, SLOT(fontBigger()));
     connect(aFontSmaller, SIGNAL(triggered()), this, SLOT(fontSmaller()));
+    connect(aTypeWriterFont, SIGNAL(triggered()), this, SLOT(typeWriterFont()));
     connect(aFontFormat, SIGNAL(triggered()), this, SLOT(fontFormat()));
     connect(aColor, SIGNAL(triggered()), this, SLOT(textColor()));
     connect(aLeft, SIGNAL(triggered()), this, SLOT(textAlign()));
@@ -385,7 +386,8 @@ void EditorActionHandler::connectActions()
     connect(aSplitCells, SIGNAL(triggered()), this, SLOT(tableSplitCells()));
     connect(aOpen, SIGNAL(triggered()), this, SLOT(fileOpen()));
     connect(aSave, SIGNAL(triggered()), this, SLOT(saveAs()));
-    connect(aAddDate, SIGNAL(triggered()), this, SLOT(addDate()));
+    connect(aAddDateLong, SIGNAL(triggered()), this, SLOT(addDate()));
+    connect(aAddDateShort, SIGNAL(triggered()), this, SLOT(addDate()));
     connect(aAddUserName, SIGNAL(triggered()), this, SLOT(addUserName()));
     connect(aAddPatientName, SIGNAL(triggered()), this, SLOT(addPatientName()));
 
@@ -455,12 +457,12 @@ void EditorActionHandler::setCurrentEditor(TextEditor *editor)
 {
     // disconnect old editor
     if (m_CurrentEditor) {
-        disconnect( m_CurrentEditor->textEdit(), SIGNAL( currentCharFormatChanged( const QTextCharFormat & ) ),
-                    this, SLOT( currentCharFormatChanged( const QTextCharFormat & ) ) );
-        disconnect( m_CurrentEditor->textEdit(), SIGNAL( cursorPositionChanged() ),
-                    this, SLOT( cursorPositionChanged() ) );
-        disconnect( m_CurrentEditor->textEdit(), SIGNAL( customContextMenuRequested( const QPoint & ) ),
-                    m_CurrentEditor, SLOT( contextMenu( const QPoint & ) ) );
+        disconnect(m_CurrentEditor->textEdit(), SIGNAL(currentCharFormatChanged(QTextCharFormat)),
+                   this, SLOT(currentCharFormatChanged(QTextCharFormat)));
+        disconnect(m_CurrentEditor->textEdit(), SIGNAL(cursorPositionChanged()),
+                   this, SLOT(cursorPositionChanged()));
+        disconnect(m_CurrentEditor->textEdit(), SIGNAL(customContextMenuRequested(QPoint)),
+                   m_CurrentEditor, SLOT(contextMenu(QPoint)));
         disconnect(m_CurrentEditor->textEdit(), SIGNAL(undoAvailable(bool)), this, SLOT(updateUndoAction()));
         disconnect(m_CurrentEditor->textEdit(), SIGNAL(redoAvailable(bool)), this, SLOT(updateRedoAction()));
         disconnect(m_CurrentEditor->textEdit(), SIGNAL(copyAvailable(bool)), this, SLOT(updateCopyAction()));
@@ -474,15 +476,15 @@ void EditorActionHandler::setCurrentEditor(TextEditor *editor)
     }
 
     // reconnect some actions
-    connect( m_CurrentEditor->textEdit(), SIGNAL( currentCharFormatChanged( const QTextCharFormat & ) ),
-             this, SLOT( currentCharFormatChanged( const QTextCharFormat & ) ) );
-    connect( m_CurrentEditor->textEdit(), SIGNAL( cursorPositionChanged() ),
-             this, SLOT( cursorPositionChanged() ) );
-    connect( m_CurrentEditor->textEdit(), SIGNAL( customContextMenuRequested( const QPoint & ) ),
-             m_CurrentEditor, SLOT( contextMenu( const QPoint & ) ) );
-    connect(m_CurrentEditor->textEdit(), SIGNAL(undoAvailable(bool)), this, SLOT(updateUndoAction()));
-    connect(m_CurrentEditor->textEdit(), SIGNAL(redoAvailable(bool)), this, SLOT(updateRedoAction()));
-    connect(m_CurrentEditor->textEdit(), SIGNAL(copyAvailable(bool)), this, SLOT(updateCopyAction()));
+    connect(m_CurrentEditor->textEdit(), SIGNAL(currentCharFormatChanged(QTextCharFormat)),
+            this, SLOT(currentCharFormatChanged(QTextCharFormat)), Qt::UniqueConnection);
+    connect(m_CurrentEditor->textEdit(), SIGNAL(cursorPositionChanged()),
+            this, SLOT(cursorPositionChanged()), Qt::UniqueConnection);
+    connect(m_CurrentEditor->textEdit(), SIGNAL(customContextMenuRequested(QPoint)),
+            m_CurrentEditor, SLOT(contextMenu(QPoint)), Qt::UniqueConnection);
+    connect(m_CurrentEditor->textEdit(), SIGNAL(undoAvailable(bool)), this, SLOT(updateUndoAction()), Qt::UniqueConnection);
+    connect(m_CurrentEditor->textEdit(), SIGNAL(redoAvailable(bool)), this, SLOT(updateRedoAction()), Qt::UniqueConnection);
+    connect(m_CurrentEditor->textEdit(), SIGNAL(copyAvailable(bool)), this, SLOT(updateCopyAction()), Qt::UniqueConnection);
 
     // show toolbar
     m_CurrentEditor->toogleToolbar(m_CurrentEditor->toolbarIsVisible());
@@ -540,15 +542,8 @@ void EditorActionHandler::toogleToolbar()
 
 void EditorActionHandler::print()
 {
-    if (m_CurrentEditor) {
-        Core::IDocumentPrinter *p = printer();
-        p->clearTokens();
-        QHash<QString, QVariant> tokens;
-        tokens.insert(Core::Constants::TOKEN_DOCUMENTTITLE, "");
-        p->addTokens(Core::IDocumentPrinter::Tokens_Global, tokens);
-        // TODO: add more options for the user : select papers, print duplicatas...
-        p->print(m_CurrentEditor->textEdit()->document(), Core::IDocumentPrinter::Papers_Generic_User, false);
-    }
+    if (m_CurrentEditor)
+        m_CurrentEditor->print();
 }
 
 void EditorActionHandler::fileOpen()
@@ -648,6 +643,13 @@ void EditorActionHandler::fontSmaller()
     m_CurrentEditor->textEdit()->setFocus();
 }
 
+void EditorActionHandler::typeWriterFont()
+{
+    if (m_CurrentEditor)
+        m_CurrentEditor->typeWriterFont();
+    m_CurrentEditor->textEdit()->setFocus();
+}
+
 void EditorActionHandler::textColor()
 {
     if (m_CurrentEditor) {
@@ -702,8 +704,16 @@ void EditorActionHandler::tableSplitCells()
 
 void EditorActionHandler::addDate()
 {
-    if (m_CurrentEditor)
-        m_CurrentEditor->addDate();
+    QAction *a = qobject_cast<QAction*>(sender());
+    if (!a)
+        return;
+
+    if (m_CurrentEditor) {
+        if (a == aAddDateLong)
+            m_CurrentEditor->addDate(TextEditor::LongFormat);
+        else if (a == aAddDateShort)
+            m_CurrentEditor->addDate(TextEditor::ShortFormat);
+    }
 }
 
 void EditorActionHandler::addUserName()
