@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -25,8 +25,8 @@
  ***************************************************************************/
 
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
 /**
@@ -50,15 +50,19 @@
 #include <translationutils/trans_menu.h>
 #include <translationutils/trans_filepathxml.h>
 
-#include <coreplugin/contextmanager/contextmanager.h>
-#include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/itheme.h>
 #include <coreplugin/iuser.h>
 #include <coreplugin/ipatient.h>
-#include <coreplugin/uniqueidmanager.h>
-#include <coreplugin/constants_menus.h>
+#include <coreplugin/ipadtools.h>
 #include <coreplugin/isettings.h>
+#include <coreplugin/constants_menus.h>
+#include <coreplugin/constants_tokensandsettings.h>
+#include <coreplugin/contextmanager/contextmanager.h>
+#include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
+
+#include <extensionsystem/pluginmanager.h>
 
 #include <QLocale>
 #include <QAction>
@@ -71,7 +75,6 @@
 #include <QFontComboBox>
 #include <QFontDatabase>
 #include <QMenu>
-#include <QMenuBar>
 #include <QTextCodec>
 #include <QTextEdit>
 #include <QToolBar>
@@ -82,6 +85,7 @@
 #include <QStringList>
 #include <QFontDialog>
 #include <QPointer>
+#include <QMimeData>
 
 using namespace Editor;
 using namespace Editor::Internal;
@@ -90,6 +94,8 @@ using namespace Trans::ConstantTranslations;
 static inline Core::IUser *user() {return Core::ICore::instance()->user();}
 static inline Core::IPatient *patient() {return Core::ICore::instance()->patient();}
 static inline Core::ISettings *settings() {return Core::ICore::instance()->settings();}
+static inline Core::IDocumentPrinter *printer() {return ExtensionSystem::PluginManager::instance()->getObject<Core::IDocumentPrinter>();}
+static inline Core::IPadTools *padTools() {return Core::ICore::instance()->padTools();}
 
 namespace Editor {
 namespace Internal {
@@ -161,9 +167,12 @@ public:
     TextEditorPrivate(QWidget *parent , TextEditor::Types type) :
         m_Type(type),
         m_Context(0),
+        m_ToolBar(0),
         textEdit(0),
         m_Parent(parent),
-        m_ToolBarIsVisible(false)
+        m_ToolBarIsVisible(false),
+        _papers(Core::IDocumentPrinter::Papers_Generic_User),
+        _alwaysPrintDuplicata(false)
     {
         textEdit = new TextEditorWithControl(m_Parent);
         //textEdit = new TextEditorHtmlPaster(m_Parent);
@@ -194,52 +203,41 @@ public:
         m_ToolBar->clear();
         QStringList actions;
 
-        // IO Actions
+        // Actions
         if (m_Type & TextEditor::WithIO) {
             actions << Core::Constants::A_EDITOR_FILEOPEN
                     << Core::Constants::A_EDITOR_FILESAVE
                     << Core::Constants::A_EDITOR_FILEPRINT
+                    << "--"
                     ;
-            foreach(const QString &a, actions) {
-                Core::Command *cmd = am->command(a);
-                if (cmd)
-                    m_ToolBar->addAction(cmd->action());
-            }
-            m_ToolBar->addSeparator();
         }
 
-        Core::Command *cmd = am->command(Core::Constants::A_FILE_PRINT);
-        if (cmd) {
-            m_ToolBar->addAction(cmd->action());
-            m_ToolBar->addSeparator();
-        }
+        actions << Core::Constants::A_FILE_PRINT
+                << "--";
 
-        actions.clear();
         if (m_Type & TextEditor::Clipboard) {
-            actions
-                    << Core::Constants::A_EDIT_COPY
+            actions << Core::Constants::A_EDIT_COPY
                     << Core::Constants::A_EDIT_PASTE
                     << Core::Constants::A_EDIT_CUT
-                       ;
-            foreach(const QString &a, actions) {
-                Core::Command *cmd = am->command(a);
-                if (cmd)
-                    m_ToolBar->addAction(cmd->action());
-            }
-            m_ToolBar->addSeparator();
+                    << "--"
+                    ;
         }
 
-        actions.clear();
         actions << Core::Constants::A_EDIT_UNDO
                 << Core::Constants::A_EDIT_REDO
+                << "--"
                 ;
         foreach(const QString &a, actions) {
-            Core::Command *cmd = am->command(a);
+            if (a=="--") {
+                m_ToolBar->addSeparator();
+                continue;
+            }
+            Core::Command *cmd = am->command(Core::Id(a));
             if (cmd)
                 m_ToolBar->addAction(cmd->action());
         }
-        m_ToolBar->addSeparator();
 
+        // Menu contents
         QAction *previous = 0;
         actions.clear();
         if (m_Type & TextEditor::CharFormat) {
@@ -253,7 +251,7 @@ public:
         }
 
         foreach(const QString &m, actions) {
-            Core::ActionContainer *ac = am->actionContainer(m);
+            Core::ActionContainer *ac = am->actionContainer(Core::Id(m));
             if (ac) {
                 foreach (QAction *action, ac->menu()->actions()) {
                     if (previous) {
@@ -280,8 +278,8 @@ public:
             cursor.select(QTextCursor::WordUnderCursor);
         cursor.mergeCharFormat(format);
         textEdit->mergeCurrentCharFormat(format);
-    }           
-    
+    }
+
 public:
     TextEditor::Types m_Type;
     EditorContext *m_Context;
@@ -290,6 +288,10 @@ public:
     //TextEditorHtmlPaster *textEdit;
     QWidget *m_Parent;
     bool m_ToolBarIsVisible;
+
+    Core::IDocumentPrinter::PapersToUse _papers;
+    bool _alwaysPrintDuplicata;
+    QString _docTitle;
 };
 
 }  // End Internal
@@ -308,7 +310,7 @@ TextEditor::TextEditor(QWidget *parent, Types type) :
 
     // instanciate private part
     d = new TextEditorPrivate(this, type);
-    // instanciate editor manager
+    // TODO: remove this: instanciate editor manager
     EditorManager::instance();
 
     // create ToolBar
@@ -317,8 +319,8 @@ TextEditor::TextEditor(QWidget *parent, Types type) :
 
     // instanciate context for actions
     d->m_Context = new EditorContext(this);
-    setTypes(type);
     Core::ICore::instance()->contextManager()->addContextObject(d->m_Context);
+    setTypes(type);
 
     // create QWidget
     QVBoxLayout * vb = new QVBoxLayout(this);
@@ -328,13 +330,15 @@ TextEditor::TextEditor(QWidget *parent, Types type) :
 
     vb->addWidget(d->m_ToolBar);
     vb->addWidget(d->textEdit);
+
+    setFocusProxy(d->textEdit);
 }
 
 TextEditor::~TextEditor()
 {
     // unreg context
     // FIXME: manage a segfault with freepad when updating context.
-//    Core::ICore::instance()->contextManager()->removeContextObject(d->m_Context);
+    Core::ICore::instance()->contextManager()->removeContextObject(d->m_Context);
     if (d) {
         delete d;
         d=0;
@@ -354,33 +358,56 @@ void TextEditor::setTextControl(ITextControl *control)
 void TextEditor::setTypes(Types type)
 {
     d->m_Type = type;
-    d->m_Context->clearContext();
-    Core::UniqueIDManager *uid = Core::ICore::instance()->uniqueIDManager();
-    d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_BASIC));
-    if (type & TextEditor::CharFormat) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_CHAR_FORMAT));
-    }
-    if (type & TextEditor::ParagraphFormat) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_PARAGRAPH));
-    }
-    if (type & TextEditor::Clipboard) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_CLIPBOARD));
-    }
-
-    if (type & TextEditor::WithTables) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_TABLE));
-    }
-    if (type & TextEditor::WithIO) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_IO));
-    }
-    if (type & TextEditor::WithTextCompleter) {
-        d->m_Context->addContext(uid->uniqueIdentifier(Core::Constants::C_EDITOR_ADDTEXT));
-    }
+    Core::Context context;
+    if (type & TextEditor::CharFormat)
+        context.add(Core::Constants::C_EDITOR_CHAR_FORMAT);
+    if (type & TextEditor::ParagraphFormat)
+        context.add(Core::Constants::C_EDITOR_PARAGRAPH);
+    if (type & TextEditor::Clipboard)
+        context.add(Core::Constants::C_EDITOR_CLIPBOARD);
+    if (type & TextEditor::WithTables)
+        context.add(Core::Constants::C_EDITOR_TABLE);
+    if (type & TextEditor::WithIO)
+        context.add(Core::Constants::C_EDITOR_IO);
+    if (type & TextEditor::WithTextCompleter)
+        context.add(Core::Constants::C_EDITOR_ADDTEXT);
+    d->m_Context->setContext(context);
     // update toolbar
     d->populateToolbar();
     Core::ICore::instance()->contextManager()->updateContext();
 }
 
+/**
+ * Define the papers for the document. Papers are header, footer and watermark.
+ * By default, set to Core::IDocumentPrinter::Papers_Generic_User.
+ * \sa Core::IDocumentPrinter
+ */
+void TextEditor::setUserDocumentForPrintingProcess(Core::IDocumentPrinter::PapersToUse papers)
+{
+    d->_papers = papers;
+}
+
+/** Define the duplicata printing. By default, set to false. */
+void TextEditor::setAlwaysPrintDuplicata(bool printDuplicata)
+{
+    d->_alwaysPrintDuplicata = printDuplicata;
+}
+
+/** Define the document title to use for the print process (token content) */
+void TextEditor::setDocumentTitle(const QString &title)
+{
+    d->_docTitle = title;
+}
+
+/** Adds a Core::Context to this contextual widget */
+void TextEditor::addContext(const Core::Context &context)
+{
+    Core::Context ctx = d->m_Context->context();
+    ctx.add(context);
+    d->m_Context->setContext(ctx);
+    Core::ICore::instance()->contextManager()->updateContext();
+}
+
 /** Creates the context menu for the Editor::TextEditor according to its Editor::TextEditor::Type. The menu is not executed. */
 QMenu *TextEditor::getContextMenu()
 {
@@ -396,7 +423,7 @@ QMenu *TextEditor::getContextMenu()
     Core::Command *cmd = 0;
     QStringList actions;
 
-    cmd = am->command(Core::Constants::A_EDITOR_TOOGLETOOLBAR);
+    cmd = am->command(Core::Id(Core::Constants::A_EDITOR_TOOGLETOOLBAR));
     if (cmd) {
         mc->addAction(cmd->action());
         mc->addSeparator();
@@ -405,12 +432,13 @@ QMenu *TextEditor::getContextMenu()
     if (d->m_Type & WithTextCompleter) {
         QMenu *m = new QMenu(this);
         m->setTitle(tkTr(Trans::Constants::EDITOR_ADDTEXTMENU_TEXT).remove("&"));
-        actions << Core::Constants::A_EDITOR_ADDDATE
+        actions << Core::Constants::A_EDITOR_ADDDATE_LONG
+                << Core::Constants::A_EDITOR_ADDDATE_SHORT
                 << Core::Constants::A_EDITOR_ADDUSERNAME
                 << Core::Constants::A_EDITOR_ADDPATIENTNAME
                 ;
         foreach(const QString &a, actions) {
-            cmd = am->command(a);
+            cmd = am->command(Core::Id(a));
             if (cmd)
                 m->addAction(cmd->action());
         }
@@ -430,7 +458,7 @@ QMenu *TextEditor::getContextMenu()
                 << Core::Constants::A_FILE_PRINTPREVIEW
                 ;
         foreach(const QString &a, actions) {
-            cmd = am->command(a);
+            cmd = am->command(Core::Id(a));
             if (cmd)
                 m->addAction(cmd->action());
         }
@@ -445,7 +473,7 @@ QMenu *TextEditor::getContextMenu()
             << Core::Constants::A_EDIT_CUT
             ;
     foreach(const QString &a, actions) {
-        Core::Command *cmd = am->command(a);
+        Core::Command *cmd = am->command(Core::Id(a));
         if (cmd)
             medit->addAction(cmd->action());
     }
@@ -456,7 +484,7 @@ QMenu *TextEditor::getContextMenu()
             << Core::Constants::A_EDIT_REDO
             ;
     foreach(const QString &a, actions) {
-        Core::Command *cmd = am->command(a);
+        Core::Command *cmd = am->command(Core::Id(a));
         if (cmd)
             medit->addAction(cmd->action());
     }
@@ -475,7 +503,7 @@ QMenu *TextEditor::getContextMenu()
     }
 
     foreach(const QString &m, actions) {
-        Core::ActionContainer *ac = am->actionContainer(m);
+        Core::ActionContainer *ac = am->actionContainer(Core::Id(m));
         if (ac) {
             QMenu *menu = new QMenu(this);
             menu->setTitle(ac->menu()->title());
@@ -546,7 +574,17 @@ void TextEditor::fileOpen()
                                                 &selected);
     if (file.isEmpty())
         return;
-    QString str = Utils::readTextFile(file, Utils::WarnUser, this);
+    QString str = Utils::readTextFile(file, Utils::WarnUser);
+    // run token if FreeMedForms
+#ifdef FREEMEDFORMS
+    patient()->replaceTokens(str);
+    user()->replaceTokens(str);
+
+#ifdef WITH_PAD
+    str = padTools()->processPlainText(str);
+#endif
+
+#endif
     if (Qt::mightBeRichText(str)) {
         textEdit()->setHtml(str);
     } else {
@@ -554,6 +592,7 @@ void TextEditor::fileOpen()
     }
 }
 
+/** Save the content of the text editor */
 void TextEditor::saveAs()
 {
     QAction *a = qobject_cast<QAction *>(sender());
@@ -573,7 +612,7 @@ void TextEditor::saveAs()
     if (fileName.isEmpty())
         return;
     if (selected==tkTr(Trans::Constants::FILE_FILTER_HTML)) {
-        if (Utils::saveStringToFile(Utils::toHtmlAccent(textEdit()->document()->toHtml("UTF-8")),
+        if (Utils::saveStringToFile(Utils::htmlReplaceAccents(textEdit()->document()->toHtml("UTF-8")),
                                     fileName, Utils::Overwrite, Utils::WarnUser, this))
             textEdit()->document()->setModified(false);
     } else if (selected==tkTr(Trans::Constants::FILE_FILTER_TXT)) {
@@ -590,6 +629,16 @@ void TextEditor::saveAs()
     }
 }
 
+void TextEditor::print()
+{
+    Core::IDocumentPrinter *print = printer();
+    print->clearTokens();
+    QHash<QString, QVariant> tokens;
+    tokens.insert(Core::Constants::TOKEN_DOCUMENTTITLE, d->_docTitle);
+    print->addTokens(Core::IDocumentPrinter::Tokens_Global, tokens);
+    print->print(d->textEdit->document(), d->_papers, d->_alwaysPrintDuplicata);
+}
+
 //--------------------------------------------------------------------------------------------------------
 //--------------------------------------------- Text Decoration ------------------------------------------
 //--------------------------------------------------------------------------------------------------------
@@ -630,6 +679,15 @@ void TextEditor::fontSmaller()
     d->mergeFormatOnWordOrSelection(fmt);
 }
 
+void TextEditor::typeWriterFont()
+{
+    QFont font = textEdit()->textCursor().charFormat().font();
+    font.setFamily("Courier");
+    QTextCharFormat fmt;
+    fmt.setFont(font);
+    d->mergeFormatOnWordOrSelection(fmt);
+}
+
 void TextEditor::textBold(bool checked)
 {
     QTextCharFormat fmt;
@@ -735,9 +793,12 @@ void TextEditor::textColor()
     d->mergeFormatOnWordOrSelection(fmt);
 }
 
-void TextEditor::addDate()
+void TextEditor::addDate(DateFormat format)
 {
-    textEdit()->insertHtml(QDateTime::currentDateTime().toString(QLocale().dateTimeFormat(QLocale::LongFormat)));
+    if (format==LongFormat)
+        textEdit()->insertHtml(QDateTime::currentDateTime().toString(QLocale().dateTimeFormat(QLocale::LongFormat)));
+    else //if (format==ShortFormat)
+        textEdit()->insertHtml(QDateTime::currentDateTime().toString(QLocale().dateTimeFormat(QLocale::ShortFormat)));
 }
 
 void TextEditor::addUserName()
