@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -37,77 +37,102 @@
 */
 
 #include "usermanagerplugin.h"
+#include "usercore.h"
 #include "usermodel.h"
 #include "database/userbase.h"
 #include "widgets/usermanager.h"
 #include "widgets/useridentifier.h"
-#include "widgets/userwizard.h"
+#include "widgets/usercreatorwizard.h"
 #include "currentuserpreferencespage.h"
-#include "userfistrunpage.h"
+#include "userfirstrunpage.h"
 #include "usermanagermode.h"
+#include "database/userbase.h"
+#include "widgets/usermanager.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/isettings.h>
-#include <coreplugin/constants.h>
-#include <coreplugin/translators.h>
 #include <coreplugin/iuser.h>
 #include <coreplugin/itheme.h>
+#include <coreplugin/constants.h>
+#include <coreplugin/isettings.h>
+#include <coreplugin/translators.h>
+#include <coreplugin/imainwindow.h>
 #include <coreplugin/icommandline.h>
 #include <coreplugin/modemanager/modemanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/contextmanager/contextmanager.h>
 
-#include <translationutils/constanttranslations.h>
 #include <utils/log.h>
 #include <utils/global.h>
 #include <utils/databaseconnector.h>
+#include <utils/widgets/databaseinformationdialog.h>
+#include <translationutils/constants.h>
+#include <translationutils/trans_database.h>
+#include <translationutils/trans_titles.h>
+#include <translationutils/trans_user.h>
+#include <translationutils/trans_menu.h>
 
 #include <extensionsystem/pluginmanager.h>
 
-#include <QtCore/QtPlugin>
+#include <QtPlugin>
 #include <QApplication>
 #include <QProgressDialog>
+#include <QTreeWidget>
+#include <QGridLayout>
+
 #include <QDebug>
 
-using namespace UserPlugin::Internal;
+enum { WithUserMode = false };
+
+using namespace UserPlugin;
+using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
 static inline Core::ActionManager *actionManager() {return Core::ICore::instance()->actionManager();}
 static inline Core::ISettings *settings() {return Core::ICore::instance()->settings();}
-static inline UserPlugin::UserModel *userModel() {return UserPlugin::UserModel::instance();}
 static inline Core::ContextManager *contextManager() { return Core::ICore::instance()->contextManager(); }
 static inline Core::ModeManager *modeManager() { return Core::ICore::instance()->modeManager(); }
 static inline Core::IUser *user() {return Core::ICore::instance()->user();}
 static inline Core::ICommandLine *commandLine() {return Core::ICore::instance()->commandLine();}
 
+static inline UserPlugin::UserCore &userCore() {return UserPlugin::UserCore::instance();}
+static inline UserPlugin::UserModel *userModel() {return userCore().userModel();}
+static inline UserPlugin::Internal::UserBase *userBase() {return userCore().userBase();}
+
 static inline Core::ITheme *theme()  { return Core::ICore::instance()->theme(); }
 static inline void messageSplash(const QString &s) {theme()->messageSplashScreen(s); }
 
 UserManagerPlugin::UserManagerPlugin() :
     aCreateUser(0), aChangeUser(0),
+    aUserManager(0),
+    aAboutDatabase(0),
     m_FirstCreation(new FirstRun_UserCreation(this)),
-    m_Mode(0)
+    m_Mode(0),
+    m_UserManagerMainWin(0)
 {
     setObjectName("UserManagerPlugin");
     if (Utils::Log::warnPluginsCreation())
         qWarning() << "creating UserManagerPlugin";
 
     // Add Translator to the Application
-    Core::ICore::instance()->translators()->addNewTranslator("usermanagerplugin");
+    Core::ICore::instance()->translators()->addNewTranslator("plugin_usermanager");
 
-    new UserBase(this);
+    new UserCore(this);
 
     addObject(m_FirstCreation);
+    connect(Core::ICore::instance(), SIGNAL(coreOpened()), this, SLOT(postCoreInitialization()));
+
+#ifdef USERMANAGERTESTER
+    // Here we only use the UserManagerMainWindow
+    m_UserManagerMainWin = new UserManagerMainWindow;
+    Core::ICore::instance()->setMainWindow(m_UserManagerMainWin);
+#endif
 }
 
 UserManagerPlugin::~UserManagerPlugin()
 {
-    qWarning() << "UserManagerPlugin::~UserManagerPlugin()";
-    if (m_FirstCreation) {
-        removeObject(m_FirstCreation);
-        delete m_FirstCreation;
-        m_FirstCreation = 0;
-    }
+    if (Utils::Log::warnPluginsCreation())
+        WARN_FUNC;
 }
 
 bool UserManagerPlugin::initialize(const QStringList &arguments, QString *errorString)
@@ -119,16 +144,16 @@ bool UserManagerPlugin::initialize(const QStringList &arguments, QString *errorS
 
     messageSplash(tr("Initializing user manager plugin..."));
 
-    // is UserBase reachable ?
-    Internal::UserBase *base = UserBase::instance();
-    base->initialize();
-
-    if (!base->isInitialized()) {
-        Utils::warningMessageBox(tr("Unable to connect to the user database."),
-                                 tr("The user database is not reachable. Please check your configuration.\n"
-                                    "Application will stop."));
-        LOG_ERROR(settings()->databaseConnector().toString());
-        return false;
+    if (!userCore().initialize()
+            || !userBase()->isInitialized()) {
+        LOG_ERROR("Unable to initialize user core/base. Starting the configuration dialog.");
+        if (!Core::ICore::instance()->applicationConfigurationDialog()) {
+            Utils::warningMessageBox(tr("Unable to connect to the user database."),
+                                     tr("The user database is not reachable. Please check your configuration.\n"
+                                        "Application will stop."));
+            LOG_ERROR(settings()->databaseConnector().toString());
+            return false;
+        }
     }
 
     // manage virtual user creation
@@ -140,35 +165,34 @@ bool UserManagerPlugin::initialize(const QStringList &arguments, QString *errorS
         dlg.setFocus();
         dlg.setValue(0);
 
-        bool created = true;
         // Doctors
-        created = base->createVirtualUser("d1f29ad4a4ea4dabbe40ec888d153228", "McCoy", "Leonard", Trans::Constants::Doctor, genders().indexOf(tkTr(Trans::Constants::MALE)),
-                                      QStringList() << "Medical Doctor",
-                                      QStringList() << "Chief medical officer USS Enterprise",
-                                      Core::IUser::AllRights, Core::IUser::AllRights, 0, Core::IUser::AllRights, Core::IUser::AllRights);
+        bool created = userBase()->createVirtualUser("d1f29ad4a4ea4dabbe40ec888d153228", "McCoy", "Leonard", Trans::Constants::Doctor, genders().indexOf(tkTr(Trans::Constants::MALE)),
+                                                     QStringList() << "Medical Doctor",
+                                                     QStringList() << "Chief medical officer USS Enterprise",
+                                                     Core::IUser::AllRights, Core::IUser::AllRights, 0, Core::IUser::AllRights, Core::IUser::AllRights);
         if (created) {
-            base->createVirtualUser("b5caead635a246a2a87ce676e9d2ef4d", "Phlox", "", Trans::Constants::Doctor, genders().indexOf(tkTr(Trans::Constants::MALE)),
+            userBase()->createVirtualUser("b5caead635a246a2a87ce676e9d2ef4d", "Phlox", "", Trans::Constants::Doctor, genders().indexOf(tkTr(Trans::Constants::MALE)),
                                           QStringList() << "Intergalactic medicine",
                                           QStringList() << "Chief medical officer Enterprise NX-01",
                                           Core::IUser::AllRights, Core::IUser::AllRights, 0, Core::IUser::AllRights, Core::IUser::AllRights);
             // Secretaries or so :  Uhura  U.S.S. Enterprise
-            base->createVirtualUser("0f148ea3de6e47b8bbf9c2cedea47511", "Uhura", "", Trans::Constants::Madam, genders().indexOf(tkTr(Trans::Constants::FEMALE)),
+            userBase()->createVirtualUser("0f148ea3de6e47b8bbf9c2cedea47511", "Uhura", "", Trans::Constants::Madam, genders().indexOf(tkTr(Trans::Constants::FEMALE)),
                                           QStringList() << "Communications officer",
                                           QStringList() << "Enterprise NX-01",
                                           0, 0, 0, Core::IUser::AllRights, 0);
             // Nurses : Christine Chapel U.S.S. Enterprise
-            base->createVirtualUser("b94ad4ee401a4fada0bf29fc8f8f3597", "Chapel", "Christine", Trans::Constants::Madam, genders().indexOf(tkTr(Trans::Constants::FEMALE)),
+            userBase()->createVirtualUser("b94ad4ee401a4fada0bf29fc8f8f3597", "Chapel", "Christine", Trans::Constants::Madam, genders().indexOf(tkTr(Trans::Constants::FEMALE)),
                                           QStringList() << "Space nurse",
                                           QStringList() << "Nurse, Enterprise NX-01",
                                           0, 0, 0, Core::IUser::AllRights, Core::IUser::AllRights);
             // Admins
 
             // refresh model
-            UserModel::instance()->refresh();
+            userModel()->refresh();
             // reconnect user
             Utils::DatabaseConnector c = settings()->databaseConnector();
             // clear cache, don't check preferences validity
-            UserModel::instance()->setCurrentUser(c.clearLog(), c.clearPass(), true, false);
+            userModel()->setCurrentUser(c.clearLog(), c.clearPass(), true, false);
         }
     }
 
@@ -177,9 +201,17 @@ bool UserManagerPlugin::initialize(const QStringList &arguments, QString *errorS
         if (!errorString)
             errorString = new QString();
         errorString->append(tr("User is not identified."));
+        Core::ICore::instance()->setUser(0);
         return false;
     }
 
+    if (m_UserManagerMainWin) {
+        if (!m_UserManagerMainWin->initialize()) {
+            LOG_ERROR("Main window not initialized");
+            return false;
+        }
+    }
+
     return true;
 }
 
@@ -191,19 +223,25 @@ void UserManagerPlugin::extensionsInitialized()
     messageSplash(tr("Initializing user manager plugin..."));
 
     // add UserPreferences page
-    addAutoReleasedObject(new UserPlugin::CurrentUserPreferencesPage(this));
+//    addAutoReleasedObject(new UserPlugin::CurrentUserPreferencesPage(this));
+
+#ifdef USERMANAGERTESTER
+    // Nothing to do with the tester as we only need a usermanager mainwindow
+    if (m_UserManagerMainWin)
+        m_UserManagerMainWin->extensionsInitialized();
+#endif
 
     // add UserManager toogler action to plugin menu
-#ifndef FREEMEDFORMS
-    const char * const menuId = Core::Constants::M_FILE;
-    const char * const menuNewId = Core::Constants::M_FILE;
-    const char * const groupUsers = Core::Constants::G_FILE_OTHER;
-    const char * const groupNew =  Core::Constants::G_FILE_NEW;
-#else
+#ifdef FREEMEDFORMS
     const char * const menuId = Core::Constants::M_GENERAL;
     const char * const menuNewId = Core::Constants::M_GENERAL_NEW;
     const char * const groupUsers = Core::Constants::G_GENERAL_USERS;
     const char * const groupNew = Core::Constants::G_GENERAL_NEW;
+#else
+    const char * const menuId = Core::Constants::M_FILE;
+    const char * const menuNewId = Core::Constants::M_FILE_NEW;
+    const char * const groupUsers = Core::Constants::G_FILE_OTHER;
+    const char * const groupNew =  Core::Constants::G_FILE_NEW;
 #endif
 
     Core::ActionContainer *menu = actionManager()->actionContainer(menuId);
@@ -211,11 +249,10 @@ void UserManagerPlugin::extensionsInitialized()
     if (!menu)
         return;
     Core::ActionContainer *newmenu = actionManager()->actionContainer(menuNewId);
-    Q_ASSERT(newmenu);
     if (!newmenu)
-        return;
+        newmenu = menu;
 
-    QList<int> ctx = QList<int>() << Core::Constants::C_GLOBAL_ID;
+    Core::Context ctx(Core::Constants::C_GLOBAL);
     QAction *a = 0;
     Core::Command *cmd = 0;
 
@@ -224,11 +261,11 @@ void UserManagerPlugin::extensionsInitialized()
     a = aCreateUser = new QAction(this);
     a->setObjectName("aCreateUser");
     a->setIcon(QIcon(Core::Constants::ICONNEWUSER));
-    cmd = actionManager()->registerAction(aCreateUser, Core::Constants::A_CREATEUSER, ctx);
+    cmd = actionManager()->registerAction(aCreateUser, Core::Id(Core::Constants::A_CREATEUSER), ctx);
     Q_ASSERT(cmd);
     cmd->setDefaultKeySequence(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_U));
     cmd->setTranslations(Trans::Constants::USER);
-    newmenu->addAction(cmd, groupNew);
+    newmenu->addAction(cmd, Core::Id(groupNew));
     cmd->retranslate();
     connect(aCreateUser, SIGNAL(triggered()), this, SLOT(createUser()));
 
@@ -236,19 +273,41 @@ void UserManagerPlugin::extensionsInitialized()
     a = aChangeUser = new QAction(this);
     a->setObjectName("aChangeUser");
     a->setIcon(QIcon(Core::Constants::ICONUSER));
-    cmd = actionManager()->registerAction(aChangeUser, "aChangeCurrentUser", ctx);
+    cmd = actionManager()->registerAction(aChangeUser, Core::Id("aChangeCurrentUser"), ctx);
     Q_ASSERT(cmd);
-    cmd->setTranslations(tr("Change current user"));
-    menu->addAction(cmd, groupUsers);
+    cmd->setTranslations(Trans::Constants::CHANGE_USER);
+    menu->addAction(cmd, Core::Id(groupUsers));
     cmd->retranslate();
     connect(aChangeUser, SIGNAL(triggered()), this, SLOT(changeCurrentUser()));
 
+    a = aUserManager = new QAction(this);
+    a->setObjectName("aUserManager");
+    a->setIcon(QIcon(Core::Constants::ICONUSERMANAGER));
+    cmd = actionManager()->registerAction(aUserManager, Core::Id("aUserManager"), ctx);
+    Q_ASSERT(cmd);
+    cmd->setTranslations(Trans::Constants::USERMANAGER_TEXT);
+    menu->addAction(cmd, Core::Id(groupUsers));
+    cmd->retranslate();
+    connect(aUserManager, SIGNAL(triggered()), this, SLOT(showUserManager()));
+
+    Core::ActionContainer *hmenu = actionManager()->actionContainer(Core::Id(Core::Constants::M_HELP_DATABASES));
+    if (hmenu) {
+        a = aAboutDatabase = new QAction(this);
+        a->setObjectName("aAboutDatabase");
+        a->setIcon(QIcon(Core::Constants::ICONHELP));
+        cmd = actionManager()->registerAction(aAboutDatabase, Core::Id("aAboutDatabase"), ctx);
+        Q_ASSERT(cmd);
+        cmd->setTranslations(Trans::Constants::USER_DATABASE_INFORMATION);
+        hmenu->addAction(cmd, Core::Id(Core::Constants::G_HELP_DATABASES));
+        cmd->retranslate();
+        connect(aAboutDatabase, SIGNAL(triggered()), this, SLOT(showDatabaseInformation()));
+    }
+
     updateActions();
 
     // create the mode
-    m_Mode = new Internal::UserManagerMode(this);
-
-    connect(Core::ICore::instance(), SIGNAL(coreOpened()), this, SLOT(postCoreInitialization()));
+    if (WithUserMode)
+        m_Mode = new Internal::UserManagerMode(this);
 }
 
 bool UserManagerPlugin::identifyUser()
@@ -259,10 +318,12 @@ bool UserManagerPlugin::identifyUser()
     QString pass;
     bool sqliteVersion = (settings()->databaseConnector().driver()==Utils::Database::SQLite);
     bool usingCommandLine = false;
+#ifdef WITH_USER_AUTOLOGIN
     if (sqliteVersion) {
         log = settings()->databaseConnector().clearLog();
         pass = settings()->databaseConnector().clearPass();
     }
+#endif
     if (commandLine()->value(Core::ICommandLine::UserClearLogin).isValid()) {
         log = commandLine()->value(Core::ICommandLine::UserClearLogin).toString();
         pass = commandLine()->value(Core::ICommandLine::UserClearPassword).toString();
@@ -317,19 +378,16 @@ bool UserManagerPlugin::identifyUser()
 void UserManagerPlugin::postCoreInitialization()
 {
     if (Utils::Log::warnPluginsCreation())
-        qWarning() << Q_FUNC_INFO;
-    userModel()->checkUserPreferencesValidity();
-    // be sure everyone is informed of the currently connected user
-    userModel()->emitUserConnected();
-    // and be sure that ui is translated in the correct language
-    Core::ICore::instance()->translators()->changeLanguage(settings()->value(Core::Constants::S_PREFERREDLANGUAGE, user()->value(Core::IUser::LanguageISO).toString()).toString());
+        WARN_FUNC;
+    userCore().postCoreInitialization();
 }
 
 /** \brief Create a new user is connected to Core::Constants::A_CREATEUSER. */
 void UserManagerPlugin::createUser()
 {
-    UserWizard wiz;
-//    wiz.createUser(true);
+    UserCreatorWizard wiz(Core::ICore::instance()->mainWindow());
+    wiz.show();
+    Utils::resizeAndCenter(&wiz, Core::ICore::instance()->mainWindow());
     wiz.exec();
 }
 
@@ -359,7 +417,7 @@ void UserManagerPlugin::updateActions()
 {
     if (user()) {
         Core::IUser::UserRights umRights(user()->value(Core::IUser::ManagerRights).toInt());
-//        Core::IUser::UserRights adminRights(user()->value(Core::IUser::AdministrativeRights));
+        aUserManager->setEnabled(umRights & Core::IUser::AllRights);
         if ((umRights & Core::IUser::AllRights) ||
             (umRights & Core::IUser::ReadAll)) {
             aCreateUser->setEnabled(true);
@@ -372,4 +430,40 @@ void UserManagerPlugin::updateActions()
     }
 }
 
+void UserManagerPlugin::showUserManager()
+{
+    UserManagerDialog dlg(Core::ICore::instance()->mainWindow());
+//    dlg.setModal(true);
+    dlg.initialize();
+    Utils::resizeAndCenter(&dlg, Core::ICore::instance()->mainWindow());
+    dlg.show();
+    dlg.initializeAfterShowing();
+    dlg.exec();
+}
+
+void UserManagerPlugin::showDatabaseInformation()
+{
+    Utils::DatabaseInformationDialog dlg(Core::ICore::instance()->mainWindow());
+    dlg.setTitle(tkTr(Trans::Constants::TEMPLATE_DATABASE_INFORMATION));
+    dlg.setDatabase(*userBase());
+    Utils::resizeAndCenter(&dlg);
+    dlg.exec();
+}
+
+ExtensionSystem::IPlugin::ShutdownFlag UserManagerPlugin::aboutToShutdown()
+{
+    if (Utils::Log::warnPluginsCreation())
+        WARN_FUNC;
+    // Save settings
+    // Disconnect from signals that are not needed during shutdown
+    // Hide UI (if you add UI that is not in the main window directly)
+    // Remove preferences pages to plugins manager object pool
+    if (m_FirstCreation) {
+        removeObject(m_FirstCreation);
+        delete m_FirstCreation;
+        m_FirstCreation = 0;
+    }
+    return SynchronousShutdown;
+}
+
 Q_EXPORT_PLUGIN(UserManagerPlugin)
