@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -41,36 +41,36 @@
 #include <coreplugin/isettings.h>
 #include <coreplugin/constants_tokensandsettings.h>
 
+#include <formmanagerplugin/formcore.h>
 #include <formmanagerplugin/iformitem.h>
 #include <formmanagerplugin/iformwidgetfactory.h>
 #include <formmanagerplugin/iformio.h>
+#include <formmanagerplugin/formmanager.h>
 
 #include <categoryplugin/categoryitem.h>
 #include <categoryplugin/categorycore.h>
 
-#include <pmhplugin/pmhcore.h>
-#include <pmhplugin/pmhcategorymodel.h>
-
 #include <QApplication>
 #include <QDir>
 #include <QTextCodec>
 #include <QFileInfo>
 #include <QDomDocument>
 #include <QDomElement>
-#include <QtScript/QScriptEngine>
-#include <QtScript/QScriptSyntaxCheckResult>
+#include <QScriptEngine>
+#include <QScriptSyntaxCheckResult>
 
 #include <QDebug>
+enum {WarnTabOrder = false};
 
 using namespace XmlForms;
 using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
-//inline static Form::FormManager *formManager() { return Form::FormManager::instance(); }
-inline static ExtensionSystem::PluginManager *pluginManager() {return ExtensionSystem::PluginManager::instance();}
+static inline Form::FormManager &formManager() {return Form::FormCore::instance().formManager();}
+static inline ExtensionSystem::PluginManager *pluginManager() {return ExtensionSystem::PluginManager::instance();}
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
+static inline Core::IPatient *patient()  { return Core::ICore::instance()->patient(); }
 static inline Category::CategoryCore *categoryCore() {return  Category::CategoryCore::instance();}
-static inline PMH::PmhCore *pmhCore() {return PMH::PmhCore::instance();}
 static inline Internal::XmlFormContentReader *reader() {return Internal::XmlFormContentReader::instance();}
 static inline Internal::XmlIOBase *base() {return Internal::XmlIOBase::instance();}
 
@@ -98,8 +98,15 @@ XmlFormContentReader *XmlFormContentReader::instance()
 }
 
 XmlFormContentReader::XmlFormContentReader() :
-    m_Mute(false)
+    m_Mute(false),
+    m_Initialized(false)
 {
+}
+
+bool XmlFormContentReader::initialize()
+{
+    if (m_Initialized)
+        return true;
    m_ScriptsTypes.clear();
    m_ScriptsTypes.insert(Constants::TAG_SCRIPT_ONLOAD, Form::FormItemScripts::Script_OnLoad);
    m_ScriptsTypes.insert(Constants::TAG_SCRIPT_POSTLOAD, Form::FormItemScripts::Script_PostLoad);
@@ -107,6 +114,7 @@ XmlFormContentReader::XmlFormContentReader() :
    m_ScriptsTypes.insert(Constants::TAG_SCRIPT_ONVALUECHANGED, Form::FormItemScripts::Script_OnValueChanged);
    m_ScriptsTypes.insert(Constants::TAG_SCRIPT_ONVALUEREQUIERED, Form::FormItemScripts::Script_OnValueRequiered);
    m_ScriptsTypes.insert(Constants::TAG_SCRIPT_ONDEPENDENCIESCHANGED, Form::FormItemScripts::Script_OnDependentValueChanged);
+   m_ScriptsTypes.insert(Constants::TAG_SCRIPT_ONCLICKED, Form::FormItemScripts::Script_OnClicked);
 
    m_ValuesTypes.clear();
    m_ValuesTypes.insert(Constants::TAG_VALUE_UUID, Form::FormItemValues::Value_Uuid);
@@ -126,28 +134,27 @@ XmlFormContentReader::XmlFormContentReader() :
    m_SpecsTypes.insert(Constants::TAG_SPEC_BIBLIOGRAPHY, Form::FormItemSpec::Spec_Bibliography);
    m_SpecsTypes.insert(Constants::TAG_SPEC_DESCRIPTION, Form::FormItemSpec::Spec_Description);
    m_SpecsTypes.insert(Constants::TAG_SPEC_LABEL, Form::FormItemSpec::Spec_Label);
+   m_SpecsTypes.insert(Constants::TAG_SPEC_EXTRALABEL, Form::FormItemSpec::Spec_ExtraLabel);
    m_SpecsTypes.insert(Constants::TAG_SPEC_VERSION, Form::FormItemSpec::Spec_Version);
    m_SpecsTypes.insert(Constants::TAG_SPEC_ICON, Form::FormItemSpec::Spec_IconFileName);
    m_SpecsTypes.insert(Constants::TAG_SPEC_TOOLTIP, Form::FormItemSpec::Spec_Tooltip);
+   m_SpecsTypes.insert(Constants::TAG_SPEC_PLACEHOLDER, Form::FormItemSpec::Spec_PlaceHolder);
+   m_SpecsTypes.insert(Constants::TAG_SPEC_PRIORITY, Form::FormItemSpec::Spec_Priority);
+   m_SpecsTypes.insert(Constants::TAG_SPEC_HPRIM, Form::FormItemSpec::Spec_UseForHprimImportation);
 
-   m_PatientDatas.clear();
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_DRUGSALLERGIES, Core::IPatient::DrugsAtcAllergies);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_DRUGSCHRONIC, Core::IPatient::DrugsChronicTherapeutics);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_WEIGHT, Core::IPatient::Weight);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_WEIGHTUNIT, Core::IPatient::WeightUnit);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_HEIGHT, Core::IPatient::Height);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_HEIGHTUNIT, Core::IPatient::HeightUnit);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_CREAT, Core::IPatient::Creatinine);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_CREATUNIT, Core::IPatient::CreatinineUnit);
-
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_EMAIL, Core::IPatient::Mails);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_FAX, Core::IPatient::Faxes);
-   m_PatientDatas.insert(Constants::TAG_DATAPATIENT_TELS, Core::IPatient::Tels);
+   m_PatientData.clear();
+   // use the Core::IPatient::enumToString()
+   for(int i=0; i< Core::IPatient::NumberOfColumns; ++i) {
+       m_PatientData.insert(patient()->enumToString(Core::IPatient::PatientDataRepresentation(i)).toLower(), i);
+   }
+   m_Initialized = true;
+   return true;
 }
 
 XmlFormContentReader::~XmlFormContentReader()
 {}
 
+/** Clear internal cache */
 void XmlFormContentReader::clearCache()
 {
     m_ReadableForms.clear();
@@ -238,6 +245,7 @@ bool XmlFormContentReader::checkFileContent(const QString &formUidOrFullAbsPath,
             warnXmlReadError(m_Mute, formUidOrFullAbsPath, error);
             ok = false;
         }
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         if (ok)
             m_DomDocFormCache.insert(formUidOrFullAbsPath, doc);
     }
@@ -256,6 +264,7 @@ bool XmlFormContentReader::checkFileContent(const QString &formUidOrFullAbsPath,
             m_Error.append(msg);
             ok = false;
         }
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     }
     return ok;
 }
@@ -276,8 +285,8 @@ QMultiHash<QString, QString> XmlFormContentReader::readUuidEquivalence(const QDo
     while (!item.isNull()) {
         if (item.attribute(Constants::ATTRIB_OLD_UUID).isEmpty() || item.attribute(Constants::ATTRIB_NEW_UUID).isEmpty()) {
             LOG_ERROR_FOR("XmlFormContentReader", "Field equivalence missing old/new attributes");
-            continue;
             item = item.nextSiblingElement(Constants::TAG_UUID_EQUIVALENCE_ITEM);
+            continue;
         }
         oldToNew.insertMulti(item.attribute(Constants::ATTRIB_OLD_UUID), item.attribute(Constants::ATTRIB_NEW_UUID));
         item = item.nextSiblingElement(Constants::TAG_UUID_EQUIVALENCE_ITEM);
@@ -311,38 +320,36 @@ Form::FormIODescription *XmlFormContentReader::readFileInformation(const QString
     root = root.firstChildElement(Constants::TAG_FORM_DESCRIPTION);
     toReturn = readXmlDescription(root, formUidOrFullAbsPath);
 
-    // get screenshots if requiered
+    // has screenshots?
     XmlFormName form(formUidOrFullAbsPath);
-    if (query.getScreenShots()) {
-        if (query.forceFileReading()) {
-            // Get from local files
-            QString shotPath = form.absPath + QDir::separator() + "shots" + QDir::separator();
-            QStringList lang;
-            lang << QLocale().name().left(2).toLower() << "en" << "xx" << "all";
-            bool found = false;
-            foreach(const QString &l, lang) {
-                if (QDir(shotPath + l).exists()) {
-                    found = true;
-                    shotPath = shotPath + l;
-                    break;
-                }
-            }
-            if (found) {
-                QDir dir(shotPath);
-                //            qWarning() << "Trying to read shots" << dir.absolutePath();
-                foreach(const QFileInfo &file, dir.entryInfoList(QStringList() << "*.png" << "*.jpg" << "*.jpeg" << "*.gif")) {
-                    QPixmap pix(file.absoluteFilePath());
-                    toReturn->addScreenShot(file.absoluteFilePath().remove(shotPath), pix);
-                }
-            }
-        } else {
-            // Get from database
-            QHash<QString, QPixmap> pix = base()->getScreenShots(form.uid, QLocale().name().left(2).toLower());
-//            qWarning() << "xxxxxxxx FROMBASE" << formUidOrFullAbsPath << pix.keys();
-            foreach(const QString &k, pix.keys()) {
-                toReturn->addScreenShot(k, pix.value(k));
+    if (query.forceFileReading()) {
+        // Get from local files
+        QString shotPath = form.absPath + QDir::separator() + "shots" + QDir::separator();
+        QStringList lang;
+        lang << QLocale().name().left(2).toLower() << "en" << "xx" << "all";
+        bool found = false;
+        foreach(const QString &l, lang) {
+            if (QDir(shotPath + l).exists()) {
+                found = true;
+                shotPath = shotPath + l;
+                break;
             }
         }
+        if (found) {
+            QDir dir(shotPath);
+            qWarning() << "Trying to read shots" << dir.absolutePath();
+            toReturn->setData(Form::FormIODescription::HasScreenShot, true);
+            // foreach(const QFileInfo &file, dir.entryInfoList(QStringList() << "*.png" << "*.jpg" << "*.jpeg" << "*.gif")) {
+            //     QPixmap pix(file.absoluteFilePath());
+            //     toReturn->addScreenShot(file.absoluteFilePath().remove(shotPath), pix);
+            // }
+        }
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    } else {
+        // Get from database
+        // qWarning() << "Trying to read shots from database";
+        toReturn->setData(Form::FormIODescription::HasScreenShot, base()->hasScreenShots(form.uid));
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     }
     return toReturn;
 }
@@ -385,6 +392,7 @@ bool XmlFormContentReader::loadForm(const XmlFormName &form, Form::FormMain *roo
 {
 //    qWarning() << Q_FUNC_INFO << form.uid << form.absFileName << m_ActualForm;
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     QDomDocument *doc = 0;
     if (!m_DomDocFormCache.keys().contains(form.absFileName)) {
         LOG_ERROR_FOR("XmlFormContentReader","Form not in cache: " + form.absFileName);
@@ -401,34 +409,36 @@ bool XmlFormContentReader::loadForm(const XmlFormName &form, Form::FormMain *roo
             warnXmlReadError(m_Mute, form.uid, tkTr(Trans::Constants::XML_WRONG_ROOT_TAG_1_2).arg(root.tagName()).arg(Constants::TAG_NEW_FORM));
             return false;
         }
-//        rootForm = createNewForm(newForm, m_ActualForm);
     }
     m_ActualForm = rootForm;
 
     if (!loadElement(rootForm, root, form)) {
-        LOG_ERROR_FOR("XmlFormContentReader", "Unable to load form " + form.uid);
+        LOG_ERROR_FOR("XmlFormContentReader", "Unable to load form: " + form.uid);
         return false;
     }
 
 //    rootForm->createDebugPage();
-    createWidgets(rootForm);
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+//    createWidgets(rootForm);
+//    if (!setTabOrder(rootForm, ))
 
     // Manage uuid equivalence
     QMultiHash<QString, QString> oldToNew = readUuidEquivalence(doc);
     if (!oldToNew.isEmpty()) {
         QStringList newUids = oldToNew.values();
         newUids.removeDuplicates();
-        foreach(Form::FormMain *main, rootForm->flattenFormMainChildren()) {
+        foreach(Form::FormMain *main, rootForm->flattenedFormMainChildren()) {
             if (newUids.contains(main->uuid(), Qt::CaseInsensitive)) {
-                main->setEquivalentUuid(oldToNew.keys(main->uuid()));
+                main->spec()->setEquivalentUuid(oldToNew.keys(main->uuid()));
             }
-            foreach(Form::FormItem *item, main->flattenFormItemChildren()) {
+            foreach(Form::FormItem *item, main->flattenedFormItemChildren()) {
                 if (newUids.contains(item->uuid(), Qt::CaseInsensitive)) {
-                    item->setEquivalentUuid(oldToNew.keys(item->uuid()));
+                    item->spec()->setEquivalentUuid(oldToNew.keys(item->uuid()));
                 }
             }
         }
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
 
     return true;
 }
@@ -507,7 +517,7 @@ bool XmlFormContentReader::loadElement(Form::FormItem *item, QDomElement &rootEl
 
         // Patient Data Representation ?
         if (element.tagName().compare(Constants::TAG_DATAPATIENT, Qt::CaseInsensitive)==0) {
-            i = m_PatientDatas.value(element.text(), -1);
+            i = m_PatientData.value(element.text().toLower(), -1);
             if (i != -1) {
                 item->setPatientDataRepresentation(i);
             }
@@ -516,15 +526,18 @@ bool XmlFormContentReader::loadElement(Form::FormItem *item, QDomElement &rootEl
         }
 
     //             // optional?
-//            if (element.hasAttribute(Constants::ATTRIB_OPTIONNAL)) {
-//                if (element.attribute(Constants::ATTRIB_OPTIONNAL).compare("true",Qt::CaseInsensitive)==0)
-//                    item->valueReferences()->setOptionnal(true);
+//            if (element.hasAttribute(Constants::ATTRIB_OPTIONAL)) {
+//                if (element.attribute(Constants::ATTRIB_OPTIONAL).compare("true",Qt::CaseInsensitive)==0)
+//                    item->valueReferences()->setOptional(true);
 //                else
-//                    item->valueReferences()->setOptionnal(false);
+//                    item->valueReferences()->setOptional(false);
 
         // Add a form file ?
         if (element.tagName().compare(Constants::TAG_ADDFILE, Qt::CaseInsensitive)==0) {
-            addFile(element, form);
+            if (!addFile(element, form, item)) {
+                LOG_ERROR_FOR("XmlFormContentReader", QString("Unable to add file: %1; in form: %2")
+                              .arg(element.text()).arg(form.absFileName));
+            }
             element = element.nextSiblingElement();
             continue;
         }
@@ -551,7 +564,7 @@ bool XmlFormContentReader::loadElement(Form::FormItem *item, QDomElement &rootEl
 
         // All others add them in extraData
         item->addExtraData(element.tagName(), element.text());
-//        qWarning() << "°°°°°°°°°°°°°° XML adding other tag" << element.tagName() << element.text() << item->extraDatas();
+        // qWarning() << "°° XML adding other tag" << element.tagName() << element.text() << item->extraData();
 
         element = element.nextSiblingElement();
     }
@@ -577,11 +590,17 @@ bool XmlFormContentReader::createElement(Form::FormItem *item, QDomElement &elem
                 child->spec()->setValue(Form::FormItemSpec::Spec_Plugin, element.attribute(Constants::ATTRIB_TYPE), Trans::Constants::ALL_LANGUAGE);
 
             if (element.hasAttribute(Constants::ATTRIB_PATIENTREPRESENTATION)) {
-                int i = m_PatientDatas.value(element.attribute(Constants::ATTRIB_PATIENTREPRESENTATION), -1);
+                int i = m_PatientData.value(element.attribute(Constants::ATTRIB_PATIENTREPRESENTATION).toLower(), -1);
                 if (i != -1) {
                     child->setPatientDataRepresentation(i);
                 }
             }
+
+            if (element.hasAttribute(Constants::ATTRIB_UIFILE)) {
+                QString content = base()->getFormContent(form.uid, XmlIOBase::UiFile, element.attribute(Constants::ATTRIB_UIFILE));
+                child->spec()->setValue(Form::FormItemSpec::Spec_UiFileContent, content, Trans::Constants::ALL_LANGUAGE);
+            }
+
             loadElement(child, element, form);
             return true;
         }
@@ -608,13 +627,32 @@ bool XmlFormContentReader::createElement(Form::FormItem *item, QDomElement &elem
             if (element.hasAttribute(Constants::ATTRIB_TYPE))
                 item->spec()->setValue(Form::FormItemSpec::Spec_Plugin, element.attribute(Constants::ATTRIB_TYPE), Trans::Constants::ALL_LANGUAGE);
 
+            if (element.hasAttribute(Constants::ATTRIB_ISIDENTITYFORM)) {
+                const QString &att = element.attribute(Constants::ATTRIB_ISIDENTITYFORM);
+                bool isIdentity =
+                        (att.compare("yes", Qt::CaseInsensitive) == 0) ||
+                        (att.compare("true", Qt::CaseInsensitive) == 0) ||
+                        att == "1";
+                item->spec()->setValue(Form::FormItemSpec::Spec_IsIdentityForm, isIdentity, Trans::Constants::ALL_LANGUAGE);
+            }
+
             if (element.hasAttribute(Constants::ATTRIB_UIFILE)) {
                 QString content = base()->getFormContent(form.uid, XmlIOBase::UiFile, element.attribute(Constants::ATTRIB_UIFILE));
                 item->spec()->setValue(Form::FormItemSpec::Spec_UiFileContent, content, Trans::Constants::ALL_LANGUAGE);
             }
 
+            // load all sub-items
             loadElement(item, element, form);
-            // read specific form's datas
+
+            // when the form is loaded -> create the formWidgets
+            createFormWidget(m_ActualForm);
+
+            // then set the taborder
+            if (!setTabOrder(m_ActualForm, element)) {
+                LOG_ERROR_FOR("XmlFormContentReader", "Unable to set tabsorder: " + form.uid);
+            }
+
+            // read specific form's data
             m_ActualForm = oldRootForm;
             return true;
         }
@@ -624,26 +662,23 @@ bool XmlFormContentReader::createElement(Form::FormItem *item, QDomElement &elem
 
     if (element.tagName().compare(Constants::TAG_NEW_PAGE, Qt::CaseInsensitive)==0) {
         // create a new page
-        item = item->createPage(element.firstChildElement(Constants::TAG_NAME).text());
-        // TODO: add page to a form
+        QString uuid = element.attribute(Constants::ATTRIB_UUID);
+        if (uuid.isEmpty())
+            uuid = element.attribute(Constants::ATTRIB_NAME);
+        if (uuid.isEmpty())
+            uuid = element.firstChildElement(Constants::TAG_NAME).text();
+        if (uuid.isEmpty())
+            LOG_ERROR_FOR("XMLIO", "No uuid specified for the FormPage");
+
+        Form::FormPage *page = formManager().createFormPage(uuid);
         if (item) {
-            QString uidNS = getNamespace(item);
-            // read attributes (type, uid/name, patient representation...)
-            if (element.hasAttribute(Constants::ATTRIB_UUID))
-                item->setUuid(uidNS + element.attribute(Constants::ATTRIB_UUID));
-
-            if (element.hasAttribute(Constants::ATTRIB_NAME))
-                item->setUuid(uidNS + element.attribute(Constants::ATTRIB_NAME));
-
-            if (element.hasAttribute(Constants::ATTRIB_TYPE))
-                item->spec()->setValue(Form::FormItemSpec::Spec_Plugin, element.attribute(Constants::ATTRIB_TYPE), Trans::Constants::ALL_LANGUAGE);
-
+            item = page;
             loadElement(item, element, form);
-            // read specific page's datas
+            page->specLoaded();
             return true;
-        }
-        else
+        } else {
             return false;
+        }
     }
 
     return false;
@@ -695,9 +730,11 @@ bool XmlFormContentReader::populateScripts(Form::FormItem *item, const QDomEleme
 }
 
 /** Read a new file. Called when matched XML tag \e file. */
-bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName &formName)
+bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName &formName, Form::FormItem *item)
 {
     XmlFormName form = formName;
+    if (!item)
+        item = m_ActualForm;
     // Get file content from database
     QString fileName = element.text();
     // Guess the type of file according to its extension
@@ -712,15 +749,18 @@ bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName
             form = XmlFormName(fileName);
             fileName = form.modeName;
         }
-    }
-    else if (fileName.endsWith(".js", Qt::CaseInsensitive))
+    } else if (fileName.endsWith(".js", Qt::CaseInsensitive)) {
         basetype = XmlIOBase::ScriptFile;
-    else if (fileName.endsWith(".ui", Qt::CaseInsensitive))
+    } else if (fileName.endsWith(".ui", Qt::CaseInsensitive)) {
         basetype = XmlIOBase::UiFile;
-    else if (fileName.endsWith(".html", Qt::CaseInsensitive))
+    } else if (fileName.endsWith(".html", Qt::CaseInsensitive)) {
         basetype = XmlIOBase::HtmlFile;
-//    else if (fileName.endsWith(".qml", Qt::CaseInsensitive))
+    } else if (fileName.endsWith(".pdf", Qt::CaseInsensitive)) {
+        basetype = XmlIOBase::PdfFile;
+    }
+    //    else if (fileName.endsWith(".qml", Qt::CaseInsensitive)) {
 //        basetype = XmlIOBase::QmlContent;
+//        }
 
     QString content = base()->getFormContent(form.uid, basetype, fileName);
     if (content.isEmpty()) {
@@ -736,14 +776,14 @@ bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName
     fileName = QDir::cleanPath(fileName);
 //    QString content = Utils::readTextFile(fileName, Utils::DontWarnUser);
 
-    // Check file content for script addition
+    // Check file content for script addition / printmask types
     const QString &type = element.attribute(Constants::ATTRIB_TYPE);
     if (type.compare(Constants::FILETYPE_SCRIPT, Qt::CaseInsensitive)==0) {
         if (checkFileContent(fileName, content)) {
             // Add script to the empty root FormMain
             Form::FormMain *parent = m_ActualForm;
-            while (m_ActualForm->parentFormMain()) {
-                parent = m_ActualForm->parentFormMain();
+            while (parent->parentFormMain()) {
+                parent = parent->parentFormMain();
             }
             const QString &lang = element.attribute(Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE);
             const QString &scriptType = element.attribute(Constants::ATTRIB_NAME, Trans::Constants::ALL_LANGUAGE);
@@ -756,6 +796,26 @@ bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName
                 parent->scripts()->setScript(Form::FormItemScripts::Script_PostLoad, content, lang);
             return true;
         }
+    } else if (type.compare(Constants::FILETYPE_PRINTHTMLMASK, Qt::CaseInsensitive)==0) {
+        const QString &lang = element.attribute(Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE);
+        if (item)
+            item->spec()->setValue(Form::FormItemSpec::Spec_HtmlPrintMask, content, lang);
+        return true;
+    } else if (type.compare(Constants::FILETYPE_PRINTPLAINTEXTMASK, Qt::CaseInsensitive)==0) {
+        const QString &lang = element.attribute(Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE);
+        if (item)
+            item->spec()->setValue(Form::FormItemSpec::Spec_PlainTextPrintMask, content, lang);
+        return true;
+    } else if (type.compare(Constants::FILETYPE_EXPORTHTMLMASK, Qt::CaseInsensitive)==0) {
+        const QString &lang = element.attribute(Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE);
+        if (item)
+            item->spec()->setValue(Form::FormItemSpec::Spec_HtmlExportMask, content, lang);
+        return true;
+    } else if (type.compare(Constants::FILETYPE_EXPORTPLAINTEXTMASK, Qt::CaseInsensitive)==0) {
+        const QString &lang = element.attribute(Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE);
+        if (item)
+            item->spec()->setValue(Form::FormItemSpec::Spec_PlainTextExportMask, content, lang);
+        return true;
     }
 
     // Check file content (for forms file)
@@ -773,16 +833,20 @@ bool XmlFormContentReader::addFile(const QDomElement &element, const XmlFormName
     return true;
 }
 
+/** Create a formitemwidget for the \e item with the specified \e parent */
 bool XmlFormContentReader::createItemWidget(Form::FormItem *item, QWidget *parent)
 {
+    if (!item)
+        return false;
+
 //    qWarning() << Q_FUNC_INFO;
 //    qWarning() << item << item->uuid() << item->spec()->pluginName();
-    QString requestedWidget = item->spec()->pluginName().toLower();
-    Form::IFormWidgetFactory *factory = m_PlugsFactories.value(requestedWidget);
+    QString requestedWidgetName = item->spec()->pluginName().toLower();
+    Form::IFormWidgetFactory *factory = m_PlugsFactories.value(requestedWidgetName);
     Form::IFormWidget *w = 0;
 
     // does plugin was inform in the xml file ?
-    if (item->spec()->pluginName().isEmpty()) {
+    if (requestedWidgetName.isEmpty()) {
         LOG_ERROR_FOR("XmlFormContentReader", "No plugin name for item: " + item->uuid());
         factory = m_PlugsFactories.value("helptext");
         w = factory->createWidget("helptext", item);
@@ -791,9 +855,8 @@ bool XmlFormContentReader::createItemWidget(Form::FormItem *item, QWidget *paren
     }
 
     // does requested widget exist in the plugins ?
-    factory = m_PlugsFactories.value(requestedWidget);
-    if (!m_PlugsFactories.keys().contains(requestedWidget)) {
-        LOG_ERROR_FOR("XmlFormContentReader", QString("Form error in item '%1': Requested widget '%2' does not exist in plugin's widgets list.").arg(item->uuid()).arg(requestedWidget));
+    if (!factory) {
+        LOG_ERROR_FOR("XmlFormContentReader", QString("Form error in item '%1': Requested widget '%2' does not exist in plugin's widgets list.").arg(item->uuid()).arg(requestedWidgetName));
         // Add a default widget for the error log
         factory = m_PlugsFactories.value("helptext");
         item->spec()->setValue(Form::FormItemSpec::Spec_Label, QString("XML FORM ERROR: Requested widget does not exist for item '%1'").arg(item->uuid()));
@@ -802,7 +865,7 @@ bool XmlFormContentReader::createItemWidget(Form::FormItem *item, QWidget *paren
     }
 
     // get the widget
-    w = factory->createWidget(requestedWidget, item);
+    w = factory->createWidget(requestedWidgetName, item, parent);
     if (w->isContainer()) {
         foreach(Form::FormItem *child, item->formItemChildren()) {
 //            Form::IFormWidget *wchild = factory->createWidget(child->spec()->pluginName(),child,w);
@@ -811,12 +874,15 @@ bool XmlFormContentReader::createItemWidget(Form::FormItem *item, QWidget *paren
             createItemWidget(child, w);
         }
     }
+
+    // FIXME: With QtUi forms we should not do this
     Form::IFormWidget *p = qobject_cast<Form::IFormWidget*>(parent);
     if (p)
         p->addWidgetToContainer(w);
     return true;
 }
 
+/** Create the formitemwidget for a main form (and also its children) */
 bool XmlFormContentReader::createFormWidget(Form::FormMain *form)
 {
     if (!createItemWidget(form, 0))
@@ -824,84 +890,60 @@ bool XmlFormContentReader::createFormWidget(Form::FormMain *form)
     return true;
 }
 
+/** Create all the formitemwidgets for a main form */
 bool XmlFormContentReader::createWidgets(const Form::FormMain *rootForm)
 {
     // foreach FormMain children
-    foreach(Form::FormMain *form, rootForm->flattenFormMainChildren()) {
+    foreach(Form::FormMain *form, rootForm->flattenedFormMainChildren()) {
         // create the form
         createFormWidget(form);
     }
     return true;
 }
 
-bool XmlFormContentReader::loadPmhCategories(const QString &uuidOrAbsPath)
+bool XmlFormContentReader::setTabOrder(Form::FormMain *rootForm, const QDomElement &root)
 {
-    Q_UNUSED(uuidOrAbsPath);
-//    QString file = QFileInfo(uuidOrAbsPath).absolutePath() + QDir::separator() + Constants::PMHXCATOGORIES_FILENAME;
-//    // replace path TAGs
-//    file.replace(Core::Constants::TAG_APPLICATION_COMPLETEFORMS_PATH, settings()->path(Core::ISettings::CompleteFormsPath));
-//    file.replace(Core::Constants::TAG_APPLICATION_SUBFORMS_PATH, settings()->path(Core::ISettings::SubFormsPath));
-//    file.replace(Core::Constants::TAG_APPLICATION_RESOURCES_PATH, settings()->path(Core::ISettings::BundleResourcesPath));
-
-//    QDomDocument *doc = 0;
-//    if (!reader()->isInCache(file)) {
-//        if (!canReadForms(file)) {
-//            LOG_ERROR("Unable to read Pmh Category file: " + file);
-//            return false;
-//        }
-//    }
-//    categoryCore()->removeAllExistingCategories("PMHx");
-//    doc = reader()->fromCache(file);
-//    Q_ASSERT(doc);
-//    if (!doc) {
-//        LOG_ERROR("No category document in XmlFormIO::loadPmhCategories("+file+")");
-//        return false;
-//    }
-//    QDomElement root = doc->firstChildElement(Constants::TAG_MAINXMLTAG);
-//    QDomElement element = root.firstChildElement(Constants::TAG_PMHX_CATEGORIES);
-//    element = element.firstChildElement(::Constants::TAG_CATEGORY);
-//    while (!element.isNull()) {
-//        createCategory(element, 0);
-//        element = element.nextSiblingElement(::Constants::TAG_CATEGORY);
-//    }
-//    pmhCore()->pmhCategoryModel()->refreshFromDatabase();
-    return true;
-}
-
-bool XmlFormContentReader::createCategory(const QDomElement &element, Category::CategoryItem *parent)
-{
-    Q_UNUSED(element);
-    Q_UNUSED(parent);
-//    // create the category
-//    Category::CategoryItem *item = new Category::CategoryItem;
-//    item->setData(Category::CategoryItem::DbOnly_Mime, "PMHx");
-//    item->setData(Category::CategoryItem::ThemedIcon, element.attribute(::Constants::ATTRIB_ICON));
-
-//    // read the labels
-//    QDomElement label = element.firstChildElement(::Constants::TAG_SPEC_LABEL);
-//    while (!label.isNull()) {
-//        item->setLabel(label.text(), label.attribute(::Constants::ATTRIB_LANGUAGE, Trans::Constants::ALL_LANGUAGE));
-//        label = label.nextSiblingElement(::Constants::TAG_SPEC_LABEL);
-//    }
-
-//    // get ExtraTag content -> CategoryItem::ExtraXml
-//    QDomElement extra = element.firstChildElement(::Constants::TAG_SPEC_EXTRA);
-//    if (!extra.isNull()) {
-//        item->setData(Category::CategoryItem::ExtraXml, extra.toDocument().toString(2));
-//    }
-
-//    // save to database
-//    if (parent) {
-//        parent->addChild(item);
-//        item->setParent(parent);
-//    }
-//    categoryCore()->saveCategory(item);
-
-//    // has children ?
-//    QDomElement child = element.firstChildElement(::Constants::TAG_CATEGORY);
-//    while (!child.isNull()) {
-//        createCategory(child, item);
-//        child = child.nextSiblingElement(::Constants::TAG_CATEGORY);
-//    }
+    QDomElement tabs = root.firstChildElement(Constants::TAG_TABSTOPS);
+    if (tabs.isNull())
+        return true;
+    if (WarnTabOrder)
+        qWarning() << "SETABS" << rootForm->uuid();
+//    QString ns = root.attribute("ns");
+//    if (!ns.isEmpty())
+//        ns.append("::");
+    const QList<Form::FormItem *> &items = rootForm->flattenedFormItemChildren();
+    Form::IFormWidget *first = 0;
+    Form::IFormWidget *second = 0;
+    QDomElement element = tabs.firstChildElement(Constants::TAG_TABSTOP);
+    QString warn;
+    while (!element.isNull()) {
+        const QString &widgetName = element.text();
+        foreach(Form::FormItem *item, items) {
+            if (item->uuid().endsWith(widgetName)) {
+                // get it
+                if (!first) {
+                    if (WarnTabOrder)
+                        qWarning() << "---------------------------- first" << item->uuid();
+                    first = item->formWidget();
+                    first->setTabOrder(WarnTabOrder);
+                    warn = "    first: " + item->uuid() + "\n";
+                } else {
+                    warn += "   second: " + item->uuid() + "\n";
+                    second = item->formWidget();
+                    if (WarnTabOrder) {
+//                        qWarning() << QString("  setTabOrder\n" + warn);
+                        qWarning() << "    " << first->lastTabWidget() << "\n    " << second->focusedWidget() << "\n\n";
+                    }
+                    QWidget::setTabOrder(first->lastTabWidget(), second->focusedWidget());
+                    first = second;
+                    first->setTabOrder(WarnTabOrder);
+                    warn = "    first: " + item->uuid() + "\n";
+                    second = 0;
+                }
+                break;
+            }
+        }
+        element = element.nextSiblingElement(Constants::TAG_TABSTOP);
+    }
     return true;
 }
