@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -61,11 +61,13 @@ using namespace XmlForms;
 using namespace Internal;
 using namespace Trans::ConstantTranslations;
 
+static inline Core::IPatient *patient() {return Core::ICore::instance()->patient();}
+
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////  Inline static functions  //////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-inline static ExtensionSystem::PluginManager *pluginManager() {return ExtensionSystem::PluginManager::instance();}
+static inline ExtensionSystem::PluginManager *pluginManager() {return ExtensionSystem::PluginManager::instance();}
 static inline Core::ISettings *settings()  { return Core::ICore::instance()->settings(); }
 static inline Category::CategoryCore *categoryCore() {return  Category::CategoryCore::instance();}
 static inline PMH::PmhCore *pmhCore() {return PMH::PmhCore::instance();}
@@ -84,7 +86,9 @@ static inline XmlFormName &formName(const QString &uuid, QHash<QString, XmlFormN
 //////////////////////////////////////////////  XmlFormIO  /////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 XmlFormIO::XmlFormIO(QObject *parent) :
-        IFormIO(parent), m_Mute(false)
+    IFormIO(parent),
+    m_Mute(false),
+    alreadyCheckedForUpdates(false)
 {
     setObjectName("XmlFormIO");
 }
@@ -93,7 +97,7 @@ XmlFormIO::~XmlFormIO()
 {
 }
 
-QString XmlFormIO::lastestXmlVersion()
+QString XmlFormIO::latestXmlVersion()
 {
     return "0.3.0";
 }
@@ -134,6 +138,7 @@ bool XmlFormIO::canReadForms(const Form::FormIOQuery &query) const
         if (formFile.suffix().toLower()=="xml")
             content = Utils::readTextFile(formFile.absoluteFilePath(), Utils::DontWarnUser);
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
 
     // check form content
     if (!reader()->checkFileContent(formFile.absoluteFilePath(), content)) {
@@ -148,6 +153,7 @@ bool XmlFormIO::canReadForms(const Form::FormIOQuery &query) const
         m_ReadableForms.insert(form.uid, false);
         return false;
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
 
     // check included xml files too
     QDomDocument *doc = reader()->fromCache(formFile.absoluteFilePath());
@@ -165,6 +171,7 @@ bool XmlFormIO::canReadForms(const Form::FormIOQuery &query) const
                 if (!canReadForms(queryInclude))
                     LOG_ERROR("Unable to read included form: " + include);
             }
+            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         }
     }
 
@@ -174,60 +181,88 @@ bool XmlFormIO::canReadForms(const Form::FormIOQuery &query) const
     return true;
 }
 
-bool XmlFormIO::canReadScripts(const Form::FormIOQuery &query) const
-{
-//    XmlFormName form(query.formUuid());
-    XmlFormName &form = formName(query.formUuid(), m_FormNames);
-//    qWarning() << Q_FUNC_INFO << query.formUuid() << form.uid << form.absFileName << form.modeName;
+// UNUSED
+//bool XmlFormIO::canReadScripts(const Form::FormIOQuery &query) const
+//{
+////    XmlFormName form(query.formUuid());
+//    XmlFormName &form = formName(query.formUuid(), m_FormNames);
+////    qWarning() << Q_FUNC_INFO << query.formUuid() << form.uid << form.absFileName << form.modeName;
 
-    if (m_ReadableForms.keys().contains(form.absFileName)) {
-        return m_ReadableForms.value(form.absFileName);
-    }
-    m_Error.clear();
-    m_AbsFileName.clear();
-    QFileInfo scriptFile(form.absFileName);
+//    if (m_ReadableForms.keys().contains(form.absFileName)) {
+//        return m_ReadableForms.value(form.absFileName);
+//    }
+//    m_Error.clear();
+//    m_AbsFileName.clear();
+//    QFileInfo scriptFile(form.absFileName);
 
-    QString content;
-    if (!query.forceFileReading()) {
-        // Get it from database
-        if (!base()->isFormExists(form, XmlIOBase::ScriptFile, form.modeName)) {
-            base()->saveForm(form);
-        }
-        content = base()->getFormContent(form.uid, XmlIOBase::ScriptFile, form.modeName);
-    } else {
-        if (!scriptFile.exists()) {
-            LOG_ERROR(tkTr(Trans::Constants::FILE_1_DOESNOT_EXISTS).arg(scriptFile.absoluteFilePath()));
-            m_Error.append(tkTr(Trans::Constants::FILE_1_DOESNOT_EXISTS).arg(scriptFile.absoluteFilePath()));
-            return false;
-        }
-        if (scriptFile.suffix().toLower()=="js")
-            content = Utils::readTextFile(scriptFile.absoluteFilePath(), Utils::DontWarnUser);
-    }
+//    QString content;
+//    if (!query.forceFileReading()) {
+//        // Get it from database
+//        if (!base()->isFormExists(form, XmlIOBase::ScriptFile, form.modeName)) {
+//            base()->saveForm(form);
+//        }
+//        content = base()->getFormContent(form.uid, XmlIOBase::ScriptFile, form.modeName);
+//    } else {
+//        if (!scriptFile.exists()) {
+//            LOG_ERROR(tkTr(Trans::Constants::FILE_1_DOESNOT_EXISTS).arg(scriptFile.absoluteFilePath()));
+//            m_Error.append(tkTr(Trans::Constants::FILE_1_DOESNOT_EXISTS).arg(scriptFile.absoluteFilePath()));
+//            return false;
+//        }
+//        if (scriptFile.suffix().toLower()=="js")
+//            content = Utils::readTextFile(scriptFile.absoluteFilePath(), Utils::DontWarnUser);
+//    }
 
-    // check form content
-    if (!reader()->checkFileContent(scriptFile.absoluteFilePath(), content)) {
-        LOG_ERROR(tr("Invalid script file detected: %1").arg(scriptFile.absoluteFilePath()));
-        Utils::warningMessageBox(tr("Invalid file detected."),
-                                 tr("An invalid file was found. Please contact your software administrator.\n"
-                                    "Wrong file: %1\n"
-                                    "Error: %2")
-                                 .arg(form.absFileName)
-                                 .arg(reader()->lastError()));
-        m_ReadableScripts.insert(form.absFileName, false);
-        return false;
-    }
-    m_AbsFileName = form.absFileName;
-    m_ReadableScripts.insert(form.absFileName, true);
-    return true;
-}
+//    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+//    // check form content
+//    if (!reader()->checkFileContent(scriptFile.absoluteFilePath(), content)) {
+//        LOG_ERROR(tr("Invalid script file detected: %1").arg(scriptFile.absoluteFilePath()));
+//        Utils::warningMessageBox(tr("Invalid file detected."),
+//                                 tr("An invalid file was found. Please contact your software administrator.\n"
+//                                    "Wrong file: %1\n"
+//                                    "Error: %2")
+//                                 .arg(form.absFileName)
+//                                 .arg(reader()->lastError()));
+//        m_ReadableScripts.insert(form.absFileName, false);
+//        return false;
+//    }
+//    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+//    m_AbsFileName = form.absFileName;
+//    m_ReadableScripts.insert(form.absFileName, true);
+//    return true;
+//}
 
 Form::FormIODescription *XmlFormIO::readFileInformation(const QString &uuidOrAbsPath) const
 {
     return reader()->readFileInformation(uuidOrAbsPath);
 }
 
+// Check the descriptions according to the query settings. Warning: \e forms is modified
+static void checkFormIODescription(QList<Form::FormIODescription *> &forms, const Form::FormIOQuery &query, const XmlFormIO *reader)
+{
+    // Check gender specific
+    if (query.excludeGenderSpecific()) {
+        QString patientGender = patient()->data(Core::IPatient::Gender).toString().toUpper(); // M F H
+        for(int i = forms.count() - 1; i > 0 ; --i) {
+            Form::FormIODescription *descr = forms.at(i);
+            const QString &gender = descr->data(Form::FormIODescription::GenderLimitation).toString();
+            if (!gender.isEmpty()) {
+                if (gender.toUpper() != patientGender)
+                    forms.removeAt(i);
+            }
+        }
+    }
+    // TODO: check all forms for params of Query, check forms versions, remove duplicates
+    // Set IOReader to descriptions
+    const Form::IFormIO *const_iformio = qobject_cast<const Form::IFormIO*>(reader);
+    Form::IFormIO *iformio = const_cast<Form::IFormIO*>(const_iformio);
+    for(int i = 0; i < forms.count(); ++i) {
+        forms.at(i)->setIoFormReader(iformio);
+    }
+}
+
 QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::FormIOQuery &query) const
 {
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     QList<Form::FormIODescription *> toReturn;
     QStringList includedUids;
 //    qWarning() << query.formUuid() << query.forceFileReading();
@@ -235,13 +270,16 @@ QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::
     if (!query.forceFileReading()) {
         // Get from database
         toReturn = base()->getFormDescription(query);
-        if (!toReturn.isEmpty() && !query.getAllAvailableFormDescriptions())
+        if (!toReturn.isEmpty() && !query.getAllAvailableFormDescriptions()) {
+            checkFormIODescription(toReturn, query, this);
             return toReturn;
+        }
         for(int i=0; i<toReturn.count(); ++i) {
             includedUids << toReturn.at(i)->data(Form::FormIODescription::UuidOrAbsPath).toString();
         }
     }
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     // Get a specific form description
     if (!query.formUuid().isEmpty()) {
         XmlFormName &form = formName(query.formUuid(), m_FormNames);
@@ -250,6 +288,9 @@ QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::
             Form::FormIODescription *desc = reader()->readFileInformation(form.absFileName, query);
             if (desc) {
                 desc->setData(Form::FormIODescription::IsCompleteForm, true);
+                const Form::IFormIO *const_iformio = qobject_cast<const Form::IFormIO*>(this);
+                Form::IFormIO *iformio = const_cast<Form::IFormIO*>(const_iformio);
+                desc->setIoFormReader(iformio);
                 toReturn.append(desc);
                 return toReturn;
             }
@@ -257,6 +298,7 @@ QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::
         return toReturn;
     }
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     // Get all form files
     if (query.typeOfForms() & Form::FormIOQuery::CompleteForms) {
         QStringList path;
@@ -282,6 +324,8 @@ QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::
             }
         }
     }
+
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (query.typeOfForms() & Form::FormIOQuery::SubForms) {
         QStringList path;
         path << settings()->path(Core::ISettings::SubFormsPath);
@@ -306,10 +350,8 @@ QList<Form::FormIODescription *> XmlFormIO::getFormFileDescriptions(const Form::
         }
     }
 
-    // TODO: check all forms for params of Query, check forms versions, remove duplicates
-    for(int i = 0; i < toReturn.count(); ++i) {
-        toReturn.at(i)->setIoFormReader((Form::IFormIO*)this);
-    }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    checkFormIODescription(toReturn, query, this);
     return toReturn;
 }
 
@@ -329,6 +371,7 @@ QList<Form::FormMain *> XmlFormIO::loadAllRootForms(const QString &uuidOrAbsPath
             uuid = m_AbsFileName;
         }
     }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!canReadForms(form.uid)) {
         LOG_ERROR("Can not read form " + form.uid);
         return toReturn;
@@ -347,16 +390,18 @@ QList<Form::FormMain *> XmlFormIO::loadAllRootForms(const QString &uuidOrAbsPath
         base()->saveForm(form);
     }
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     QHash<QString, QString> mode_contents = base()->getAllFormFullContent(form.uid);
 
     reader()->refreshPluginFactories();
+
     QHashIterator<QString, QString> it(mode_contents);
     while (it.hasNext()) {
         it.next();
         Form::FormMain *root = m_ActualForm = new Form::FormMain;
         root->setModeUniqueName(it.key());
         root->setUuid(form.uid);
-        root->setIoFormReader((XmlFormIO*)this);
+        root->setIoFormReader(const_cast<XmlFormIO*>(this));
         QString fakeFileName;
         QFileInfo info(form.absFileName);
         if (formFile.isDir())
@@ -370,6 +415,7 @@ QList<Form::FormMain *> XmlFormIO::loadAllRootForms(const QString &uuidOrAbsPath
         }
         XmlFormName mode(form.uid);
         mode.absFileName = fakeFileName;
+        mode.modeName = it.key();
 //        qWarning() << "MODE" << mode.absFileName << mode.uid;
         if (!reader()->loadForm(mode, root)) {
             LOG_ERROR("Form not readable: " + fakeFileName);
@@ -377,7 +423,7 @@ QList<Form::FormMain *> XmlFormIO::loadAllRootForms(const QString &uuidOrAbsPath
             toReturn.append(root);
         }
         // Emit the formLoaded signal for all subForms loaded
-        QList<Form::FormMain *> forms = root->flattenFormMainChildren();
+        QList<Form::FormMain *> forms = root->flattenedFormMainChildren();
         for(int i=0; i < forms.count(); ++i) {
             forms.at(i)->emitFormLoaded();
         }
@@ -387,20 +433,19 @@ QList<Form::FormMain *> XmlFormIO::loadAllRootForms(const QString &uuidOrAbsPath
 
 bool XmlFormIO::loadPmhCategories(const QString &uuidOrAbsPath) const
 {
-    XmlFormName &form = formName(uuidOrAbsPath, m_FormNames);
+    Q_UNUSED(uuidOrAbsPath);
+    // TODO: remove dead code
+//    XmlFormName &form = formName(uuidOrAbsPath, m_FormNames);
 //    WARN_FUNC << uuidOrAbsPath << form;
-    pmhCore()->pmhCategoryModel()->setRootFormUid(form.uid);
-    pmhCore()->pmhCategoryModel()->refreshFromDatabase();
-    LOG("Category retreived");
+//    pmhCore()->pmhCategoryModel()->setRootFormUid(form.uid);
+//    pmhCore()->pmhCategoryModel()->refreshFromDatabase();
+//    LOG("--------------------------- PMH Category retreived");
     return true;
 }
 
-
 QList<QPixmap> XmlFormIO::screenShots(const QString &uuidOrAbsPath) const
 {
-    Q_UNUSED(uuidOrAbsPath);
-    QList<QPixmap> toReturn;
-    return toReturn;
+    return base()->getScreenShots(uuidOrAbsPath).values();
 }
 
 QPixmap XmlFormIO::screenShot(const QString &uuidOrAbsPath, const QString &name) const
@@ -408,85 +453,136 @@ QPixmap XmlFormIO::screenShot(const QString &uuidOrAbsPath, const QString &name)
     return base()->getScreenShot(uuidOrAbsPath, name);
 }
 
+QString XmlFormIO::extractFileToTmpPath(const QString &uuidOrAbsPath, const QString &name) const
+{
+    // Get the extension of the requested file
+    QFileInfo info(name);
+    int type = XmlIOBase::ExtraFiles;
+    if (info.suffix().compare("pdf", Qt::CaseInsensitive) == 0) {
+        type = XmlIOBase::PdfFile;
+    } else if ((info.suffix().compare("html", Qt::CaseInsensitive) == 0)
+               || (info.suffix().compare("htm", Qt::CaseInsensitive) == 0)) {
+        type = XmlIOBase::HtmlFile;
+    }
+
+    // Get the file content
+    QString content = base()->getFormContent(uuidOrAbsPath, type, name);
+    if (content.isEmpty())
+        return QString::null;
+
+    // Create output absolutefilepath
+    QString output = settings()->path(Core::ISettings::ApplicationTempPath) + QDir::separator();
+    output += XmlIOBase::createUid() + "." + info.suffix();
+
+    // Save the content to a tmp path
+    if (type == XmlIOBase::PdfFile) {
+        QByteArray save = QByteArray::fromBase64(content.toAscii());
+        QFile file(output);
+        if (!file.open(QFile::WriteOnly)) {
+            LOG_ERROR("Unable to open file: " + output);
+            return QString::null;
+        }
+        file.write(save);
+        file.close();
+    } else {
+        if (!Utils::saveStringToFile(content, output, Utils::Overwrite, Utils::DontWarnUser)) {
+            LOG_ERROR("Unable to open file: " + output);
+            return QString::null;
+        }
+    }
+    return output;
+}
+
 void XmlFormIO::checkForUpdates() const
 {
     reader()->clearCache();
     checkDatabaseFormFileForUpdates();
+    alreadyCheckedForUpdates = true;
+}
+
+const QList<Form::FormIODescription> &XmlFormIO::availableUpdates() const
+{
+    if (!alreadyCheckedForUpdates)
+        checkForUpdates();
+    return m_FormUpdatesList;
+}
+
+bool XmlFormIO::updateForms()
+{
+    if (!alreadyCheckedForUpdates)
+        checkForUpdates();
+    if (formsToUpdate.empty())
+        return false;
+
+    foreach(XmlFormName form, formsToUpdate) {
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+        if (!base()->saveForm(form)) {
+            LOG_ERROR("Unable to update form database. Form: " + form.uid + " " + form.absFileName);
+        } else {
+            LOG("Form updated: "  + form.uid + " " + form.absFileName);
+        }
+    }
+    return true;
 }
 
 /** Check the database form version and try to update them with the local files. */
 bool XmlFormIO::checkDatabaseFormFileForUpdates() const
 {
-    QList<Form::FormIODescription *> fromFiles;
-    QList<Form::FormIODescription *> fromDb;
+    m_FormUpdatesList.clear();
+    QList<Form::FormIODescription *> fileDescriptionList;
+    QList<Form::FormIODescription *> dbDescriptionList;
     LOG("Checking for form update");
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     // get all available descriptions from database
     Form::FormIOQuery querydb;
     querydb.setGetAllAvailableFormDescriptions(true);
-    fromDb = base()->getFormDescription(querydb);
+    dbDescriptionList = base()->getFormDescription(querydb);
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+
+    // Test all database forms for an update and populate a list
+    // iterate through all FormIO descriptions in database
+    foreach(Form::FormIODescription *dbDescription, dbDescriptionList) {
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+        qDeleteAll(fileDescriptionList);
+        fileDescriptionList.clear();
 
-    // Test all forms for an update and populate a list
-    QList<XmlFormName> formsToUpdate;
-    bool readError = false;
-    QStringList msg;
-    foreach(Form::FormIODescription *descDb, fromDb) {
-        qDeleteAll(fromFiles);
-        fromFiles.clear();
         Form::FormIOQuery query;
-        query.setFormUuid(descDb->data(Form::FormIODescription::UuidOrAbsPath).toString());
+        query.setFormUuid(dbDescription->data(Form::FormIODescription::UuidOrAbsPath).toString());
         query.setForceFileReading(true);
-        Utils::VersionNumber db(descDb->data(Form::FormIODescription::Version).toString());
-        fromFiles = getFormFileDescriptions(query);
-        foreach(Form::FormIODescription *descFile , fromFiles) {
-            // check version number of forms
-            Utils::VersionNumber file(descFile->data(Form::FormIODescription::Version).toString());
-            if (file.versionString()=="test" || file>db) {
+
+        // get actual version number of a form in database
+        Utils::VersionNumber dbVersion(dbDescription->data(Form::FormIODescription::Version).toString());
+
+        // get file description list of that form and iterate through this list
+        fileDescriptionList = getFormFileDescriptions(query);
+        foreach(Form::FormIODescription *fileDescription, fileDescriptionList) {
+
+            // check version number of forms in the file
+            Utils::VersionNumber fileVersion(fileDescription->data(Form::FormIODescription::Version).toString());
+
+            if (fileVersion.versionString() == "test" || fileVersion > dbVersion) {
                 // update database
-                XmlFormName &form = formName(descFile->data(Form::FormIODescription::UuidOrAbsPath).toString(), m_FormNames);
-                // Construct the detailled text of the user's question messagebox
-                QString html;
-                html = QString("<b>%1</b><br />&nbsp;&nbsp;•&nbsp;%2<br />&nbsp;&nbsp;•&nbsp;%3<br />")
-                        .arg(tr("Form: ") + descFile->data(Form::FormIODescription::ShortDescription).toString())
-                        .arg(tr("New version: %1").arg(file.versionString()))
-                        .arg(tr("Database version: %1").arg(db.versionString()))
-                        ;
-                foreach(const Utils::GenericUpdateInformation &u, descFile->updateInformationForVersion(db)) {
-                    html += "&nbsp;&nbsp;&nbsp;&nbsp;-&nbsp;" + Utils::firstLetterUpperCase(tkTr(Trans::Constants::FROM_1_TO_2).arg(u.fromVersion()).arg(u.toVersion())) + "&nbsp;: " + u.text() + "<br />";
-                }
-                msg << html;
-                if (!formsToUpdate.contains(form))
+                XmlFormName &form = formName(fileDescription->data(Form::FormIODescription::UuidOrAbsPath).toString(), m_FormNames);
+                if (!formsToUpdate.contains(form)) {
                     formsToUpdate << form;
+                    fileDescription->setData(Form::FormIODescription::UpdateAvailable_OldVersion, dbVersion.versionString());
+                    m_FormUpdatesList << Form::FormIODescription(*fileDescription);
+                }
             }
         }
     }
 
-    if (!readError && !formsToUpdate.isEmpty()) {
-        // Ask user for update
-        bool yes = Utils::yesNoMessageBox(tr("Form update detected."),
-                                          tr("A form update has been detected. Do you want to update the forms?"),
-                                          msg.join("<br /><br />"));
-        if (yes) {
-            // Update all checked forms
-            for(int i = 0; i < formsToUpdate.count(); ++i) {
-                XmlFormName &form = formsToUpdate[i];
-                if (!base()->saveForm(form)) { //, doc->toString(2), XmlIOBase::FullContent, file.baseName(), QDateTime::currentDateTime())) {
-                    LOG_ERROR("Unable to update form database. Form: " + form.uid + " " + form.absFileName);
-                } else {
-                    LOG("Form updated: "  + form.uid + " " + form.absFileName);
-                }
-            }
-        }
-    }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
 
     // Clear cache
     m_ReadableForms.clear();
     reader()->clearCache();
 
     // Clear pointers
-    qDeleteAll(fromFiles);
-    fromFiles.clear();
-    qDeleteAll(fromDb);
-    fromDb.clear();
+    qDeleteAll(fileDescriptionList);
+    fileDescriptionList.clear();
+    qDeleteAll(dbDescriptionList);
+    dbDescriptionList.clear();
     return true;
 }
