@@ -1,7 +1,7 @@
 /***************************************************************************
  *  The FreeMedForms project is a set of free, open source medical         *
  *  applications.                                                          *
- *  (C) 2008-2012 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
+ *  (C) 2008-2013 by Eric MAEKER, MD (France) <eric.maeker@gmail.com>      *
  *  All rights reserved.                                                   *
  *                                                                         *
  *  This program is free software: you can redistribute it and/or modify   *
@@ -19,8 +19,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.                            *
  ***************************************************************************/
 /***************************************************************************
- *   Main developers : Eric MAEKER, <eric.maeker@gmail.com>                *
- *   Contributors :                                                        *
+ *  Main developer: Eric MAEKER, <eric.maeker@gmail.com>                   *
+ *  Contributors:                                                          *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  *       NAME <MAIL@ADDRESS.COM>                                           *
  ***************************************************************************/
@@ -43,6 +43,10 @@
 
 #include <pmhplugin/constants.h>
 
+#ifdef WITH_ALERTS
+#   include <alertplugin/alertcore.h>
+#endif
+
 #include <utils/log.h>
 #include <utils/global.h>
 #include <utils/databaseconnector.h>
@@ -87,7 +91,7 @@ static inline QString normalizedFormUid(const QString &formUid)
     return newUid;
 }
 
-// Initializing static datas
+// Initializing static data
 XmlIOBase *XmlIOBase::m_Instance = 0;
 
 /**
@@ -102,7 +106,8 @@ XmlIOBase *XmlIOBase::instance()
 
 XmlIOBase::XmlIOBase(QObject *parent) :
     QObject(parent), Utils::Database(),
-    m_initialized(false)
+    m_initialized(false),
+    _transaction(false)
 {
     m_Instance = this;
     setObjectName("XmlIOBase");
@@ -224,7 +229,7 @@ bool XmlIOBase::createDatabase(const QString &connectionName , const QString &db
         DB = QSqlDatabase::addDatabase("QSQLITE", connectionName);
         if (!QDir(pathOrHostName).exists())
             if (!QDir().mkpath(pathOrHostName))
-                tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName);
+                LOG(tkTr(Trans::Constants::_1_ISNOT_AVAILABLE_CANNOTBE_CREATED).arg(pathOrHostName));
         DB.setDatabaseName(QDir::cleanPath(pathOrHostName + QDir::separator() + dbName));
         if (!DB.open())
             LOG(tkTr(Trans::Constants::DATABASE_1_CANNOT_BE_CREATED_ERROR_2).arg(dbName).arg(DB.lastError().text()));
@@ -265,13 +270,16 @@ bool XmlIOBase::createDatabase(const QString &connectionName , const QString &db
     }
 
     // Table INFORMATION
+    DB.transaction();
     QSqlQuery query(DB);
     query.prepare(prepareInsertQuery(Constants::Table_VERSION));
     query.bindValue(Constants::VERSION_ACTUAL, Constants::DB_VERSION);
     if (!query.exec()) {
         LOG_QUERY_ERROR(query);
+        DB.rollback();
     }
     query.finish();
+    DB.commit();
 
     // database is readable/writable
     LOG(tkTr(Trans::Constants::DATABASE_1_CORRECTLY_CREATED).arg(pathOrHostName + QDir::separator() + dbName));
@@ -297,9 +305,10 @@ void XmlIOBase::onCoreFirstRunCreationRequested()
 }
 
 /**
-  Return true if the form \e form, XmlIOBase::TypeOfForm \e type and \e modeName exists in the database.\n
-  Empty \e modeName are interpreted as the central form.\n
-  The \e form is modified and populate with the database content.
+ * Return true if the form \e form, XmlIOBase::TypeOfForm \e type
+ * and \e modeName exists in the database.\n
+ * Empty \e modeName are interpreted as the central form.\n
+ * The \e form is modified and populate with the database content.
 */
 bool XmlIOBase::isFormExists(XmlFormName &form, const int type, QString modeName)
 {
@@ -361,6 +370,7 @@ QList<Form::FormIODescription *> XmlIOBase::getFormDescription(const Form::FormI
 
     // Just manage type of forms
     DB.transaction();
+    _transaction =true;
     QSqlQuery query(DB);
     Utils::FieldList gets;
     gets << Utils::Field(Constants::Table_FORMS, Constants::FORM_UUID);
@@ -378,7 +388,7 @@ QList<Form::FormIODescription *> XmlIOBase::getFormDescription(const Form::FormI
             conds << Utils::Field(Constants::Table_FORMS, Constants::FORM_UUID, QString("like '%1%'").arg(Core::Constants::TAG_APPLICATION_SUBFORMS_PATH));
         }
     }
-    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("=%1").arg(Description));
+    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("='%1'").arg(Description));
     conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_ISVALID, QString("=1"));
 
     QString req = select(gets, joins, conds);
@@ -390,22 +400,19 @@ QList<Form::FormIODescription *> XmlIOBase::getFormDescription(const Form::FormI
             if (descr) {
                 descr->setData(Form::FormIODescription::Category, descr->data(Form::FormIODescription::Category).toString(), QLocale().name().left(2));
                 descr->setData(Form::FormIODescription::FromDatabase, true);
-
-                if (formQuery.getScreenShots()) {
-                    QHash<QString, QPixmap> pix = getScreenShots(query.value(0).toString(), QLocale().name().left(2).toLower());
-                    foreach(const QString &k, pix.keys()) {
-                        descr->addScreenShot(k, pix.value(k));
-                    }
-                }
+                descr->setData(Form::FormIODescription::HasScreenShot, hasScreenShots(descr->data(Form::FormIODescription::UuidOrAbsPath).toString()));
                 toReturn << descr;
             }
         }
     } else {
         LOG_QUERY_ERROR(query);
+        query.finish();
         DB.rollback();
+        return toReturn;
     }
     query.finish();
     DB.commit();
+    _transaction = false;
     return toReturn;
 }
 
@@ -419,6 +426,7 @@ QHash<QString, QString> XmlIOBase::getAllFormFullContent(const QString &formUid)
     // Get all formIds from Forms table
     QVector<int> ids;
     DB.transaction();
+    _transaction = true;
     QSqlQuery query(DB);
     QHash<int, QString> where;
     where.insert(Constants::FORM_UUID, QString("='%1'").arg(normalizedFormUid(formUid)));
@@ -430,6 +438,7 @@ QHash<QString, QString> XmlIOBase::getAllFormFullContent(const QString &formUid)
     } else {
         LOG_QUERY_ERROR(query);
         DB.rollback();
+        _transaction = false;
         return toReturn;
     }
     query.finish();
@@ -444,7 +453,7 @@ QHash<QString, QString> XmlIOBase::getAllFormFullContent(const QString &formUid)
     for(int i = 0; i < ids.count(); ++i) {
         int id = ids.at(i);
         Utils::FieldList conds;
-        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("=%1").arg(FullContent));
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("='%1'").arg(FullContent));
         conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_ISVALID, QString("=1"));
         conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_FORM_ID, QString("=%1").arg(id));
         req = select(gets, joins, conds);
@@ -459,10 +468,18 @@ QHash<QString, QString> XmlIOBase::getAllFormFullContent(const QString &formUid)
         }
         query.finish();
     }
+    _transaction = false;
+    DB.commit();
     return toReturn;
 }
 
-/** Return the XML content of the form according to the XmlIOBase::TypeOfForm \e type and the \e formUid */
+/**
+ * Return the XML content of the form according to the XmlIOBase::TypeOfForm \e type
+ * and the \e formUid. \n
+ * This member can be called inside or outside a database transaction. If it is called
+ * inside a transaction it does not call neither QSqlDatabase::commit(), nor
+ * QSqlDatabase::rollback().
+*/
 QString XmlIOBase::getFormContent(const QString &formUid, const int type, const QString &modeName)
 {
 //    qWarning() << Q_FUNC_INFO << formUid << type << modeName;
@@ -470,7 +487,12 @@ QString XmlIOBase::getFormContent(const QString &formUid, const int type, const
     QSqlDatabase DB = database();
     if (!connectedDatabase(DB, __LINE__))
         return QString::null;
-    DB.transaction();
+    bool insideTransaction = true;
+    if (!_transaction) {
+        DB.transaction();
+        _transaction = true;
+        insideTransaction = false;
+    }
     QSqlQuery query(DB);
     Utils::FieldList gets;
     gets << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_CONTENT);
@@ -478,7 +500,7 @@ QString XmlIOBase::getFormContent(const QString &formUid, const int type, const
     joins << Utils::Join(Constants::Table_FORMS, Constants::FORM_ID, Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_FORM_ID);
     Utils::FieldList conds;
     conds << Utils::Field(Constants::Table_FORMS, Constants::FORM_UUID, QString("='%1'").arg(normalizedFormUid(formUid)));
-    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("=%1").arg(type));
+    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("='%1'").arg(type));
     conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_ISVALID, QString("=1"));
     if (modeName.isEmpty()) {
         conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("='central'"));
@@ -490,17 +512,30 @@ QString XmlIOBase::getFormContent(const QString &formUid, const int type, const
     if (query.exec(req)) {
         if (query.next()) {
             QString var = query.value(0).toString();
-            DB.commit();
+            if (!insideTransaction) {
+                DB.commit();
+                _transaction = false;
+            }
             return var;
         }
     } else {
         LOG_QUERY_ERROR(query);
-        DB.rollback();
+        if (!insideTransaction) {
+            DB.rollback();
+            _transaction = false;
+        }
     }
+    if (!insideTransaction)
+        _transaction = false;
     return QString();
 }
 
-/** Returns the shot \e shotName associated to the form \e formUid */
+/**
+ * Returns the shot \e shotName associated to the form \e formUid
+ * This member can be called inside or outside a database transaction. If it is called
+ * inside a transaction it does not call neither QSqlDatabase::commit(), nor
+ * QSqlDatabase::rollback().
+*/
 QPixmap XmlIOBase::getScreenShot(const QString &formUid, const QString &shotName)
 {
     // Get shot content
@@ -509,19 +544,92 @@ QPixmap XmlIOBase::getScreenShot(const QString &formUid, const QString &shotName
         return QPixmap();
     // Create pixmap
     QPixmap pix;
-    pix.loadFromData(QByteArray::fromBase64(content.toAscii()));
+    if (!content.isEmpty())
+        pix.loadFromData(QByteArray::fromBase64(content.toUtf8()));
     return pix;
 }
 
-/** Returns the shots for the language \e lang associated to the form \e formUid */
+/**
+ * Returns \e true if the database contains a screenshot for the for \e formUid and
+ * the language \e lang, otherwise return \e false.\n
+ * This member can be called inside or outside a database transaction. If it is called
+ * inside a transaction it does not call neither QSqlDatabase::commit(), nor
+ * QSqlDatabase::rollback().
+*/
+bool XmlIOBase::hasScreenShots(const QString &formUid, const QString &lang)
+{
+    // can be accessed within ou without a transaction
+    QSqlDatabase DB = database();
+    if (!connectedDatabase(DB, __LINE__))
+        return false;
+    bool insideTransaction = true;
+    if (!_transaction) {
+        DB.transaction();
+        _transaction = true;
+        insideTransaction = false;
+    }
+    QSqlQuery query(DB);
+    QString req;
+    Utils::FieldList gets;
+    gets << Utils::Field(Constants::Table_FORMS, Constants::FORM_ID);
+    Utils::JoinList joins;
+    joins << Utils::Join(Constants::Table_FORMS, Constants::FORM_ID, Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_FORM_ID);
+    Utils::FieldList conds;
+    conds << Utils::Field(Constants::Table_FORMS, Constants::FORM_UUID, QString("='%1'").arg(normalizedFormUid(formUid)));
+    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("='%1'").arg(ScreenShot));
+    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_ISVALID, QString("=1"));
+    if (!lang.isEmpty()) {
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(lang));
+        req = select(gets, joins, conds);
+    } else {
+        req = select(gets, joins, conds);
+        conds.clear();
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(QLocale().name().left(2)));
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE 'xx/%'"));
+        req += " AND " + getWhereClause(conds, OR);
+    }
+    bool hasShot = false;
+    if (query.exec(req)) {
+        if (query.next())
+            hasShot = true;
+    } else {
+        LOG_QUERY_ERROR(query);
+        query.finish();
+        if (!insideTransaction) {
+            DB.rollback();
+            _transaction = false;
+        }
+    }
+    if (!insideTransaction) {
+        query.finish();
+        DB.commit();
+        _transaction = false;
+    }
+    return hasShot;
+}
+
+/**
+ * Returns the shots for the language \e lang associated to the form \e formUid.\n
+ * The returned QHash contains the shot uuid as key and the shot pixmap as value.\n
+ * This member can be called inside or outside a database transaction. If it is called
+ * inside a transaction it does not call neither QSqlDatabase::commit(), nor
+ * QSqlDatabase::rollback().
+*/
 QHash<QString, QPixmap> XmlIOBase::getScreenShots(const QString &formUid, const QString &lang)
 {
+    // can be accessed within ou without a transaction
     QHash<QString, QPixmap> pixmaps;
     QSqlDatabase DB = database();
     if (!connectedDatabase(DB, __LINE__))
         return pixmaps;
-    DB.transaction();
+    bool insideTransaction = true;
+    if (!_transaction) {
+        DB.transaction();
+        _transaction = true;
+        insideTransaction = false;
+    }
     QSqlQuery query(DB);
+    QString req;
     Utils::FieldList gets;
     gets << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME);
     gets << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_CONTENT);
@@ -529,30 +637,72 @@ QHash<QString, QPixmap> XmlIOBase::getScreenShots(const QString &formUid, const
     joins << Utils::Join(Constants::Table_FORMS, Constants::FORM_ID, Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_FORM_ID);
     Utils::FieldList conds;
     conds << Utils::Field(Constants::Table_FORMS, Constants::FORM_UUID, QString("='%1'").arg(normalizedFormUid(formUid)));
-    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("=%1").arg(ScreenShot));
+    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_TYPE, QString("='%1'").arg(ScreenShot));
     conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_ISVALID, QString("=1"));
-    conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(lang));
-    QString req = select(gets, joins, conds);
+    if (!lang.isEmpty()) {
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(lang));
+        req = select(gets, joins, conds);
+    } else {
+        req = select(gets, joins, conds);
+        conds.clear();
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(QLocale().name().left(2)));
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE 'xx/%'"));
+        req += " AND " + getWhereClause(conds, OR);
+    }
 
+    int nbShotRead = 0;
     if (query.exec(req)) {
         while (query.next()) {
             QPixmap pix;
-            pix.loadFromData(QByteArray::fromBase64(query.value(1).toByteArray()));
-            pixmaps.insert(query.value(0).toString() ,pix);
+            if (!query.value(1).toByteArray().isEmpty())
+                pix.loadFromData(QByteArray::fromBase64(query.value(1).toByteArray()));
+            pixmaps.insert(query.value(0).toString(), pix);
+            ++nbShotRead;
         }
     } else {
         LOG_QUERY_ERROR(query);
-        DB.rollback();
+        if (!insideTransaction) {
+            DB.rollback();
+            _transaction = false;
+        }
         return pixmaps;
     }
-    DB.commit();
+    query.finish();
+
+    if (nbShotRead==0) {
+        // Try with xx language
+        conds.takeLast();
+        conds << Utils::Field(Constants::Table_FORM_CONTENT, Constants::FORMCONTENT_MODENAME, QString("LIKE '%1/%'").arg(Trans::Constants::ALL_LANGUAGE));
+        req = select(gets, joins, conds);
+        if (query.exec(req)) {
+            while (query.next()) {
+                QPixmap pix;
+                if (!query.value(1).toByteArray().isEmpty())
+                    pix.loadFromData(QByteArray::fromBase64(query.value(1).toByteArray()));
+                pixmaps.insert(query.value(0).toString(), pix);
+                ++nbShotRead;
+            }
+        } else {
+            LOG_QUERY_ERROR(query);
+            if (!insideTransaction) {
+                DB.rollback();
+                _transaction = false;
+            }
+            return pixmaps;
+        }
+    }
+    if (!insideTransaction) {
+        DB.commit();
+        _transaction = false;
+    }
     return pixmaps;
 }
 
 /**
- Save the \e content of the form \e form to the database and return the used formUid.\n
- If the \e content is empty the form file is accessed.\n
- The \e form is modified and populated with the database content.
+ * Save the \e content of the form \e form to the database and return the used formUid.\n
+ * If the \e content is empty the form file is accessed.\n
+ * The \e form is modified and populated with the database content. \n
+ * This member creates its own transaction.
 */
 bool XmlIOBase::saveForm(XmlFormName &form)
 {
@@ -560,17 +710,19 @@ bool XmlIOBase::saveForm(XmlFormName &form)
     if (!connectedDatabase(DB, __LINE__))
         return false;
 
-    if (isFormExists(form))
-        return true;
+//    if (isFormExists(form))
+//        return true;
 
     LOG("Saving forms to database: " + form.uid);
 
     QDir dir(form.absPath);
 
     database().transaction();
+    _transaction = true;
 
     // save all XML files && files included in the form using the <file> XML tag
     foreach(const QFileInfo &f, dir.entryInfoList(QStringList() << "*.xml", QDir::Files | QDir::Readable)) {
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         // save the content of the form file
         QString modeName = f.baseName();
         QString content = Utils::readTextFile(f.absoluteFilePath(), Utils::DontWarnUser);
@@ -578,6 +730,7 @@ bool XmlIOBase::saveForm(XmlFormName &form)
             if (!saveContent(form.uid, content, XmlIOBase::PmhCategories, modeName)) {
                 LOG_ERROR("Can not save pmhx categories associated with the form to database");
                 database().rollback();
+                _transaction = false;
                 return false;
             } else {
                 LOG("Saving pmhx categories file to database " + f.absoluteFilePath());
@@ -585,24 +738,28 @@ bool XmlIOBase::saveForm(XmlFormName &form)
             if (!savePmhxCategories(form, content)) {
                 LOG_ERROR("Can not save pmhx categories associated with the form to database");
                 database().rollback();
+                _transaction = false;
                 return false;
             }
         } else {
             if (!saveContent(form.uid, content, XmlIOBase::FullContent, modeName)) {
                 LOG_ERROR("Can not save form to database");
                 database().rollback();
+                _transaction = false;
                 return false;
             } else {
                 LOG("Saving attached *xml files to database " + f.absoluteFilePath());
             }
         }
 
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         // Try to catch file addition
         // File addition is done using the tag ‘file’ or an attrib of the same name
         QDomDocument doc;
         if (!doc.setContent(content)) {
             LOG_ERROR("XML Error");
             database().rollback();
+            _transaction = false;
             return false;
         }
         QDomNodeList list = doc.elementsByTagName("file");
@@ -615,6 +772,7 @@ bool XmlIOBase::saveForm(XmlFormName &form)
                 if (!saveForm(includeForm)) {
                     LOG_ERROR("unable to save included form: " + includeForm.uid);
                     database().rollback();
+                    _transaction = false;
                     return false;
                 }
             }
@@ -622,36 +780,71 @@ bool XmlIOBase::saveForm(XmlFormName &form)
     }
 
     // Save scripts
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!saveFiles(form, "scripts", "js", XmlIOBase::ScriptFile)) {
         LOG_ERROR("Unable to save script files");
         database().rollback();
+        _transaction = false;
         return false;
     }
 
     // Save uis
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!saveFiles(form, "ui", "ui", XmlIOBase::UiFile)) {
         LOG_ERROR("Unable to save UI files");
         database().rollback();
+        _transaction = false;
         return false;
     }
     // Save qml
 //    saveFiles(form, "qml", "qml", XmlIOBase::QmlFile);
 
-    // Save html
+    // Save html && css
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!saveFiles(form, "html", "html", XmlIOBase::HtmlFile)) {
         LOG_ERROR("Unable to save HTML files");
         database().rollback();
+        _transaction = false;
+        return false;
+    }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    if (!saveFiles(form, "html", "css", XmlIOBase::HtmlFile)) {
+        LOG_ERROR("Unable to save HTML files");
+        database().rollback();
+        _transaction = false;
         return false;
     }
 
     // Save screenshots
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!saveScreenShots(form)) {
         LOG_ERROR("Unable to save screenshot files");
         database().rollback();
+        _transaction = false;
         return false;
     }
 
+    // Register AlertPacks -- manages save/update of alertpacks
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    if (!registerAlertPacks(form)) {
+        LOG_ERROR("Unable to save screenshot files");
+        database().rollback();
+        _transaction = false;
+        return false;
+    }
+
+    // Save PDF
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
+    if (!saveFiles(form, "pdf", "pdf", XmlIOBase::PdfFile)) {
+        LOG_ERROR("Unable to save PDF files");
+        database().rollback();
+        _transaction = false;
+        return false;
+    }
+
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     database().commit();
+    _transaction = false;
 
     if (isFormExists(form))
         return true;
@@ -659,8 +852,14 @@ bool XmlIOBase::saveForm(XmlFormName &form)
     return true;
 }
 
+/**
+ * Creates the PMHx categories related to the \e form
+ * reading the QDomDocument from the \e element
+ * with the parent category \e parent.
+ */
 Category::CategoryItem * XmlIOBase::createCategory(const XmlFormName &form, const QDomElement &element, Category::CategoryItem *parent) const
 {
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     // create the category
     Category::CategoryItem *item = new Category::CategoryItem;
     item->setData(Category::CategoryItem::DbOnly_Mime, QString("%1@%2").arg(PMH::Constants::CATEGORY_MIME).arg(form.uid));
@@ -693,16 +892,23 @@ Category::CategoryItem * XmlIOBase::createCategory(const XmlFormName &form, cons
     // has children ?
     QDomElement child = element.firstChildElement(::Constants::TAG_CATEGORY);
     while (!child.isNull()) {
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         createCategory(form, child, item);
         child = child.nextSiblingElement(::Constants::TAG_CATEGORY);
     }
     return item;
 }
 
-/** Save PMHx categories to the categories database. The pmhx category file must be already in the database. */
+/**
+ * Save PMHx categories to the categories database.
+ * The PMHx category file must be already in the database.
+*/
 bool XmlIOBase::savePmhxCategories(const XmlFormName &form, const QString &content)
 {
     Q_UNUSED(form);
+//    QTime chr;
+//    chr.start();
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (content.isEmpty()) {
         LOG_ERROR("Empty content.");
         return false;
@@ -713,7 +919,7 @@ bool XmlIOBase::savePmhxCategories(const XmlFormName &form, const QString &conte
     int col = -1;
     QString error;
     if (!doc.setContent(content, &error, &line, &col)) {
-        LOG_ERROR(QString("Error while oading PMHxCategories XML files.\n  %1: %2;%3").arg(error).arg(line).arg(col));
+        LOG_ERROR(QString("Error while loading PMHxCategories XML files.\n  %1: %2;%3").arg(error).arg(line).arg(col));
         return false;
     }
     QDomElement root = doc.firstChildElement(Constants::TAG_MAINXMLTAG);
@@ -724,22 +930,36 @@ bool XmlIOBase::savePmhxCategories(const XmlFormName &form, const QString &conte
         rootCategories << createCategory(form, element, 0);
         element = element.nextSiblingElement(::Constants::TAG_CATEGORY);
     }
-
+    // Utils::Log::logTimeElapsed(chr, "---", "create categories");
     // save categories in the categories plugin
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     if (!categoryCore()->saveCategories(rootCategories)) {
         LOG_ERROR(tr("Error while saving PMHxCateogries (%1)").arg(form.uid));
         return false;
     }
+    // Utils::Log::logTimeElapsed(chr, "---", "save categories");
     return true;
 }
 
-/** Save screenshots files associated with the forms. */
+/**
+ * Save screenshots files associated with the \e form.
+*/
 bool XmlIOBase::saveScreenShots(const XmlFormName &form)
 {
     QDir shotPath(form.absPath + QDir::separator() + "shots");
     if (shotPath.exists()) {
         LOG_FOR("XmlFormIO","Saving attached screenshots to database " + form.uid);
         QFileInfoList files = Utils::getFiles(shotPath, "*.png", Utils::Recursively);
+        if (files.isEmpty())
+            return true;
+
+        // Start a transaction
+        QSqlDatabase DB = database();
+        if (!connectedDatabase(DB, __LINE__))
+            return false;
+        DB.transaction();
+        _transaction = true;
+
         foreach(const QFileInfo &f, files) {
             QString fp = f.absoluteFilePath();
             QFile file(fp);
@@ -749,12 +969,42 @@ bool XmlIOBase::saveScreenShots(const XmlFormName &form)
             QString mode = fp.mid(begin, end-begin) + "/" + f.fileName();
             if (file.open(QFile::ReadOnly)) {
                 QByteArray ba = file.readAll();
-                if (!saveContent(form.uid, ba.toBase64(), XmlIOBase::ScreenShot, mode))
+                if (!saveContent(form.uid, ba.toBase64(), XmlIOBase::ScreenShot, mode)) {
+                    DB.rollback();
+                    _transaction = false;
                     return false;
+                }
+            }
+        }
+
+        DB.commit();
+        _transaction = false;
+    }
+    return true;
+}
+
+/** Register associated alertpacks for the \e form. */
+bool XmlIOBase::registerAlertPacks(const XmlFormName &form)
+{
+    Q_UNUSED(form)
+#ifndef WITH_ALERTS
+    // No alerts -> return true
+    return true;
+#else
+    QDir alertPacksPath(form.absPath + QDir::separator() + "alertpacks");
+    bool ok = true;
+    if (alertPacksPath.exists()) {
+        LOG_FOR("XmlFormIO","Saving attached alertpacks to database " + form.uid);
+        QFileInfoList files = Utils::getFiles(alertPacksPath, "packdescription.xml", Utils::Recursively);
+        foreach(const QFileInfo &f, files) {
+            if (!Alert::AlertCore::instance().registerAlertPack(f.absolutePath())) {
+                LOG_ERROR("Unable to read alertpack: "+ f.absolutePath());
+                ok = false;
             }
         }
     }
-    return true;
+    return ok;
+#endif
 }
 
 /** Save files associated with the forms. */
@@ -767,25 +1017,72 @@ bool XmlIOBase::saveFiles(const XmlFormName &form, const QString &subDir, const
     if (path.exists()) {
         LOG_FOR("XmlFormIO","Saving attached *." + fileExtension + " files to database " + form.uid);
         QFileInfoList files = Utils::getFiles(path, "*." + fileExtension, Utils::Recursively);
+        if (files.isEmpty())
+            return true;
+
+        // Start a transaction
+        QSqlDatabase DB = database();
+        if (!connectedDatabase(DB, __LINE__))
+            return false;
+        DB.transaction();
+        _transaction = true;
+
         foreach(const QFileInfo &f, files) {
+            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
             QString fp = f.absoluteFilePath();
             QString mode = fp;
             mode = "." + mode.remove(form.absPath);
-            if (!saveContent(form.uid, Utils::readTextFile(fp, Utils::DontWarnUser), type, mode))
-                return false;
+            if (type == PdfFile) {
+                // Save base64
+                QFile file(fp);
+                if (!file.open(QFile::ReadOnly)) {
+                    LOG_ERROR("Unable to open file: " + fp);
+                    DB.rollback();
+                    _transaction = false;
+                    return false;
+                }
+                QString content = file.readAll().toBase64();
+                if (!saveContent(form.uid, content, type, mode)) {
+                    DB.rollback();
+                    _transaction = false;
+                    return false;
+                }
+            } else {
+                // Save qstring content
+                if (!saveContent(form.uid, Utils::readTextFile(fp, Utils::DontWarnUser), type, mode)) {
+                    DB.rollback();
+                    _transaction = false;
+                    return false;
+                }
+            }
         }
+
+        DB.commit();
+        _transaction = false;
     }
     return true;
 }
 
-/** Save the content of an XML form content \e xmlContent into the database using the Form Uuid \e formUid, XmlIOBase::TypeOfForm \e type, Mode name \e modeName and \e date */
+/**
+ * Save the content of an XML form content \e xmlContent into the database
+ * using the Form Uuid \e formUid, XmlIOBase::TypeOfForm \e type,
+ * Mode name \e modeName and \e date
+ * This member can be called inside or outside a database transaction. If it is called
+ * inside a transaction it does not call neither QSqlDatabase::commit(), nor
+ * QSqlDatabase::rollback().
+*/
 bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, const int type, const QString &modeName, const QDateTime &date)
 {
     QSqlDatabase DB = database();
     if (!connectedDatabase(DB, __LINE__))
         return false;
 
-    DB.transaction();
+    bool insideTransaction = true;
+    if (!_transaction) {
+        insideTransaction = false;
+        DB.transaction();
+        _transaction = true;
+    }
     QSqlQuery query(DB);
     int formId = -1;
     QString req;
@@ -795,6 +1092,7 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
     }
     mode.remove(".xml");
 
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     QHash<int, QString> where;
     // create in form table ?
     where.insert(FORM_UUID, QString("='%1'").arg(normalizedFormUid(formUid)));
@@ -811,7 +1109,10 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
             formId = query.lastInsertId().toInt();
         } else {
             LOG_QUERY_ERROR(query);
-            DB.rollback();
+            if (!insideTransaction) {
+                DB.rollback();
+                _transaction = false;
+            }
             return false;
         }
         query.finish();
@@ -824,7 +1125,10 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
             }
         } else {
             LOG_QUERY_ERROR(query);
-            DB.rollback();
+            if (!insideTransaction) {
+                DB.rollback();
+                _transaction = false;
+            }
             return false;
         }
         query.finish();
@@ -833,7 +1137,7 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
     // update or insert content table ?
     where.clear();
     where.insert(FORMCONTENT_FORM_ID, QString("=%1").arg(formId));
-    where.insert(FORMCONTENT_TYPE, QString("=%1").arg(type));
+    where.insert(FORMCONTENT_TYPE, QString("='%1'").arg(type));
     where.insert(FORMCONTENT_MODENAME, QString("='%1'").arg(mode));
     where.insert(FORMCONTENT_CONTENT, "IS NOT NULL");
     nb = count(Table_FORM_CONTENT, FORMCONTENT_FORM_ID, getWhereClause(Table_FORM_CONTENT, where));
@@ -850,18 +1154,25 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
         query.bindValue(FORMCONTENT_CONTENT, xmlContent);
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
-            DB.rollback();
+            if (!insideTransaction) {
+                DB.rollback();
+                _transaction = false;
+            }
             return false;
         }
         query.finish();
 
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         // add the description
         if (mode.compare("central",Qt::CaseInsensitive)==0) {
             int beg = xmlContent.indexOf(QString("<%1>").arg(Constants::TAG_FORM_DESCRIPTION));
             int end = xmlContent.indexOf(QString("</%1>").arg(Constants::TAG_FORM_DESCRIPTION), beg);
             if (beg==-1 || end==-1) {
                 LOG_ERROR("no description found");
-                DB.rollback();
+                if (!insideTransaction) {
+                    DB.rollback();
+                    _transaction = false;
+                }
                 return false;
             }
             end += QString("</%1>").arg(Constants::TAG_FORM_DESCRIPTION).length();
@@ -877,7 +1188,10 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
             query.bindValue(FORMCONTENT_CONTENT, descr);
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
-                DB.rollback();
+                if (!insideTransaction) {
+                    DB.rollback();
+                    _transaction = false;
+                }
                 return false;
             }
             query.finish();
@@ -890,36 +1204,50 @@ bool XmlIOBase::saveContent(const QString &formUid, const QString &xmlContent, c
         query.bindValue(0, xmlContent);
         if (!query.exec()) {
             LOG_QUERY_ERROR(query);
-            DB.rollback();
+            if (!insideTransaction) {
+                DB.rollback();
+                _transaction = false;
+            }
             return false;
         }
         query.finish();
 
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         // update the description
         if (mode.compare("central",Qt::CaseInsensitive)==0) {
             int beg = xmlContent.indexOf(QString("<%1>").arg(Constants::TAG_FORM_DESCRIPTION));
             int end = xmlContent.indexOf(QString("</%1>").arg(Constants::TAG_FORM_DESCRIPTION), beg);
             if (beg==-1 || end==-1) {
                 LOG_ERROR("no description found");
-                DB.rollback();
+                if (!insideTransaction) {
+                    DB.rollback();
+                    _transaction = false;
+                }
                 return false;
             }
             end += QString("</%1>").arg(Constants::TAG_FORM_DESCRIPTION).length();
             QString descr = xmlContent.mid(beg, end-beg);
 
-            where.insert(FORMCONTENT_TYPE, QString("=%1").arg(Description));
+            where.insert(FORMCONTENT_TYPE, QString("='%1'").arg(Description));
             req = prepareUpdateQuery(Table_FORM_CONTENT, FORMCONTENT_CONTENT, where);
             query.prepare(req);
             query.bindValue(0, descr);
             if (!query.exec()) {
                 LOG_QUERY_ERROR(query);
-                DB.rollback();
+                if (!insideTransaction) {
+                    DB.rollback();
+                    _transaction = false;
+                }
                 return false;
             }
             query.finish();
         }
     }
-    DB.commit();
+    if (!insideTransaction) {
+        DB.commit();
+        _transaction = false;
+    }
+    qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     return true;
 }
 
